// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: db_worker.cc,v $ - directory server working thread
 *            implemented as a class.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/25 21:37:42 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <string>

# include "udp_srv.h"
# include "dir_handler.h"
# include "db_syncer.h"
# include "dir_pub.h"

# include "db_worker.h"

DB_Worker::DB_Worker(DirSrvConf &dconf) :
	srv(NULL),
	dirhandler(NULL),
	syncer(NULL),
	publisher(NULL)
{
	srv=new Udp_srv(NULL, dconf.port());
	if ((!srv)||(!srv->good())) {
		throw std::string("Cannot create UDP server.");
	}

	dirhandler=new DIR_Handler(srv, dconf.backstore(), dconf.ident());
	syncer=new DB_Syncer(5, dirhandler);
	publisher=new DirPublisher(dconf, srv->port());
}

int DB_Worker::get_port(void)
{
	return srv->port();
}

DB_Worker::~DB_Worker()
{
	stop();
	finalize();

	if (syncer)
		delete syncer;

	if (dirhandler)
		delete dirhandler;

	if (srv)
		delete srv;
}

bool DB_Worker::start()
{
	return (syncer->start() && publisher->start() && CBS_Thread::start());
}

bool DB_Worker::is_running()
{
	return (syncer->is_running() && publisher->is_running() && CBS_Thread::is_running());
}

void DB_Worker::stop(bool right_now)
{
	if (publisher->is_running()) {
		publisher->stop();
		if (!right_now)
			sleep(5);
	}

	if (syncer->is_running()) {
		syncer->stop();
	}

	if (CBS_Thread::is_running()) {
		CBS_Thread::stop();
	}

	srv->stop();
}

void DB_Worker::finalize()
{
	publisher->finalize();
	syncer->finalize();
	CBS_Thread::finalize();
}

void * DB_Worker::run(void)
{
	//printf("Entering the run() loop.\n");
	while (1) {
		Udp_srv_req *q=srv->wait_for_request();
		//printf("Got a request.\n");

		if (should_quit()) {
			if (q)
				delete q;

			break;
		}

		if (!q) {
			continue;
		}

		if (q->is_stop_req()) {
			delete q;
			stop();
			break;
		}

		dirhandler->handle_request(q);
	}
	//printf("Exiting the run() loop.\n");

	return NULL;
}
