// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_cli.cc,v $ - directory server client library.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/17 23:36:12 $
 * Version :  $Revision: 1.11 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <netinet/in.h>
# include <time.h>
# include "sock_conn.h"
/***************************************
 * The following are used for lookup() *
 ***************************************
# include "mcast_annc.h"
# include "../shared/daemon_common.h"
 ***************************************/
# include "udp_cli.h"
# include "dir_common.h"
# include "dir_cli.h"

SCS_DIR::SCS_DIR() : 
	srv_pt(0),
	dir_cli(NULL)
{
	ident[0]='\0';

	return;
}

SCS_DIR::SCS_DIR(char *dir=NULL, unsigned int flags) :
	srv_pt(0),
	dir_cli(NULL)
{
	srv_ad[0]='\0';
	if (dir) {
		change(dir, flags);
	}
}

SCS_DIR::~SCS_DIR()
{
	if (dir_cli)
		delete dir_cli;
}

/**
 * status code translation.
 */
static int udp_to_dir_status(int status)
{
	switch (status) {
		case UDP_RES_OKAY : return DIR_RES_OKAY;
		case UDP_RES_TOUT : return DIR_RES_ETOUT;
		case UDP_RES_SYST : return DIR_RES_ENETWORK;
		case UDP_RES_PORT :
		case UDP_RES_ADDR :
		case UDP_RES_LENG : return DIR_RES_EDATA;
		default			  : return DIR_RES_ECLIENT;
	}
}

int SCS_DIR::check_udp_cli()
{
	if (!dir_cli) { // not initialized yet.
		dir_cli=new Udp_cli("", 0, 1000); // set timeout to be 1 second.
		if ((!dir_cli)||(!dir_cli->good())) {
			if (dir_cli) {
				delete dir_cli;
				dir_cli=NULL;
			}
			return -1;
		}
	}

	return 0;
}

DIR_REQ *SCS_DIR::new_req_ident(int *len)
{
	if (!len)
		return NULL;

	*len=DIR_REQ_HEADER_SZ+sizeof(DIR_REQ_IDENT);
	DIR_REQ *ret=(DIR_REQ *)malloc(*len);
	if (!ret)
		return NULL;
	ret->type=DIR_REQ_TYPE_IDENT;

	return ret;
}

DIR_RES *SCS_DIR::new_res_dummy()
{
	DIR_RES *res=(DIR_RES *)malloc(sizeof(DIR_RES));;

	if (res)
		memset(res, 0, sizeof(DIR_RES));
	return res;
}

// The ident of the Sorrento deployment serviced by 
// the current directory server.
int SCS_DIR::get_ident(char *id, int len)
{
	if ((!ident)) {
		return DIR_RES_EINVALID;
	}

	if (check_udp_cli()) {
		return DIR_RES_ECLIENT;
	}

	// has queried before.
	if (ident[0] != '\0') {
		strncpy(id, ident, len-1);
		id[len-1]='\0';

		return DIR_RES_OKAY;
	}

	// construct a request.
	int req_len;
	DIR_REQ *req=new_req_ident(&req_len);
	if (!req) {
		return DIR_RES_EMEM;
	}

	Udp_res **ress=dir_cli->sync_req(srv_ad, srv_pt, req, req_len, 10);
	free(req);

	int ret=DIR_RES_OKAY;
	DIR_RES *res=NULL;

	if ((!ress)||(!ress[0])) {
		ret=DIR_RES_EMEM;
		goto _exit;
	}

	if ((ress[0]->status!=UDP_RES_OKAY)) {
		ret=udp_to_dir_status(ress[0]->status);
		goto _exit;
	}

	res=(DIR_RES *)ress[0]->result();

	if (res->status!=DIR_RES_OKAY) {
		ret=res->status;
		goto _exit;
	}

	strncpy(id, res->u.d_ident.ident, len-1);
	id[len-1]='\0';

	strncpy(ident, res->u.d_ident.ident, 127);
	ident[127]='\0';

_exit:
	if (ress)
		dir_cli->free_results(ress);

	return ret;
}

void SCS_DIR::free_res(DIR_RES *res)
{
	free(res);
}

DIR_RES * SCS_DIR::hook_common(DIR_REQ *req, int req_len)
{
	DIR_RES *res=NULL;

	if (check_udp_cli()) {
		res=new_res_dummy();
		if (res)
			res->status=DIR_RES_ECLIENT;
		return res;
	}

	Udp_res **ress=dir_cli->sync_req(srv_ad, srv_pt, req, req_len, 10);

	if ((!ress)||(!ress[0])) {
		res=new_res_dummy();
		if (res)
			res->status=DIR_RES_EMEM;
		goto _exit;
	}

	if (ress[0]->status!=UDP_RES_OKAY) {
		res=new_res_dummy();
		if (res)
			res->status=udp_to_dir_status(ress[0]->status);
		goto _exit;
	}

	res=(DIR_RES *)malloc(ress[0]->length());

	if (!res) {
		goto _exit;
	}

	memcpy(res, ress[0]->result(), ress[0]->length());;

_exit:
	if (ress) {
		dir_cli->free_results(ress);
	}

	return res;
}

// path must have already been standardized
DIR_RES * SCS_DIR::hook_query(DIR_REQ *req, int req_len)
{
	return hook_common(req, req_len);
}

DIR_RES * SCS_DIR::hook_del(DIR_REQ *req, int req_len)
{
	return hook_common(req, req_len);
}

DIR_RES * SCS_DIR::hook_setp(DIR_REQ *req, int req_len)
{
	return hook_common(req, req_len);
}

int SCS_DIR::change_server(char *n_host, int n_port)
{
	if (n_port<=0) {
		n_port=DIRSRV_PORT_DEFAULT;
	}

	if (!n_host) {
		n_host=srv_ad;
	}
	else {
		// convert to IP address.
		struct sockaddr_in sin;
		if (sock_filladdr(n_host, n_port, &sin)<0) {
			return DIR_RES_EINVALID; // bad address
		}

		if (sock_dotaddr(n_host, 16, &(sin.sin_addr))<0) {
			return DIR_RES_EINVALID;
		}
	}

	if ( (strcmp(n_host, srv_ad)) || (n_port!=srv_pt) ) { // server change
		// new server, starts at ROOT. must invalidate all the cached stuff.		
		nid=ROOT;
		strcpy(pwd,"/");
		cur_param.reset();
		child=ERR;
		parent=ROOT;
		next_node=ERR;
		strcpy(srv_ad, n_host);
		srv_pt=n_port;
		ident[0]='\0';
	}

	return DIR_RES_OKAY;
}
