// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_cli_ap_file.cc,v $ - the Directory Service 
 *            Client access point through direct file access.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/18 23:25:07 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include "dir_cli_ap_file.h"

/** The constructor */
DIR_Cli_AP_FILE::DIR_Cli_AP_FILE(DIR_DB *db) :
	dir_db(db)
{ }

/** The destructor */
DIR_Cli_AP_FILE::~DIR_Cli_AP_FILE() 
{ }

/** Create response structures, common routine. */
DIR_RES *DIR_Cli_AP_FILE::new_res_common(int len)
{
	DIR_RES *ret=(DIR_RES *)calloc(1, len);

	if (!ret)
		return NULL;

	return ret;
}

/** Create query response structures. */
DIR_RES *DIR_Cli_AP_FILE::new_res_query(int buf_len, int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+DIR_RES_QUERY_HEADER_SZ+((buf_len<=0)?1:buf_len);

	DIR_RES *ret=new_res_common(*len);

	if (ret)
		ret->u.d_query.buf_len=buf_len;

	return ret;
}

/** Create del response structures. */
DIR_RES *DIR_Cli_AP_FILE::new_res_del(int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+sizeof(DIR_RES_DEL);
	return new_res_common(*len);
}

/** Create setp response structures. */
DIR_RES *DIR_Cli_AP_FILE::new_res_setp(int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+sizeof(DIR_RES_SETP);
	return new_res_common(*len);
}

/** free the response structure */
void DIR_Cli_AP_FILE::free_response(DIR_RES *res)
{
	free(res);
}

/** delegate the request handling */
DIR_RES *DIR_Cli_AP_FILE::request(DIR_REQ *req, int req_len)
{
	DIR_RES *res=NULL;
	int res_len;

	switch (req->type) {
	 	case DIR_REQ_TYPE_QUERY: 
			{
				res=new_res_query(req->u.d_query.buf_len, &res_len);
				if (res) {
					res->status=dir_db->query((&req->u.d_query), &(res->u.d_query));
				}
				break;
			}
		case DIR_REQ_TYPE_DEL:
			{
				res=new_res_del(&res_len);
				if (res) {
					res->status=dir_db->del((&req->u.d_del), &(res->u.d_del));
				}
				break;
			}
		case DIR_REQ_TYPE_SETP:
			{
				res=new_res_setp(&res_len);
				if (res) {
					res->status=dir_db->setp((&req->u.d_setp), &(res->u.d_setp));
				}
				break;
			}
		default:
			{
				res=new_res_common(sizeof(DIR_RES));
				if (res) {
					res->status=DIR_RES_EINVALID;
				}
				break;
			}
	}

	return res;
}
