// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_cli_ap_file.h,v $ - the Directory Service 
 *            Client access point through direct file access.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/18 23:25:07 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _DIR_CLI_AP_FILE_H_
# define _DIR_CLI_AP_FILE_H_

# include "dir_cli_ap.h"
# include "dirdb.h"
# include <string>

/**
 * Directory service Client Access Point using file: It is a subclass of 
 * DIR_Cli_AP.
 */
class DIR_Cli_AP_FILE : public DIR_Cli_AP {
 private:
	DIR_DB *dir_db; /**< The dirdb object. */
	DIR_RES *new_res_common(int len);
	DIR_RES *new_res_query(int buf_len, int *len);
	DIR_RES *new_res_del(int *len);
	DIR_RES *new_res_setp(int *len);

 public:
	DIR_Cli_AP_FILE(DIR_DB *db); /**< The constructor. */
	virtual ~DIR_Cli_AP_FILE(); /**< The destructor */
	/** sending a request and wait for the response. */
	virtual DIR_RES *request(DIR_REQ *req, int req_len); 
	virtual void free_response(DIR_RES *res);
	/** Get the identity of the directory server */
	virtual int get_ident(const char * &id)
	{
		if (!dir_db)
			return DIR_RES_ESERVER;

		id=dir_db->get_ident();
		return 0;
	}

	virtual int get_protocol(const char * &proto)
	{
		proto = "file://";
		return 0;
	}
};

# endif // _DIR_CLI_AP_FILE_H_
