// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_cli_base.h,v $ - header file for directory server common
 *            client functionality API. It is the base for both
 *            remote client library and in-process client library.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/08/01 11:21:20 $
 * Version :  $Revision: 1.11 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _DIR_CLI_BASE_H_
# define _DIR_CLI_BASE_H_

# include <string>

# include "dir_common.h"
// # include "dirdb.h"

# if defined(__CHECK_MEM__)
# include "asmalloc.h"
# endif

class DIR_Base {
public:
	typedef dir_node_t node_t;
	static const node_t NIL; /* 0 */
	static const node_t ROOT; /* 1 */
	static const node_t ERR; /* 2 */
	static const int MAX_NAME_LEN;
	static const int MAX_PATH_LEN;

protected:
	node_t nid;
	char pwd[DIR_MAX_PATH_LEN]; // current absolute working direcoty
	DIR_Param cur_param; // parameters for the current directory

	time_t expire; // the two cached values are only valid for 60 seconds.
	node_t child; // cached attributes
	node_t parent; // cached attributes
	node_t next_node; // only used by list() when path==NULL.

	struct QUERY_RES {
		int status;
		node_t nid;
		node_t parent;
		node_t child;
		node_t next;
		DIR_Param param;
	};

	int query_(char *node_path, QUERY_RES *meta, char *buf, int *len, unsigned int flags);

	DIR_REQ *new_req_query(char *path, int buf_len, unsigned int flags, int *len);
	DIR_REQ *new_req_del(char *path, unsigned int flags, int *len);
	DIR_REQ *new_req_setp(char *path, DIR_Param *p, int *len);
	int do_local_cd_dotdot();
	void check_expire(bool force_expire=false);
	void set_expire();
	bool is_valid_local_path(char *path);
	virtual DIR_RES *hook_query(DIR_REQ *req, int req_len)=0;
	virtual DIR_RES *hook_del(DIR_REQ *req, int req_len)=0;
	virtual DIR_RES *hook_setp(DIR_REQ *req, int req_len)=0;
	virtual int change_server(char *s, int p)=0;
	virtual void free_res(DIR_RES *res)=0;

public:
	DIR_Base();
	virtual ~DIR_Base() { return; }
	int change(char *dir=NULL, unsigned int flags=0);
	int list(char *node, DIR_Param *nparam, char *buf, int *len, unsigned int flags=0);
	int del(char *node, unsigned int flags=0);
	int set_param(char *node, DIR_Param *p);
	int read_list(char *buf, int len, char **p, char **iname, DIR_Param *iparam);
	char *cwd() { if (pwd[0]=='\0') return NULL; return pwd; }
	int get_param(DIR_Param *pm); // get the parameters of the current directory.
	virtual bool good()=0; /* virtuals */
	void sync() { check_expire(true); }
	virtual int get_ident(char *id, int len)=0;
};

# endif
