// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_cli_opt.h,v $ - header file for directory server client 
 *            command-line utlities' option parser.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.6 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _DIR_CLI_OPT_H_
# define _DIR_CLI_OPT_H_

# include <stdlib.h>
# include "dir_common.h"
# include "tmpi_debug.h"

# if defined(__CHECK_MEM__)
# include "asmalloc.h"
# endif

struct DIR_Cli_opt {
public:
	char *idir;
	char *bsf;
	char *srv;
	int port;
	bool creat;
	bool excl;
	bool recur;

private:
	enum {
		IDIR_INIT=0x1, 
		IDIR_FREE=0x2, 
		BSF_INIT=0x4, 
		BSF_FREE=0x8, 
		SRV_INIT=0x10, 
		SRV_FREE=0x20,
		PORT_INIT=0x40,
		CREAT_INIT=0x80,
		EXCL_INIT=0x100,
		RECUR_INIT=0x200
	};

	int flag;

# define IDIR_DEFAULT "/"
# define BSF_DEFAULT ""
# define SRV_DEFAULT ""
# define PORT_DEFAULT DIRSRV_PORT_DEFAULT

	int set_var(char *s, bool cpref, bool set_init, char *&var, int init_mask, int free_mask);
	bool is_valid_num(char *p);
	int set_bool(char *b, bool set_init, bool &boolean, int init_mask);
	int set_num(char *n, bool set_init, int &num, int init_mask);

public:
	DIR_Cli_opt() : 
		idir(IDIR_DEFAULT),
		bsf(BSF_DEFAULT),
		srv(SRV_DEFAULT),
		port(PORT_DEFAULT),
		creat(false),
		excl(false),
		recur(false),
		flag(0)
	{
		return;
	}

	~DIR_Cli_opt(); 

	int set_idir(char *s, bool set_init, bool cpref)
	{
		if (s[0]!='/') {
			tmpi_error(DBG_CALLER, "Initial directory must be an absolute path.\n");
			return -1;
		}

		return set_var(s, cpref, set_init, idir, IDIR_INIT, IDIR_FREE);
	}
	
	int set_bsf(char *s, bool set_init, bool cpref)
	{
		return set_var(s, cpref, set_init, bsf, BSF_INIT, BSF_FREE);
	}

	int set_srv(char *s, bool set_init, bool cpref)
	{
		return set_var(s, cpref, set_init, srv, SRV_INIT, SRV_FREE);
	}

	int set_port(char *p, bool set_init=true, bool dummy=true)
	{
		return set_num(p, set_init, port, PORT_INIT);
	}

	int set_creat(char *p, bool set_init=true, bool dummy=true)
	{
		return set_bool(p, set_init, creat, CREAT_INIT);
	}

	int set_excl(char *p, bool set_init=true, bool dummy=true)
	{
		return set_bool(p, set_init, excl, EXCL_INIT);
	}

	int set_recur(char *p, bool set_init=true, bool dummy=true)
	{
		return set_bool(p, set_init, recur, RECUR_INIT);
	}

	// for configuration file parsing
	int set_var_val(char *var, char *val);
};

void dump_dir_cli_opt(DIR_Cli_opt *opt);

void dir_cli_usage(int status);

void parse_dir_cli_opt(int argc, char *argv[], DIR_Cli_opt *opt, 
		void (*version)());

int parse_dir_cli_config(DIR_Cli_opt *opt, char *config);

# endif
