// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_cli_simple.h,v $ - header file for the simple directory server client
 *            which access the DB directly as a local private file.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/17 23:36:13 $
 * Version :  $Revision: 1.12 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _DIR_CLI_SIMPLE_H_
# define _DIR_CLI_SIMPLE_H_

# include "dir_common.h"
# include "dirdb.h"
# include "dir_cli_base.h"

# if defined(__CHECK_MEM__)
# include "asmalloc.h"
# endif

/**
 * The directory db client library
 */

class DIR : public DIR_Base {
 private:
	DIR_DB *dir_db;

	DIR_RES *new_res_common(int len);
	DIR_RES *new_res_query(int buf_len, int *len);
	DIR_RES *new_res_del(int *len);
	DIR_RES *new_res_setp(int *len);
	void inc_cnt();
	int req_cnt;

 protected:
	virtual DIR_RES *hook_query(DIR_REQ *req, int req_len);
	virtual DIR_RES *hook_del(DIR_REQ *req, int req_len);
	virtual DIR_RES *hook_setp(DIR_REQ *req, int req_len);
	virtual int change_server(char *s, int p);
	virtual void free_res(DIR_RES *res);

 public:
	DIR(DIR_DB *db, char *dir=NULL, unsigned int flags=0);
	virtual ~DIR() { return; }
	virtual bool good() { if (dir_db) return true; return false; }
	virtual int get_ident(char *id, int len)
	{
		if ((!id)||(len<=0))
			return DIR_RES_EINVALID;

		if (!dir_db)
			return DIR_RES_ESERVER;

		strncpy(id, dir_db->get_ident(), len-1);
		id[len-1]='\0';

		return DIR_RES_OKAY;
	}
};

# endif
