// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_common.cc,v $ - directory server/client comman facilities.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/19 08:20:45 $
 * Version :  $Revision: 1.11 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <string.h>
# include <ctype.h>
# include <stdlib.h>
# include "dir_common.h"

# if defined(__DEBUG__)
# include "asmalloc.h"
# endif

static bool is_host_char(int c)
{
	if (isalnum(c)||(c=='.'))
		return true;
	return false;
}

/**
 * Format: [//host[:port]:]path (':' is not allowed in the path)
 * note the second semi-colon is introduced to avoid
 * the ambiguity of possible redundant "/".
 * when host is present, path must be absolute path.
 * On error: 0
 * Otherwise, return a bit-wise OR of the following flags:
 * 	DIR_PATH_LPATH, DIR_PATH_PORT, DIR_PATH_HOST
 */
int dir_parse_path(char *path, char *host, int len, int *port, char *lpath, int len2)
{
	if (!path)
		return 0;

	char *begin_host=NULL;
	char *end_host=NULL;
	char *begin_port=NULL;
	char *end_port=NULL;
	char *begin_path=path;
	char *end_path=path+strlen(path);
	int ret=0;
	
	if ((path[0]=='/')&&(path[1]=='/')&&(strchr(path, ':'))) {
		end_host=begin_host=path+2;
		while (*end_host) {
			if (is_host_char(*end_host))
				end_host++;
			else {
				break;
			}
		}
		if (*end_host=='\0')
			return 0;

		if (end_host==begin_host) {
			return 0;
		}

		begin_path=end_host;

		if ((*end_host!=':'))
				return 0;

		if (strchr(begin_path+1, ':')) {
			end_port=begin_port=end_host+1;
			while (*end_port) {
				if (isdigit(*end_port))
					end_port++;
				else 
					break;
			}

			if (*end_port=='\0')
				return 0;

			if (end_port==begin_port) {
				return 0;
			}

			begin_path=end_port;
		}

		if (*begin_path!=':')
			return 0;

		begin_path++;

		// ":" is illegal in the path
		if (strchr(begin_path, ':')) {
				return 0;
		}

		// if (*begin_path!='/')
		//	return 0;
	}

	if ((end_path-begin_path)<=0)
		return 0;

	if (begin_host) {
		int hlen=end_host-begin_host;
		if (hlen>=len)
			return 0;
		memcpy(host, begin_host, hlen);
		host[hlen]='\0';
		ret|=DIR_PATH_HOST;
	}

	if (begin_port) {
		*port=atoi(begin_port);
		ret|=DIR_PATH_PORT;
	}

	if (begin_path) {
		int plen=end_path-begin_path;
		if (plen>=len2)
			return 0;
		memcpy(lpath, begin_path, plen);
		lpath[plen]='\0';
		ret|=DIR_PATH_LPATH;
	}

	return ret;
}

static int cd_parent(char *beginpath, char *endpath, char **p)
{
	char *q=*p;
	char *r;

	// handle the empty path case
	if (q==beginpath) {
		if ((endpath-beginpath)>2) {
			memcpy(beginpath, "..", 2);
			q+=2;
			goto _exit;
		}
		else return -1;
	}

	// handle the root case: parent(/) == /
	if (((q-beginpath)==1)&&(beginpath[0]=='/')) {
		// q=beginpath+1;
		goto _exit;
	}

	// atempt to go back
	// find the previous path component
	r=q-1;
	while (r>=beginpath) {
		if (*r=='/')
			break;
		r--;
	}

	if (r>=beginpath) {
		if (((q-r)==3)&&(r[0]=='/')&&(r[1]=='.')&&(r[2]=='.')) {
			// nothing
			;
		}
		else {
			q=r;
			if (q==beginpath)
				q++;
			goto _exit;
		}
	}
	else { // only one component, 
		if (((q-beginpath)==2)&&(beginpath[0]=='.')&&(beginpath[1]=='.')) {
			// the one component is ..
		}
		else { 
			// get rid of it.
			q=beginpath;
			goto _exit;
		}
	}

	if ((endpath-q)>3) {
		*q++='/';
		*q++='.';
		*q++='.';
		goto _exit;
	}
	else {
		return -1;
	}

_exit:
	*p=q;
	return 0;
}

int dir_std_path(const char *base, const char *path, char *std_path, int len)
{
	// parameter checking
	if ((!path)||(!std_path)||(len<=1)) {
		return -1;
	}

	// base must be an absolute path
	if (base&&(base[0]!='/')) {
		return -1;
	}

	char *p=std_path;
	char *endp=std_path+len;

	// Initialization:
	// Purpose: 
	// path will be pointing to the first none-'/'
	// std_path holds the prefix of the final path
	// p points to the next char after the prefix.
	// The prefix should not end with / unless it
	// is the root.
	//
	// If original path is relative, use base
	// as the prefix and no change to path
	// Otherwise, set / as the prefix and change path
	// to skip the initial '/'s.
	//
	if (path[0]=='/') { // absolute path, replace.
		std_path[0]='/';
		p++;
		while (*path) {
			if (*path=='/')
				path++;
			else
				break;
		}
	}
	else if (base) {
		int blen=strlen(base);
		if (len<=blen)
			return -1;
		strncpy(std_path, base, blen); // asumming base is standardized
		p+=blen;
		if ((*(p-1)=='/')&&((p-1)>std_path)) { // get rid of the trailing '/'
			p--;
		}
	}

	const char *bp=path;
	const char *endpath=path+strlen(path);

	while ((bp<endpath)) {

		// bp might point to a / when the loop runs over.
		while ((*bp)&&(*bp=='/'))
			bp++;

		// search for the next componet in the path starting with bp.
		// bp points to a char other than '/'.

		const char *ep=bp;
		while (*ep) {
			if (*ep=='/') {
				break;
			}
			ep++;
		}
		if ((ep-bp)==0) { // seems we've been to the end.
			// bp=ep+1;
		}
		else if (((ep-bp)==1)&&(bp[0]=='.')) {
			// bp=ep+1;
		}
		else if (((ep-bp)==2)&&(bp[0]=='.')&&(bp[1]=='.')) {
			if (cd_parent(std_path, endp, &p)<0)
				return -1;
		}
		else {
			if (p>std_path) {
				if (*(p-1)=='/') { // add a slash or not? root (no slash)!
					if ((endp-p)<=(ep-bp)) {
						return -1;
					}
				}
				else {
					if ((endp-p)<=(ep-bp+1)) {
						return -1;
					}
					*p='/';
					p++;
				}
			}
			else {
				// epsilon component.
				if ((endp-p)<=(ep-bp)) {
					return -1;
				}
			}
			memcpy(p, bp, ep-bp);
			p+=(ep-bp);
		}

		bp=ep+1;
	}

	if (p==std_path) {
		*p++='.';
	}

	*p='\0';

	return 0;
}

int dir_opt_path(const char *base, const char *std_path, char *opt_path, int len)
{
	// parameter checking
	if ((!base)||(!std_path)||(!opt_path)||(len<=1)) {
		return -1;
	}

	// base must be an absolute path
	if (base&&(base[0]!='/')) {
		return -1;
	}

	char abs_path[DIR_MAX_PATH_LEN];
	if (std_path[0]=='/') {
		strncpy(abs_path, std_path, DIR_MAX_PATH_LEN-1);
		abs_path[DIR_MAX_PATH_LEN-1]='\0';
	}
	else {
		if (dir_std_path(base, std_path, abs_path, DIR_MAX_PATH_LEN)) {
			return -1;
		}
	}

	int base_len=strlen(base);
	char *rel_path=NULL;
	if (strncmp(base, abs_path, base_len)==0) {
		rel_path=abs_path+base_len;
		if (*rel_path == '\0') {
		}
		else if (*rel_path == '/') {
			rel_path++;
		}
		else { /* not really a match */
			rel_path=NULL;
		}
	}

	if (rel_path) {
		if (*rel_path) {
			// use rel_path
			int rel_len=strlen(rel_path);
			if (len<=rel_len)
				return -1;
			strcpy(opt_path, rel_path);
		}
		else {
			// "."
			strcpy(opt_path, ".");
		}
	}
	else {
		if (std_path[0]=='/') {
			// I do not try to reverse construct relative path
			// unless abs_path happens to be a prefix of base.
			bool done=false;
			int abs_len=strlen(abs_path);

			if (strncmp(abs_path, base, abs_len)==0) {
				const char *p=base+abs_len;
				if ((*p!='\0') && (*p!='/' ) ) {
					// false alarm
				}
				else {
					done=true;
					int par_levels=0;
					while (p&&*p) {
						if (*p=='/')
							p++;
						else {
							par_levels++;
							p=strchr(p, '/');
						}
					}
					if (len<3*par_levels) {
						return -1;
					}
					char *q=opt_path;
					bool first_time=true;
					while (par_levels) {
						if (!first_time)
							*q++='/';
						else
							first_time=false;
						*q++='.';
						*q++='.';
						par_levels--;
					}
					*q='\0';
				}
			}

			if (!done) {
				// giving up, use absolute.
				int abs_len=strlen(abs_path);
				if (len<=abs_len)
					return -1;
				strcpy(opt_path, abs_path);
			}
		}
		else {
			// destination is not a sub-node of base
			// compare the cost of a relative approach
			// and absolute approach.
			// for absolute path: 1+2x
			// for relative path: c+2x
			// where x is the number of regular components
			// c is the number of ".." (parent) components
			// in a normalized relative path, ".." must be
			// at the beginning of the path.
			int abs_cost=1;
			int rel_cost=0;
			{
				const char *p=abs_path;
				while ((p=strchr(p, '/'))) {
					abs_cost+=2;
					p++;
					if (*p=='\0') { /* trailing '/' */
						abs_cost-=2;
						break;
					}
				}
			}

			{
				const char *p=std_path;
				while ((p[0]=='.')&&(p[1]=='.')) {
					if ((*(p+2)!='\0')&&(*(p+2)!='/'))
						break;
					rel_cost++;
					p+=2;
					if (*p=='/')
						p++;
				}

				if (*p) // cannot be "/"
					rel_cost+=2;

				while ((p=strchr(p, '/'))) {
					rel_cost+=2;
					p++;
					if (*p=='\0') { /* trailing '/' */
						rel_cost-=2;
						break;
					}
				}
			}

			{
				const char *p=NULL;
				/* if it is a tie, we favor for relative path */
				if (abs_cost<rel_cost) 
					p=abs_path;
				else
					p=std_path;
				int opt_len=strlen(p);
				if (len<=opt_len)
					return -1;

				strcpy(opt_path, p);
			}
		}

	}

	return 0;
}
