// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_handler.cc,v $ - directory server server-side bridging handler.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/08/01 08:25:39 $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include "tmpi_debug.h"
# include "udp_srv.h"
# include "dirdb.h"
# include "dir_common.h"
# include "dir_handler.h"

int DIR_Handler::handle_request(Udp_srv_req *q)
{
	DIR_REQ *req=(DIR_REQ *)(q->request());
	int ret = DIR_RES_OKAY;
	switch (req->type) {
		case DIR_REQ_TYPE_IDENT:
			ret = handle_ident(q, (DIR_REQ_IDENT *)&(req->u.d_ident));	
			break;
		case DIR_REQ_TYPE_QUERY:
			ret = handle_query(q, (DIR_REQ_QUERY *)&(req->u.d_query));	
			break;
		case DIR_REQ_TYPE_DEL:
			ret = handle_del(q, (DIR_REQ_DEL *)&(req->u.d_del));
			break;
		case DIR_REQ_TYPE_SETP:
			ret = handle_setp(q, (DIR_REQ_SETP *)&(req->u.d_setp));	
			break;
		default:
			tmpi_error(DBG_APP, "Unknown request type %d.\n", req->type);
			ret = -1;
	}

	delete q;

	return ret;
}

DIR_RES * DIR_Handler::new_res_common(int len)
{
	DIR_RES *ret=(DIR_RES *)malloc(len);

	if (!ret)
		return NULL;

	memset(ret, 0, len);

	return ret;
}

DIR_RES * DIR_Handler::new_res_ident(int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+ident.length()+1;

	return new_res_common(*len);
}

DIR_RES * DIR_Handler::new_res_query(int buf_len, int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+DIR_RES_QUERY_HEADER_SZ+((buf_len<=0)?1:buf_len);

	DIR_RES *ret=new_res_common(*len);

	if (ret)
		ret->u.d_query.buf_len=buf_len;

	return ret;
}

DIR_RES * DIR_Handler::new_res_del(int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+sizeof(DIR_RES_DEL);
	return new_res_common(*len);
}

DIR_RES * DIR_Handler::new_res_setp(int *len)
{
	if (!len)
		return NULL;

	*len=DIR_RES_HEADER_SZ+sizeof(DIR_RES_SETP);
	return new_res_common(*len);
}

int DIR_Handler::handle_ident(Udp_srv_req *q, DIR_REQ_IDENT *req)
{
	int len;
	DIR_RES *res=new_res_ident(&len);

	if (!res) {
		tmpi_error(DBG_APP, "Out of memory!\n");
		return -1;
	}

	res->status=DIR_RES_OKAY;
	strcpy(res->u.d_ident.ident, ident.c_str());
	srv->respond(q, res, len);
	free(res);

	return 0;
}

int DIR_Handler::handle_query(Udp_srv_req *q, DIR_REQ_QUERY *req)
{
	int len;
	DIR_RES *res=new_res_query(req->buf_len, &len);

	if (!res) {
		tmpi_error(DBG_APP, "Out of memory!\n");
		return -1;
	}

	res->status=dirdb->query(req, &(res->u.d_query));
	len=DIR_RES_HEADER_SZ+DIR_RES_QUERY_HEADER_SZ+res->u.d_query.buf_len;
	srv->respond(q, res, len);
	free(res);

	return 0;
}

int DIR_Handler::handle_del(Udp_srv_req *q, DIR_REQ_DEL *req)
{
	int len;
	DIR_RES *res=new_res_del(&len);

	if (!res) {
		tmpi_error(DBG_APP, "Out of memory!\n");
		return -1;
	}

	res->status=dirdb->del(req, &(res->u.d_del));
	srv->respond(q, res, len);
	free(res);

	return 0;
}

int DIR_Handler::handle_setp(Udp_srv_req *q, DIR_REQ_SETP *req)
{
	int len;
	DIR_RES *res=new_res_setp(&len);

	if (!res) {
		tmpi_error(DBG_APP, "Out of memory!\n");
		return -1;
	}

	res->status=dirdb->setp(req, &(res->u.d_setp));
	srv->respond(q, res, len);
	free(res);

	return 0;
}
