// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_handler.h,v $ - header file for server-side directory server
 *            request handling.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/16 06:40:22 $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _DIR_HANDLER_H_
# define _DIR_HANDLER_H_

# include <string>

# include "udp_srv.h"
# include "dirdb.h"

/**
 * This is the glue class that takes UDP requests, 
 * operates on a dirdb object, and return back UDP 
 * responses.
 */

class DIR_Handler {
private:
	std::string ident;
	Udp_srv *srv;
	DIR_DB *dirdb;
	
	DIR_RES * new_res_common(int len);
	DIR_RES * new_res_ident(int *len);
	DIR_RES * new_res_query(int buf_len, int *len);
	DIR_RES * new_res_del(int *len);
	DIR_RES * new_res_setp(int *len);

	int handle_ident(Udp_srv_req *q, DIR_REQ_IDENT *req);
	int handle_query(Udp_srv_req *q, DIR_REQ_QUERY *req);
	int handle_del(Udp_srv_req *q, DIR_REQ_DEL *req);
	int handle_setp(Udp_srv_req *q, DIR_REQ_SETP *req);

public:
	DIR_Handler(Udp_srv *s, char *db_file, char *id): 
		ident(id),
		srv(s),
		dirdb(NULL)
	{ 
		dirdb=new DIR_DB(db_file); 
		if ((!dirdb)||(!dirdb->good())) {
			throw std::string("Cannot create directory database: ")+db_file;
		}
	}

	// a sink, request will be freed after being handled.
	int handle_request(Udp_srv_req *q); 
	void sync_db() { dirdb->sync(); }
	~DIR_Handler() { delete dirdb; }
};

# endif
