// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_pub.cc,v $ - dir_srv publisher.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/25 22:37:43 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
/**
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <unistd.h>
# include <stdio.h>
# include <mntent.h>
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
**/
# include <netinet/in.h>
# include "tmpi_debug.h"
# include "sock_conn.h"
# include "../shared/daemon_common.h"
# include "dirsrv_conf.h"
# include "dir_pub.h"

DirPublisher::DirPublisher(DirSrvConf &dconf, int port) : 
	ident(NULL),
	ancer(NULL),
	interval(1000),
	lis_port(port)
{
	ident=dconf.ident();
	if (!ident) {
		throw "No identity string defined.";
	}

	// char *home=dconf.home_dir();
	// if (!home) {
		// throw "Cannot determine Dir_srv home.";
	// }

	// char host_id_file[1024];
	// sprintf(host_id_file, "%s/etc/host_id.%s", home, ident);

	// free(home);

	// if (access(host_id_file, R_OK)) {
		// host-ID file not found, create it.
		// int fd=open(host_id_file, O_CREAT|O_EXCL|O_WRONLY, S_IRUSR|S_IRGRP);
		// if (fd<0) {
			// throw "Failed to create host-ID file: permission denied.";
		// }

		// Guid guid;
		// guid.gen();
		// guid.pack(&host_id, sizeof(HostID));

		// char guid_str[128];
		// guid.to_string(guid_str, 128);
		// write(fd, guid_str, strlen(guid_str));
		// close(fd);
	// }
	// else {
		// int fd=open(host_id_file, O_RDONLY);
		// if (fd<0) {
			// throw "Failed to read host-ID file: permission denied.";
		// }

		// char guid_str[128];
		// int n=read(fd, guid_str, 127);

		// close(fd);

		// guid_str[n]='\0';

		// Guid guid;
		// guid.parse(guid_str);
		// guid.pack(&host_id, sizeof(HostID));
	// }

	char *mc_group=dconf.mc_group();
	int mc_port=dconf.mc_port();

	struct sockaddr_in sin;

	if ((!mc_group)||(mc_port<=0)) {
		tmpi_error(DBG_CALLER, "Bad parameters: mc_group=%s, mc_port=%d.\n", mc_group, mc_port);
		throw "Bad parameters to Publisher()";
	}

	if (sock_filladdr(mc_group, mc_port, &sin)<0) {
		tmpi_error(DBG_CALLER, "Bad address: %s:%d.\n", mc_group, mc_port);
		throw "Bad parameters to Publisher()";
	}

	if (sock_is_mcast_addr(&sin, sizeof(sin))!=1) {
		tmpi_error(DBG_CALLER, "Bad multi-cast address: %s:%d.\n", mc_group, mc_port);
		throw "Bad parameters to Publisher()";
	}

	ancer=new Mc_ancer(mc_group, mc_port);

	free(mc_group);

	if (!ancer->good()) {
		tmpi_error(DBG_CALLER, "Mc_ancer() failed.\n");
		throw "Cannot create multicast publisher.";
	}

	interval=dconf.mc_interval();

	return;
}

DirPublisher::~DirPublisher(void)
{
	if (ancer) {
		delete ancer;
		ancer=NULL;
	}

	free(ident);
}

static void random_usleep(unsigned long usec)
{
	int least=usec/2;
	usleep(least+random()%usec);
}

# define MAX_ERRORS 1000

void *DirPublisher::run()
{
	int errors=0;

	while (!should_quit()) {
		if (errors>MAX_ERRORS) {
			tmpi_error(DBG_APP, "DirPublisher is quitting due to too many errors ...\n");
			return NULL;
		}

		int annc_error=0;
		if ((annc_error=announce())) {
			tmpi_error(DBG_INTERNAL, "Publisher.announce failed.\n");
			errors+=annc_error;
		}
		else {
			//tmpi_error(DBG_VERBOSE, "Load announced.\n");
		}

		random_usleep(interval*1000);
	}

	//tmpi_error(DBG_VERBOSE, "Load Publisher is quitting normally ...\n");

	return NULL;
}

// TO-DO: it might be better to use XDR to marshal data, currently there is
// an obvious space waste.
int DirPublisher::announce()
{
	int errors=0;

	Mc_anc_data data;

	if (data.add(KEY_SOR_IDENT, ident, strlen(ident)+1)<0) {
		tmpi_error(DBG_INTERNAL, "Mc_annc_data.add(%d) failed.\n", KEY_SOR_IDENT);
		errors++;
	}

	// if (data.add(KEY_SVR_IDENT, &host_id, sizeof(HostID))<0) {
		// tmpi_error(DBG_INTERNAL, "Mc_annc_data.add(%d) failed.\n", KEY_SVR_IDENT);
		// errors++;
	// }

	if (data.add(KEY_DIRSRV_PORT, &lis_port, sizeof(int))<0) {
		tmpi_error(DBG_INTERNAL, "Mc_annc_data.add(%d) failed.\n", KEY_DIRSRV_PORT);
		errors++;
	}

	return ancer->announce(&data);
}
