// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dirdb_cache.h,v $ - header file for directory server db cache.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _DIRDB_CACHE_H_
# define _DIRDB_CACHE_H_

# include <string.h>
# include <pthread.h>
# include "md5.h"

/**
 * I decided to use the direct mapping cache
 * because really I need very fast cache update speed.
 */

# define DIRDB_CACHE_SIZE (256*1024)

class DIRDB_Cache {
 private:
	 typedef unsigned int node_t;
	 struct Entry {
		unsigned int sum; /* sum */
# if defined(__USE_MD5__)
		char tag[MD5_LEN];
# endif
		node_t inode; /* value */
		void set(
# if defined(_USE_MD5__)
				unsigned char *digest, 
# endif
				unsigned int sum, node_t inode) {
			this->sum=sum;
# if defined(__USE_MD5__)
			memcpy(this->tag, digest, MD5_LEN);
# endif
			this->inode=inode;
		}

		void clear() {
			sum=0;
# if defined(__USE_MD5__)
			memset(tag, 0, MD5_LEN);
# endif
			inode=0;
		}
	 };

	 pthread_mutex_t lock;
	 Entry *cache;
	 bool cache_on;
	 unsigned int n_ins;
	 unsigned int n_del;
	 unsigned int n_miss;
	 unsigned int n_hit;

# if defined(__USE_MD5__)
	 void md5(node_t parent, char *path, unsigned char *digest);
# endif
	 void pjw(node_t parent, char *path, unsigned int &sum);

 public:
	 DIRDB_Cache(bool enable=true);
	 ~DIRDB_Cache();

	 void ins(node_t parent, char *path, node_t node);
	 void del(node_t parent, char *path);
	 int find(node_t parent, char *path, node_t &node);
	 void report();
};	

# endif
