// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dirdb_dump.cc,v $ - directory db dumper.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/20 07:50:58 $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <stdio.h>
# include <getopt.h>
# include <stdlib.h>
# include "dirdb.h"

char *prog=NULL;

void do_help()
{
	printf("\
Usage: %s <options>:\n\
   -[s|b|d|f|p] <db_file>   DIR DB dump, s/b/d/f/p are the dumping methods\n\
                            s: sequence, b: breadth, d: depth, f: full, p: plain\n\
   -h                       Help, this screen.\n\
", prog);
	exit(0);
}

int main(int argc, char *argv[])
{
	enum METHOD {SEQUENCE, BREADTH, DEPTH, FULL, PLAIN};
	enum METHOD method=PLAIN;

	prog=argv[0];

	int c;
	while ((c=getopt(argc, argv, "sbdfp"))!=EOF) {
		switch (c) {
			case 's':
				method=SEQUENCE;
				break;
			case 'b':
				method=BREADTH;
				break;
			case 'd':
				method=DEPTH;
				break;
			case 'f':
				method=FULL;
				break;
			case 'p':
				method=PLAIN;
				break;
			case 'h':
				do_help();
				break;
			default:
				do_help();
				break;
		}
	}
	if ((argc-optind)<=0) {
		fprintf(stderr, "No db file specified.\n");
		do_help();
	}
	DIR_DB db(argv[optind]);

	if (!db.good()) {
		fprintf(stderr, "Cannot read db file.\n");
		exit(-1);
	}

	switch (method) {
		case SEQUENCE: 
			db.dump_sequence();
			break;
		case BREADTH: 
			db.dump_breadth();
			break;
		case DEPTH: 
			db.dump_depth();
			break;
		case FULL: 
			db.dump_full();
			break;
		case PLAIN: 
			db.dump_plain();
			break;
		default:
			break;
	}

	return 0;
}
