// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dirdb_dup.cc,v $ - directory db duplicator.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/05/06 09:19:22 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <sys/stat.h>
# include <unistd.h>
# include <stdio.h>
# include <stdlib.h>
# include <getopt.h>
# include "dirdb.h"

char *prog=NULL;

void do_help()
{
	printf("\
Sweeping the content of the source DIRDB file and store the tree\n\
to the <output> file.\n\
\n\
Usage: %s [options] [source] [output]\n\
   -s <source>            Source file.\n\
   -o <output>            Output file.\n\
   -f                     Overwrite output file if existent.\n\
   -F                     Force the operation even though there\n\
                          might be other processes using the \n\
						  source file. It is necessary when last\n\
						  invocation on the source failed to\n\
						  clean up. \n\
   -h                     Help, this screen.\n\
", prog);
	exit(0);
}

int main(int argc, char *argv[])
{
	char *src=NULL;
	char *dst=NULL;
	bool force=false;
	bool db_force=false;

	prog=argv[0];

	int c;
	while ((c=getopt(argc, argv, "s:o:fFh"))!=EOF) {
		switch (c) {
			case 's':
				if (src) {
					fprintf(stderr, "Source is already specified.\n");
					do_help();
				}
				src=optarg;
				break;
			case 'o':
				if (dst) {
					fprintf(stderr, "Output is already specified.\n");
					do_help();
				}
				dst=optarg;
				break;
			case 'f':
				force=true;
				break;
			case 'F':
				db_force=true;
				break;
			case 'h':
				do_help();
				break;
			default:
				do_help();
				break;
		}
	}

	int remain_arg=2;
	if (src)
		remain_arg--;
	if (dst)
		remain_arg--;

	if ((argc-optind)>remain_arg) {
		fprintf(stderr, "Too many arguments.\n");
		do_help();
	}
	else if ((argc-optind)<remain_arg) {
		fprintf(stderr, "Too few arguments.\n");
		do_help();
	}

	if (!src) {
		src=argv[optind++];
	}

	if (!dst) {
		dst=argv[optind++];
	}

	struct stat st;
	if (stat(dst, &st)==0) {
		if (!force) {
			fprintf(stderr, "Output file already exists.\n");
			exit(-1);
		}
		if (unlink(dst)<0) {
			fprintf(stderr, "Cannot remove output file.\n");
			exit(-1);
		}
	}

	DIR_DB old_db(src);

	if (!old_db.good()) {
		fprintf(stderr, "Cannot read db file.\n");
		exit(-1);
	}

	if (old_db.copy(dst, db_force)<0) {
		fprintf(stderr, "Failed to copy %s to %s.\n", src, dst);
		unlink(dst); // desperate attempt.
		return -1;
	}

	return 0;
}
