// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dirsrv_conf.cc,v $ - directory server configuration
 *            manager (implementation).
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/28 07:20:18 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdlib.h>
# include <stdio.h>
# include <string>
# include "sock_conn.h"
# include "dirsrv_conf.h"

using namespace std;

DirSrvConf::DirSrvConf(char *f) : Configuration()
{
	if (ReadConfigFile(*this, f) == false) {
		throw string("Failed to read configuration file: '")+f+"'";
	}
}

char *DirSrvConf::ident()
{
	string def=Find("DirSrvDefault::Ident");
	string ret=Find("DirSrv::Ident", def.c_str());
	if (ret.c_str()[0]=='\0') {
		throw string("No identity defined.");
	}
	return strdup(ret.c_str());
}

int DirSrvConf::port()
{
	int def=FindI("DirSrvDefault::Port", SOCK_PORT_ANY);
	int port=FindI("DirSrv::Port", def);

	return port;
}

char *DirSrvConf::home_dir()
{
	char *p=getenv("DIRSRV_HOME");
	if (p==NULL) {
		p="/var/lib/dirsrv";
	}

	string defaultHome=Find("DirSrvDefault::Home", p);
	string home=Find("DirSrv::Home", defaultHome.c_str());

	return strdup(home.c_str());
}

char *DirSrvConf::log_dir()
{
	char *home=home_dir();

	string defaultLogDir=Find("DirSrvDefault::LogDir", "log");
	string logDir=Find("DirSrv::LogDir", defaultLogDir.c_str());
	char abs_logdir[1024];

	if (logDir.c_str()[0]=='/') {
		strcpy(abs_logdir, logDir.c_str());
	}
	else {
		sprintf(abs_logdir, "%s/%s", home, logDir.c_str());
	}

	free(home);

	return strdup(abs_logdir);
}

char *DirSrvConf::log_file()
{
	char *abs_logdir=log_dir();

	string defaultLogName=Find("DirSrvDefault::LogName", "dirsrv");
	string logName=Find("DirSrv::LogName", defaultLogName.c_str());

	char abs_logname[1024];
	sprintf(abs_logname, "%s/%s.log", abs_logdir, logName.c_str());

	free(abs_logdir);
	
	return strdup(abs_logname);
}

char *DirSrvConf::backstore()
{
	char *home=home_dir();

	string defaultBackStore=Find("DirSrvDefault::BackStore", "dirsrv.db");
	string backStore=Find("DirSrv::BackStore", defaultBackStore.c_str());
	char abs_backstore[1024];

	if (backStore.c_str()[0]=='/') {
		strcpy(abs_backstore, backStore.c_str());
	}
	else {
		sprintf(abs_backstore, "%s/%s", home, backStore.c_str());
	}

	free(home);

	return strdup(abs_backstore);
}

char *DirSrvConf::mc_group()
{
	string def=Find("DirSrvDefault::Channel");
	string channel=Find("DirSrv::Channel", def.c_str());

	char *group=NULL, *p=NULL;

	if (channel.c_str()[0]=='\0') {
		goto _fail;
		
	}

	group=strdup(channel.c_str());
	p=strchr(group, ':');

	if (p) {
		*p='\0';
		return group;
	}

	free(group);

_fail:
	throw "mc_group(): No mcast group defined.";
}

int DirSrvConf::mc_port()
{
	string def=Find("DirSrvDefault::Channel");
	string channel=Find("DirSrv::Channel", def.c_str());

	char *group=NULL, *p=NULL;
	int port=-1;

	if (channel.c_str()[0]=='\0') {
		goto _fail;
		
	}

	group=strdup(channel.c_str());
	p=strchr(group, ':');

	if (p) {
		p++;
		port=atoi(p);
		free(group);
		return port;
	}

	free(group);

_fail:
	throw "mc_port(): No mcast group defined.";
}

int DirSrvConf::mc_interval()
{
	int def=FindI("DirSrvDefault::AnnounceInterval", 1000);
	return FindI("DirSrv::AnnounceInterval", def);
}
