// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: port_announcer.cc,v $ - directory server port announcer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.6 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <netinet/in.h>
# include <unistd.h>
# include <stdio.h>
# include <mntent.h>
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
# include "tmpi_debug.h"
# include "sock_conn.h"
# include "mcast_annc.h"
# include "../shared/daemon_common.h"
# include "port_announcer.h"

PortAnnouncer::PortAnnouncer(char *mgrp, int port, int lport, int itvl) : 
	ancer(NULL),
	lis_port(lport),
	interval(itvl)
{
	struct sockaddr_in sin;

	if ((!mgrp)||(port<=0)||(lport<=0)) {
		tmpi_error(DBG_CALLER, "Bad parameters: mgrp=%s, port=%d, lport=%d.\n", mgrp, port, lport);
		return;
	}

	if (sock_filladdr(mgrp, port, &sin)<0) {
		tmpi_error(DBG_CALLER, "Bad address: %s:%d.\n", mgrp, port);
		return;
	}

	if (sock_is_mcast_addr(&sin, sizeof(sin))!=1) {
		tmpi_error(DBG_CALLER, "Bad multi-cast address: %s:%d.\n", mgrp, port);
		return;
	}

	ancer=new Mc_ancer(mgrp, port);

	if (!ancer->good()) {
		tmpi_error(DBG_CALLER, "Cannot create multi-cast announcer.\n");
		goto _exit;
	}

	return;

_exit:
	release();
	return;
}

# define MAX_ERRORS 1000

static void	random_usleep(unsigned long usec)
{
	int least=usec/2;
	usleep(least+random()%usec);
}

void *PortAnnouncer::run()
{
	if (!good()) {
		tmpi_error(DBG_CALLER, "Attempt to run an uninitialized Port Announcer.\n");
		return NULL;
	}

	int errors=0;

	while (!should_quit()) {
		if (errors>MAX_ERRORS) {
			tmpi_error(DBG_APP, "Port Announcer quitting due to too many errors ...\n");
			return NULL;
		}

		if (announce()<0) {
			tmpi_error(DBG_INTERNAL, "PortAnnouncer.announce failed.\n");
			errors++;
		}

		random_usleep(interval*1000);
	}

	tmpi_error(DBG_APP, "Port Announcer quitting normally ...\n");

	return NULL;
}

int PortAnnouncer::announce()
{
	if ((!is_running())||(!good()))
		return -1;

	Mc_anc_data data;

	if (data.add(KEY_DIRSRV_PORT, &(lis_port), sizeof(lis_port))<0) {
			return -1;
	}

	return ancer->announce(&data);
}
