// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t_dirdb2.cc,v $ - directory server client shell 
 *            program II. This program tests the more extensive interface
 *            provided by dir_cli_mt.h 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/09/12 00:29:15 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <stdio.h>
# include <stdlib.h>
# include <ctype.h>
# include <unistd.h>
# include <getopt.h>

extern "C" {
# include <readline/readline.h>
# include <readline/history.h>
}

# include "usc_time.h"
# include "dir_cli_mt.h"
# include "dir_cli_ap_udp.h"
# include "dir_cli_ap_file.h"
# include "dirdb.h"
# include "dir_cli_opt.h"
# include "helper.h"

# include <vector>

typedef std::vector<const char *> VecArg;

static char *program_name=NULL;
static DIR_DB *db=NULL;
static DIR_Cli_AP *ap=NULL;
static DIR_Entry *cwd=NULL;
char *Id="$Id: t_dirdb2.cc,v 1.5 2003/09/12 00:29:15 htang Exp $";

static char *chop(char *line)
{
	
	while (*line) {
		if (isspace(*line))
			line++;
		else
			break;
	}

	char *p=line+strlen(line)-1;
	while ((p>=line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	return line;
}

static char *chop1(char *line)
{
	
	while (*line) {
		if ((isspace(*line))||(isdigit(*line))||(*line=='>'))
			line++;
		else
			break;
	}

	char *p=line+strlen(line)-1;
	while ((p>=line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	return line;
}

static char *get_RCS_val(char *var)
{
	static char buf[1024];
	char *p;

	if ((p=strchr(var, ':'))==NULL) {
		return var;
	}
	buf[1023]='\0';
	strncpy(buf, p+1, 1024);
	p=buf+strlen(buf)-1;
	while ((p>=buf)&&(*p=='$')) {
		*p='\0';
		p--;
	}

	return chop(buf);
}

static void do_version()
{
	static char *version="$Revision: 1.5 $";
	static char *author="Hong Tang";
	static char *date="$Date: 2003/09/12 00:29:15 $";

	{
		printf("Directory Server/DB Client II (%s) %s.\n", program_name, get_RCS_val(version));
		printf("Copyright (c) University of California, Santa Barbara.\n");
		printf("Author: %s.\n", get_RCS_val(author));
		printf("Last Modification Date: %s GMT.\n", get_RCS_val(date));
		printf("Specify the --help option for further information.\n");
	}

	exit(0);
}

static void do_help(char *cmd)
{
	if (cmd) {
		printf("Unknow command: %s. Type help for more info.\n", cmd);
	}
	else {
		printf("\n\
The shell suports the following commands:\n\
 Short  Full     arguments       meaning\n\
 ===================================================================\n\
 cd     chdir    <path>          Change the current directory.\n\
 ls     list     <path>          List the content of a directory.\n\
 ga     getattr  <path>          Get the attribute of a node.\n\
 sa     setattr  <path> <size>   Set the size of a node (file only).\n\
 md     mkdir    <path>          Make a directory.\n\
 rd     rmdir    <path>          Remove a directory.\n\
 dt     deltree  <path>          Delete a sub-tree.\n\
 cr     create   <path>          Create a file.\n\
 rm     remove   <path>          Remove a file.\n\
 pwd                             Show current working directory.\n\
 id     ident                    Show the identity of db source.\n\
 h      help                     Print this message.\n\
 q      quit                     Quit the shell.\n\
");
	}

	return;
}

static void usage(int status)
{
	if (status != 0) {
		fprintf(stderr, "Try '%s --help' for more information.\n", program_name);
	}
	else {
		printf ("Usage: %s [options].\n", program_name);
		printf ("\
A Directory Service Command Line Shell.\n\
\n\
1. DIR Database Location:\n\
    -f, --dir-db <db_file>    Set dir_db file (cannot combine with -S).\n\
    -S, --dir-srv <server>    Set dir-server address (cannot combine with -f).\n\
    -p, --port <port>         Set dir-server port (only combine with -S).\n\
 \n\
2. Debugging:\n\
    -s, --silent              Display/log only fatal information.\n\
    -d, --debug               Keep a detailed log.\n\
  \n\
3. Information:\n\
    -h, --help                Display this help screen.\n\
    -V, --version             Display version number.\n\
");
	}

	exit(status);
}

static void parse_cmd_args(int argc, char *argv[])
{
	static struct option const long_options[] = {
		{"dir-db", required_argument, 0, 'f'},
		{"dir-srv", required_argument, 0, 'S'},
		{"port", required_argument, 0, 'p'},
		{"silent", no_argument, 0, 's'},
		{"debug", no_argument, 0, 'd'},
		{"help", no_argument, 0, 'h'},
		{"version", no_argument, 0, 'V'},
		{NULL, 0, 0, 0}
	};

	int optc;
	char *bsf=NULL;
	char *dir_srv=NULL;
	int port=-1;

	program_name=argv[0];

	while ((optc=getopt_long(argc, argv, \
					"f:S:p:sdhV", long_options, NULL)) !=-1) {
		switch (optc) {
			case 0:
				break;
			case 'f':
				if (bsf) {
					printf("dir_db file already specified.\n");
					usage(0);
				}
				bsf=optarg;
				break;
			case 'S':
				if (dir_srv) {
					printf("dir_srv addr already specified.\n");
					usage(0);
				}
				dir_srv=optarg;
				break;
			case 'p':
				if (port>0) {
					printf("dir_srv port already specified.\n");
					usage(0);
				}
				port=atoi(optarg);
				if (port<=0) {
					printf("invalid port.\n");
					usage(0);
				}
				break;
			case 's':
				tmpi_set_dbg_level(DBG_FATAL-1);
				tmpi_hide_location();
				break;
			case 'd':
				tmpi_set_dbg_level(DBG_ALL);
				tmpi_show_location();
				break;
			case 'V':
				do_version();
				break;
			case 'h':
				usage(0);
				break;
			default:
				usage(1);
		}
	}

	if (optind<argc) {
		if (optind<argc-1) {
			usage(1);
		}

		if (bsf) {
			printf("dir_db file already specified.\n");
			usage(0);
		}

		bsf=argv[optind];
	}

	if ((bsf)&&(dir_srv)) {
		printf("Both dir_db file and dir_srv address specified.\n");
		usage(0);
	}

	if ((!bsf)&&(!dir_srv)) {
		printf("Neither dir_db nor dir_srv specified.\n");
		usage(0);
	}

	if (port<=0) {
		port=DIRSRV_PORT_DEFAULT;
	}

	if (bsf) {
		db=new DIR_DB(bsf);
		if (!db->good()) {
			tmpi_error(DBG_APP, "Cannot initialize DIR DB file <%s>.\n", bsf);
			delete db;
			db=NULL;
			exit(-1);
		}

		ap=(DIR_Cli_AP *) new DIR_Cli_AP_FILE(db);
	}
	else {
		ap=(DIR_Cli_AP *) new DIR_Cli_AP_UDP(dir_srv, port);
	}

	if (!ap) {
		tmpi_error(DBG_APP, "Cannot create directory service access point.\n");
		exit(-1);
	}

	cwd=new DIR_Entry;

	if (!cwd) {
		tmpi_error(DBG_APP, "Cannot create directory entry object.\n");
		exit(-1);
	}

	return;
}

enum CMDS {
	CMD_CHDIR, /* lookup */
	CMD_LIST, /* lookup, readdir, readdir_cont */
	CMD_GETATTR, /* lookup, getattr */
	CMD_SETATTR, /* lookup, setattr */
	CMD_MKDIR, /* mkdir */
	CMD_RMDIR, /* rmdir */
	CMD_DELTREE, /* deltree */
	CMD_CREATE, /* create */
	CMD_REMOVE, /* remove */
	CMD_PWD, /* path */
	CMD_HELP, /* help */
	CMD_QUIT, /* quit */
	CMD_IDENT, /* get_ident, get_protocol */
	//-------//
	CMD_MAX /* The maximum of the command enums */
};

struct CMD_TABLE {
	char *cmd;
	int cid;
} cmd_to_cid[] =
{
	{"cd", 		CMD_CHDIR},
	{"chdir", 	CMD_CHDIR},
	{"ls", 		CMD_LIST},
	{"list", 	CMD_LIST},
	{"ga", 		CMD_GETATTR},
	{"getattr", CMD_GETATTR},
	{"sa", 		CMD_SETATTR},
	{"setattr", CMD_SETATTR},
	{"md", 		CMD_MKDIR},
	{"mkdir", 	CMD_MKDIR},
	{"rd", 		CMD_RMDIR},
	{"rmdir", 	CMD_RMDIR},
	{"dt", 		CMD_DELTREE},
	{"deltree", CMD_DELTREE},
	{"cr", 		CMD_CREATE},
	{"create", 	CMD_CREATE},
	{"rm", 		CMD_REMOVE},
	{"remove", 	CMD_REMOVE},
	{"pw", 		CMD_PWD},
	{"pwd", 	CMD_PWD},
	{"h", 		CMD_HELP},
	{"help", 	CMD_HELP},
	{"?", 		CMD_HELP},
	{"q", 		CMD_QUIT},
	{"quit", 	CMD_QUIT},
	{"id", 		CMD_IDENT},
	{"ident", 	CMD_IDENT},
	// ... //
	{NULL, 	-1}
};

static int handle_chdir(VecArg &, bool);
static int handle_list(VecArg &, bool);
static int handle_getattr(VecArg &, bool);
static int handle_setattr(VecArg &, bool);
static int handle_mkdir(VecArg &, bool);
static int handle_rmdir(VecArg &, bool);
static int handle_deltree(VecArg &, bool);
static int handle_create(VecArg &, bool);
static int handle_remove(VecArg &, bool);
static int handle_pwd(VecArg &, bool);
static int handle_ident(VecArg &, bool);

struct CMD_HANDLER {
	unsigned int n_arg;
	int (*f)(VecArg &, bool);
	char *help_str;
} handlers[CMD_MAX];

struct CMD_HANDLER_INFO {
	int id;
	unsigned int n_arg;
	int (*f)(VecArg &, bool);
	char *help_str;
} handler_info[] =
{
	{ CMD_CHDIR, 	1, &handle_chdir,  	"cd     chdir    <path>          Change the current directory." },
	{ CMD_LIST, 	1, &handle_list,   	"ls     list     <path>          List the content of a directory." },
	{ CMD_GETATTR, 	1, &handle_getattr, "ga     getattr  <path>          Get the attribute of a node." },
	{ CMD_SETATTR, 	2, &handle_setattr, "sa     setattr  <path> <size>   Set the size of a node (file only)." },
	{ CMD_MKDIR, 	1, &handle_mkdir, 	"md     mkdir    <path>          Make a directory." },
	{ CMD_RMDIR, 	1, &handle_rmdir, 	"rd     rmdir    <path>          Remove a directory." },
	{ CMD_DELTREE, 	1, &handle_deltree, "dt     deltree  <path>          Delete a sub-tree." },
	{ CMD_CREATE, 	1, &handle_create, 	"cr     create   <path>          Create a file." },
	{ CMD_REMOVE, 	1, &handle_remove, 	"rm     remove   <path>          Remove a file." },
	{ CMD_PWD, 		0, &handle_pwd, 	"pwd                             Show current working directory." },
	{ CMD_IDENT, 	0, &handle_ident, 	"id     ident                    Show the identity of db source." },
	{ CMD_HELP, 	0, NULL,  		    "h      help                     Print this message." },
	{ CMD_QUIT, 	0, NULL, 			"q      quit                     Quit the shell." },
	{ -1, 0, NULL, NULL}
};

static void init_handlers()
{
	struct CMD_HANDLER_INFO *p=handler_info;
	while (p->id>=0) {
		handlers[p->id].n_arg=p->n_arg;
		handlers[p->id].f=p->f;
		handlers[p->id].help_str=p->help_str;
		p++;
	}

	return;
}

static int handle_cmd(char *cmd, VecArg &args, bool do_output)
{
	if (!cmd)
		return 0;

	struct CMD_TABLE *p=cmd_to_cid;

	while (p->cmd) {
		if (strcmp(cmd, p->cmd)==0) {
			break;
		}
		p++;
	}

	if (p->cmd) {
		struct CMD_HANDLER *h=&(handlers[p->cid]);
		if (args.size() != h->n_arg) {
			if (do_output) {
				if (args.size() < h->n_arg) {
					printf("Too few arguments. Usage of '%s':\n", cmd);
					printf("%s\n", "SHORT  FULL     ARGUMENTS       MEANING");
					printf("%s\n", h->help_str);
				}
				else if (args.size() > h->n_arg) {
					printf("Too many arguments. Usage of '%s':\n", cmd);
					printf("%s\n", "SHORT  FULL     ARGUMENTS       MEANING");
					printf("%s\n", h->help_str);
				}
			}

			return 0;
		}

		if (h->f) {
			return (*h->f)(args, do_output);
		}

		switch (p->cid) {
			case CMD_HELP : 
				do_help(NULL); return 0;
			case CMD_QUIT : 
				printf("Good bye!\n"); return 1;
			default : 
				if (do_output) 
					tmpi_error(DBG_APP, "Internal error: Bad command code.\n"); 
				return 0;
		}
	}
	else {
		do_help(cmd);
		return 0;
	}
}

static char *get_token(char * &t, char * & t_end)
{
	if (!t)
		return NULL;

	while ((*t)&&(isspace(*t))) {
		t++;
	}

	t_end=t;
	while ((*t_end)&&(!isspace(*t_end))) {
		t_end++;
	}
	
	if (t==t_end)
		return NULL;

	return t;
}

static bool is_empty_line(char *line)
{
	while (*line) {
		if (!isspace(*line))
			return false;
		line++;
	}

	return true;
}

/**
 * parse a command line.
 * @param line the command line.
 * @param cmd the command name.
 * @param args a vector of arguments.
 * @return status (-1: error; 0: success).
 */
static int parse_line(char *line, char *&cmd, VecArg &args)
{
	if (!line)
		return -1;
	
	cmd=NULL;

	line=chop1(line);

	if (is_empty_line(line)) {
		return 0;
	}

	char *t_begin;
	char *t_end;

	t_begin=line;
	t_begin=get_token(t_begin, t_end);
	
	if (!t_begin) 
		return 0;

	*t_end='\0';
	cmd=t_begin;

	t_begin=t_end+1;
	t_begin=get_token(t_begin, t_end);

	while (t_begin) {
		*t_end='\0';
		args.push_back(t_begin);

		t_begin=t_end+1;
		t_begin=get_token(t_begin, t_end);
	}

	return 0;
}

static int output_error(int cid, int errno, bool do_output)
{
	if (!do_output)
		return 0;

	switch (errno) {
		case DIR_RES_ENONEXIST: 
			if ((cid==CMD_MKDIR)||(cid==CMD_CREATE))
				printf("Unable to create file or directory now, try again.\n"); 
			else
				printf("File or directory does not exist.\n"); 
			return 0;
		case DIR_RES_EEXIST: 
			if (cid==CMD_RMDIR)
				printf("Directory not empty.\n"); 
			else
				printf("File or directory already exists.\n"); 
			return 0;
		case DIR_RES_ESELF: 
			printf("Current directory no longer exists.\n"); return 0;
		case DIR_RES_EINVALID: 
			printf("Invalid argument.\n"); return 0;
		case DIR_RES_EISDIR: 
			printf("Target is a directory.\n"); return 0;
		case DIR_RES_ENOTDIR: 
			printf("Target is a not a directory.\n"); return 0;
		case DIR_RES_EBUFFER: 
		case DIR_RES_EMEM: 
			printf("Out of memory.\n"); return 0;
		case DIR_RES_ESERVER: 
			printf("Server error.\n"); return 0;
		case DIR_RES_ETOUT: 
			printf("Request timed out. Maybe network error.\n"); return 0;
		case DIR_RES_ENETWORK: 
			printf("Network error.\n"); return 0;
		case DIR_RES_ECLIENT: 
			printf("Client error.\n"); return 0;
		default:
			printf("Internal error. Code: %d.\n", errno);
			return 1;
	}
}

int main(int argc, char *argv[])
{
# if defined(__DEBUG__)
	tmpi_set_dbg_level(DBG_ALL);
	tmpi_show_location();
# else
	tmpi_set_dbg_level(DBG_VERBOSE);
	tmpi_hide_location();
# endif
	
	bool use_gnu_getline=false;
	bool do_output=false;
	bool do_timing=true;

	if (isatty(fileno(stdin))) {
		use_gnu_getline=true;
		do_output=true;
		do_timing=false;
	}

	// might be no return.
	parse_cmd_args(argc, argv);
	init_handlers();

	usc_time_t t_start=0, t_end=0;
	int cmd_cnt=0;

	if (do_timing) {
		t_start=usc_gettime();
	}

	while (1) {
		char *cmd; 
		VecArg args;
		char line[1024];
		if (use_gnu_getline) {
			char *l=readline("> ");
			strncpy(line, l, 1023);
			line[1023]='\0';
			free_line(l);

			if (line&&(line[0]))
				add_history(line);
		}
		else {
			if (fgets(line, 1024, stdin)==NULL) {
				break;
			}
			if (do_output)
				printf("> %s", line);
		}

		if (parse_line(line, cmd, args)<0) {
			printf("Bad command.\n");
			continue;
		}

		if (handle_cmd(cmd, args, do_output))  {
			break;
		}
		if (do_timing)
			cmd_cnt++;
	}

	if (do_timing) {
		t_end=usc_gettime();
	}

	if (db) {
		db->cache_report();
		delete db;
	}

	if (ap)
		delete ap;

	if (do_timing) {
		printf("Total %d commands executed, execution time %.3fms.\n", cmd_cnt, usc_difftime(t_end, t_start)/1000);
	}

	return 0;
}

static int handle_chdir(VecArg &args, bool do_output)
{
	DIR_Entry entry;
	int ret=cwd->lookup(ap, args[0], &entry);

	if (ret==DIR_RES_OKAY) {
		if (!entry.is_dir(ap)) {
			ret=DIR_RES_ENOTDIR;
		}

		memcpy(cwd, &entry, sizeof(DIR_Entry));
		return 0;
	}
	else {
		return output_error(CMD_CHDIR, ret, do_output);
	}
}

static int time_to_str(time_t tt, char *s, int l)
{
	if (!s)
		return -1;

    struct tm ttm;

    memcpy(&ttm, gmtime(&tt), sizeof(struct tm));
    if (strftime(s, l, "%b %e %T %Y", &ttm)==0)
		return -1;

    return 0;
}

static void print_item(DIR_Param *param, const char *name)
{
	char ts[256];
	time_to_str(param->t_create, ts, 256);
	bool is_dir=DIR_IS_DIR(param->options);
	printf("%crw%c------ %10lld %24s %s\n", is_dir?'d':'-', 
			is_dir?'x':'-', param->size, ts, name);
}

static void print_item_detail(DIR_Param *param, const char *name)
{
	char ts[256];
	bool is_dir=DIR_IS_DIR(param->options);
	time_to_str(param->t_create, ts, 256);
	char guid_str[256];
	Guid guid(param->guid);
	guid.to_string(guid_str, 256);
	printf("%s: %8s %10lld %24s [%s]\n", name, 
			is_dir?"[dir]":"[file]", param->size, ts, guid_str);
}

static int handle_list(VecArg &args, bool do_output)
{
	DIR_Entry entry;
	int ret=cwd->lookup(ap, args[0], &entry);

	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_LIST, ret, do_output);
	}

	if (entry.is_dir(ap)) {
		DIR_List list;
		ret=entry.readdir(ap, list, 3000);
		while (ret==DIR_RES_OKAY) {
			ret=entry.readdir_cont(ap, list, 3000);
		}
		if (ret != DIR_RES_EEND) {
			return output_error(CMD_LIST, ret, do_output);
		}
		if (do_output) {
			printf("List directory <%s>:\n", args[0]);
			for (unsigned int i=0; i<list.size(); i++) {
				print_item(&(list[i].param), list[i].name.c_str());
			}
		}
	}
	else {
		DIR_Param param;
		ret=entry.getattr(ap, &param);
		if (ret != DIR_RES_OKAY) {
			return output_error(CMD_GETATTR, ret, do_output);
		}
		if (do_output) {
			printf("List file <%s>:\n", args[0]);
			print_item(&param, args[0]);
		}
	}

	return 0;
}

static int handle_getattr(VecArg &args, bool do_output)
{
	DIR_Entry entry;
	int ret=cwd->lookup(ap, args[0], &entry);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_LIST, ret, do_output);
	}
	DIR_Param param;
	ret=entry.getattr(ap, &param);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_GETATTR, ret, do_output);
	}
	if (do_output) {
		print_item_detail(&param, args[0]);
	}

	return 0;
}

static int handle_setattr(VecArg &args, bool do_output)
{
	DIR_Entry entry;
	int ret=cwd->lookup(ap, args[0], &entry);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_LIST, ret, do_output);
	}
	DIR_Param param;
	ret=entry.getattr(ap, &param);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_GETATTR, ret, do_output);
	}
	param.size=atoi(args[1]);
	ret=entry.setattr(ap, &param);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_SETATTR, ret, do_output);
	}

	return 0;
}

static int handle_mkdir(VecArg & args, bool do_output)
{
	DIR_Entry entry;
	int ret=cwd->mkdir(ap, args[0], &entry);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_MKDIR, ret, do_output);
	}

	return 0;
}

static int handle_rmdir(VecArg & args, bool do_output)
{
	int ret=cwd->rmdir(ap, args[0]);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_RMDIR, ret, do_output);
	}

	return 0;
}

static int handle_deltree(VecArg & args, bool do_output)
{
	tmpi_error(DBG_INTERNAL, "deltree no longer supported.\n");
	/**
	int ret=cwd->deltree(ap, args[0]);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_DELTREE, ret, do_output);
	}
	**/

	return 0;
}

static int handle_create(VecArg & args, bool do_output)
{
	DIR_Entry entry;
	DIR_Param param;
	param.reset();
	param.size=rand();
	param.options=DIR_OPTION_FILE;

	int ret=cwd->create(ap, args[0], &param, &entry);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_CREATE, ret, do_output);
	}

	return 0;
}

static int handle_remove(VecArg & args, bool do_output)
{
	int ret=cwd->remove(ap, args[0]);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_REMOVE, ret, do_output);
	}

	return 0;
}

static int handle_pwd(VecArg & args, bool do_output)
{
	printf("%s\n", cwd->path());

	return 0;
}

static int handle_ident(VecArg & args, bool do_output)
{
	const char *id, *proto;
	int ret=ap->get_ident(id);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_IDENT, ret, do_output);
	}
	ret=ap->get_protocol(proto);
	if (ret != DIR_RES_OKAY) {
		return output_error(CMD_IDENT, ret, do_output);
	}

	printf("%s%s\n", proto, id);

	return 0;
}

