// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t_tracegen.cc,v $ - generate trace for dirdb/dir_srv testing.
 * Authors :  Lingkun Chu, lkchu@cs.ucsb.edu
 *            Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <getopt.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/types.h>
#include <dirent.h>

#define MAX_DIR_NUM 2048
#define MAX_PATH 4096

class Dir {
    struct {
	char path[MAX_PATH];
	int valid;
    } dirs[MAX_DIR_NUM];
    char root[MAX_PATH];
    int pwd;
    int idx;
public:
    void init(char *r) {
	strcpy(root, r);
	pwd=0;
	memset(dirs, 0, sizeof(dirs));
	dirs[0].valid = 1;
	strcpy(dirs[0].path, "/");
	idx = 1;
    }
    void set_cwd(int p) {
	pwd = p;
    }
    int pick_path(void) {
	int i, j, k;

	k = 0;
	for (i=0; i<MAX_DIR_NUM; i++) {
	    if (strlen(dirs[i].path) > 0) k++;
	}
	j =  (int)(drand48()*k);
	k = 0;
	for (i=0; i<MAX_DIR_NUM; i++) {
	    if (strlen(dirs[i].path) > 0) {
		if (k==j) 
		    return i;
		k++;
	    }
	}
	return -1;
    }
    char *long_path(int p) {
	char tmp[2*MAX_PATH];

	sprintf(tmp, "%s/%s", root, dirs[p].path);
	return strdup(tmp);
    }
    char *short_path(char *p) {
	char tmp[MAX_PATH];

	strcpy(tmp, p+strlen(root));
	if (strlen(tmp) == 0) strcpy(tmp, "/");
	return strdup(tmp);
    }
    char *get_path(int p) {
	return dirs[p].path;
    }
    void invalidate(int p) {
	if ( p> 0) dirs[p].valid = 0;
    }
    void push_path(char *p) {
	int i;
	for (i=idx; i<MAX_DIR_NUM; i++) {
	    if (dirs[i].valid == 0) {
		strcpy(dirs[i].path, p);
		dirs[i].valid = 1;
		idx = i;
		break;
	    }
	}
	if (i==MAX_DIR_NUM) {
	    for (i=0; i<idx; i++) {
		if (dirs[i].valid == 0) {
		    strcpy(dirs[i].path, p);
		    dirs[i].valid = 1;
		    idx = i;
		    break;
		}
	    }
	}
    }

};

static Dir gdir;

#define MAX_TST_CMD 5

typedef void (*TSTFUNC)(int cnt, FILE *tfp, FILE *rfp);

static void get_dir(int cnt, FILE *tfp, FILE *rfp);
static void rm_dir(int cnt, FILE *tfp, FILE *rfp);
static void mk_dir(int cnt, FILE *tfp, FILE *rfp);
static void chg_dir(int cnt, FILE *tfp, FILE *rfp);
static void ls_dir(int cnt, FILE *tfp, FILE *rfp);

struct Command {
    double prob;
    TSTFUNC func;
} cmds[MAX_TST_CMD] = {
    { 0.1, get_dir },
    { 0.1, rm_dir },
    { 0.2, mk_dir },
    { 0.3, chg_dir },
    { 0.3, ls_dir }
};

void usage(char *prog)
{
    fprintf(stderr,"Usage: %s -n command_number -t trace_file -r result_file\n", prog);
}

int main(int argc, char *argv[])
{
    char c;
    uint32_t count=0;
    double p, q;
    char *trace_fn = NULL;
    char *result_fn = NULL;
    FILE *trace_fp;
    FILE *result_fp;
    char root[MAX_PATH];

    while ((c = getopt(argc, argv, "n:t:r:")) != EOF) {
        switch (c) {
        case 'n':
	    count = atoi(optarg);
	    break;
	case 't':
	    trace_fn = strdup(optarg);
	    break;
	case 'r':
	    result_fn = strdup(optarg);
	    break;
	default:
	    usage(argv[0]);
	    exit(1);
	}
    }
    if (count == 0 || trace_fn == NULL || result_fn == NULL) {
	usage(argv[0]);
	exit(1);
    }
    getcwd(root, MAX_PATH);
    gdir.init(root);
    srand48(time(NULL));
    trace_fp = fopen(trace_fn, "w");
    result_fp = fopen(result_fn, "w");
    if (trace_fp == NULL || result_fp == NULL) {
	usage(argv[0]);
	exit(1);
    }
    for (uint32_t i = 0; i < count; i++) {
	p = drand48();
	q = 0;
	for (int j = 0; j < MAX_TST_CMD; j++) {
	    q += cmds[j].prob;
	    if (p < q) {
		cmds[j].func(i, trace_fp, result_fp);
		break;
	    }
	}
    }
    free(trace_fn);
    free(result_fn);
    fclose(trace_fp);
    fclose(result_fp);
}

#define NAME_LEN 8
static char *new_name(void)
{
    char tmp[MAX_PATH];
    
    for (int i=0; i<NAME_LEN; i++) {
	tmp[i] = 'a'+ (int)(drand48()*('z'-'a'));
    }
    tmp[NAME_LEN] = '\0';
    return strdup(tmp);
}

static void get_dir(int cnt, FILE *tfp, FILE *rfp)
{
    char pwd[MAX_PATH];
    char *p;

    fprintf(tfp, "%d> pwd\n", cnt);
    if (getcwd(pwd, MAX_PATH) == NULL) {
	fprintf(rfp, "%d> :-1\n", cnt);
    } else { 
	p = gdir.short_path(pwd);
	fprintf(rfp, "%d> %s\n", cnt, p);
	free(p);
    }
}

static void rm_dir(int cnt, FILE *tfp, FILE *rfp)
{
    char *path;
    int p;
    char cmd[2*MAX_PATH];

    p = gdir.pick_path();
    if (p == -1) {
	fprintf(stderr, "error in pick_path\n");
    } else {
	path = gdir.long_path(p);
	if ((p > 0) && (drand48() > 0.9)) {
	    fprintf(tfp, "%d> rm -r %s\n", cnt, gdir.get_path(p));
	    sprintf(cmd, "rm -rf %s\n", path);
	    system(cmd);
	    fprintf(rfp, "%d> :0\n", cnt);
	} else {
	    fprintf(tfp, "%d> rm %s\n", cnt, gdir.get_path(p));
	    if (p == 0) {
		fprintf(rfp, "%d> :-1\n", cnt);
	    } else {
		fprintf(rfp, "%d> :%d\n", cnt, rmdir(path));
	    }
	}
	gdir.invalidate(p);
	free(path);
    }
}

static void mk_dir(int cnt, FILE *tfp, FILE *rfp)
{
    int p;
    char *path;
    char *newdir;
    char tmpstr1[2*MAX_PATH];
    char tmpstr2[2*MAX_PATH];

    p = gdir.pick_path();
    if (p == -1) {
	fprintf(stderr, "error in pick_path\n");
    } else {
	path = gdir.long_path(p);
	newdir = new_name();
	if (strlen(newdir)+strlen(gdir.get_path(p))+1 < MAX_PATH) {
	    sprintf(tmpstr1, "%s/%s", gdir.get_path(p), newdir);
	    fprintf(tfp, "%d> md %s\n", cnt, tmpstr1);
	    sprintf(tmpstr2, "mkdir -p %s/%s", path, newdir);
            system(tmpstr2);
	    //rst = mkdir(tmpstr2, S_IRWXU); 
	    //fprintf(rfp, "%d> :%d\n", cnt, rst);
	    fprintf(rfp, "%d> :0\n", cnt);
	    gdir.push_path(tmpstr1);
	}
	free(newdir);
	free(path);
    }
}

static void ls_dir(int cnt, FILE *tfp, FILE *rfp)
{
    DIR *d;
    struct dirent *en;
    int p;
    char *path;

    p = gdir.pick_path();
    if (p == -1) {
	fprintf(stderr, "error in pick_path\n");
    } else {
	path = gdir.long_path(p);
	fprintf(tfp, "%d> ls %s\n", cnt, gdir.get_path(p));
	d = opendir(path);
	if (d == NULL) {
	    fprintf(rfp, "%d> :-1\n", cnt);
	} else {
	    fprintf(rfp, "%d> :0\n", cnt);
	    while ((en=readdir(d))!=NULL) {
		if ((strcmp(en->d_name, ".")) && (strcmp(en->d_name, ".."))) {
		    fprintf(rfp, "%d> %s\n", cnt, en->d_name);
		}
	    }
	    closedir(d);
	}
	free(path);
    }
}

static void chg_dir(int cnt, FILE *tfp, FILE *rfp)
{    
    int p;
    char *path;
    int rst;

    p = gdir.pick_path();
    if (p == -1) {
	fprintf(stderr, "error in pick_path\n");
    } else {
	path = gdir.long_path(p);
	fprintf(tfp, "%d> cd %s\n", cnt, gdir.get_path(p));
	rst = chdir(path);
	fprintf(rfp, "%d> :%d\n", cnt, rst);
        if (rst) {
	    gdir.set_cwd(p);
	}
	free(path);
    }
}
