/* -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- */

#include "fuse_i.h"
#include "sor_proc.h"

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>
#include <linux/string.h>

/** Sorrento VFS statistics */
struct sor_vfs_stats sor_vfs_stats;


/** fill in a buffer page with Sorrento VFS statistics */
int
sor_vfs_stats_read(char *buffer,
				   char **start,
				   off_t offset,
				   int length,
				   int *eof,
				   void *data)
{
    struct sor_vfs_stats *ps = &sor_vfs_stats;
    int len;
    const int LIMIT = PAGE_SIZE - 80; /* to be safe */

    len = 0;
    do {
	len += sprintf(buffer, "%-79s\n", "Sorrento VFS Statistics");
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-79s\n\n", "=====================");
	if (len >= LIMIT) break;
	
	len += sprintf(buffer + len, "%-79s\n", "File Operations:");
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "open:", ps->open);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "release:", ps->release);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "fsync:", ps->fsync);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "read:", ps->read);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "write:", ps->write);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "mmap:", ps->mmap);
	if (len >= LIMIT) break;

	len += sprintf(buffer + len, "%-79s\n", "inode Operations:");
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "lookup:", ps->lookup);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "create:", ps->create);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "mknod:", ps->mknod);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "mkdir:", ps->mkdir);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "symlink:", ps->symlink);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "unlink:", ps->unlink);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "rmdir:", ps->rmdir);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "rename:", ps->rename);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "link:", ps->link);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "setattr:", ps->setattr);
	if (len >= LIMIT) break;
	len += sprintf(buffer + len, "%-12s %9d\n", "permission:", ps->permission);
	if (len >= LIMIT) break;
#ifdef KERNEL_2_6
	len += sprintf(buffer + len, "%-12s %9d\n", "getattr:", ps->getattr);
#else
	len += sprintf(buffer + len, "%-12s %9d\n", "revalidate:", ps->revalidate);
#endif
	if (len >= LIMIT) break;
    } while (0);

    len -= offset;

    if (len > length)
	len = length;
    if (len < 0)
	len = 0;

    *start = buffer + offset;
    *eof = 1;

    return len;
}


/** Sorrento file system root entry in /proc/fs */
struct proc_dir_entry *proc_fs_sor;


#define sor_proc_create(name, get_info) \
create_proc_read_entry(name, 0, proc_fs_sor, get_info, NULL)

/** 
 * Initialize Sorrento FS in /proc/fs 
 * @return 0 for success
 */
int
sor_proc_init(void)
{
	struct proc_dir_entry *entry = NULL;

	reset_sor_vfs_stats();

	proc_fs_sor = proc_mkdir("sorrento", proc_root_fs);
	if ( !proc_fs_sor ) {
		printk(KERN_INFO "Could not create /proc/fs/sorrento\n");
		return 1;
	}

	proc_fs_sor->owner = THIS_MODULE;
	if ( (entry = sor_proc_create("vfs_stats", sor_vfs_stats_read)) == NULL ) {
		printk(KERN_INFO "Could not create vfs_stat in /proc/fs/sorrento\n");
	}

	return 0;
}

/** Cleanup Sorrento FS in /proc/fs */
void
sor_proc_cleanup()
{
    remove_proc_entry("vfs_stats", proc_fs_sor);
    remove_proc_entry("sorrento", proc_root_fs);
}


void
reset_sor_vfs_stats()
{
    memset(&sor_vfs_stats, 0, sizeof(struct sor_vfs_stats));
}
