/* -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- */

#ifndef _IO_CACHE_H_
#define _IO_CACHE_H_

#include <pthread.h>
#include <vector>

#include "sorrento_types.h"
#include "cbs_thread.h"

class SorFile;
class FileTable;

/* I/O Job Types */
#define IO_JOB_READ  0x1
#define IO_JOB_WRITE 0x2

#define IO_JOB_IDLE  0    /* no thread is working on the job */
#define IO_JOB_BUSY  1    /* some thread is working on the job */

struct io_job {
    int type;         /**< Job type */
    off64_t offset;   /**< starting offset */
    off_seg_t len;    /**< buffer length   */
    char *buf;        /**< buffer          */
    SorFile *file;    /**< File handle (is needed??)    */
    unsigned idx;     /**< Index in FileTable */
    int done:1;       /**< flag if the job is done. This is used to indicate
					   * the job is forced done in close() call.
					   */

	short unsigned busy; /**< indicate if a thread is work on the job */

    struct io_job *prev; /**< previous I/O buffer */
    struct io_job *next; /**< next I/O buffer     */

    io_job(int t, off64_t off, off_seg_t l, char *p, unsigned i)
	: type(t), offset(off), len(l), buf(p), file(NULL), idx(i), 
	 done(0), busy(IO_JOB_IDLE), prev(NULL), next(NULL)
    {}
    io_job() {} /* not used, just make compiler happy */
};

/** IO Cache class: implements asynchronous I/O */
class IOCache : public CBS_Thread {
 private:
    std::vector<struct io_job*> dirty_list; /**< Write job list */
    pthread_mutex_t dirty_lock; /**< lock for dirty job list */
    pthread_cond_t dirty_cond;  /**< condition signal there are jobs pending */

    FileTable *ftable;  /**< SClientStub's file table */

    /* perform asynchronous I/O */
    virtual void* run();

	unsigned _limit;    /**< Maximum buffer size allowed */
	unsigned _csize;    /**< current buffer size */
	pthread_mutex_t size_lock; /**< lock for buffer size */
	pthread_cond_t size_cond;  /**< condition for buffer size */

 public:
    IOCache(FileTable *, unsigned limit = 64*1024*1024);
    ~IOCache();

    bool add_job(io_job *job, int type);
};


#endif
