// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

# include <stdio.h>
# include <signal.h>
# include <string>

//# include "sor_client.h"
#include "SClientStub.h"
# include "sor_fs.h"
# include "subscriber.h"
# include "tmpi_debug.h"


static Subscriber *sub=NULL;
static SClientStub *scli=NULL;
static SorFS *sorfs=NULL;


/** 
 * Initialize a Sorrento client from a config file
 * @param config Sorrento configuration file
 * @return Sorrento client stub
 */
SClientStub*
init_sorrento(char *config)
{
	tmpi_set_dbg_level(DBG_ALL);
	tmpi_show_location();
	signal(SIGPIPE, SIG_IGN);

	try {
		SorrentoConf sconf(config);
		sub= new Subscriber(sconf);
		sub->start();
		printf("Starting up ...");
		fflush(stdout);
		bool done=false;
		int max_retry=10;
		
		while ((!done) && (max_retry--)) {
			printf(".");
			fflush(stdout);
			try {
				sleep(1); 
				sorfs=new SorFS(sub, std::string(sconf.ident()));
				scli=new SClientStub(sorfs);
				done=true;
			}
			catch (const std::string &errmsg) {
				if (sorfs) {
					delete sorfs;
					sorfs=NULL;
				}
				if (scli) {
					delete scli;
					scli=NULL;
                }
			}
		}
		
		if (scli == NULL) {
			printf("\nCannot locate the specified service (%s).\n", sconf.ident());
			return NULL;
		}
		else {
			printf("\nSorrento file server started...\n");
			return scli;
		}
	}
	catch (const std::string &errmsg) {
		tmpi_error(DBG_INTERNAL, "%s : %s\n", config, errmsg.c_str());
	}
	catch (int errnum) {
		tmpi_error(DBG_INTERNAL,"%s : error caught (code=%d).\n", config, errnum);
	}
	catch (...) {
		tmpi_error(DBG_INTERNAL, "Unknown error, initialization failed.\n");
	}

	return NULL;
}


void
cleanup_sorrento()
{
	if (sub) {
		sub->stop();
		sub->finalize();
		delete sub;
	}

	if (sorfs) {
		delete sorfs;
	}

	if (scli) {
		delete scli;
	}

	sub = NULL;
	sorfs = NULL;
	scli = NULL;
	return;
}
