// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: cbs_bf.h,v $ - header file for Bloom filters with incremantal
 *            update capabilities.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/05/06 09:19:22 $
 * Version :  $Revision: 1.17 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _CBS_BF_H_
# define _CBS_BF_H_

# include <stdio.h>
# include "bloom.h"
# include "guid.h"

# define MAX_LAB_ENTRIES_PER_PKT (30*1024/sizeof(unsigned int))

// BloomFilter packet data (directly converted from/to flat-memory buffer).
struct CBS_BF_PKT_POD {
	unsigned int n_bits;
	unsigned short n_lab_entries;
	unsigned short n_bf_pages;
 // protected:
	// lab_uint_array, bf_pgs, bf_pg_idx
	unsigned int data[1]; // set data to be aligned by unsigned integer
};

struct CBS_BF_PKT : public CBS_BF_PKT_POD {
 public:
	static unsigned int packet_header_len() 
	{ return (unsigned int)&(((CBS_BF_PKT_POD *)0L)->data); }
	unsigned int *lab_entries() 
	{ return data; }
	unsigned char *bf_pages() 
	{ return (unsigned char *)data+n_lab_entries*sizeof(unsigned int); }
	unsigned int *bf_pg_idx() 
	{ return (unsigned int *)((unsigned char *)bf_pages()+BF_PAGE_SIZE*n_bf_pages); }
	static unsigned int pkt_size(int n_lab_ent, int n_bf_pg); 
	unsigned int pkt_size() { return pkt_size(n_lab_entries, n_bf_pages); }

	// Two variables: # of LAB entries and # of BF pages. The one to be
	// determined must be assigned -1 and the other one fixed.
	static bool fit_packet
	(unsigned int pkt_len, int &n_lab_ent, int &n_bf_pg);
	void dump();
};

// all baseBF methods are still valid.
class CBS_BaseBF : public BaseBF {
 public:
	CBS_BaseBF(int max_objs) : BaseBF(max_objs) {}
	// the update() method will be used to do TCP stream update
	// and the following one for UDP update.
	bool update_from_packet(CBS_BF_PKT *pkt);
	static void prepare_check(guid_t &key, check_handle_t &ch) 
		{ BaseBF::prepare_check(&key, GUID_LEN, ch); }
	static void prepare_check(Guid &key, check_handle_t &ch) 
		{ guid_t guid; key.pack(guid); prepare_check(guid, ch); }
	bool check(guid_t &key) 
		{ return BaseBF::check(&key, GUID_LEN); }
	bool check(Guid &key) 
		{ guid_t guid; key.pack(guid); return check(guid); }
};

class CBS_BF : public BF {
 public:
	CBS_BF(int max_objs) : 
		BF(max_objs, MAX_LAB_ENTRIES_PER_PKT*10)
# ifdef __OVERLAP_UPDATE__
		, n_overlap_lab_entries(0)
# endif
# ifdef __CONST_FREQ_UPDATE__
		, next_chunk_id(0)
# endif
		{}
	bool add(const guid_t &key) 
		{ return BF::add(&key, GUID_LEN); }
	bool add(Guid & key) 
		{ guid_t guid; key.pack(guid); return add(guid); }
	bool del(const guid_t &key) 
		{ return BF::del(&key, GUID_LEN); }
	bool del(Guid & key) 
		{ guid_t guid; key.pack(guid); return del(guid); }

	bool make_bf_update_packet(CBS_BF_PKT *pkt, unsigned short &pkt_len);
	bool check(const guid_t &key) 
		{ return BaseBF::check(&key, GUID_LEN); }
	bool check(Guid &key) 
		{ guid_t guid; key.pack(guid); return check(guid); }
	bool reset() {
		bool ret=BF::reset();

# ifdef __OVERLAP_UPDATE__
		n_overlap_lab_entries=0;
# endif

# ifdef __CONST_FREQ_UPDATE__
		next_chunk_id=0;
# endif

		return ret;
	}

	bool update_reset() {
		bool ret=BF::update_reset();

# ifdef __OVERLAP_UPDATE__
		n_overlap_lab_entries=0;
# endif

# ifdef __CONST_FREQ_UPDATE__
		next_chunk_id=0;
# endif

		return ret;
	}

	static char *update_policy_string() {
		static char policy[1024]="";
		if (policy[0]=='\0') {
			snprintf(policy, 1024, "%s w/ %s", bit_flip_policy, chunk_update_policy);
		}

		return policy;
	}

 private:
	static char *bit_flip_policy;
# ifdef __OVERLAP_UPDATE__
	unsigned int overlap_lab_entries[65536/sizeof(unsigned int)];
	unsigned int n_overlap_lab_entries;
# endif

	static char *chunk_update_policy;
# ifdef __CONST_FREQ_UPDATE__
	unsigned int next_chunk_id;
# endif
};
# endif
