// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: cbs_bf_test.cc,v $ - testing CBS_BF.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include "cbs_bf.h"
# include "guid.h"

# define TOTAL_ITEMS 1000000
# define UPDATE_FREQ 100

// # define DUMP_PKT do { pkt->dump() } while (0)
# define DUMP_PKT do { } while (0)

int main(int argc, char *argv[])
{
	char *random_file="/dev/urandom";
	int ret=0;
	int i;
	int byte_update=0;
	int time_update=0;
	int lag_update=0;
	int byte_update2=0;
	int time_update2=0;
	int lag_update2=0;
# define PKT_BUF_SIZE (12*1024)
	// we declare the buffer as an array of intergers
	// to get it aligned.
	unsigned int pkt_buf[PKT_BUF_SIZE/sizeof(unsigned int)];
	CBS_BF_PKT *pkt=(CBS_BF_PKT *)pkt_buf;
	int miss_item=0;
	int miss_item2=0;
	int n_pages=0;

	if (argc==2) {
		random_file=argv[1];
	}

	int random_fd=open(random_file, O_RDONLY);
	Guid::set_random_fd(random_fd);
	printf("Testing page modification/updating II.\n");
	fflush(stdout);

	CBS_BF snd(TOTAL_ITEMS);
	CBS_BaseBF rcv(TOTAL_ITEMS);
	Guid *gi=new Guid[TOTAL_ITEMS];
	Guid *uigi=new Guid[TOTAL_ITEMS];
	for (i=0; i<TOTAL_ITEMS; i++) {
		gi[i].gen();
		if (!snd.add(gi[i])) {
			printf("\rBF.add() failed!\n");
			fflush(stdout);
			goto _exit;
		}
		if (!((i+1)%UPDATE_FREQ)) {
			unsigned short pkt_size=PKT_BUF_SIZE;
			if (snd.make_bf_update_packet(pkt, pkt_size)) {
				DUMP_PKT;
				rcv.update_from_packet(pkt);
				byte_update+=pkt_size;
				time_update++;
			}
		}
		if (i%1000==0) {
			printf("\r%.2f%% completed.", (double)i*100/TOTAL_ITEMS);
			fflush(stdout);
		}
		uigi[i].gen();
	}

	printf("\rRandom number generation done, filter partially replicated.\n");
	fflush(stdout);

	for (i=0; i<TOTAL_ITEMS; i++) {
		if (!rcv.check(gi[i])) {
			miss_item++;
		}
		if (i%1000==0) {
			printf("\r%.2f%% completed.", (double)i*100/TOTAL_ITEMS);
			fflush(stdout);
		}
	}

	n_pages=snd.get_num_page();
	while (1) {
		unsigned short pkt_size=PKT_BUF_SIZE;
		if (!snd.make_bf_update_packet(pkt, pkt_size)) {
			printf("\rCBS_BF::make_bf_update_packet() failed.\n");
			fflush(stdout);
			ret=-1;
			goto _exit;
		}
		DUMP_PKT;
		byte_update+=pkt_size;
		time_update++;
		lag_update++;
		if (lag_update>n_pages)
			break;

# ifdef __NO_CHUNK_UPDATE__
		if (pkt->n_lab_entries==0)
			break;
# else
		if ( (pkt->n_bf_pages==0) && (pkt->n_lab_entries==0))
			break;
# endif
		rcv.update_from_packet(pkt);
	}

	printf("\rFilter fully replicated, checking consistency.\n");
	fflush(stdout);

	for (i=0; i<TOTAL_ITEMS; i++) {
		if (rcv.check(gi[i])!=snd.check(gi[i])) {
			printf("\rInconsistent replication!\n");
			fflush(stdout);
			ret=-1;
			goto _exit;
		}
		if (rcv.check(uigi[i])!=snd.check(uigi[i])) {
			printf("\rInconsistent replication!\n");
			fflush(stdout);
			ret=-1;
			goto _exit;
		}
		if (i%1000==0) {
			printf("\r%.2f%% completed.", (double)i*100/TOTAL_ITEMS);
			fflush(stdout);
		}
	}

	printf("\rRemove items from source filter.\n");
	fflush(stdout);

	for (i=0; i<TOTAL_ITEMS; i++) {
		if (!snd.del(gi[i])) {
			printf("\rCBS_BF::del() failed!\n");
			fflush(stdout);
			ret=-1;
			goto _exit;
		}
		if (!((i+1)%UPDATE_FREQ)) {
			unsigned short pkt_size=PKT_BUF_SIZE;
			if (snd.make_bf_update_packet(pkt, pkt_size)) {
				DUMP_PKT;
				rcv.update_from_packet(pkt);
				byte_update2+=pkt_size;
				time_update2++;
			}
		}
		if (i%1000==0) {
			printf("\r%.2f%% completed.", (double)i*100/TOTAL_ITEMS);
			fflush(stdout);
		}
	}

	printf("\rItems removed, filter partially replicated.\n");
	fflush(stdout);

	for (i=0; i<TOTAL_ITEMS; i++) {
		if (rcv.check(gi[i])) {
			miss_item2++;
		}
		if (i%1000==0) {
			printf("\r%.2f%% completed.", (double)i*100/TOTAL_ITEMS);
			fflush(stdout);
		}
	}

	n_pages=snd.get_num_page();
	while (1) 
	{
		unsigned short pkt_size=PKT_BUF_SIZE;
		if (!snd.make_bf_update_packet(pkt, pkt_size)) {
			printf("\rCBS_BF::make_bf_update_packet() failed.\n");
			fflush(stdout);
			ret=-1;
			goto _exit;
		}
		DUMP_PKT;
		byte_update2+=pkt_size;
		time_update2++;
		lag_update2++;
		if (lag_update2>=n_pages)
			break;

# ifdef __NO_CHUNK_UPDATE__
		if (pkt->n_lab_entries==0)
			break;
# else
		if ( (pkt->n_bf_pages==0) && (pkt->n_lab_entries==0))
			break;
# endif
		rcv.update_from_packet(pkt);
	}

	printf("\rFilter fully replicated, checking consistency.\n");
	fflush(stdout);
	
	for (i=0; i<TOTAL_ITEMS; i++)
	{
		if (rcv.check(gi[i])!=snd.check(gi[i])) {
			printf("\rInconsistent replication!\n");
			ret=-1;
			goto _exit;
		}
		if (rcv.check(uigi[i])!=snd.check(uigi[i])) {
			printf("\rInconsistent replication!\n");
			ret=-1;
			goto _exit;
		}
		if (i%1000==0) {
			printf("\r%.2f%% completed.", (double)i*100/TOTAL_ITEMS);
			fflush(stdout);
		}
	}

	printf("\rPhase 1: Adding items:\nBFilter size: %d (KB), %d (pages) \n",
			snd.get_num_page()*BF_PAGE_SIZE/1024, snd.get_num_page());
	printf("Total bytes: %d\n", byte_update);
	printf("Total times: %d\n", time_update);
	printf("Lagged update: %d\n", lag_update);
	printf("False miss due to lag: %.3f%%\n", (double)miss_item*100.0/TOTAL_ITEMS);
	printf("Phase 2: Removing items:\n");
	printf("Total bytes: %d\n", byte_update2);
	printf("Total times: %d\n" , time_update2);
	printf("Lagged update: %d\n" , lag_update2);
	printf("False Hit due to lag: %.3f%%\n" , (double)miss_item2*100.0/TOTAL_ITEMS);
	printf("Test Successful");

	if (gi) {
		delete []gi;
	}
	if (uigi) {
		delete []uigi;
	}

	return 0;

 _exit:
	printf("\rTest failed\n");

	if (gi) {
		delete []gi;
	}

	if (uigi) {
		delete []uigi;
	}

	return ret;
}
