// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: load_announcer.cc,v $ - load announcer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/05 07:20:10 $
 * Version :  $Revision: 1.13 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <netinet/in.h>
# include <unistd.h>
# include <stdio.h>
# include <mntent.h>
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
# include "tmpi_debug.h"
# include "sock_conn.h"
# include "mcast_annc.h"
# include "resmon.h"
# include "cbs_thread.h"
# include "../shared/daemon_common.h"
# include "../shared/mcast_struct.h"
# include "load_announcer.h"

LoadAnnouncer::LoadAnnouncer(char *mgrp, int port, char **mnts, int size, int cport, int itvl) : 
	ancer(NULL),
	mnt_ptrs(NULL),
	n_mnt(0),
	con_port(0),
	interval(itvl)
{
	struct sockaddr_in sin;

	if ((!mgrp)||(port<=0)||(cport<=0)) {
		tmpi_error(DBG_CALLER, "Bad parameters: mgrp=%s, port=%d, con_port=%d.\n", mgrp, port, cport);
		return;
	}

	if (sock_filladdr(mgrp, port, &sin)<0) {
		tmpi_error(DBG_CALLER, "Bad address: %s:%d.\n", mgrp, port);
		return;
	}

	if (sock_is_mcast_addr(&sin, sizeof(sin))!=1) {
		tmpi_error(DBG_CALLER, "Bad multi-cast address: %s:%d.\n", mgrp, port);
		return;
	}

	if (mnts&&(size>0)) {
		mnt_ptrs=new char*[size+1];
		memset(mnt_ptrs, 0, sizeof(char*)*(size+1));
		for (int i=0; i<size; i++) {
			if ((mnt_ptrs[i]=strdup(mnts[i]))==NULL) {
				tmpi_error(DBG_INTERNAL, "Out of memory.");
				goto _exit;
			}
		}
		n_mnt=size;
	}

	ancer=new Mc_ancer(mgrp, port);

	if (!ancer->good()) {
		tmpi_error(DBG_CALLER, "Cannot create multi-cast announcer.\n");
		goto _exit;
	}

	con_port=cport;

	return;

_exit:
	release();
	return;

}

void LoadAnnouncer::release()
{
	if (mnt_ptrs) {
		for (int i=0; i<n_mnt; i++) {
			if (mnt_ptrs[i]) {
				free(mnt_ptrs[i]);
				mnt_ptrs[i]=NULL;
			}
		}
		delete [] mnt_ptrs;
		mnt_ptrs=NULL;
	}

	if (ancer) {
		delete ancer;
		ancer=NULL;
	}
}

# define MAX_ERRORS 1000

static void random_usleep(unsigned long usec)
{
	int least=usec/2;
	usleep(least+random()%usec);
}

void *LoadAnnouncer::run()
{
	if (!good()) {
		tmpi_error(DBG_CALLER, "Attempt to run an uninitialized LoadAnnouncer\n");
		return NULL;
	}

	int errors=0;

	while (!should_quit()) {
		if (errors>MAX_ERRORS) {
			tmpi_error(DBG_APP, "Load Announcer quitting due to too many errors ...\n");
			return NULL;
		}

		int annc_error=0;
		if ((annc_error=announce())) {
			tmpi_error(DBG_INTERNAL, "LoadAnnouncer.announce failed.\n");
			errors+=annc_error;
		}

		random_usleep(interval*1000);
	}

	tmpi_error(DBG_APP, "Load Announcer quitting normally ...\n");

	return NULL;
}

int LoadAnnouncer::announce()
{
	if (!good()) {
		tmpi_error(DBG_INTERNAL, "LoadAnnouncer not initialized.\n");

		return 1;
	}

	int errors=0;

	int use_cpu=1;
	int use_mem=1;

	ResMon::CPU cpu;

	if (ResMon::get_cpu(&cpu)<0) {
		tmpi_error(DBG_INTERNAL, "Cannot obtain CPU load.\n");
		use_cpu=0;
		errors++;
	}

	ResMon::MEM mem;
	if (ResMon::get_mem(&mem)<0) {
		tmpi_error(DBG_INTERNAL, "Cannot obtain mem load.\n");
		use_mem=0;
		errors++;
	}

	Mc_anc_data data;

	struct {
		unsigned char key;
		void *buf;
		unsigned int len;
	} *p, 

	recs_cpu[] = {
		{KEY_CPU_TOT, &(cpu.cputotal), sizeof(cpu.cputotal)},
		{KEY_CPU_USR, &(cpu.user), sizeof(cpu.user)},
		{KEY_CPU_KER, &(cpu.kernel), sizeof(cpu.kernel)},
		{KEY_CPU_IOW, &(cpu.iowait), sizeof(cpu.iowait)},
		{KEY_CPU_IDL, &(cpu.idle), sizeof(cpu.idle)},
		{KEY_CPU_NUM, &(cpu.cpus), sizeof(cpu.cpus)},
		{KEY_CPU_CXW, &(cpu.cswitches), sizeof(cpu.cswitches)},
		{0, NULL, 0}
	}, 

	recs_mem[] = {
		{KEY_MEM_TOT, &(mem.mtotal), sizeof(mem.mtotal)},
		{KEY_MEM_USE, &(mem.mused), sizeof(mem.mused)},
		{KEY_MEM_FRE, &(mem.mfree), sizeof(mem.mfree)},
		/* {KEY_MEM_SHR, &(mem.mshared), sizeof(mem.mshared)}, */
		{KEY_MEM_BUF, &(mem.mbuffers), sizeof(mem.mbuffers)},
		{KEY_MEM_CCH, &(mem.mcached), sizeof(mem.mcached)},
		{KEY_SWP_TOT, &(mem.stotal), sizeof(mem.stotal)},
		{KEY_SWP_FRE, &(mem.sfree), sizeof(mem.sfree)},
		{0, NULL, 0}
	};

	if (use_cpu) {
		p=recs_cpu;
		while (p->buf) {
			if (data.add(p->key, p->buf, p->len)<0) {
				tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
				errors++;
			}
			p++;
		}
	}	

	if (use_mem) {
		p=recs_mem;
		while (p->buf) {
			if (data.add(p->key, p->buf, p->len)<0) {
				tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
				errors++;
			}
			p++;
		}
	}	

# if (0)
	/**
	 * The following has to be modified to read the
	 * configuration file and send out only the 
	 * managed volumns. 
	 */
	static const char *MTAB_NAME = "/etc/mtab";
	FILE *fp;
	struct mntent *mnt;
	int ndisks=0;
	int use_disks=1;

	if ((fp=setmntent(MTAB_NAME, "r"))==NULL) {
		tmpi_error(DBG_INTERNAL, "Cannot open mtab.\n");
		errors++;
		use_disks=0;
	}

	if (use_disks) {
		while ((mnt=getmntent(fp))) {
			if (strcmp(mnt->mnt_fsname, "none")==0) {
				continue;
			}
			ResMon::DISK *d=ResMon::get_disk(mnt->mnt_dir);
			if ((d->good)&&(d->btotal>0)&&(d->ftotal>0)) {
				ndisks++;

				DiskInfo dsk;
				dsk.btotal=d->btotal;
				dsk.bfree=d->bavail;

				if (data.add(KEY_DSK_FRE, &dsk, sizeof(dsk))<0) {
					tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
					errors++;
				}

				delete d;
			}
		}
		endmntent(fp);
	}


	if (ndisks>0) {
		if (data.add(KEY_DSK_NUM, &ndisks, sizeof(ndisks))<0) {
			tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
			errors++;
		}
	}
# endif

	int ndisks=0;

	if (n_mnt>0) {
		for (int i=0; i<n_mnt; i++) {
			ResMon::DISK *d=ResMon::get_disk(mnt_ptrs[i]);
			if ((d->good)&&(d->btotal>0)&&(d->ftotal>0)) {
				ndisks++;

				DiskInfo dsk;
				dsk.btotal=d->btotal;
				dsk.bfree=d->bavail;

				if (data.add(KEY_DSK_FRE, &dsk, sizeof(dsk))<0) {
					tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
					errors++;
				}

				delete d;
			}
		}
	}

	if (ndisks>0) {
		if (data.add(KEY_DSK_NUM, &(ndisks), sizeof(ndisks))<0) {
			tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
			errors++;
		}
	}

	if (data.add(KEY_SCSD_PORT, &(con_port), sizeof(con_port))<0) {
		tmpi_error(DBG_INTERNAL, "Mc_annc_data.add failed.\n");
		errors++;
	}

# if defined(__DEBUG__)
	else {
		// dump_anc(&data);
	}
# endif

	return ancer->announce(&data);
}
