// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: load_announcer.h,v $ - header file for provider load announcer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _LOAD_ANNOUNCER_H_
# define _LOAD_ANNOUNCER_H_
# include "cbs_thread.h"
# include "mcast_annc.h"

/**
 * Usage:
 * LoadAnnouncer *la=new(group, port);
 * if (!la->good()) 
 * 	goto _exit;
 * 
 * if (la->run()<0) {
 *	failed to create a thread to run the announcer.
 * 	goto _exit;
 * }
 *
 * do other stuff. (e.g. create other daemons)
 * ...
 * finally
 * bind to a control port
 * wait for shutdown message
 * la->stop_announcer();
 * la->wait_for_shutdown();
 *
 * exit(0);
 * }
 */
class LoadAnnouncer : public CBS_Thread {
private:
	Mc_ancer *ancer;
	char **mnt_ptrs;
	int n_mnt;
	int con_port;
	int interval;

	int announce();
	void release();

public:
	LoadAnnouncer(); // illegal constructor
	LoadAnnouncer(char *mgrp, int port, char *mnts[], int size, int cport, int itvl);
	virtual ~LoadAnnouncer() { release(); }
	virtual void * run();
	bool good() { return (ancer!=NULL); }
};

# endif
