
// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: client.h,v $ - Client class for sending requests to the server
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/10 03:12:16 $ by $Author: bill $
 * Version :  $Revision: 1.17 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu, gulbeden@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */


#ifndef __CLIENT_H__
#define __CLIENT_H__

# include <string.h>
# include <stdlib.h>
# include "udp_cli.h"
# include "guid.h"
# include "sorrento_types.h"

/* descriptor consists of host-address port# and descriptor at server */
class Descriptor
{
public:
	ServerAddr saddr;
	int desc;

	Descriptor(ServerAddr0 &sa, int d) : saddr(sa), desc(d) { }
	// Descriptor(ServerAddr0 &sa, int d) : saddr(sa), desc(d) { }
	Descriptor() : desc(-1) { }
	bool valid() { return ((desc>=0) && saddr.valid()); }
	void reset() { *this = Descriptor(); }
};

class ClientID{
    public:
        long id;
        ClientID(long theID) : id(theID) {}
};

# define STG_REDIR_LOOP_DEFAULT 10
class Client
{
private:
	Udp_cli *cli;			/* to send requests */
	int stop_server_normal(const ServerAddr & saddr);

public:
	Client();

	int create(ServerAddr & saddr, ObjID &guid, Descriptor &desc, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int open(ServerAddr & saddr, ObjID &guid, Descriptor &desc, int flags, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int close(Descriptor &desc, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int truncate(Descriptor &desc, off_seg_t offset, off_seg_t &new_size, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int remove(ServerAddr & saddr, ObjID &guid, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int read(Descriptor &desc, off_seg_t offset, char *buffer, off_seg_t length, off_seg_t &read_bytes, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
  	int write(Descriptor &desc, off_seg_t offset, char *buffer, off_seg_t length, off_seg_t &write_bytes, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int append(Descriptor &desc, char *buffer, off_seg_t length, off_seg_t &append_bytes, int max_redir_loop=STG_REDIR_LOOP_DEFAULT);
	int stop_server(const ServerAddr & saddr);
};

#endif
