// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: protocol.cc,v $ - The protocol that servers use
 *            communicating each other.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/02/19 13:14:51 $ by $Author: gulbeden $
 * Version :  $Revision: 1.14 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu, gulbeden@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

# include <stdlib.h>
# include <string>
# include <errno.h>

# include "protocol.h"
# include "sock_conn.h"
# include "sock_dg.h"
# include "sock_sr.h"
# include "udp_cli.h"
# include "udp_srv.h"
# include "tmpi_debug.h"
# include "sp_singleton.h"

using namespace std;

Stg_Request *stg_req_alloc(int op_type, off_seg_t len)
{
	Stg_Request *ret=(Stg_Request *)calloc(1, STG_REQ_HEADER_SZ+len);
	if (ret) {
		ret->header.tag=op_type;
		ret->header.size=len;
	}

	return ret;
}

Stg_Request *new_stg_req_create1(ObjID &oid)
{
	off_seg_t len=sizeof(Stg_Req_Create1);
	Stg_Request *ret=stg_req_alloc(STG_REQ_CREATE1, len);
	if (ret) {
		ret->u.u_create1.oid=oid;
	}

	return ret;
}

Stg_Request *new_stg_req_open(ObjID &oid, int flags)
{
	off_seg_t len=sizeof(Stg_Req_Open);
	Stg_Request *ret=stg_req_alloc(STG_REQ_OPEN, len);
	if (ret) {
		ret->u.u_open.oid=oid;
		ret->u.u_open.flags=flags;
	}

	return ret;
}

Stg_Request *new_stg_req_close(int desc, int flags)
{
	off_seg_t len=sizeof(Stg_Req_Close);
	Stg_Request *ret=stg_req_alloc(STG_REQ_CLOSE, len);
	if (ret) {
		ret->u.u_close.desc=desc;
		ret->u.u_close.flags=flags;
	}

	return ret;
}

Stg_Request *new_stg_req_read(int desc, off_seg_t offset, off_seg_t length)
{
	off_seg_t len=sizeof(Stg_Req_Read);
	Stg_Request *ret=stg_req_alloc(STG_REQ_READ, len);
	if (ret) {
		ret->u.u_read.desc=desc;
		ret->u.u_read.offset=offset;
		ret->u.u_read.length=length;
	}

	return ret;
}

Stg_Request *new_stg_req_write(int desc, off_seg_t offset, char *buf, off_seg_t length)
{
	if (length <= STG_MAX_UDP_SIZE) {
		off_seg_t len=STG_REQ_WRITE_SMALL_HEADER_SZ+length;
		Stg_Request *ret=stg_req_alloc(STG_REQ_WRITE_SMALL, len);
		if (ret) {
			ret->u.u_write_small.desc=desc;
			ret->u.u_write_small.offset=offset;
			ret->u.u_write_small.length=length;
			// memcpy(ret->u.u_write_small.data, buf, length);
		}

		return ret;
	}
	else {
		off_seg_t len=sizeof(Stg_Req_Write_Big);
		Stg_Request *ret=stg_req_alloc(STG_REQ_WRITE_BIG, len);
		if (ret) {
			ret->u.u_write_big.desc=desc;
			ret->u.u_write_big.offset=offset;
			ret->u.u_write_big.length=length;
		}

		return ret;
	}
}

Stg_Request *new_stg_req_append(int desc, char *buf, off_seg_t length)
{
	if (length <= STG_MAX_UDP_SIZE) {
		off_seg_t len=STG_REQ_APPEND_SMALL_HEADER_SZ+length;
		Stg_Request *ret=stg_req_alloc(STG_REQ_APPEND_SMALL, len);
		if (ret) {
			ret->u.u_append_small.desc=desc;
			ret->u.u_append_small.length=length;
			// memcpy(ret->u.u_append_small.data, buf, length);
		}

		return ret;
	}
	else {
		off_seg_t len=sizeof(Stg_Req_Append_Big);
		Stg_Request *ret=stg_req_alloc(STG_REQ_APPEND_BIG, len);
		if (ret) {
			ret->u.u_append_big.desc=desc;
			ret->u.u_append_big.length=length;
		}

		return ret;
	}
}

Stg_Request *new_stg_req_remove(ObjID &oid)
{
	off_seg_t len=sizeof(Stg_Req_Remove);
	Stg_Request *ret=stg_req_alloc(STG_REQ_REMOVE, len);
	if (ret) {
		ret->u.u_remove.oid=oid;
	}

	return ret;
}

Stg_Request *new_stg_req_truncate(int desc, off_seg_t size)
{
	off_seg_t len=sizeof(Stg_Req_Truncate);
	Stg_Request *ret=stg_req_alloc(STG_REQ_TRUNCATE, len);
	if (ret) {
		ret->u.u_truncate.desc=desc;
		ret->u.u_truncate.size=size;
	}

	return ret;
}

Stg_Request *new_stg_req_add_entry(HostID &hid, ObjID &oid, int version)
{
	off_seg_t len=sizeof(Stg_Req_Add_Entry);
	Stg_Request *ret=stg_req_alloc(STG_REQ_ADD_ENTRY, len);
	if (ret) {
		ret->u.u_add_entry.hid=hid;
		ret->u.u_add_entry.oid=oid;
		ret->u.u_add_entry.version=version;
	}

	return ret;
}

Stg_Request *new_stg_req_del_entry(HostID &hid, ObjID &oid)
{
	off_seg_t len=sizeof(Stg_Req_Del_Entry);
	Stg_Request *ret=stg_req_alloc(STG_REQ_DEL_ENTRY, len);
	if (ret) {
		ret->u.u_del_entry.hid=hid;
		ret->u.u_del_entry.oid=oid;
	}

	return ret;
}

Stg_Request *new_stg_req_get_range(HostID &hid)
{
	off_seg_t len=sizeof(Stg_Req_Get_Range);
	Stg_Request *ret=stg_req_alloc(STG_REQ_GET_RANGE, len);
	if (ret) {
		ret->u.u_get_range.hid=hid;
	}

	return ret;
}

Stg_Request *new_stg_req_sync_host(ServerAddr &sa, ObjID oid, int otherV)
{
	off_seg_t len=sizeof(Stg_Req_Sync_Host);
	Stg_Request *ret=stg_req_alloc(STG_REQ_SYNC_HOST, len);
	if (ret) {
		ret->u.u_sync_host.sa=sa;
		ret->u.u_sync_host.oid=oid;
		ret->u.u_sync_host.otherV=otherV;
	}

	return ret;
}

Stg_Request *new_stg_req_notify_ver_ch(ServerAddr &sa, ObjID oid, int version)
{
	off_seg_t len=sizeof(Stg_Req_Notify_Ver_Ch);
	Stg_Request *ret=stg_req_alloc(STG_REQ_NOTIFY_VER_CH, len);
	if (ret) {
		ret->u.u_notify_ver_ch.sa=sa;
		ret->u.u_notify_ver_ch.oid=oid;
		ret->u.u_notify_ver_ch.version=version;
	}

	return ret;
}

Stg_Request *new_stg_req_stop(void)
{
	off_seg_t len=sizeof(Stg_Req_Stop);
	Stg_Request *ret=stg_req_alloc(STG_REQ_STOP, len);
	if (ret) {
		ret->u.u_stop.flags=0;
	}

	return ret;
}

/**
 * The transport management.
 */

# define STG_MAX_RETRY 10
// client read timeout 10 seconds
# define STG_CLI_READ_TOUT 10000
// server read timeout 100 mili-second.
# define STG_SRV_READ_TOUT 100

/**
 * The operation matrix:
 * Start_request:
 *             No Payload              With Payload
 *     UDP     All Op                  !USE_TCP
 *     TCP     All Op                  All Op
 *	
 * The following requests have USE_TCP set:
 * 	write_big, append_big, read_big.
 *
 * Wait_response:
 *             No Payload              With Payload
 *     UDP     All Op                  !USE_TCP
 *     TCP     All Op                  All Op
 * The following response(s) have USE_TCP set:
 * 	get_range_big, redir1_big..
 *
 * The plan: for both start_request and wait_response, there will
 * be three possible transports (or even more): TCP, UDP, Cached_TCP
 * and two possible scenarios: No-payload and with payload. (totally
 * N*4 functions, where N is the number of transports).
 * The transport selection is done by four functions:
 *     start_request_no_payload, start_request_with_payload
 *     wait_response_no_payload, stop_response_with_payload
 * The actual Client APIs use the following matrix to decide
 * which of those four functions should be used.
 *                With payload           No payload
 * Start_request  write/append           (all others)
 * Wait_response  read/get_range/redir1  (all others)
 * 
 * The selection can be done by using hints.
 */

static int start_request_no_payload_udp(Udp_cli *cli, const ServerAddr &saddr, Stg_Request *req, Stg_Req_Handle &handle)
{
	// we first try cached connection, then try udp
	int fd = sp_client_get_one(saddr.ip, saddr.tcp_port);
	if (fd >= 0) {
		if (sock_write(fd, (char *)req, stg_req_size(req))==0) {
			handle.tcp_sock=fd;
			return STG_RES_OKAY;
		}
        else {
            ::close(fd);
        }
	}

	handle.udp_seq=cli->async_req(saddr.ip, saddr.udp_port, req, stg_req_size(req), STG_MAX_RETRY);
	if (handle.udp_seq == UDP_SEQ_ERR) {
		return STG_RES_ENETWORK;
	}

	return STG_RES_OKAY;
}

static int start_request_no_payload_tcp(const ServerAddr &saddr, Stg_Request *req, Stg_Req_Handle &handle)
{
	// we first try cached connection, then try udp
	int fd = sp_client_get_one(saddr.ip, saddr.tcp_port);
	if (fd >= 0) {
		if (sock_write(fd, (char *)req, stg_req_size(req))==0) {
			handle.tcp_sock=fd;
			return STG_RES_OKAY;
		}
        else {
            ::close(fd);
        }
	}

	handle.tcp_sock=sock_conn(saddr.ip, saddr.tcp_port);
	if (handle.tcp_sock<0) {
		return STG_RES_ENETWORK;
	}

	if (sock_write(handle.tcp_sock, (char *)req, stg_req_size(req))<0) {
		return STG_RES_ENETWORK;
	}

	return STG_RES_OKAY;
}

// everything is in req, can use either TCP or UDP.
// unless we are told to use TCP.
int stg_start_request_no_payload(Udp_cli *cli, const ServerAddr &saddr, Stg_Request *req, Stg_Req_Handle &handle, int hint)
{
	assert (!(req->header.tag & STG_DATA_FOLLOWS));
	int status;
	// choice logic: always use UDP unless hint says no.
	if (hint & STG_USE_TCP) {
		// possible when we expect a large response.
		status = start_request_no_payload_tcp(saddr, req, handle);
	}
	else {
		status = start_request_no_payload_udp(cli, saddr, req, handle);
	}

	handle.status = (status == STG_RES_OKAY)?STG_RES_IN_USE:status;

	return status;
}

static int start_request_with_payload_udp(Udp_cli *cli, const ServerAddr &saddr, Stg_Request *req, char *payload, off_seg_t length, Stg_Req_Handle &handle)
{
	// copy payload into the request.
	memcpy(req->payload(), payload, length);

	return start_request_no_payload_udp(cli, saddr, req, handle);
}

static int start_request_with_payload_tcp(const ServerAddr &saddr, Stg_Request *req, char *payload, off_seg_t length, Stg_Req_Handle &handle)
{
	if (!(req->header.tag & STG_DATA_FOLLOWS)) {
		// payload should be embeded in the request.
		// TODO: The following memcpy could be elimiated.
		memcpy(req->payload(), payload, length);
		return start_request_no_payload_tcp(saddr, req, handle);
	}

	// payload should be sent after the request.
	int ret=start_request_no_payload_tcp(saddr, req, handle);
	if (ret != STG_RES_OKAY)
		return ret;
	
	// should we wait till we get an acknowledge
	// from the server?
	if (sock_write(handle.tcp_sock, payload, length)<0) {
		return STG_RES_ENETWORK;
	}
	return STG_RES_OKAY;
}

int stg_start_request_with_payload(Udp_cli *cli, const ServerAddr &saddr, Stg_Request *req, char *payload, off_seg_t length, Stg_Req_Handle &handle, int hint)
{
	assert(req->payload_size() == length);

	// choice logic: always use UDP unless hint says no, or when 
	// USE_TCP is set.
	int status;
	if ((hint & STG_USE_TCP) || (req->header.tag & STG_DATA_FOLLOWS)) {
		// possible when we expect a large response.
		status = start_request_with_payload_tcp(saddr, req, payload, \
			length, handle);
	}
	else {
		status = start_request_with_payload_udp(cli, saddr, req, payload, \
			length, handle);
	}
	handle.status = (status == STG_RES_OKAY)?STG_RES_IN_USE:status;
	return status;
}

static void drain_tcp_data(int sock, off_seg_t len)
{
	int buf_len;
	const unsigned BUF_LEN = (64*1024U);
	char buf[BUF_LEN];
	
	while (len>0) {
		buf_len=(int)((len>BUF_LEN)?BUF_LEN:len);
		(void)sock_read_tout(sock, buf, &buf_len, STG_CLI_READ_TOUT);
		if (buf_len>0) {
			// we got some data.
			len-=buf_len;
		}
		else {
			// bail-out
			break;
		}
	}

	return;
}

static int wait_response_no_payload_udp(Udp_cli *cli, seq_t seq, Stg_Res_Handle &handle)
{
	handle.udp_res=cli->wait_one(seq);
	if ((!handle.udp_res)||(!handle.udp_res[0])) {
		return STG_RES_ENETWORK;
	}

	if (handle.udp_res[0]->status != UDP_RES_OKAY) {
		return STG_RES_ENETWORK;
	}

	Stg_Response *res = handle.response();

	if (res->header.tag == STG_RES_ERROR) {
		return res->u.u_error.status;
	}

	return STG_RES_OKAY;
}

static int wait_response_no_payload_tcp(int sock, Stg_Res_Handle &handle)
{
	Stg_Header header;
	int len=sizeof(header);
	int errnum=sock_read_tout(sock, (char *)&header, &len, STG_CLI_READ_TOUT);
	if (len != sizeof(header)) {
		tmpi_error(DBG_VERBOSE, "errnum=%d, Got %d bytes, expecting %d bytes.\n", errnum, len, sizeof(header));
		return (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
	}

	if (header.tag == STG_RES_ERROR) {
		assert(header.size == sizeof(struct Stg_Res_Error));
		struct Stg_Res_Error st_err;
		len=sizeof(st_err);
		int errnum =sock_read_tout(sock, (char *)&st_err, &len, STG_CLI_READ_TOUT);
		if (len != sizeof(st_err)) {
			tmpi_error(DBG_VERBOSE, "errnum=%d, Got %d bytes, expecting %d bytes.\n", errnum, len, sizeof(st_err));
			return (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
		}

		tmpi_error(DBG_VERBOSE, "Server responds error (%d).\n", st_err.status);
		return st_err.status;
	}

	handle.tcp_res=stg_res_alloc(header.tag, header.size);
	if (!handle.tcp_res) {
		// try to drain the data. (is it necessary?)
		// drain_tcp_data(sock, header.size);
		tmpi_error(DBG_VERBOSE, "Cannot allocate (tag: %d) %d bytes.\n", header.tag, header.size);
		return STG_RES_EMEM;
	}

	if (header.size != 0) {
		len=(int)header.size;
		int errnum = sock_read_tout(sock, (char *)&(handle.tcp_res->u), &len, STG_CLI_READ_TOUT);

		if (len != (int)header.size) {
			tmpi_error(DBG_VERBOSE, "errnum=%d, Got %d bytes, expecting %d bytes.\n", errnum, len, (int)(header.size));
			return (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
		}
	}

	return STG_RES_OKAY;
}

int stg_wait_response_no_payload(Udp_cli *cli, Stg_Req_Handle &req_handle, \
	Stg_Res_Handle &res_handle)
{
	if (req_handle.status != STG_RES_IN_USE) {
		return req_handle.status;
	}

	int status;
	if (req_handle.use_tcp()) {
		status = wait_response_no_payload_tcp(req_handle.tcp_sock, res_handle);
	}
	else {
		status = wait_response_no_payload_udp(cli, req_handle.udp_seq, res_handle);
	}

	switch (status) {
		case STG_RES_ENOTFOUND:
		case STG_RES_EEXIST:
		case STG_RES_EIO:
		case STG_RES_ESPACE:
			req_handle.status=STG_RES_OKAY; break;
		default:
			req_handle.status=status;
	}

	return status;
}

static int wait_response_with_payload_cont_udp(Stg_Res_Handle &handle, char *payload, off_seg_t length)
{
	off_seg_t payload_size=handle.response()->payload_size();
	if (length < payload_size) {
		return STG_RES_EBUFFER;
	}

	if ((payload_size>0)&&(handle.response()->payload())) {
		memcpy(payload, handle.response()->payload(), payload_size);
	}

	return STG_RES_OKAY;
}

static int wait_response_with_payload_udp
	(Udp_cli *cli, seq_t seq, Stg_Res_Handle &handle, char *payload, off_seg_t length)
{
	int ret=wait_response_no_payload_udp(cli, seq, handle);
	if (ret != STG_RES_OKAY) {
		return ret;
	}

	return wait_response_with_payload_cont_udp(handle, payload, length);
}

// Since the length could be very long, we do retry as long as
// we get something.
static int wait_response_with_payload_cont_tcp
	(int sock, Stg_Res_Handle &handle, char *payload, off_seg_t length)
{
	Stg_Response *res=handle.response();
	if (length < res->payload_size()) {
		return STG_RES_EBUFFER;
	}

	if (res->payload()) {
		// payload is inside the response.
		memcpy(payload, res->payload(), res->payload_size());
		return STG_RES_OKAY;
	}

	char *p=payload;
	off_seg_t remain_bytes=handle.response()->payload_size();
	while (remain_bytes>0) {
		off_seg_t len=remain_bytes;
		int errnum = sock_read_tout(sock, p, (int *)&len, STG_CLI_READ_TOUT);
		if (len > 0) {
			p+=len;
			remain_bytes-=len;
		}
		else {
			return (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
		}
	}

	return STG_RES_OKAY;
}

int stg_wait_response_with_payload_cont
	(Udp_cli *cli, Stg_Req_Handle &reqh, Stg_Res_Handle &resh, \
			char *payload, off_seg_t length)
{
	if (reqh.status != STG_RES_EBUFFER) {
		return reqh.status;
	}

	int status;
	if (reqh.use_tcp()) {
		 status = wait_response_with_payload_cont_tcp(reqh.tcp_sock, \
				resh, payload, length);
	}
	else {
		status = wait_response_with_payload_cont_udp(resh, payload, length);
	}

	reqh.status=status;

	return status;
}

static int wait_response_with_payload_tcp
	(int sock, Stg_Res_Handle &handle, char *payload, off_seg_t length)
{
	int ret=wait_response_no_payload_tcp(sock, handle);
	if (ret != STG_RES_OKAY) {
		return ret;
	}

	return wait_response_with_payload_cont_tcp(sock, handle, payload, length);
}

// If the user provided payload buffer is too small, we 
// may return EAGAIN when using UDP, or EBUFFER when using
// TCP. The desired buffer size can be retrieved by using
// res_handle.response()->payload_size().
// For EAGAIN, we need to restart the request, for EBUFFER,
// we need to call wait_response_with_payload_tcp_cont().
int stg_wait_response_with_payload(Udp_cli *cli, Stg_Req_Handle &req_handle, Stg_Res_Handle &res_handle, char *payload, off_seg_t length)
{
	if (req_handle.status != STG_RES_IN_USE) {
		return req_handle.status;
	}

	int status;

	if (req_handle.use_tcp()) {
		status = wait_response_with_payload_tcp(req_handle.tcp_sock, res_handle, payload, length);
	}
	else {
		status = wait_response_with_payload_udp(cli, req_handle.udp_seq, res_handle, payload, length);
	}

	switch (status) {
		case STG_RES_ENOTFOUND:
		case STG_RES_EEXIST:
		case STG_RES_EIO:
		case STG_RES_ESPACE:
			req_handle.status=STG_RES_OKAY; break;
		default:
			req_handle.status=status;
	}

	return status;
}

Stg_Res_Handle::~Stg_Res_Handle()
{
	if (tcp_res) {
		stg_res_free(tcp_res);
	}
	if (udp_res) {
		Udp_cli::free_results(udp_res);
	}
}

////////////////////////////////////////////////////////

Stg_Response *stg_res_alloc(int op_type, off_seg_t len)
{
	Stg_Response *ret=(Stg_Response *)calloc(1, STG_RES_HEADER_SZ+len);
	if (ret) {
		ret->header.tag=op_type;
		ret->header.size=len;
	}

	return ret;
}

void stg_res_adjust(Stg_Response *res, int op_type, off_seg_t len)
{
	if (res) {
		if (res->header.size<len) // can only shrink an allocated structure.
			throw STG_RES_ESIZE;
		res->header.tag=op_type;
		res->header.size=len;
	}
}

Stg_Response *new_stg_res_okay(void)
{
	return stg_res_alloc(STG_RES_OKAY, 0);
}

Stg_Response *new_stg_res_error(int errnum)
{
	off_seg_t len=sizeof(Stg_Res_Error);
	Stg_Response *ret=stg_res_alloc(STG_RES_ERROR, len);
	if (ret) {
		ret->u.u_error.status=errnum;
	}

	return ret;
}

Stg_Response *new_stg_res_redir1(const ServerAddr0* sas, int nSrvrs, \
								 bool use_tcp)
{
	Stg_Response *ret=NULL;
	int payloadSize = nSrvrs*sizeof(ServerAddr0);

	if (use_tcp) {
 		// TCP
 		off_seg_t len=sizeof(Stg_Res_Redir1_Big);
 		ret=stg_res_alloc(STG_RES_REDIR1_BIG, len);
 		if (ret) {
 			ret->u.u_redir1_big.nHosts = nSrvrs;
 		}
	}

	else if (STG_RES_REDIR1_SMALL_HEADER_SZ + payloadSize 
			   <= STG_MAX_UDP_SIZE){
		// UDP
		off_seg_t len = STG_RES_REDIR1_SMALL_HEADER_SZ + payloadSize;
		
		ret=stg_res_alloc(STG_RES_REDIR1_SMALL, len);
		if (ret) {
			memcpy((char*) (ret->u.u_redir1_small.saddrs), (char *)sas, payloadSize);
			ret->u.u_redir1_small.nHosts = nSrvrs;
		}
	}

	else {
		throw STG_RES_ESIZE;
	}

	return ret;
}

Stg_Response *new_stg_res_redir2(const ServerAddr &saddr, int desc)
{
	off_seg_t len=sizeof(Stg_Res_Redir2);
	Stg_Response *ret=stg_res_alloc(STG_RES_REDIR2, len);
	if (ret) {
		ret->u.u_redir2.saddr=saddr;
		ret->u.u_redir2.desc=desc;
	}

	return ret;
}

Stg_Response *new_stg_res_create(int desc, int version)
{
	off_seg_t len=sizeof(Stg_Res_Create);
	Stg_Response *ret=stg_res_alloc(STG_RES_CREATE, len);
	if (ret) {
		ret->u.u_create.desc=desc;
		ret->u.u_create.version=version;
	}

	return ret;
}

Stg_Response *new_stg_res_open(int desc)
{
	off_seg_t len=sizeof(Stg_Res_Open);
	Stg_Response *ret=stg_res_alloc(STG_RES_OPEN, len);
	if (ret) {
		ret->u.u_open.desc=desc;
	}

	return ret;
}

Stg_Response *new_stg_res_truncate(off_seg_t new_size)
{
	off_seg_t len=sizeof(Stg_Res_Truncate);
	Stg_Response *ret=stg_res_alloc(STG_RES_TRUNCATE, len);
	if (ret) {
		ret->u.u_truncate.new_size=new_size;
	}

	return ret;
}

Stg_Response *new_stg_res_write(off_seg_t written_bytes)
{
	off_seg_t len=sizeof(Stg_Res_Write);
	Stg_Response *ret=stg_res_alloc(STG_RES_WRITE, len);
	if (ret) {
		ret->u.u_write.written_bytes=written_bytes;
	}

	return ret;
}

Stg_Response *new_stg_res_append(off_seg_t append_bytes)
{
	off_seg_t len=sizeof(Stg_Res_Append);
	Stg_Response *ret=stg_res_alloc(STG_RES_APPEND, len);
	if (ret) {
		ret->u.u_append.append_bytes=append_bytes;
	}

	return ret;
}

Stg_Response *new_stg_res_close(int version)
{
	off_seg_t len=sizeof(Stg_Res_Close);
	Stg_Response *ret=stg_res_alloc(STG_RES_CLOSE, len);
	if (ret) {
		ret->u.u_close.version=version;
	}

	return ret;
}


Stg_Response *new_stg_res_read(off_seg_t read_bytes, bool use_tcp)
{
	Stg_Response *ret=NULL;

	if (use_tcp) {
		// TCP
		off_seg_t len=sizeof(Stg_Res_Read_Big);
		ret=stg_res_alloc(STG_RES_READ_BIG, len);
		if (ret) {
			ret->u.u_read_big.read_bytes=read_bytes;
		}
	}
	else if ((read_bytes <= STG_MAX_UDP_SIZE)) {
		// UDP
		off_seg_t len=STG_RES_READ_SMALL_HEADER_SZ+read_bytes;
		ret=stg_res_alloc(STG_RES_READ_SMALL, len);
		if (ret) {
			ret->u.u_read_small.read_bytes=read_bytes;
		}
	}
	else {
		throw STG_RES_ESIZE;
	}

	return ret;
}


void adjust_stg_res_read(Stg_Response *res, off_seg_t read_bytes, bool use_tcp)
{
	if (use_tcp) {
		// TCP
		off_seg_t len=sizeof(Stg_Res_Read_Big);
		stg_res_adjust(res, STG_RES_READ_BIG, len);
		res->u.u_read_big.read_bytes=read_bytes;
	}
	else if ((read_bytes <= STG_MAX_UDP_SIZE)) {
		// UDP
		off_seg_t len=STG_RES_READ_SMALL_HEADER_SZ+read_bytes;
		stg_res_adjust(res, STG_RES_READ_SMALL, len);
		res->u.u_read_small.read_bytes=read_bytes;
	}
	else {
		throw STG_RES_ESIZE;
	}
}

Stg_Response *new_stg_res_get_range(HostID &hid, ObjID *objs, int n_objs, bool use_tcp)
{
	Stg_Response *ret=NULL;
	off_seg_t length=n_objs*sizeof(ObjID);

	if (use_tcp) {
		off_seg_t len=sizeof(Stg_Res_Get_Range_Big);
		ret=stg_res_alloc(STG_RES_GET_RANGE_BIG, len);
		if (ret) {
			ret->u.u_get_range_big.hid=hid;
			ret->u.u_get_range_big.n_objs=n_objs;
		}
	}
	else if (length <= STG_MAX_UDP_SIZE) {
		off_seg_t len=STG_RES_GET_RANGE_SMALL_HEADER_SZ+length;
		ret=stg_res_alloc(STG_RES_GET_RANGE_SMALL, len);
		if (ret) {
			ret->u.u_get_range_small.hid=hid;
			ret->u.u_get_range_small.n_objs=n_objs;
			memcpy(ret->u.u_get_range_small.objs, objs, length);
		}
	}
	else {
		throw STG_RES_ESIZE;
		return NULL;
	}

	return ret;
}

void Stg_Srv_Request::respond(Udp_srv *srv, Stg_Response *res) 
{
    if (use_tcp()) {
        if (sock_write(tcp_sock, (char *)res, stg_res_size(res))<0) {
			status = STG_RES_ENETWORK;
			tmpi_error(DBG_APP, "Netowrk problem.\n");
		}
		else {
			status = STG_RES_OKAY;
		}
    }
	else {
    	if (srv->respond(udp_req, res, stg_res_size(res))<0) {
			status = STG_RES_ENETWORK;
			tmpi_error(DBG_APP, "Netowrk problem.\n");
		}
		else {
			status = STG_RES_OKAY;
		}
	}
}

void Stg_Srv_Request::respond_error(Udp_srv *srv, int errnum)
{
	Stg_Response *res=new_stg_res_error(errnum);
	if (res) {
		respond(srv, res);
		stg_res_free(res);
	}
	else {
		status = STG_RES_EMEM;
		tmpi_error(DBG_INTERNAL, "Out of memory.\n");
	}
}

void Stg_Srv_Request::respond_okay(Udp_srv *srv)
{
	Stg_Response *res=new_stg_res_okay();
	if (res) {
		respond(srv, res);
		stg_res_free(res);
	}
	else {
		status = STG_RES_EMEM;
		tmpi_error(DBG_APP, "Out of memory.\n");
	}
}

bool Stg_Srv_Request::read_more(char *buffer, off_seg_t len, off_seg_t &read_bytes)
{
    if (tcp_sock<0)
        return false;

    read_bytes=len;

    // 100ms timeout.
    int errnum = sock_read_tout(tcp_sock, buffer, (int *)&read_bytes, 100);
	if (read_bytes == 0) {
		status = (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
        return false;
    }

    return true;
}

bool Stg_Srv_Request::write_more(char *buffer, off_seg_t len)
{
    if (tcp_sock<0)
        return false;

    // 100ms timeout.
    if (sock_write(tcp_sock, buffer, len)<0) {
		status = STG_RES_ENETWORK;
        return false;
    }

    return true;
}

Stg_Srv_Request::Stg_Srv_Request(int sock) :
	udp_req(NULL),
	tcp_req(NULL),
	tcp_sock(sock),
	status(STG_RES_IN_USE)
{
	Stg_Header header;
	int len = sizeof(header);
	int errnum = sock_read_tout(sock, (char *)&header, &len, STG_SRV_READ_TOUT);

	if (len != sizeof(header)) {
		status = (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
		if (status == STG_RES_ETOUT) {
			throw string("Stg_Srv_Request::Stg_Srv_Request: read time out.");
		}
		else {
			throw string("Stg_Srv_Request::Stg_Srv_Request: network problem.");
		}
	}

	tcp_req=stg_req_alloc(header.tag, header.size);
	if (!tcp_req) {
		status = STG_RES_EMEM;
		throw string("Stg_Srv_Request::Stg_Srv_Request: out of memory.");
	}
	len = (int)header.size;
	errnum = sock_read_tout(sock, (char *)&(tcp_req->u), &len, STG_SRV_READ_TOUT);
	if (len != (int)header.size) {
		status = (errnum==ETIMEDOUT)?STG_RES_ETOUT:STG_RES_ENETWORK;
		if (status == STG_RES_ETOUT) {
			throw string("Stg_Srv_Request::Stg_Srv_Request: read time out.");
		}
		else {
			throw string("Stg_Srv_Request::Stg_Srv_Request: network problem.");
		}
	}
}

Stg_Srv_Request::Stg_Srv_Request(Udp_srv_req *uq) :
	udp_req(uq),
	tcp_req(NULL),
	tcp_sock(-1),
	status(STG_RES_IN_USE)
{
	if ((request()->header.tag & STG_DATA_FOLLOWS)) {
		status=STG_RES_ECLIENT;
		throw string("Stg_Srv_Request::Stg_Srv_Request: Bad request");
	}
}

Stg_Srv_Request::~Stg_Srv_Request()
{
	if (udp_req) {
		delete udp_req;
	}
	if (tcp_sock>=0) {
		if (status == STG_RES_OKAY) {
			sp_server_put_one(tcp_sock);
		}
		else {
			::close(tcp_sock);
		}
	}
	if (tcp_req) {
		stg_req_free(tcp_req);
	}
}

Stg_Request *Stg_Srv_Request::request() const
{
	return (use_tcp())?tcp_req:(Stg_Request *)(udp_req->request());
}
