// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: protocol.h,v $ - The protocol that servers use
 *            communicating each other.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/05 23:34:47 $ by $Author: bill $
 * Version :  $Revision: 1.25 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu, gulbeden@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

#ifndef __PROTOCOL_H__
#define __PROTOCOL_H__

# include <unistd.h>
# include <vector>
# include "udp_common.h"
# include "sorrento_types.h"

/**
 * We use a generic header for both request and
 * response. The tag is used to determine the
 * type of the request or response. The size
 * is the size of the variable length structure
 * followed by the header.
 */
struct Stg_Header {
	// the tag for selecting the size 
	// of the following structure.
	int tag; 
	off_seg_t size; /*< the size of the following structure */
};

/* Request Types */
# define STG_USE_TCP 0x400
# define STG_USE_UDP 0x800
# define STG_DATA_FOLLOWS 0x1000
# define STG_REQ_PRIORITY 0x2000

/* File System Operations */
#define STG_REQ_CREATE1 1
#define STG_REQ_CREATE2 2
#define STG_REQ_OPEN 3
#define STG_REQ_CLOSE 4
#define STG_REQ_READ 6
#define STG_REQ_WRITE_SMALL 7
#define STG_REQ_WRITE_BIG  (8|STG_DATA_FOLLOWS)
#define STG_REQ_APPEND_SMALL 9
#define STG_REQ_APPEND_BIG (10|STG_DATA_FOLLOWS)
#define STG_REQ_REMOVE 11
#define STG_REQ_TRUNCATE 12

/* Server Index Operations, set the priority flag */
/* push information */
#define STG_REQ_ADD_ENTRY (101 | STG_REQ_PRIORITY)
/* push information */
#define STG_REQ_DEL_ENTRY (102 | STG_REQ_PRIORITY)
/* pull information */
#define STG_REQ_GET_RANGE (103 | STG_REQ_PRIORITY)

/* for migration */
#define STG_REQ_DEL_RANGE (104 | STG_REQ_PRIORITY)

/* for replication*/
#define STG_REQ_SYNC_HOST (105 | STG_REQ_PRIORITY)
#define STG_REQ_NOTIFY_VER_CH (106 | STG_REQ_PRIORITY)

/* Remote management */
#define STG_REQ_STOP 150

/* VFS Operations */
#define STG_REQ_DUMP 200

//#define MAX_UDP_SIZE 0
#define STG_MAX_UDP_SIZE UDP_SAFE_SZ

struct Stg_Req_Create1 {
	ObjID oid;
	//ClientID cid;
};

struct Stg_Req_Create2 {
	ObjID old_old;
	ObjID oid;
	//ClientID cid;
};

struct Stg_Req_Open {
	ObjID oid;
	int flags;
};

struct Stg_Req_Close {
	int desc;
	int flags; // currently 0, may be STG_ABORT or STG_COMMIT.
};

struct Stg_Req_Truncate {
	int desc;
	off_seg_t size;
};

struct Stg_Req_Remove {
	ObjID oid;
};

struct Stg_Req_Read {
	//ClientID cid;
	int desc;
	off_seg_t offset;
	off_seg_t length;
};

struct Stg_Req_Write_Small {
	int desc;
	off_seg_t offset;
	off_seg_t length;
	char data[1];
};

# define STG_REQ_WRITE_SMALL_HEADER_SZ \
	(offsetof(Stg_Req_Write_Small, data))

struct Stg_Req_Write_Big {
	//ClientID cid;
	int desc;
	off_seg_t offset;
	off_seg_t length;
};

struct Stg_Req_Append_Small {
	//ClientID cid;
	int desc;
	off_seg_t length;
	char data[1];
};

# define STG_REQ_APPEND_SMALL_HEADER_SZ (offsetof(Stg_Req_Append_Small, data))

struct Stg_Req_Append_Big {
	//ClientID cid;
	int desc;
	off_seg_t length;
};

struct Stg_Req_Stop {
	int flags; // not used currently
};

struct Stg_Req_Add_Entry {
	HostID hid;
	ObjID oid;
	int version;
};

struct Stg_Req_Del_Entry {
	HostID hid;
	ObjID oid;
};

struct Stg_Req_Get_Range {
	HostID hid;
};

struct Stg_Req_Sync_Host {
	ServerAddr0 sa;
	ObjID oid;
	int otherV;
};

struct Stg_Req_Notify_Ver_Ch {
	ServerAddr0 sa;
	ObjID oid;
	int version;
};

struct Stg_Req_Dump {
	int desc;
};

/**
To be implemented.
struct Stg_Req_Del_Range {
	HostID hid;
	int n_obj;
	ObjID objs[1];
};
**/

// For UDP, we need to calculate the size carefully.
// For TCP, we simply send sizeof(STG_REQ) bytes,
// followed by the actual data.
struct Stg_Request {
	struct Stg_Header header;
	union {
		// BIG requests can only be initiated through TCP
		struct Stg_Req_Write_Big u_write_big;
		struct Stg_Req_Append_Big u_append_big;

		// all other requests can be sent through both
		// UDP and TCP.
		struct Stg_Req_Create1 u_create1;
		// struct Stg_Req_Create2 u_create2; // to be implemented
		struct Stg_Req_Open u_open;
		struct Stg_Req_Close u_close;
		struct Stg_Req_Read u_read;
		struct Stg_Req_Write_Small u_write_small;
		struct Stg_Req_Append_Small u_append_small;
		struct Stg_Req_Stop u_stop;
		struct Stg_Req_Remove u_remove;
		struct Stg_Req_Truncate u_truncate;

		struct Stg_Req_Add_Entry u_add_entry;
		struct Stg_Req_Del_Entry u_del_entry;
		struct Stg_Req_Get_Range u_get_range;
		struct Stg_Req_Sync_Host u_sync_host;
		struct Stg_Req_Notify_Ver_Ch u_notify_ver_ch;
	} u;

	off_seg_t payload_size(void) const {
		switch (header.tag) {
			case STG_REQ_WRITE_SMALL:
				return u.u_write_small.length;
			case STG_REQ_WRITE_BIG:
				return u.u_write_big.length;
			case STG_REQ_APPEND_SMALL:
				return u.u_append_small.length;
			case STG_REQ_APPEND_BIG:
				return u.u_append_big.length;
			default:
				return 0;
		}
	}
	void * payload(void) {
		switch (header.tag) {
			case STG_REQ_WRITE_SMALL:
				return u.u_write_small.data;
			case STG_REQ_APPEND_SMALL:
				return u.u_append_small.data;
			default:
				return NULL;
		}
	}
};

# define STG_REQ_HEADER_SZ (offsetof(Stg_Request, u))

/* OKAY will be used for CLOSE, REMOVE, STOP, ADD_ENTRY, DEL_ENTRY, DEL_RANGE */
# define STG_RES_OKAY 0
# define STG_RES_ERROR -1
// REDIR1 for open/create/remove
# define STG_RES_REDIR1_SMALL 1
# define STG_RES_REDIR1_BIG (15 | STG_DATA_FOLLOWS)
// REDIR2 for close/truncate/read/write/append
# define STG_RES_REDIR2 2
# define STG_RES_CREATE 3
// # define STG_RES_CREATE1 3
// # define STG_RES_CREATE2 4
# define STG_RES_OPEN 5
# define STG_RES_TRUNCATE 7
# define STG_RES_WRITE 9
# define STG_RES_APPEND 10
# define STG_RES_READ_SMALL 11 
# define STG_RES_READ_BIG (12 | STG_DATA_FOLLOWS)
# define STG_RES_GET_RANGE_SMALL 13
# define STG_RES_GET_RANGE_BIG (14 | STG_DATA_FOLLOWS)
# define STG_RES_CLOSE 16

// Data too large to be sent over the transport media (likely UDP)
// Try TCP instead.
# define STG_RES_ESIZE -1
// User-buffer too small in wait_response, increase buffer size,
// and do wait_response_cont if possible.
# define STG_RES_EBUFFER -2
// User-buffer too small, increase buffer size, and restart the
// request (UDP)
// # define STG_RES_EAGAIN -3
// Generic server side error
# define STG_RES_ESERVER -4
// Generic client-side error
# define STG_RES_ECLIENT -5
// Generic network problem
# define STG_RES_ENETWORK -6
// timed out
# define STG_RES_ETOUT -7
// Server side I/O error
# define STG_RES_EIO -8
// Server side out-of-space
# define STG_RES_ESPACE -9
// Out of memory
# define STG_RES_EMEM -10
// Object not found
# define STG_RES_ENOTFOUND -11
// already exists
# define STG_RES_EEXIST  -12
// special code for Stg_Res_Handle
# define STG_RES_IN_USE -13

struct Stg_Res_Error {
	int status;
};

// redirection for a file that is not opened.
struct Stg_Res_Redir1_Small {
	int nHosts;
	ServerAddr0 saddrs[1];
};

struct Stg_Res_Redir1_Big {
	int nHosts;
};

# define STG_RES_REDIR1_SMALL_HEADER_SZ \
	(offsetof(Stg_Res_Redir1_Small, saddrs))

// redirection for a file that is currently opened.
struct Stg_Res_Redir2 {
	ServerAddr0 saddr;
	int desc;
};

struct Stg_Res_Create {
	int desc;
	int version;
};

struct Stg_Res_Open {
	int desc;
};

struct Stg_Res_Truncate {
	off_seg_t new_size;
};

struct Stg_Res_Write {
	off_seg_t written_bytes;
};

struct Stg_Res_Append {
	off_seg_t append_bytes;
};

struct Stg_Res_Read_Small {
	off_seg_t read_bytes;
	char data[1];
};

# define STG_RES_READ_SMALL_HEADER_SZ \
	(offsetof(Stg_Res_Read_Small, data))

struct Stg_Res_Read_Big {
	off_seg_t read_bytes;
};

struct Stg_Res_Get_Range_Small {
	HostID hid; // the owner of these objects.
	int n_objs;
	ObjID objs[1];
};

# define STG_RES_GET_RANGE_SMALL_HEADER_SZ \
	(offsetof(Stg_Res_Get_Range_Small, objs))

struct Stg_Res_Get_Range_Big {
	HostID hid;
	int n_objs;
};

struct Stg_Res_Close {
	int version;
};

struct Stg_Response {
	struct Stg_Header header;
	union {
		struct Stg_Res_Error u_error;
		struct Stg_Res_Create u_create;
		struct Stg_Res_Open u_open;
		struct Stg_Res_Redir1_Small u_redir1_small;
		struct Stg_Res_Redir1_Big u_redir1_big;
		struct Stg_Res_Redir2 u_redir2;
		struct Stg_Res_Read_Small u_read_small;
		struct Stg_Res_Read_Big u_read_big;
		struct Stg_Res_Write u_write;
		struct Stg_Res_Append u_append;
		struct Stg_Res_Truncate u_truncate;
		struct Stg_Res_Get_Range_Small u_get_range_small;
		struct Stg_Res_Get_Range_Big u_get_range_big;
		struct Stg_Res_Close u_close;
	} u;

	off_seg_t payload_size(void) const {
		switch (header.tag) {
			case STG_RES_READ_SMALL:
				return u.u_read_small.read_bytes;
			case STG_RES_READ_BIG:
				return u.u_read_big.read_bytes;
			case STG_RES_GET_RANGE_SMALL:
				return u.u_get_range_small.n_objs*sizeof(ObjID);
			case STG_RES_GET_RANGE_BIG:
				return u.u_get_range_big.n_objs*sizeof(ObjID);
		    case STG_RES_REDIR1_SMALL:
				return u.u_redir1_small.nHosts*sizeof(ServerAddr0);
		    case STG_RES_REDIR1_BIG:
				return u.u_redir1_big.nHosts*sizeof(ServerAddr0);
				
			default:
				return 0;
		}
	}

	void * payload(void) {
		switch (header.tag) {
			case STG_RES_READ_SMALL:
				return u.u_read_small.data;
			case STG_RES_GET_RANGE_SMALL:
				return u.u_get_range_small.objs;
		    case STG_RES_REDIR1_SMALL:
				return u.u_redir1_small.saddrs;
			default:
				return NULL;
		}
	}
};

# define STG_RES_HEADER_SZ (offsetof(Stg_Response, u))

inline off_seg_t stg_req_size(Stg_Request *req)
{
	return sizeof(Stg_Header)+req->header.size;
}

inline off_seg_t stg_res_size(Stg_Response *res)
{
	return sizeof(Stg_Header)+res->header.size;
}

Stg_Request *stg_req_alloc(int op_type, off_seg_t len);
Stg_Request *new_stg_req_create1(ObjID &oid);
// Stg_Request *new_stg_req_create2(ObjID &oid);
Stg_Request *new_stg_req_open(ObjID &oid, int flags);
Stg_Request *new_stg_req_close(int desc, int flags);
Stg_Request *new_stg_req_read(int desc, off_seg_t offset, off_seg_t len);
Stg_Request *new_stg_req_write(int desc, off_seg_t offset, char *buf, off_seg_t len);
Stg_Request *new_stg_req_append(int desc, char *buf, off_seg_t len);
Stg_Request *new_stg_req_remove(ObjID &oid);
Stg_Request *new_stg_req_truncate(int desc, off_seg_t size);
Stg_Request *new_stg_req_add_entry(HostID &hid, ObjID &oid, int version);
Stg_Request *new_stg_req_del_entry(HostID &hid, ObjID &oid);
Stg_Request *new_stg_req_get_range(HostID &hid);
Stg_Request *new_stg_req_sync_host(ServerAddr &sa, ObjID oid, int otherV);
Stg_Request *new_stg_req_notify_ver_ch(ServerAddr &sa, ObjID oid, int otherV);
// Stg_Request *new_stg_req_del_range(...);
Stg_Request *new_stg_req_stop(void);
inline void stg_req_free(Stg_Request *req) { free(req); }

Stg_Response *stg_res_alloc(int op_type, off_seg_t len);
void stg_res_adjust(Stg_Response *res, int op_type, off_seg_t len);
Stg_Response *new_stg_res_okay(void);
Stg_Response *new_stg_res_error(int errnum);
Stg_Response *new_stg_res_redir1(const ServerAddr0 *sas, int nSrvrs, \
								 bool use_tcp);
Stg_Response *new_stg_res_redir2(const ServerAddr &saddr, int desc);
Stg_Response *new_stg_res_create(int desc, int version);
Stg_Response *new_stg_res_open(int desc);
Stg_Response *new_stg_res_close(int version);
Stg_Response *new_stg_res_truncate(off_seg_t new_size);
Stg_Response *new_stg_res_write(off_seg_t written_bytes);
Stg_Response *new_stg_res_append(off_seg_t append_bytes);
Stg_Response *new_stg_res_read(off_seg_t read_bytes, bool use_tcp);
void adjust_stg_res_read(Stg_Response *res, off_seg_t read_bytes, bool use_tcp);
Stg_Response *new_stg_res_get_range(HostID &hid, ObjID *objs, int n_objs, bool use_tcp);
inline void stg_res_free(Stg_Response *res) { free(res); }

# include "udp_cli.h"
# include "sp_singleton.h"

struct Stg_Req_Handle {
	int tcp_sock; // for receiving TCP response.
	seq_t udp_seq; // for receiving UDP response.
	int status;
	Stg_Req_Handle() : tcp_sock(-1), udp_seq(UDP_SEQ_ERR), status(STG_RES_OKAY) {}
	bool use_tcp() const { return (tcp_sock>=0); }
	~Stg_Req_Handle() {
		if (tcp_sock>=0) {
			if (status == STG_RES_OKAY) {
				// specifying NULL for ip because I forgot the destination. :(
				sp_client_put_one(tcp_sock, NULL, 0); 
			}
			else {
				::close(tcp_sock);
			}
		}
	}
};

struct Stg_Res_Handle {
	Stg_Response *tcp_res; // for TCP response.
	Udp_res **udp_res; // for UDP response.
	Stg_Res_Handle() : tcp_res(NULL), udp_res(NULL) {}
	~Stg_Res_Handle();

	Stg_Response *response() const {
		return (tcp_res)?tcp_res:(Stg_Response *)(udp_res[0]->result());
	}
};

int stg_start_request_no_payload(
	Udp_cli *cli, 
	const ServerAddr &saddr, 
	Stg_Request *req, 
	Stg_Req_Handle &handle, 
	int hint
);

int stg_start_request_with_payload(
	Udp_cli *cli, 
	const ServerAddr &saddr, 
	Stg_Request *req, 
	char *payload, 
	off_seg_t length, 
	Stg_Req_Handle &handle, 
	int hint
);

int stg_wait_response_no_payload(
	Udp_cli *cli, 
	Stg_Req_Handle &req_handle, 
	Stg_Res_Handle &res_handle
);

int stg_wait_response_with_payload_cont(
	Udp_cli *cli, 
	Stg_Req_Handle &reqh, 
	Stg_Res_Handle &resh, 
	char *payload, 
	off_seg_t length
);

int stg_wait_response_with_payload(
	Udp_cli *cli, 
	Stg_Req_Handle &req_handle, 
	Stg_Res_Handle &res_handle, 
	char *payload, 
	off_seg_t length
);

class Udp_srv;
class Udp_srv_req;

struct Stg_Srv_Request {
    Udp_srv_req *udp_req; // for UDP request.
	Stg_Request *tcp_req; // for TCP request.
    int tcp_sock;
	int status;

	Stg_Srv_Request(int accepted_sock);
	Stg_Srv_Request(Udp_srv_req *uq);
	~Stg_Srv_Request();
    bool use_tcp() const { return ((tcp_sock>=0)&&(tcp_req)); }
	Stg_Request *request() const; 
	void respond(Udp_srv *srv, Stg_Response *res); 
	void respond_error(Udp_srv *srv, int errnum); 
	void respond_okay(Udp_srv *srv);
    bool read_more(char *buffer, off_seg_t len, off_seg_t &read_bytes);
    bool write_more(char *buffer, off_seg_t len);
	bool valid() { return ((udp_req) || (tcp_req && (tcp_sock >=0))); }
	void set_status(int st) { status = st; }
};

# endif
