// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: replicator.h,v $ - Module that is responsible from
 *            replication. 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/29 02:17:14 $ by $Author: gulbeden $
 * Version :  $Revision: 1.8 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef __REPLICATOR_H__
# define __REPLICATOR_H__

#include <pthread.h>

# include "cbs_thread.h"
# include "map.h"
# include "sorrento_types.h"

using namespace std;

class Client;
class HostMap;
class Udp_cli;
class Verfs;

/**
 * This structure stores the replication degree and the latest committed 
 * version's version number.
 */
class DegVer {
public:
	int degree;
	int version;
	DegVer(int d, int v) :
		degree(d),
		version(v)
	{}
};

/**
 * This structure stores a server address and the version of the segment stored
 * there.
 */
class SadVer {
public:
	ServerAddr sa;
	int version;
	SadVer(ServerAddr s, int v) : 
		sa(s),
		version(v)
	{}
};

typedef multimap<const ObjID, SadVer, ObjID_cmp> ownSegsType;
typedef map<const ObjID, DegVer, ObjID_cmp> repDegreesType;

class Replicator: public CBS_Thread
{

private:
	bool determineReceiver(ObjID &oid, ServerAddr &sa);
	bool determineSender(ObjID &oid, ServerAddr &sa, int &latVer);
	bool sendData(ObjID &oid, ServerAddr &sa);
	bool setLatestVersion(const ObjID &oid, int ver);
	bool sendSyncReq(ServerAddr &saSend, ServerAddr &saRecv, 
					 const ObjID &oid, const int latestVer) const;

	ownSegsType::iterator checkOidSaddr(const ObjID &oid, const ServerAddr &saddr);

	double compAvg();

	
	Client *cli;

	ownSegsType ownSegs;
	repDegreesType repDegrees;

	HostMap *hm;
	Verfs *verfs;
	HostID *myHid;
	ServerAddr *myAddr;

	pthread_mutex_t ownSegLock;
	pthread_mutex_t rdLock;
	Udp_cli *udpCli;
	
public:
	Replicator(HostMap *hm, Verfs* verfs, HostID *hid, ServerAddr *ma);
	~Replicator();
	bool addSegment(const ObjID &oid, const ServerAddr &saddr, int version=0);
	bool setRepDegree(const ObjID &oid, int deg, int v);
	bool setVersion(const ObjID &oid, const ServerAddr &saddr, int v);
	bool removeSegment(const ObjID &oid);
   
	virtual void *run();
	bool notify_host_join();
	bool notify_host_leave(ServerAddr &sad);
	bool start();

	bool syncObject(/*const */ObjID &oid);

	static bool setRepDegree(int rd);
	static int repDegree;
};


#endif // __REPLICATOR_H__
