// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sor_client.h,v $ - the Sorrento FS client class. 
 *            It is built upon the following three classes: DIR_Entry,
 *            SorFS and DIR_Cli_AP. This class is only used to build the 
 *            trace player.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/09/12 05:39:53 $
 * Version :  $Revision: 1.8 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _SOR_CLIENT_H_
# define _SOR_CLIENT_H_

# include "sorrento_types.h"
# include "host_map.h"
# include "../dir_srv/dir_cli_mt.h"

class DIR_Cli_AP;
class Client;
class SorFile;
class SorFS;

/**
 * Sorrento Client: maintaining per-client state information such
 * as current working directory (within a certain Sorrento file 
 * system). We also keep a DIR_Srv_AP pointer that can be used
 * to request dir_srv related functionalities. This
 */
class SorClient {
 private:
	/** 
	 * The Sorrento file system where that the client is operating 
	 * on. Each client can only operate on one SorFS at a time.
	 */
	// SorFS *sp;
	HostMap *hmap; /**< The host map structure */
	DIR_Cli_AP *ap; /**< The Directory Server Access Point. */
	Client *cli; /**< The stoarge access point. */
	DIR_Entry cwd; /**< The current working directory. */
	int errnum; /**< per client previous error number */
	int callid; /**< per client previous call ID */

	void set_param(DIR_Param *param, ObjID &guid, off64_t size);
	void init_param(DIR_Param *param);

 public:
	typedef DIR_Entry Cookie;
	SorClient(SorFS *sp); /**< The constructor. */
	~SorClient(); /**< The destructor. */
	bool chdir(const char *path); /**< Change the current working directory. */
	bool mkdir(const char *path); /**< Make a new directory. */
	bool rmdir(const char *path); /**< Remove an existing directory. */
	// bool deltree(char *path); /**< Remove an existing directory tree. */
	/** Read the content of a directory. */
	bool readdir(const char *path, DIR_List &list, int buf_len, Cookie *cookie);
	/** 
	 * Is end-of-directory? Check whether we need to make another 
	 * readdir_cont() call. 
	 */
	bool is_eod(const DIR_Entry *cookie) const;
	/** Continue the previous reading. */
	bool readdir_cont(Cookie *cookie, DIR_List &list, int buf_len);
	bool unlink(const char *path); /**< remove an existing file node. */
	SorFile *create(const char *path); 
	SorFile *open(const char *path, int flags); /**< Open a file for access */
	bool close(SorFile *sf); /**< Close a file */
	/** Read a number of bytes from a certain offset */
	bool read(SorFile *sf, off64_t offset, char *buf, off_seg_t len, off_seg_t & read_bytes); 
	/** Write a number of bytes from a certain offset */
	bool write(SorFile *sf, off64_t offset, char *buf, off_seg_t len, off_seg_t & written_bytes);
	/** Change the size of the current file */
	bool ftruncate(SorFile *sf, off64_t offset);
	bool truncate(const char *path, off64_t offset);
	/** Get attribute of a certain file. */
	bool fstat(SorFile *sf, DIR_Param *param);
	bool stat(const char *path, DIR_Param *param);
	const char *get_pwd(void) const { return cwd.path(); }
	const char *ident(void) const { return hmap->ident(); }

	/** Print out a string corresponding to previous error state. */
	std::string str_error(); 
	void dump_view(void);
	void stop_service(void);
};

# endif // _SOR_CLIENT_H_
