// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sor_dll.h,v $ - a simple double link list.
 *            It supports three types of operations:
 *            three types of operations:
 *            (1) Add a new item (of base type DoubleLinkListable)
 *                to the end of the list.
 *            (2) Unlink an item from anywhere of the queue.
 *            (3) Get the reference to first item in the queue.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/08/02 00:06:25 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _SOR_DLL_H_
# define _SOR_DLL_H_

# include <stdio.h>

class DoubleLinkList;

class DoubleLinkListable {
 private:
	DoubleLinkListable **prev; 
	DoubleLinkListable *next;

	bool link(DoubleLinkListable **p, DoubleLinkListable *n)
	{
		if ((prev!=NULL)||(next!=NULL))
			return false;

		prev=p;
		next=n;
		*p=this;
		n->prev=&next;

		return true;
	}

 public:
	DoubleLinkListable() : prev(NULL), next(NULL) {}
	~DoubleLinkListable() {}

	// get myself out of the list.
	// returns:
	// true: success.
	// false: failed (maybe because it is not currently 
	// linked in a double link list).
	bool unlink()
	{
		if ((prev==NULL)||(next==NULL))
			return false; 
		next->prev=prev;
		*prev=next;

		prev=NULL;
		next=NULL;

		return true;
	}

	friend class DoubleLinkList;
};

class DoubleLinkList {
 private:
	DoubleLinkListable *head;
	// end is a sentinel node that will never be unlinked because
	// its next field is NULL, and will fail the entry test in
	// the unlink() function.
	DoubleLinkListable end; 

 public:
	DoubleLinkList() : 
		head(&end),
		end()
	{
		end.prev=&head;
	}

	bool push_back(DoubleLinkListable *i)
	{
		DoubleLinkListable **p=end.prev;
		DoubleLinkListable *n=&end;
		return i->link(p, n);
	}

	DoubleLinkListable *get_head()
	{
		if (head==&end)
			return NULL;
		return head;
	}

	DoubleLinkListable *pop_head()
	{
		if (head==&end)
			return NULL;

		DoubleLinkListable *ret=head;
		if (ret->unlink())
			return ret;
		else {
			throw std::string("DoubleLinkList::pop_head: Internal error.");
		}
	}
};

# endif
