// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sor_file.h,v $ - the Sorrento File class.
 *            It represents a logical file.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/02/19 13:14:51 $
 * Version :  $Revision: 1.13 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _SOR_FILE_H_
# define _SOR_FILE_H_

# include <sys/types.h>
# include <vector>

# include "sorrento_types.h"
# include "../dir_srv/dir_cli_mt.h"
// # include "guid.h"

class Client;
class Descriptor;
class HostMap;

class Segment;
class Attachment;
class DataSegments;

/**
 * Sorrento File Object.
 */
class SorFile {
 private:
	DIR_Entry root; /**< The root block entry */
	ObjID root_guid;
	Segment *root_segment; /**< The root segment (meta-data). */
	/** only one of attachment and data_segs will be valid. */
	Attachment *attachment; /**< The attached data, if any. */
	DataSegments *data_segs; /**< The data segments. */
	int flags;

 public:
	SorFile(Client *cli, HostMap *hmap, DIR_Cli_AP *ap, DIR_Entry *entry, int flags, bool new_file=false); 
	~SorFile();
	int truncate(Client *cli, HostMap *hmap, off64_t size);
	int stat(DIR_Param *param);
	int read(Client *cli, HostMap *hmap, off64_t offset, char *buf, off_seg_t len, off_seg_t &read_bytes);
	int write(Client *cli, HostMap *hmap, off64_t offset, char *buf, off_seg_t len, off_seg_t &written_bytes);
	// int sync(DIR_Cli_AP *ap, Client *cli, HostMap *hmap);
	int close(DIR_Cli_AP *ap, Client *cli, HostMap *hmap);
	static int unlink(Client *cli, HostMap *hmap, ObjID &guid);
	int get_size(off64_t & size);
	int get_oid(ObjID &oid); 
};

# define FILE_RES_OKAY DIR_RES_OKAY
# define FILE_RES_EMEM 0x101
# define FILE_RES_ESCLIENT 0x102
# define FILE_RES_ESPACE 0x103
# define FILE_RES_EREAD 0x104
# define FILE_RES_EWRITE 0x105
# define FILE_RES_NOT_FOUND 0x106
# define FILE_RES_EMETA 0x107
# define FILE_RES_EFILE 0x108
# define FILE_RES_EDATA 0x109
# define FILE_RES_ATTACHMENT 0x10a
# define FILE_RES_ESERVER 0x10b
# define FILE_RES_ENETWORK 0x10c
# define FILE_RES_EIO 0x10d
# define FILE_RES_EEXIST 0x10e
# define FILE_RES_ECLIENT 0x10f
# define FILE_RES_RD_ONLY 0x110

# endif // _SOR_FILE_H_
