// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sor_shell.cc,v $ - sorrent client shell 
 *            This program tests the sor_client functionality.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/02/19 13:14:51 $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <errno.h>
# include <ctype.h>
# include <unistd.h>
# include <getopt.h>
# include <vector>
# include <string>

extern "C" {
# include <readline/readline.h>
# include <readline/history.h>
}

# include "usc_time.h"
# include "sor_client.h"
# include "sor_fs.h"
# include "sor_file.h"
# include "subscriber.h"
# include "tmpi_debug.h"

using namespace std;

typedef vector<const char *> VecArg;

static char *program_name=NULL;
static Subscriber *sub=NULL;
static SorClient *scli=NULL;
static SorFile *file_table[1024];

char *Id="$Id: sor_shell.cc,v 1.9 2004/02/19 13:14:51 gulbeden Exp $";

static char *chop(char *line)
{
	
	while (*line) {
		if (isspace(*line))
			line++;
		else
			break;
	}

	char *p=line+strlen(line)-1;
	while ((p>=line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	return line;
}

static char *chop1(char *line)
{
	
	while (*line) {
		if ((isspace(*line))||(isdigit(*line))||(*line=='>'))
			line++;
		else
			break;
	}

	char *p=line+strlen(line)-1;
	while ((p>=line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	return line;
}

static char *get_RCS_val(char *var)
{
	static char buf[1024];
	char *p;

	if ((p=strchr(var, ':'))==NULL) {
		return var;
	}
	buf[1023]='\0';
	strncpy(buf, p+1, 1024);
	p=buf+strlen(buf)-1;
	while ((p>=buf)&&(*p=='$')) {
		*p='\0';
		p--;
	}

	return chop(buf);
}

static void do_version()
{
	static char *version="$Revision: 1.9 $";
	static char *author="Hong Tang";
	static char *date="$Date: 2004/02/19 13:14:51 $";

	{
		printf("Sorrento Shell (%s) %s.\n", program_name, get_RCS_val(version));
		printf("Copyright (c) University of California, Santa Barbara.\n");
		printf("Author: %s.\n", get_RCS_val(author));
		printf("Last Modification Date: %s GMT.\n", get_RCS_val(date));
		printf("Specify the --help option for further information.\n");
	}

	exit(0);
}

static void do_help(char *cmd)
{
	if (cmd) {
		printf("Unknow command: %s. Type help for more info.\n", cmd);
	}
	else {
		printf("\n\
The shell suports the following commands:\n\
 Short  Full      arguments         meaning\n\
 ===================================================================\n\
 cd     chdir     <path>            Change the current directory.\n\
 md     mkdir     <path>            Make a directory.\n\
 rd     rmdir     <path>            Remove a directory.\n\
 ls     list      <path>            List the content of a directory.\n\
 rm     remove    <path>            Remove a file.\n\
 cr     create    <path>            Create a file.\n\
 op     open      <path>            Open a file.\n\
 cl     close     <fd>              Close a file.\n\
 re     read      <fd> <off> <len>  Read from a file.\n\
 ep     export    <src> <des>       Export a file.\n\
 wr     write     <fd> <off> <len>  Write to a file.\n\
 ip     import    <src> <des>       Import a file.\n\
 ft     ftruncate <fd> <size>       Truncate an open file.\n\
 tr     truncate  <path> <size>     Truncate a file.\n\
 fga    fgetattr  <fd>              Get attribute of an open file.\n\
 ga     getattr   <path>            Get the attribute of a file or directory.\n\
 of     openfile                    List open file descriptors.\n\
 vi     view                        Local view of the cluster.\n\
 pw     pwd                         Show current working directory.\n\
 id     ident                       Show the identity of sorrento system.\n\
 h      help                        Print this message.\n\
 q      quit                        Quit the shell.\n\
 ht     halt                        Halt the service.\n\
");
	}

	return;
}

static void usage(int status)
{
	if (status != 0) {
		fprintf(stderr, "Try '%s --help' for more information.\n", program_name);
	}
	else {
		printf ("Usage: %s [options] <ident>.\n", program_name);
		printf ("\
A Sorrento System Command Line Shell. <ident> is the sorrento deployment ID.\n\
\n\
1. Sorrento address:\n\
    -f, --config <config-file>  Sorrento configuration file (cannot combine with -m or -p).\n\
    -m, --mc-addr <mcast-ip>    Sorrento multicast channal address (cannot combine with -f).\n\
    -p, --mc-port <mcast-port>  Sorrento multicast channel port (must combine with -m).\n\
 \n\
2. Debugging:\n\
    -s, --silent                Display/log only fatal information.\n\
    -d, --debug                 Keep a detailed log.\n\
  \n\
3. Information:\n\
    -h, --help                  Display this help screen.\n\
    -V, --version               Display version number.\n\
");
	}

	exit(status);
}

static void parse_cmd_args_and_init(int argc, char *argv[])
{
	static struct option const long_options[] = {
		{"config", required_argument, 0, 'f'},
		{"mc-addr", required_argument, 0, 'm'},
		{"mc-port", required_argument, 0, 'p'},
		{"silent", no_argument, 0, 's'},
		{"debug", no_argument, 0, 'd'},
		{"help", no_argument, 0, 'h'},
		{"version", no_argument, 0, 'V'},
		{NULL, 0, 0, 0}
	};

	int optc;
	char *config=NULL;
	char *mc_addr=NULL;
	int mc_port=-1;
	char *ident=NULL;

	program_name=argv[0];

	while ((optc=getopt_long(argc, argv, \
					"f:m:p:sdhV", long_options, NULL)) !=-1) {
		switch (optc) {
			case 0:
				break;
			case 'f':
				if (config) {
					printf("Sorrento configuration file already specified.\n");
					usage(0);
				}
				config=optarg;
				break;
			case 'm':
				if (mc_addr) {
					printf("Sorrento multicast address already specified.\n");
					usage(0);
				}
				mc_addr=optarg;
				break;
			case 'p':
				if (mc_port>0) {
					printf("Sorrento multicast port already specified.\n");
					usage(0);
				}
				mc_port=atoi(optarg);
				if (mc_port<=0) {
					printf("invalid port.\n");
					usage(0);
				}
				break;
			case 's':
				tmpi_set_dbg_level(DBG_FATAL-1);
				tmpi_hide_location();
				break;
			case 'd':
				tmpi_set_dbg_level(DBG_ALL);
				tmpi_show_location();
				break;
			case 'V':
				do_version();
				break;
			case 'h':
				usage(0);
				break;
			default:
				usage(1);
		}
	}

	if (optind != (argc-1)) {
		printf("No demployment ID specified.\n");
		usage(1);
	}

	ident=argv[optind];

	if ((config)&&(mc_addr)) {
		printf("Both configuration file and multicast address specified.\n");
		usage(0);
	}

	if ((!config)&&(!mc_addr)) {
		printf("Neither configuration file nor multicast address specified.\n");
		usage(0);
	}

	if (mc_addr && (mc_port<=0)) {
		printf("Multicast port not specified.\n");
	}

	try {
		if (config) {
			SorrentoConf sconf(config);
			sub=new Subscriber(sconf);
		}
		else {
			sub=new Subscriber(mc_addr, mc_port, 1000);
		}
		// server = new Server(sconf, sub);
		sub->start();
		// server->start();
		printf("Starting the proxy, please wait ...\n");
		bool done=false;
		while (!done) {
			try {
				sleep(1); // wait for 3 seconds.
				SorFS sorfs(sub, string(ident));
				scli=new SorClient(&sorfs);
				done=true;
			}
			catch (const string &errmsg) {
			}
		}
	}
	catch (const string &errmsg) {
		tmpi_error(DBG_INTERNAL, "sor_shell: %s\n", errmsg.c_str());
		goto _exit;
	}
	catch (int errnum) {
		tmpi_error(DBG_INTERNAL, "sor_shell: error caught (code=%d).\n", errnum);
		goto _exit;
	}
	catch (...) {
		tmpi_error(DBG_INTERNAL, "Unknow error, initialization failed.\n");
		goto _exit;
	}

	return;

_exit:
	if (sub)
		delete sub;
	exit(-1);
}

enum CMDS {
	CMD_CHDIR, /* lookup */
	CMD_MKDIR, /* mkdir */
	CMD_RMDIR, /* rmdir */
	CMD_LIST, /* readdir, readdir_cont */
	CMD_REMOVE, /* remove */
	CMD_CREATE, /* create */
	CMD_OPEN, /* open */
	CMD_CLOSE, /* close */
	CMD_READ, /* read */
	CMD_EXPORT, /* uses read */
	CMD_WRITE, /* write */
	CMD_IMPORT, /* uses write */
	CMD_FTRUNCATE, /* ftruncate */
	CMD_TRUNCATE, /* truncate */
	CMD_FGETATTR, /* fgetattr */
	CMD_GETATTR, /* getattr */
	CMD_OPENFILE, /* file_table */
	CMD_VIEW, /* view  */
	CMD_PWD, /* path */
	CMD_IDENT, /* get_ident */
	CMD_HELP, /* help */
	CMD_QUIT, /* quit */
	CMD_HALT, /* halt */
	//-------//
	CMD_MAX /* The maximum of the command enums */
};

struct CMD_TABLE {
	char *cmd;
	int cid;
} cmd_to_cid[] =
{
	{"cd", 		CMD_CHDIR},
	{"chdir", 	CMD_CHDIR},
	{"md", 		CMD_MKDIR},
	{"mkdir", 	CMD_MKDIR},
	{"rd", 		CMD_RMDIR},
	{"rmdir", 	CMD_RMDIR},
	{"ls", 		CMD_LIST},
	{"list", 	CMD_LIST},
	{"rm", 		CMD_REMOVE},
	{"remove", 	CMD_REMOVE},
	{"cr", 		CMD_CREATE},
	{"create", 	CMD_CREATE},
	{"op", 		CMD_OPEN},
	{"open", 	CMD_OPEN},
	{"cl", 		CMD_CLOSE},
	{"close", 	CMD_CLOSE},
	{"re", 		CMD_READ},
	{"read", 	CMD_READ},
	{"ep", 		CMD_EXPORT},
	{"export", 	CMD_EXPORT},
	{"wr", 		CMD_WRITE},
	{"write", 	CMD_WRITE},
	{"ip", 		CMD_IMPORT},
	{"import", 	CMD_IMPORT},
	{"ft", 		CMD_FTRUNCATE},
	{"ftruncate",CMD_FTRUNCATE},
	{"tr", 		CMD_TRUNCATE},
	{"truncate",CMD_TRUNCATE},
	{"fga",		CMD_FGETATTR},
	{"fgetattr",CMD_FGETATTR},
	{"ga", 		CMD_GETATTR},
	{"getattr", CMD_GETATTR},
	{"of", 		CMD_OPENFILE},
	{"openfile", CMD_OPENFILE},
	{"vi",      CMD_VIEW},
	{"view",    CMD_VIEW},
	{"pw", 		CMD_PWD},
	{"pwd", 	CMD_PWD},
	{"id", 		CMD_IDENT},
	{"ident", 	CMD_IDENT},
	{"h", 		CMD_HELP},
	{"help", 	CMD_HELP},
	{"?", 		CMD_HELP},
	{"q", 		CMD_QUIT},
	{"quit", 	CMD_QUIT},
	{"ht", 		CMD_HALT},
	{"halt", 	CMD_HALT},
	// ... //
	{NULL, 	-1}
};

static int handle_chdir(VecArg &, bool);
static int handle_mkdir(VecArg &, bool);
static int handle_rmdir(VecArg &, bool);
static int handle_list(VecArg &, bool);
static int handle_remove(VecArg &, bool);
static int handle_create(VecArg &, bool);
static int handle_open(VecArg &, bool);
static int handle_close(VecArg &, bool);
static int handle_read(VecArg &, bool);
static int handle_export(VecArg &, bool);
static int handle_write(VecArg &, bool);
static int handle_import(VecArg &, bool);
static int handle_ftruncate(VecArg &, bool);
static int handle_truncate(VecArg &, bool);
static int handle_fgetattr(VecArg &, bool);
static int handle_getattr(VecArg &, bool);
static int handle_openfile(VecArg &, bool);
static int handle_view(VecArg &, bool);
static int handle_pwd(VecArg &, bool);
static int handle_ident(VecArg &, bool);
static int handle_halt(VecArg &, bool);

struct CMD_HANDLER {
	unsigned int n_arg;
	int (*f)(VecArg &, bool);
	char *help_str;
} handlers[CMD_MAX];

struct CMD_HANDLER_INFO {
	int id;
	unsigned int n_arg;
	int (*f)(VecArg &, bool);
	char *help_str;
} handler_info[] =
{
	{CMD_CHDIR,    1, &handle_chdir,     "cd     chdir     <path>            Change the current directory."},
	{CMD_MKDIR,    1, &handle_mkdir,     "md     mkdir     <path>            Make a directory."},
	{CMD_RMDIR,    1, &handle_rmdir,     "rd     rmdir     <path>            Remove a directory."},
	{CMD_LIST,     1, &handle_list,      "ls     list      <path>            List the content of a directory."},
	{CMD_REMOVE,   1, &handle_remove,    "rm     remove    <path>            Remove a file."},
	{CMD_CREATE,   1, &handle_create,    "cr     create    <path>            Create a file."},
	{CMD_OPEN,     1, &handle_open,      "op     open      <path>            Open a file."},
	{CMD_CLOSE,    1, &handle_close,     "cl     close     <fd>              Close a file."},
	{CMD_READ,     3, &handle_read,      "re     read      <fd> <off> <len>  Read from a file."},
	{CMD_EXPORT,   2, &handle_export,    "ep     export    <src> <des>       Export a file."},
	{CMD_WRITE,    3, &handle_write,     "wr     write     <fd> <off> <len>  Write to a file."},
	{CMD_IMPORT,   2, &handle_import,    "ip     import    <src> <des>       Import a file."},
	{CMD_FTRUNCATE,2, &handle_ftruncate, "ft     ftruncate <fd> <size>       Truncate an open file."},
	{CMD_TRUNCATE, 2, &handle_truncate,  "tr     truncate  <path> <size>     Truncate a file."},
	{CMD_FGETATTR, 1, &handle_fgetattr,  "fga    fgetattr  <fd>              Get attribute of an open file."},
	{CMD_GETATTR,  1, &handle_getattr,   "ga     getattr   <path>            Get the attribute of a file or directory."},
	{CMD_OPENFILE, 0, &handle_openfile,  "of     openfile                    List open file descriptors."},
	{CMD_VIEW,     0, &handle_view,      "vi     view                        Local view of the cluster."},
	{CMD_PWD,      0, &handle_pwd,       "pw     pwd                         Show current working directory."},
	{CMD_IDENT,    0, &handle_ident,     "id     ident                       Show the identity of sorrento system."},
	{CMD_HELP,     0, NULL,              "h      help                        Print this message."},
	{CMD_QUIT,     0, NULL,              "q      quit                        Quit the shell."},
	{CMD_HALT,     0, &handle_halt,      "ht     halt                        Halt the service."},
	{ -1, 0, NULL, NULL}
};

static void init_handlers()
{
	struct CMD_HANDLER_INFO *p=handler_info;
	while (p->id>=0) {
		handlers[p->id].n_arg=p->n_arg;
		handlers[p->id].f=p->f;
		handlers[p->id].help_str=p->help_str;
		p++;
	}

	return;
}

static int handle_cmd(char *cmd, VecArg &args, bool do_output)
{
	if (!cmd)
		return 0;

	struct CMD_TABLE *p=cmd_to_cid;

	while (p->cmd) {
		if (strcmp(cmd, p->cmd)==0) {
			break;
		}
		p++;
	}

	if (p->cmd) {
		struct CMD_HANDLER *h=&(handlers[p->cid]);
		if (args.size() != h->n_arg) {
			if (do_output) {
				if (args.size() < h->n_arg) {
					printf("Too few arguments. Usage of '%s':\n", cmd);
					printf("%s\n", "SHORT  FULL       ARGUMENTS        MEANING");
					printf("%s\n", h->help_str);
				}
				else if (args.size() > h->n_arg) {
					printf("Too many arguments. Usage of '%s':\n", cmd);
					printf("%s\n", "SHORT  FULL       ARGUMENTS        MEANING");
					printf("%s\n", h->help_str);
				}
			}

			return 0;
		}

		if (h->f) {
			return (*h->f)(args, do_output);
		}

		switch (p->cid) {
			case CMD_HELP : 
				do_help(NULL); return 0;
			case CMD_QUIT : 
				printf("Good bye!\n"); return 1;
			default : 
				if (do_output) 
					tmpi_error(DBG_APP, "Internal error: Bad command code.\n"); 
				return 0;
		}
	}
	else {
		do_help(cmd);
		return 0;
	}
}

static char *get_token(char * &t, char * & t_end)
{
	if (!t)
		return NULL;

	while ((*t)&&(isspace(*t))) {
		t++;
	}

	t_end=t;
	while ((*t_end)&&(!isspace(*t_end))) {
		t_end++;
	}
	
	if (t==t_end)
		return NULL;

	return t;
}

static bool is_empty_line(char *line)
{
	while (*line) {
		if (!isspace(*line))
			return false;
		line++;
	}

	return true;
}

/**
 * parse a command line.
 * @param line the command line.
 * @param cmd the command name.
 * @param args a vector of arguments.
 * @return status (-1: error; 0: success).
 */
static int parse_line(char *line, char *&cmd, VecArg &args)
{
	if (!line)
		return -1;
	
	cmd=NULL;

	line=chop1(line);

	if (is_empty_line(line)) {
		return 0;
	}

	char *t_begin;
	char *t_end;

	t_begin=line;
	t_begin=get_token(t_begin, t_end);
	
	if (!t_begin) 
		return 0;

	*t_end='\0';
	cmd=t_begin;

	t_begin=t_end+1;
	t_begin=get_token(t_begin, t_end);

	while (t_begin) {
		*t_end='\0';
		args.push_back(t_begin);

		t_begin=t_end+1;
		t_begin=get_token(t_begin, t_end);
	}

	return 0;
}

int main(int argc, char *argv[])
{
# if defined(__DEBUG__)
	tmpi_set_dbg_level(DBG_ALL);
	tmpi_show_location();
# else
	tmpi_set_dbg_level(DBG_VERBOSE);
	tmpi_hide_location();
# endif
	
	bool use_gnu_getline=false;
	bool do_output=false;
	bool do_timing=true;

	if (isatty(fileno(stdin))) {
		use_gnu_getline=true;
		do_output=true;
		do_timing=false;
	}

	// might be no return.
	parse_cmd_args_and_init(argc, argv);
	init_handlers();

	usc_time_t t_start=0, t_end=0;
	int cmd_cnt=0;

	if (do_timing) {
		t_start=usc_gettime();
	}

	while (1) {
		char *cmd; 
		VecArg args;
		char line[1024];
		if (use_gnu_getline) {
			char *l=readline("> ");
			strncpy(line, l, 1023);
			line[1023]='\0';
			free(l);

			if (line&&(line[0]))
				add_history(line);
		}
		else {
			if (fgets(line, 1024, stdin)==NULL) {
				break;
			}
			if (do_output)
				printf("> %s", line);
		}

		if (parse_line(line, cmd, args)<0) {
			printf("Bad command.\n");
			continue;
		}

		if (handle_cmd(cmd, args, do_output))  {
			break;
		}
		if (do_timing)
			cmd_cnt++;
	}

	if (do_timing) {
		t_end=usc_gettime();
		printf("Total %d commands executed, execution time %.3fms.\n", cmd_cnt, usc_difftime(t_end, t_start)/1000);
	}

	for (int i=0; i<1024; i++) {
		if (file_table[i]) {
			scli->close(file_table[i]);
			file_table[i]=NULL;
		}
	}
	delete scli;
	sub->stop();
	sub->finalize();
	delete sub;

	return 0;
}

static void output_error(const char *cmd, bool do_output)
{
	if (do_output)
		printf("%s failed: %s.\n", cmd, scli->str_error().c_str());
}

static int handle_chdir(VecArg &args, bool do_output)
{
	bool ret=scli->chdir(args[0]);
	if (!ret) {
		output_error("Chdir", do_output);
	}

	return 0;
}

static int handle_mkdir(VecArg & args, bool do_output)
{
	if (scli->mkdir(args[0]) == false) {
		output_error("Mkdir", do_output);
	}

	return 0;
}

static int handle_rmdir(VecArg & args, bool do_output)
{
	if (scli->rmdir(args[0]) == false) {
		output_error("Rmdir", do_output);
	}

	return 0;
}

static int time_to_str(time_t tt, char *s, int l)
{
	if (!s)
		return -1;

    struct tm ttm;

    memcpy(&ttm, gmtime(&tt), sizeof(struct tm));
    if (strftime(s, l, "%b %e %T %Y", &ttm)==0)
		return -1;

    return 0;
}

static void print_item(DIR_Param *param, const char *name)
{
	char ts[256];
	time_to_str(param->t_create, ts, 256);
	bool is_dir=DIR_IS_DIR(param->options);
	printf("%crw%c------ %10lld %24s %s\n", is_dir?'d':'-', 
			is_dir?'x':'-', param->size, ts, name);
}

static void print_item_detail(DIR_Param *param, const char *name)
{
	char ts[256];
	bool is_dir=DIR_IS_DIR(param->options);
	time_to_str(param->t_create, ts, 256);
	char guid_str[256];
	Guid guid(param->guid);
	guid.to_string(guid_str, 256);
	printf("%s: %8s %10lld %24s [%s]\n", name, 
			is_dir?"[dir]":"[file]", param->size, ts, guid_str);
}

static int handle_list(VecArg &args, bool do_output)
{
	DIR_Param param;

	if (scli->stat(args[0], &param) == false) {
		output_error("List/stat", do_output);
		return 0;
	}

	bool is_dir=DIR_IS_DIR(param.options);
	if (is_dir) {
		SorClient::Cookie cookie;
		DIR_List list;

		if (scli->readdir(args[0], list, 30000, &cookie) == false) {
			output_error("List/readdir", do_output);
			return 0;
		}

		while (!scli->is_eod(&cookie)) {
			if (scli->readdir_cont(&cookie, list, 30000) == false) {
				output_error("List/readdir_cont", do_output);
				return 0;
			}
		}

		if (do_output) {
			printf("List directory <%s>:\n", args[0]);
			for (unsigned int i=0; i<list.size(); i++) {
				print_item(&(list[i].param), list[i].name.c_str());
			}
		}
	}
	else {
		printf("List file <%s>:\n", args[0]);
		print_item(&param, args[0]);
	}

	return 0;
}

static int handle_remove(VecArg & args, bool do_output)
{
	if (scli->unlink(args[0]) == false) {
		output_error("Remove", do_output);
	}

	return 0;
}

static int add_to_file_table(SorFile *psf)
{
	for (int i=0; i<1024; i++) {
		if (file_table[i]==NULL) {
			file_table[i]=psf;
			return i;
		}
	}
	return -1;
}

static SorFile * get_from_file_table(int fd)
{
	if ((fd<0) || (fd>=1024))
		return NULL;

	return file_table[fd];
}

static int handle_create(VecArg & args, bool do_output)
{
	SorFile *psf=scli->create(args[0]);
	if (psf == NULL) {
		output_error("Create", do_output);
		return 0;
	}
	
	int fd=add_to_file_table(psf);
	if (fd == -1) {
		if (do_output) 
			printf("Create: file table full.\n");
		scli->close(psf);
	}
	else if (do_output) {
		printf("Create: File descriptor = %d.\n", fd);
	}
	
	return 0;
}

static int handle_open(VecArg & args, bool do_output)
{
	SorFile *psf=scli->open(args[0], SOR_WR);
	if (psf == NULL) {
		output_error("Open", do_output);
		return 0;
	}
	
	int fd=add_to_file_table(psf);
	if (fd == -1) {
		if (do_output) 
			printf("Open: file table full.\n");
		scli->close(psf);
	}
	else if (do_output) {
		printf("Open: File descriptor = %d.\n", fd);
	}
	
	return 0;
}

static int handle_close(VecArg & args, bool do_output)
{
	int fd=atoi(args[0]);
	SorFile *psf=get_from_file_table(fd);
	if (!psf) {
		if (do_output)
			printf("Close: Bad file descriptor.\n");
		return 0;
	}

	if (scli->close(psf) == false) {
		output_error("Close", do_output);
	}

	file_table[fd]=NULL;

	return 0;
}

static int handle_read(VecArg & args, bool do_output)
{
	int fd=atoi(args[0]);
	off64_t offset=atoll(args[1]);
	off_seg_t len=(unsigned int)atoll(args[2]);

	SorFile *psf=get_from_file_table(fd);

	if (psf == NULL) {
		if (do_output)
			printf("Read: bad file descriptor\n");
		return 0;
	}

	char *buf=(char *)malloc(len);
	if (!buf) {
		if (do_output) {
			printf("Read: out of memory.\n");
		}

		return 0;
	}

	off_seg_t read_len;

	if (scli->read(psf, offset, buf, len, read_len) == false) {
		output_error("Read", do_output);
	}
	else if (do_output) {
		printf("Read: %u bytes read.\n", read_len);
	}

	free(buf);

	return 0;
}

static int handle_export(VecArg & args, bool do_output)
{
	SorFile *src=scli->open(args[0], 0);
	if (src == NULL) {
		output_error("Open", do_output);
		return 0;
	}
    int des = open(args[1], O_WRONLY|O_TRUNC|O_CREAT, S_IRWXU|S_IRWXG);
    if (des < 0) {
        scli->close(src);
        if (do_output) {
		    printf("::open: cannot create file: %s.\n", args[1]);
        }
        return 0;
    }
	
    DIR_Param param;
    if (scli->fstat(src, &param) == false) {
        if (do_output) {
		    output_error("Fstat", do_output);
        }
        goto _exit;
    }
    {
        off64_t total_bytes=param.size;
        off64_t offset=0;
        const off_seg_t buf_len = 128*1024;
        char buf[buf_len];
        while (offset < total_bytes) {
            off_seg_t read_len;
            if (scli->read(src, offset, buf, buf_len, read_len) == false) {
                output_error("Read", do_output);
                break;
            }
            off_seg_t write_len=0;
            while (write_len < read_len) {
                int len = write(des, buf, read_len);
                if (len < 0) {
                    if (do_output) {
                        printf("::write: %s.\n", strerror(errno));
                    }
                    break;
                }
                write_len += len;
            }
            offset+=write_len;
            if (write_len < read_len) {
                break;
            }
        }

        if (do_output) {
            printf("%llu bytes exported to file %s.\n", offset, args[1]);
        }
    }

_exit:
    if (src)
        scli->close(src);
    if (des>=0)
        close(des);
    return 0;
}

static int handle_import(VecArg & args, bool do_output)
{
    int src = open(args[0], O_RDONLY);
    if (src < 0) {
        if (do_output) {
		    printf("::open: cannot open file: %s.\n", args[0]);
        }
        return 0;
    }

	SorFile *des=scli->create(args[1]);
	if (des == NULL) {
	    des=scli->open(args[1], 0);
        if (!des) {
		    output_error("Open", do_output);
		    return 0;
        }
	}
	
    struct stat st;
    if (fstat(src, &st)<0) {
        if (do_output) {
            printf("::fstat: %s.\n", strerror(errno));
        }
        goto _exit;
    }

    {
        if (scli->ftruncate(des, (off_seg_t)st.st_size) == false) {
		    output_error("Ftruncate", do_output);
		    return 0;
        }
        off64_t total_bytes=(off64_t)st.st_size;
        off64_t offset=0;
        const off_seg_t buf_len = 128*1024;
        char buf[buf_len];
        while (offset < total_bytes) {
            int len = read(src, buf, buf_len);
            if (len < 0) {
                if (do_output) {
                    printf("::read: %s.\n", strerror(errno));
                }
                break;
            }
            off_seg_t read_len=(off_seg_t)len;
            off_seg_t write_len;
            if (scli->write(des, offset, buf, read_len, write_len) == false) {
                output_error("Write", do_output);
                break;
            }
            offset+=write_len;
            if (write_len != read_len) {
                break;
            }
        }

        if (do_output) {
            printf("%llu bytes imported to file %s.\n", offset, args[1]);
        }
    }

_exit:
    if (src>=0)
        close(src);
    if (des)
        scli->close(des);
    return 0;
}

static int handle_write(VecArg & args, bool do_output)
{
	int fd=atoi(args[0]);
	off64_t offset=atoll(args[1]);
	off_seg_t len=(unsigned int)atoll(args[2]);

	SorFile *psf=get_from_file_table(fd);

	if (psf == NULL) {
		if (do_output)
			printf("Write: bad file descriptor\n");
		return 0;
	}

	char *buf=(char *)malloc(len);
	if (!buf) {
		if (do_output) {
			printf("Write: out of memory.\n");
		}

		return 0;
	}

	memset(buf, 0xa6, len);
	off_seg_t written_len;

	if (scli->write(psf, offset, buf, len, written_len) == false) {
		output_error("Write", do_output);
	}
	else if (do_output) {
		printf("Write: %u bytes written.\n", written_len);
	}

	free(buf);

	return 0;
}

static int handle_ftruncate(VecArg & args, bool do_output)
{
	int fd=atoi(args[0]);
	off64_t len=atoll(args[1]);

	SorFile *psf=get_from_file_table(fd);

	if (psf == NULL) {
		if (do_output)
			printf("Ftruncate: bad file descriptor\n");
		return 0;
	}

	if (scli->ftruncate(psf, len) == false) {
		output_error("Ftruncate", do_output);
	}

	return 0;
}

static int handle_truncate(VecArg & args, bool do_output)
{
	off64_t len=atoll(args[1]);

	if (scli->truncate(args[0], len) == false) {
		output_error("Truncate", do_output);
	}

	return 0;
}

static int handle_fgetattr(VecArg & args, bool do_output)
{
	int fd=atoi(args[0]);

	SorFile *psf=get_from_file_table(fd);

	if (psf == NULL) {
		if (do_output)
			printf("Fgetattr: bad file descriptor\n");
		return 0;
	}

	DIR_Param param;
	if (scli->fstat(psf, &param) == false) {
		output_error("Fgetattr", do_output);
	}

	if (do_output) {
		char name[32];
		snprintf(name, 32, "<FD>[%d]", fd);
		print_item_detail(&param, name);
	}

	return 0;
}

static int handle_getattr(VecArg &args, bool do_output)
{
	DIR_Param param;
	if (scli->stat(args[0], &param) == false) {
		output_error("Getattr", do_output);
		return 0;
	}

	if (do_output) {
		print_item_detail(&param, args[0]);
	}

	return 0;
}

static int handle_openfile(VecArg &args, bool do_output)
{
	for (int i=0; i<1024; i++) {
		ObjID oid;
		off64_t size;
		SorFile *psf=get_from_file_table(i);
		if (!psf) {
			continue;
		}

		if (psf->get_size(size) != FILE_RES_OKAY) {
			continue;
		}

		if (psf->get_oid(oid) != FILE_RES_OKAY) {
			continue;
		}
		char buf[128];
		Guid g(oid);
		g.to_string(buf, 128);

		printf("%4d%20lld%s\n", i, size, buf);
	}

	return 0;
}

static int handle_view(VecArg &args, bool do_output)
{
	scli->dump_view();
	return 0;
}

static int handle_halt(VecArg &args, bool do_output)
{
	scli->stop_service();
	if (do_output) {
		printf("System halted.\n");
	}
	return 1;
}

static int handle_pwd(VecArg & args, bool do_output)
{
	const char *pwd=scli->get_pwd();

	if (pwd == NULL) {
		output_error("Pwd", do_output);
	}

	if (do_output)
		printf("%s\n", pwd);

	return 0;
}

static int handle_ident(VecArg & args, bool do_output)
{
	const char *ident=scli->ident();

	if (ident == NULL) {
		output_error("Ident", do_output);
	}

	if (do_output)
		printf("%s\n", ident);

	return 0;
}

