// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sorrento_conf.cc,v $ - sorrento configuration
 *            manager (implementation).
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/03/29 19:17:07 $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include <string>
# include "sorrento_conf.h"

using namespace std;

SorrentoConf::SorrentoConf(char *f) : Configuration()
{
	if (ReadConfigFile(*this, f) == false) {
		throw string("SorrentoConf::SorrentoConf: Failed to read configuration file.");
	}
}

bool SorrentoConf::run_server(void)
{
	bool def=FindB("SorrentoDefault::Server::Activate", false);
	return FindB("Sorrento::Server::Activate", def);
}

bool SorrentoConf::run_proxy(void)
{
	bool def=FindB("SorrentoDefault::Proxy::Activate", false);
	return FindB("Sorrento::Proxy::Activate", def);
}

char *SorrentoConf::ident(void)
{
	string def=Find("SorrentoDefault::Ident");
	string ret=Find("Sorrento::Ident", def.c_str());
	if (ret.c_str()[0]=='\0') {
		throw string("SorrentoConf::ident: No identity defined.");
	}
	return strdup(ret.c_str());
}

char *SorrentoConf::mc_group(void)
{
	string def=Find("SorrentoDefault::Channel");
	string channel=Find("Sorrento::Channel", def.c_str());

	char *group=NULL, *p=NULL;

	if (channel.c_str()[0]=='\0') {
		goto _fail;
		
	}

	group=strdup(channel.c_str());
	p=strchr(group, ':');

	if (p) {
		*p='\0';
		return group;
	}

	free(group);

_fail:
	throw string("SorrentoConf:mc_group: No mcast group defined.");
}

int SorrentoConf::mc_port(void)
{
	string def=Find("SorrentoDefault::Channel");
	string channel=Find("Sorrento::Channel", def.c_str());

	char *group=NULL, *p=NULL;
	int port=-1;

	if (channel.c_str()[0]=='\0') {
		goto _fail;
		
	}

	group=strdup(channel.c_str());
	p=strchr(group, ':');

	if (p) {
		p++;
		port=atoi(p);
		free(group);
		return port;
	}

	free(group);

_fail:
	throw string("SorrentoConf:mc_port: No mcast group defined.");
}

char **SorrentoConf::partitions(void)
{
	const Item * head=Tree("Sorrento::Server::Partitions");

	head=(head)?head->Child:head;

	const Item *list=head;

	if (!list) {
		goto _fail;
	}

	{
		int n_part;
		char **ret;
		int i;

		n_part=0;

		while (list) {
			n_part++;
			list=list->Next;
		}

		if (n_part==0)
			goto _fail;

		ret=(char **)calloc(n_part+1, sizeof(char *));

		for (list=head, i=0; list!=NULL; list=list->Next, i++) {
			ret[i]=strdup(list->Value.c_str());
		}

		return ret;
	}

_fail:
	throw string("SorrentoConf:partitions: No parititons specified.");
}

int SorrentoConf::mc_interval(void)
{
	int def=FindI("SorrentoDefault::Server::AnnounceInterval", 1000);
	return FindI("Sorrento::Server::AnnounceInterval", def);
}

int SorrentoConf::vnodes(void)
{
	int def=FindI("SorrentoDefault::Server::Vnodes", 32);
	return FindI("Sorrento::Server::Vnodes", def);
}

int SorrentoConf::rep_degree(void)
{
	int def=FindI("SorrentoDefault::Server::ReplicationDegree", 1);
	return FindI("Sorrento::Server::ReplicationDegree", def);
}

char *SorrentoConf::home_dir(void)
{
	char *p=getenv("SORRENTO_HOME");
	if (p==NULL) {
		p="/var/lib/sorrento";
	}

	string defaultHome=Find("SorrentoDefault::Home", p);
	string home=Find("Sorrento::Home", defaultHome.c_str());

	return strdup(home.c_str());
}

char *SorrentoConf::log_dir(void)
{
	char *home=home_dir();

	string defaultLogDir=Find("SorrentoDefault::LogDir", "log");
	string logDir=Find("Sorrento::LogDir", defaultLogDir.c_str());
	char abs_logdir[1024];

	if (logDir.c_str()[0]=='/') {
		strcpy(abs_logdir, logDir.c_str());
	}
	else {
		sprintf(abs_logdir, "%s/%s", home, logDir.c_str());
	}

	free(home);

	return strdup(abs_logdir);
}

char *SorrentoConf::server_log(void)
{
	char *abs_logdir=log_dir();

	string defaultLogName=Find("SorrentoDefault::Server::LogName", "server");
	string logName=Find("Sorrento::Server::LogName", defaultLogName.c_str());

	char abs_logname[1024];
	sprintf(abs_logname, "%s/%s.log", abs_logdir, logName.c_str());

	free(abs_logdir);
	
	return strdup(abs_logname);
}

char *SorrentoConf::proxy_log(void)
{
	char *abs_logdir=log_dir();

	string defaultLogName=Find("SorrentoDefault::Proxy::LogName", "proxy");
	string logName=Find("Sorrento::Proxy::LogName", defaultLogName.c_str());

	char abs_logname[1024];
	sprintf(abs_logname, "%s/%s.log", abs_logdir, logName.c_str());

	free(abs_logdir);
	
	return strdup(abs_logname);
}
