// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sorrento_types.h,v $ - sorrento constants and
 *            typedefs. 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/02/19 13:15:31 $
 * Version :  $Revision: 1.16 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _SORRENTO_TYPES_H_
# define _SORRENTO_TYPES_H_

# include "../../shared/guid/guid.h"
# include <map>
# include <string>
# include <stdio.h>
# include <sys/types.h>
# include "verfs.h"

#if !defined __off64_t_defined
typedef __off64_t off64_t;
# define __off64_t_defined
#endif

#define SOR_WR 0
#define SOR_RD_ONLY 1

// struct HostID { 
	// char data[16]; 
// };

typedef unsigned int off_seg_t; // segment offset type

struct HostAddr {
	char ip[16];
	int port;
	HostAddr() : port(-1) { ip[0] = '\0'; }
	HostAddr(const HostAddr &ha) : port(ha.port) { strncpy(ip, ha.ip, 16); }
	HostAddr(const char *xip, int xport) : port(xport)
	{ strncpy(ip, xip, 16); }
	bool valid() { return (ip[0] && (port>0)); }
	std::string to_string() { 
		if (valid()) {
			char buf[32]; 
			snprintf(buf, 32, "%s:%d", ip, port); 
			return std::string(buf);
		}
		else {
			return std::string("<N/A>");
		}
	}
};

struct ServerAddr0 {
	char ip[16];
	int tcp_port;
	int udp_port;
};

struct ServerAddr : public ServerAddr0 {
	ServerAddr() {
		tcp_port=-1;
		udp_port=-1;
		ip[0]='\0';
	}

	ServerAddr(ServerAddr0 &sa0) {
		tcp_port=sa0.tcp_port;
		udp_port=sa0.udp_port;
		strncpy(ip, sa0.ip, 16);
	}

	ServerAddr(const char *xip, int tport, int uport) 
	{
		tcp_port=tport;
		udp_port=uport;
		strncpy(ip, xip, 16); 
	}

	bool valid() { return (ip[0] && (tcp_port>0) && (udp_port>0)); }
	void init(const char *xip, int tport, int uport)
	{ 
		strncpy(ip, xip, 16); 
		tcp_port=tport;
		udp_port=uport;
	}

	char* toString(char *st){
		strncpy(st, ip, 16);
		st[17] = '\0';
		return st;
	}

};


# endif // _SORRENTO_TYPES_H_
