// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: srvcatalog.cc,v $ 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/03/15 21:43:21 $ by $Author: gulbeden $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

#include <iostream>
#include <stdio.h>
#include "srvcatalog.h"

SrvCatalog::SrvCatalog()
{
	pthread_mutex_init(&lock, NULL);
}

SrvCatalog::~SrvCatalog()
{
	pthread_mutex_destroy(&lock);
}

/**
 * adds pait oid, hid to the map
 * @return true if add succeeds, fasle otherwise
 */
bool SrvCatalog::add(const ObjID & oid, const HostID & hid) {

	if (pthread_mutex_lock(&lock) < 0) {
		return false;
	}
	bool ret = true;
  	SrvCatalog::iterator iter = this->insert(SrvCatalog::value_type(oid, hid));
	
	if(iter == this->end())		// is this the correct way of checking error 
		ret = false;			// conditions?

	pthread_mutex_unlock(&lock);
  
  	return ret;
}

/**
 * prints out the map
 */
void SrvCatalog::dump()
{
	if (pthread_mutex_lock(&lock) < 0) {
		return;
	}
	std::cout << "MAP:\n";
	SrvCatalog::const_iterator iter;
	for(iter = this->begin(); iter != this->end(); iter++) {
		char oid_str[256];
		char hid_str[256];

		Guid guid_oid(iter->first);
		guid_oid.to_string(oid_str, 256);
		Guid guid_hid(iter->second);
		guid_hid.to_string(hid_str, 256);

		printf("%s -> %s\n", oid_str, hid_str);
	}
	pthread_mutex_unlock(&lock);
}

/**
 * Gets the host-id for an object that is remotely stored.
 * @param oid ID of the object that is possibly stored at another host
 * @param hid Reference to the HostID that will be set to the HostID of the 
 * server that stores the object.
 * @return Status of the operation.
 */
bool SrvCatalog::get_hid(const ObjID &oid, vector<HostID> &hids)
{
	if (pthread_mutex_lock(&lock) < 0) {
		return false;
	}
	bool ret;

	SrvCatalog::iterator iter;
    SrvCatalog::iterator lower;
	SrvCatalog::iterator upper;

    lower = this->lower_bound(oid);
    upper = this->upper_bound(oid);

  	if (lower == this->end()){
  		ret = false;
  	}
  	else {
		for (iter = lower; iter != upper; iter++) {
			hids.push_back(iter->second);
		}
		ret = true;
	}

	pthread_mutex_unlock(&lock);

	return ret;
}


/**
 * removes a pair from the map
 * @param oid Object ID to be deleted
 * @param hid Host ID to be deleted
 * @return false in case of an error, true otherwise
 */
bool SrvCatalog::remove(const ObjID &oid, const HostID &hid)
{
	if (pthread_mutex_lock(&lock) < 0) {
		return false;
	}

	bool ret;

	SrvCatalog::iterator iter;
    SrvCatalog::iterator lower;
	SrvCatalog::iterator upper;

    lower = this->lower_bound(oid);
    upper = this->upper_bound(oid);

	try {
		
		ret = true;
		for (iter = lower; iter != upper; iter++) {
			if(memcmp((char *)&hid, (char *)&(iter->second), \
					  sizeof(HostID)) == 0) {
				// found the one we are looking for
				this->erase(iter);
			}
		}
	}
	catch (...) {
		ret = false;
	}
	
	pthread_mutex_unlock(&lock);
	
	return ret;
}
