// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: srvcatalog.h,v $ - 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/05 23:34:47 $ by $Author: bill $
 * Version :  $Revision: 1.11 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

#ifndef _SRV_CATALOG_H_
#define _SRV_CATALOG_H_

//#include <map>
#include <map.h>
#include <vector>
#include <pthread.h>
#include "sorrento_types.h"

using namespace std;

// TODO:
// Can we manage multiple hosts for each oid?

class SrvCatalog : std::multimap<ObjID, HostID, ObjID_cmp> 
{
 private:
	pthread_mutex_t lock;

 public:
	SrvCatalog();
	~SrvCatalog();

	// If not exist, add to the catalog, if exist, add to the vector. 
	// both situation are considereed as successful.
	bool add(const ObjID &oid, const HostID &hid);

	//bool get_hid(const ObjID &oid, HostID &hid);
	bool get_hid(const ObjID &oid, vector<HostID> &hidsg);

	// If not exist, do nothing (could be a duplicate request).
	// if exist and the hid is the same, erase the mapping.
	// Otherwise, do nothing. All these cases are
	// considered as successful.
	bool remove(const ObjID &oid, const HostID &hid);
	void dump();

};

#endif
