// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: srvcatalogupdater.h,v $ - Thread that updates server
 *            catalog.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/02/19 13:15:31 $ by $Author: gulbeden $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

#ifndef __SRV_INDEX_UPDATER__
#define __SRV_INDEX_UPDATER__

#include "cbs_thread.h"
#include "sorrento_types.h"


class HostMap;
class Replicator;
class RNG;
class SrvCatalog;
class TimeHeap;
class Udp_cli;


class SrvCatalogUpdater: public CBS_Thread
{
 private:
    SrvCatalog *sc;
    HostMap *hmap;
	Udp_cli *cli;
	HostID myHID;
	TimeHeap *time_heap;
	RNG *rng;
	bool use_tcp; /**< whether we use tcp to pull from remote servers. */
	ObjID *recv_buf; /**< receiving buffer */
	off_seg_t buf_size; /** receiving buffer size */
	Replicator *replicator;

	void pull_from_host(ServerAddr &saddr);
    
 public:
    virtual void *run();
    SrvCatalogUpdater(SrvCatalog *src, HostMap *hm, HostID hid, Replicator *r);
    ~SrvCatalogUpdater();
    bool addHost(HostID &hid);
    bool removeHost(HostID &hid);
};


#endif //__SRV_INDEX_UPDATER__
