// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t4.cc,v $ - sorrento monitor.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/26 07:32:04 $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <sys/ioctl.h>
# include <sys/time.h>
# include <sys/types.h>
# include <string.h>
# include <unistd.h>
# include <stdio.h>
# include <string.h>
# include <signal.h>

# include <string>
# include <vector>

# include "myterm.h"
# include "sorrento_conf.h"
# include "subscriber.h"

using namespace std;

static bool quit_char(int c)
{
	if (c=='q')
		return true;

	return false;
}

MyTerm term(NULL);

static void handle_sigwinch (int sig)
{
	term.adj_window_size();
	signal(SIGWINCH, handle_sigwinch);
}

string * translate(MyTerm &term, const VHostLoad *hostloads)
{
	if ((!hostloads)||(hostloads->size()<=0))
		return NULL;

	// ip(16), vn(4), cpu(4), cpu_ewma(4), io(4),io_ewma(4), mem(4), swap(4), disks(...)
	string *lines=new string[hostloads->size()];
	for (unsigned int i=0; i<hostloads->size(); i++) {
		char buf[1024];
		sprintf(buf, " %-16s %-4d %04.1f   %04.1f   %04.1f   %04.1f   %04.1f   %04.1f  ", 
				(*hostloads)[i].addr.ip,
				(*hostloads)[i].vnodes,
				(*hostloads)[i].cpu_comp*100,
				(*hostloads)[i].cpu_ewma_comp*100,
				(*hostloads)[i].cpu_io*100,
				(*hostloads)[i].cpu_ewma_io*100,
				(*hostloads)[i].mem_used*100,
				(*hostloads)[i].swap_used*100
		);
		string str_disk("<");
		for (int j=0; j<(*hostloads)[i].ndisks; j++) {
			if (j>0)
				str_disk=str_disk+",";
			char tmp[100];
			sprintf(tmp, "%d", (*hostloads)[i].disks[j].bfree/1024); 
			str_disk=str_disk+tmp;
		}
		str_disk=str_disk+">";
		lines[i]=buf;
		lines[i]=lines[i]+str_disk;
	}

	return lines;
}

int main(int argc,char **argv)
{
	if (argc!=2) {
		printf("Usage: %s <config>.\n", argv[0]);
		exit(-1);
	}

	try {
		SorrentoConf sconf(argv[1]);
		Subscriber sub(sconf);
		char *ident=sconf.ident();
		int c='\0';

		signal(SIGWINCH, handle_sigwinch);

		sub.start();

		char buf[1024];
		sprintf(buf, " %-16s %4s %6s %6s %6s %6s %6s %6s %s", 
			"       IP       ", "VN ", " CPU  ", "CPU-CU", "  IO  ",
			" IO-CU", " MEM  ", " SWAP ", " DISKS ");
		string header=buf;
		while (!quit_char(c)) {
			string *lines=NULL;
			int n_lines=0;

			//printf("Getting host map\n");
			//fflush(stdout);
			HostMap *hmap=sub.get_host_map_by_ident(ident);
			string title="Sorrento Monitor";
			title=title+" ("+ident+")";
			HostAddr dir_addr;
			if (hmap->get_dir_addr(dir_addr)) {
				title=title+" [DirSrv: "+dir_addr.to_string()+"]";
			}
			else {
				title=title+" [DirSrv not available.]";
			}
			//if (hmap)
				//printf("Got host map\n");
			//else
				//printf("Got NULL host map\n");
			//fflush(stdout);
			const VHostLoad *hostloads=(hmap)?hmap->get_host_loads():NULL;
			//printf("Got host loads\n");
			//fflush(stdout);
			if (hostloads) {
				lines=translate(term, hostloads);
				n_lines=hostloads->size();
			}
			hmap->release_host_loads(hostloads); // Important: release_host_loads even if it is NULL.
			term.clr_scr();
			term.display_title(title);
			term.display_msg(header);
			if (lines) {
	    		term.display_body(lines, n_lines);
				// for (unsigned int i=0; i<n_lines; i++) {
					// printf("%s\n", lines[i].c_str());
				// }
				delete [] lines;
			}

			struct timeval tv;
			fd_set in;

			tv.tv_sec = 0;
			tv.tv_usec = 3000000; // 1s

			FD_ZERO(&in);
			FD_SET(0, &in);
			if (select(1, &in, 0, 0, &tv) > 0) 
				read(0, &c, 1);
    	}

		sub.stop();

		sub.finalize();
	}
	catch (string &msg)
	{
		printf("Error: %s.\n", msg.c_str());
		exit(-1);
	}

	return 0;
}
