// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: test_client.cc,v $ - program to test server and 
 *                      client through commands. 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/03/15 21:43:21 $ by $Author: gulbeden $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu, gulbeden@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

/**
 * List of commands:
 * c1 : create object (without a parent)
 * o index : open object
 * w desc offset length data : write to object
 * a desc length : append to object
 * r desc offset length : read from object
 * c desc : close object
 * t desc length : truncate
 * m index : remove object
 * 
 */


#include <iostream>
#include <unistd.h>		/* sleep */
#include <vector>
#include <stdio.h>

#include "client.h"		/* to relay requests to the server */

using namespace std;

char server[32];

int main(int argc, char *argv[])
{
  int udpPort;  
  int tcpPort;
  //char server[32];

  
  int arg, offset, length;

	Client *client;

	int end;

	char buffer[512];

	guid_t guids[20];
	int guidIndex=0;

	char comm[30];

    if (argc == 4) {
		strcpy(server, argv[1]);
		udpPort = atoi(argv[3]);
		tcpPort = atoi(argv[2]);
		cout << "Client will connect to "<< server << ":" << udpPort << ":" << tcpPort <<".\n";
	}

	else{
		printf("Usage: \n%s <server> <udpPort> <tcpPort>\n", argv[0]);
		exit(1);
	}
	client = new Client();

	Descriptor* descs;

	descs = new Descriptor[60];

	if (!client) {
		cout << "Client null." << endl;
		exit(-1);
	}

	for(int i=0; i<10; i++) {
		guids[i].data[0]=i;
		guids[i].data[1]=0;
		guids[i].data[2]=0;
		guids[i].data[3]=0;
		guids[i].data[4]=0;
		guids[i].data[5]=0;
		guids[i].data[6]=0;
		guids[i].data[7]=0;
		guids[i].data[8]=0;
		guids[i].data[9]=0;
		guids[i].data[10]=0;
		guids[i].data[11]=0;
		guids[i].data[12]=0;
		guids[i].data[13]=0;
		guids[i].data[14]=0;
		guids[i].data[15]=0;
	}

	ServerAddr sad(server, udpPort, tcpPort);

	do {
		end = scanf("%s", comm);

		if (end == EOF)
			exit(0);

		printf(": %s\n", comm);

		if (comm[0] == 'c' && comm[1]=='1') {
		  // create
		  
		  client->create(sad, guids[guidIndex], descs[guidIndex]);
		  guidIndex++;
		}
		else if(comm[0] == 'r') {
			// read
			scanf("%d", &arg);
			scanf("%d", &offset);
			scanf("%d", &length);

			char buffer[512];
			off_seg_t read_len;

			client->read(descs[arg], offset, buffer, length, read_len);

			printf("test client read:\n%s\n", buffer);
		}
		else if(comm[0] == 'w') {
			// write
			scanf("%d", &arg);
			scanf("%d", &offset);
			scanf("%d", &length);
			scanf("%s", buffer);

			off_seg_t write_len;
			client->write(descs[arg], (off_seg_t)offset, buffer, (off_seg_t)length, write_len);
		}
		else if(comm[0] == 'a') {
			// append
			scanf("%d", &arg);
			scanf("%d", &length);
			scanf("%s", buffer);

			off_seg_t append_len;
			client->append(descs[arg], buffer, (off_seg_t)length, append_len);
		}
		else if(comm[0] == 'c') {
			// close
			scanf("%d", &arg);
			client->close(descs[arg]);
		}
		else if(comm[0] == 'o') {
		  // open
		  
		  scanf("%d", &arg);

		  client->open(sad, (guids[arg]), descs[guidIndex], SOR_WR);
		  guidIndex++;
		}
		else if(comm[0] == 's') {
		  // stop server

		  client->stop_server(sad);
		  break;
		}
		else if(comm[0] == 'm') {
		  // remove
		  scanf("%d", &arg);

		  client->remove(sad, (guids[arg]));
		}
		else if(comm[0] == 't') {
			// truncate
			scanf("%d", &arg);
			scanf("%d", &length);
			off_seg_t trunc_len;
			client->truncate(descs[arg], (off_seg_t)length, trunc_len);
		}
		else {
			cout << "Command not recognized." << endl;
		}

		/*

		else if(comm[0] == 'd'){
		  // read
		  scanf("%d", &arg);

		  vfsDumpObject(arg);
		  if (verbose) printf("Dumped :%d\n", arg);
		}
		*/

	} while (1);
}
