// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: xput_tracker.h,v $ - Module that tracks block throughput. 
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            William Strathearn, bill@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/10 03:12:16 $ by $Author: bill $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef __XPUT_TRACKER_H__
# define __XPUT_TRACKER_H__

#include <pthread.h>

# include "cbs_thread.h"
# include "map.h"
# include "sorrento_types.h"
# include "client.h"

using namespace std;


class Client;
class HostMap;
class Udp_cli;
class Verfs;

struct cmpBlockMaps
{
    bool operator()(int b1, int b2) const
    {
       if(b1 < b2) return true;
       else return false;
    }
};

struct cmpClientID 
{
    bool operator()(ClientID c1, ClientID c2) const
    {
        if(c1.id < c2.id) return true;
        return false;
    }  
};

//map containing the number of bytes accessed for each client ID 	
typedef map<const ClientID, long, cmpClientID> blockAccessType;
//map containing the blockAccessType's for each block
typedef map<int, blockAccessType, cmpBlockMaps> blockMapType;

class Xput_Tracker: public CBS_Thread
{

private:
	HostMap *hm;
	ClientID *myHid;
	ServerAddr *myAddr;
    pthread_mutex_t blockMapLock;
	Udp_cli *udpCli;
    Client *cli;
	blockMapType blockMap;	
    void dumpAccessType(blockAccessType access);
    void dumpBlockMap();
public:
	Xput_Tracker(HostMap *hostMap);
	~Xput_Tracker();
	bool addSegment(int desc, const ClientID &cid);
	bool removeSegment(int desc);
	bool addXput(int desc, ClientID &cid, int bytes);
	virtual void *run();
	bool start();
};


#endif // __XPUT_TRACKER_H__
