// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: mcast_adaptor.h,v $ - header file for multi-cast packet 
 *            assembling and dessembling adaptors.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/14 08:21:12 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _MCAST_ADAPTOR_H_
# define _MCAST_ADAPTOR_H_

# include "mcast_annc.h"

/** MDC: multicast data collectors */
class MDC {
 public:
	/** @name public methods (adaptor interface) **/
	//@{
	/**
 	 * read the local state data and put it into a
	 * multicast announcement packet.
 	 *  @param   pkt - multicast announcement packet
 	 *  @param   ms_time - current reference timer expressed 
	 * 	         in milli-seconds, intend to be used to allow
	 * 	         data collectors implement their own update
	 * 	         frequencies. E.g. one adaptor may wish to
	 * 	         only insert an update field every 1 second,
	 * 	         it may achieve that by remembering the last
	 * 	         time it was called can check if the current
	 * 	         time is 1 second after the previous update
	 * 	         time.
 	 *  @return  true: a field is successfully added; 
	 *           false: no field is added (not necessarily 
	 *             an error).
 	 */
	virtual bool put(Mc_anc_data &pkt, int ms_time)=0; 

	/** destructor */
	virtual ~MDC()=0;
	//@}
}; // end of class MDC

/** MDE: multicast data extractors */
class MDE {
 public:
	/** @name public methods (adaptor interface) **/
	//@{
	/**
 	 * read multicast announcement packet and update
	 * the local state data
 	 *  @param   pkt - multicast announcement packet
 	 *  @return  true: a field is successfully extracted 
	 *             and local state updated; 
	 *
	 *           false: no field is extracted and no 
	 *             local state update (not signify an 
	 *             error).
 	 */
	virtual bool get(Mc_anc &pkt)=0;

	/** destructor */
	virtual ~MDE()=0;
	//@}
}; // end of class MDE

# endif // _MCAST_ADAPTOR_H_
