// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dir_install.cc,v $ - tool to format an installation
 *            of the sorrento system.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/21 18:30:04 $ by $Author: htang $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <dirent.h>
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>
# include <getopt.h>
# include <string.h>

# include "../dir_srv/dirsrv_conf.h"

# define MAX_MP_LEN 1024
# define MODE S_IRWXU

char *prog=NULL;

static void do_help()
{
	printf("\
Usage: %s [options] [config].\n\
   -f         Format even if the targets already exist. \n\
   -h         Help, this screen.\n\
", prog);
	exit(0);
}

static bool prompt_force(const char *mp, bool force=false)
{
	if (force)
		return true;

	printf("File %s already exists, removing it? [Y/N]", mp);
	fflush(stdout);
	char line[256];
	fgets(line, 256, stdin);
	if (line[0]=='Y')
		return true;
	return false;
}

static bool create_dir(const char *dir)
{
	if (access(dir, R_OK|W_OK|X_OK)) {
		if (mkdir(dir, MODE)) {
			fprintf(stderr, "Cannot create directory: %s.\n", dir);
			return false;
		}
	}

	return true;
}
		
int main(int argc, char *argv[])
{
	bool force=false;

	prog=argv[0];
	
	int c;
	
	while ((c=getopt(argc, argv, "fh"))!=EOF) 
	{
		switch (c) {
			case 'f':
				force=true;
				break;
			case 'h':
				do_help();
				break;
			default:
				do_help();
				break;
		}
	}

	if ((argc-optind)<=0) {
		fprintf(stderr, "No configuration file specified.\n");
		do_help();
	}

	if ((argc-optind)>1) {
		fprintf(stderr, "Too many arguments.\n");
		do_help();
	}

	try {
		DirSrvConf dconf(argv[optind]);
		char *home_dir=dconf.home_dir();
		char *log_dir=dconf.log_dir();
		if ( (home_dir == NULL) || (home_dir[0]=='\0')  ||
			 (log_dir == NULL) || (log_dir[0]=='\0') ) {
			fprintf(stderr, "No directory information in the configuration file.");
			exit(-1);
		}

		// create directories
		if(create_dir(home_dir)==false) printf("Unable to create home dir.\n");
		if(create_dir(log_dir)==false) printf("Unable to create log dir.\n");
		free(home_dir);
		free(log_dir);

		char *backstore=dconf.backstore();
		if (access(backstore, F_OK)==0) {
			if (prompt_force(backstore, force)) {
				unlink(backstore);
				if (access(backstore, F_OK)==0) {
					fprintf(stderr, "Failed to remove file: %s.\n", backstore);
					exit(-1);
				}
			}
		}
		free(backstore);
		printf("Format complete.\n");
	}
	catch (...) {
		printf("Error occurred during the format.\n");
		exit(-1);
	}

	return 0;
}
