// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sor_halt.cc,v $ - micro benchmark.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/02 01:55:44 $ by $Author: htang $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include <signal.h>
# include <string>

extern "C" {
# include <readline/readline.h>
# include <readline/history.h>
}

# include "sor_client.h"
# include "sor_fs.h"
# include "subscriber.h"
# include "tmpi_debug.h"

int main(int argc, char *argv[])
{
    if (argc != 2) {
        printf("Usage: %s <config>.\n", argv[0]);
        exit(-1);
    }

	tmpi_set_dbg_level(DBG_ALL);
	tmpi_show_location();
    int ret=-1;
    signal(SIGPIPE, SIG_IGN);

    Subscriber *sub=NULL;
    SorClient *scli=NULL;
    SorFS *sorfs=NULL;
	try {
        SorrentoConf sconf(argv[1]);
        sub= new Subscriber(sconf);
        sub->start();
        printf("Starting up ...");
        fflush(stdout);
		bool done=false;
        int max_retry=10;

		while ((!done) && (max_retry--)) {
            printf(".");
            fflush(stdout);
			try {
				sleep(1); 
		        sorfs=new SorFS(sub, std::string(sconf.ident()));
				scli=new SorClient(sorfs);
				done=true;
			}
			catch (const std::string &errmsg) {
                if (sorfs) {
                    delete sorfs;
                    sorfs=NULL;
                }
                if (scli) {
                    delete scli;
                    scli=NULL;
                }
			}
		}

        if (scli == NULL) {
            printf("\nCannot locate the specified service (%s).\n", sconf.ident());
            ret=-1;
        }
        else {
            printf("\nGathering information (%s).\n", sconf.ident());
            sleep(3); // wait for 3 seconds.
            scli->dump_view();
            char confirm='x';
            while ((confirm != 'y')&&(confirm != 'n')) {
                char *l=readline("Continue (Y/N)?");
                confirm=tolower(l[0]);
                free(l);
            }

            if (confirm == 'y') {
                printf("Proceed to stop the system (%s).\n", sconf.ident());
                scli->stop_service();
                printf("System halted.\n");
            }
            else {
                printf("Halting process aborted.\n");
            }
            ret=0;
        }
	}
	catch (const std::string &errmsg) {
		tmpi_error(DBG_INTERNAL, "%s : %s\n", argv[0], errmsg.c_str());
	}
	catch (int errnum) {
		tmpi_error(DBG_INTERNAL,"%s : error caught (code=%d).\n",argv[0], errnum);
	}
	catch (...) {
		tmpi_error(DBG_INTERNAL, "Unknown error, initialization failed.\n");
	}

	if (sub) {
	    sub->stop();
	    sub->finalize();
		delete sub;
    }

    if (sorfs) {
        delete sorfs;
    }

    if (scli) {
        delete scli;
    }

	return ret;
}
