/*******************************************************************************
Copyright (c) 2003 The Regents of the University of California. 
All rights reserved.

Netptune project contact: Lingkun Chu (lkchu@cs.ucsb.edu),
                          Tao Yang (tyang@cs.ucsb.edu),
                          Kai Shen (kshen@cs.rochester.edu)
Web site: http://www.cs.ucsb.edu/projects/neptune
Department of Computer Science, UC Santa Barbara.

This software is distributed to individuals for personal non-commercial 
use and to non-profit entities for non-commercial purposes only. Permission 
is also granted to commercial enterprises to use this software for evaluation 
purposes only. It is licensed on a non-exclusive basis, free of charge for 
these uses. The license is subject to the following conditions:

1. Redistribution is not permitted. UC Santa Barbara will maintain a copy 
   of this software as a directly downloadable file, available under the terms 
   of license specified in this agreement.
2. All advertising materials mentioning features or use of this software must 
   display the following acknowledgment: "This product includes software 
   developed by the University of California, Santa Barbara." 
3. Neither the name of the University nor the names of its contributors may 
   be used to endorse or promote products derived from this software without 
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*******************************************************************************/
/*******************************************************************************
 *
 * FILE: config.h
 * DESCRIPTION:	This module contains retrieval configuration routines.
 * AUTHOR: Kai Shen
 * DATE CREATED: Dec 11, 2000
 * REVISIONS:
*******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "config.h"


/*[]-------------------------------------------------------------------[]
  |  Utitily routines (copied from Linkun's aux-server components :)	|
  []-------------------------------------------------------------------[]*/

static char *trim(char *line)
{
    char *cp;
    for (cp=line; isspace(*cp) && (*cp) != 0; cp++);
    for (int i=strlen(line)-1; isspace(line[i]) && i>=0; i--)
        line[i]='\0';
    return cp;
}


static int split(char *line, char **key, char **value)
{
    char *cp = NULL;

    if ((line == NULL) || (strlen(line)==0) || (line[0]=='#'))
        return 0;
    for (size_t i=0; i<strlen(line); i++) {
        if (line[i] == '=') 
		{
            cp = &line[i];
            *cp++='\0';
            break;
        }
    }
    if (cp != NULL) 
	{
        (*key) = trim(line);
        (*value) = trim(cp);
        return 1;
    }
    else
        return 0;
}


/*[]-----------------------------------------[]
  |  Public                                   |
  []-----------------------------------------[]*/

Config::Config(const char* fname, const char* hname)
{
	file_num = 0;
    FILE* fp = fopen(fname, "r");
    if (fp == NULL) 
	{
        fprintf(stderr, "Unable to open configuration file %s", fname);
        return;
    }

    char buffer[MAX_LINE];
    char *key, *value;
	
	while(fgets(buffer, MAX_LINE, fp))
	{
		if (!split(buffer, &key, &value))
			continue;
		if (!strcmp(key, "Domain"))
		{
			if (!strcmp(value, hname))
			{
				strncpy(domain, value, MAXHOSTNAMELEN);
				break;
			}
		}
	}
	
    while(fgets(buffer, MAX_LINE, fp))
    {
        if (!split(buffer, &key, &value)) 
			continue;

        if (!strcmp(key, "Domain"))
        	break;
		if (!strcmp(key, "File_Name"))
		{
            strncpy(file_name[file_num], value, MAXFILENAMLEN);
			file_num ++;
        }
		if (!strcmp(key, "Index_Name")){
			strncpy(index_name, value, MAXFILENAMLEN);
		}
		if (!strcmp(key, "Free_Name")){
			strncpy(free_name, value, MAXFILENAMLEN);
        }
    }

    fclose(fp);
}
