#include <stdlib.h>
#include <stdio.h>
#include "freespacelist.h"

Freespace_List::Freespace_List()
{
}

Freespace_List::~Freespace_List()
{
}

void Freespace_List::Initialize(char* filename)
{
	ifmodify = 0;
	Read_fromFile(filename);
}

void Freespace_List::Finalize(char* filename)
{
	Write_toFile(filename);
	Free_Blocks* ptr1;
	Free_Blocks* ptr2;

	ptr1 = head;
	while (ptr1 != NULL)
	{
		ptr2 = ptr1->next;
		free(ptr1);
		ptr1 = ptr2;
	}
}

int Freespace_List::Read_fromFile(char* filename)
{
	FILE* systemfile;
	systemfile = fopen(filename, "r");
	if (systemfile == NULL)
	{
		fprintf(stderr, "Free Space File NOT Found! Create a NEW List...\n");
		head = (Free_Blocks*) malloc(sizeof(Free_Blocks));
		head->beginblock = 0;
		head->blocknum = -1;
		head->next = NULL;
		ifmodify = 1;
		return 0;
	}

	head = NULL;
	Save_Blocks oneunit;
	Free_Blocks* end;
	Free_Blocks* newunit;
	if (fread(&oneunit, sizeof(Save_Blocks), 1, systemfile) != 1)
	{
		fprintf(stderr, "Free Space File Read ERROR! Create a NEW List...\n");
		head = (Free_Blocks*) malloc(sizeof(Free_Blocks));
		head->beginblock = 0;
		head->blocknum = -1;
		head->next = NULL;
		return 0;
	}
	head = (Free_Blocks*) malloc(sizeof(Free_Blocks));
	head->beginblock = oneunit.beginblock;
	head->blocknum = oneunit.blocknum;
	head->next = NULL;
	end = head;
	
	while (fread(&oneunit, sizeof(Save_Blocks), 1, systemfile) == 1)
	{
		newunit = (Free_Blocks*) malloc(sizeof(Free_Blocks));
		newunit->beginblock = oneunit.beginblock;
		newunit->blocknum = oneunit.blocknum;
		newunit->next = NULL;
		end->next = newunit;
		end = newunit;
	}
	fclose(systemfile);
	return 1;
}

int Freespace_List::Write_toFile(char* filename)
{
	FILE* systemfile;
	systemfile = fopen(filename, "w");
	if (systemfile == NULL)
	{
		fprintf(stderr, "Free Space List CANNOT be Saved! List Lost...\n");
		return 0;
	}

	Save_Blocks oneunit;
	Free_Blocks* ptr;
	ptr = head;
	while(ptr != NULL)
	{
		oneunit.beginblock = ptr->beginblock;
		oneunit.blocknum = ptr->blocknum;
		if (fwrite(&oneunit, sizeof(Save_Blocks), 1, systemfile) != 1)
		{
			fprintf(stderr, "Save Free Space List Error! List Lost...\n");
			fclose(systemfile);
			return 0;
		}
		ptr = ptr->next;
	}
	fclose(systemfile);
	ifmodify = 0;
	return 1;	
}

void Freespace_List::Print_List()
{
	Free_Blocks* ptr;
	ptr = head;
	while (ptr != NULL)
	{
		fprintf(stderr, "BeginBlock = %d  ", ptr->beginblock);
		fprintf(stderr, "BlockNumber = %d\n", ptr->blocknum);
		ptr = ptr->next;
	}
}

size_t Freespace_List::Get_Freespace(int blocknum)
{
	Free_Blocks* pre;
	Free_Blocks* cur;
	size_t ret=0;
	
	cur = head;
	pre = NULL;
	while(cur != NULL)
	{
		if (cur->blocknum == -1)
		{
			ret = cur->beginblock;
			cur->beginblock += blocknum;
			break;
		}
		if (cur->blocknum == blocknum)
		{
			if (pre == NULL)
				head = cur->next;
			else
				pre->next = cur->next;
			ret = cur->beginblock;
			free(cur);
			break;
		}
		if (cur->blocknum > blocknum)
		{
			ret = cur->beginblock;
			cur->beginblock += blocknum;
			cur->blocknum -= blocknum;
			break;
		}
		pre = cur;
		cur = cur->next;
	}
	ifmodify = 1;
	return ret;
}

int Freespace_List::Return_Freespace(size_t beginblock, int blocknum)
{
	Free_Blocks* ptr;
	if (beginblock < (head->beginblock))
	{
		if ((beginblock + blocknum) >= (head->beginblock))
		{
			if (head->blocknum != -1)
				head->blocknum = (head->beginblock)-beginblock+(head->blocknum); 
			head->beginblock = beginblock;
		}
		else
		{
			ptr = (Free_Blocks*) malloc(sizeof(Free_Blocks));
			ptr->beginblock = beginblock;
			ptr->blocknum = blocknum;
			ptr->next = head;
			head = ptr;
		}
	}
	else
	{
		Free_Blocks* nxt;
		ptr = head;
		nxt = ptr->next;
		while((nxt != NULL) && (beginblock > nxt->beginblock))
		{
			ptr = nxt;
			nxt = nxt->next;
		}
		if (nxt == NULL)
			return 0;
		else
		{
			if ((beginblock + blocknum) >= nxt->beginblock)
			{
				if (nxt->blocknum != -1)
					nxt->blocknum = nxt->beginblock-beginblock+nxt->blocknum;
				nxt->beginblock = beginblock;
				if ((ptr->beginblock+ptr->blocknum) >= nxt->beginblock)
				{
					if (nxt->blocknum == -1)
						ptr->blocknum = -1;
					else
						ptr->blocknum = nxt->beginblock+nxt->blocknum-ptr->beginblock;
					ptr->next = nxt->next;
					free(nxt);
				}
			}
			else
			{
				if ((ptr->beginblock+ptr->blocknum) >= beginblock)
				{
					ptr->blocknum = beginblock+blocknum-ptr->beginblock;
				}
				else
				{
					Free_Blocks* newb;
					newb = (Free_Blocks*) malloc(sizeof(Free_Blocks));
					newb->beginblock = beginblock;
					newb->blocknum = blocknum;
					newb->next = nxt;
					ptr->next = newb;
				}
			}				
		}			
	}
	ifmodify = 1;
	return 1;	
}

int Freespace_List::Ifmodified()
{
	return ifmodify;
}

/*
int main()
{
	Freespace_List fsl;
	fsl.Get_Freespace(4);
	fsl.Get_Freespace(5);
	fsl.Get_Freespace(6);

	fsl.Return_Freespace(2, 2);
	fsl.Return_Freespace(7, 4);


	fsl.Get_Freespace(3);
	fsl.Get_Freespace(4);

	fsl.Return_Freespace(0, 2);
	fsl.Return_Freespace(7, 3);
	fsl.Return_Freespace(16, 4);
	
	fsl.Return_Freespace(4, 3);
	fsl.Return_Freespace(10, 8);
	fsl.Get_Freespace(8);
	fsl.Return_Freespace(0, 2);
	fsl.Return_Freespace(4, 2);
	fsl.Return_Freespace(2, 2);
	fsl.Print_List();
}
*/
