/**********************************************************************************
Copyright (c) 2003 The Regents of the University of California. 
All rights reserved.

Sorrento project contact: Hong Tang (hongt@cs.ucsb.edu)
			  Lingkun Chu (lkchu@cs.ucsb.edu),
                          Tao Yang (tyang@cs.ucsb.edu),
Web site: http://www.cs.ucsb.edu/projects/sorrento
Department of Computer Science, UC Santa Barbara.

This software is distributed to individuals for personal non-commercial 
use and to non-profit entities for non-commercial purposes only. Permission 
is also granted to commercial enterprises to use this software for evaluation 
purposes only. It is licensed on a non-exclusive basis, free of charge for 
these uses. The license is subject to the following conditions:

1. Redistribution is not permitted. UC Santa Barbara will maintain a copy 
   of this software as a directly downloadable file, available under the terms 
   of license specified in this agreement.
2. All advertising materials mentioning features or use of this software must 
   display the following acknowledgment: "This product includes software 
   developed by the University of California, Santa Barbara." 
3. Neither the name of the University nor the names of its contributors may 
   be used to endorse or promote products derived from this software without 
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <getopt.h>
#include <pthread.h>
#include <ctype.h>
#include <assert.h>
#include "argdef.h"
#include "socket_opt.h"
#include "webclt.h"

#define MAX_LINE MAXFILENAMLEN+20
#define MAXCLIENTTYPES 10

typedef struct
{
	char hostname[MAXHOSTNAMELEN];
	char featurelist[MAXFILENUMBER][MAXFEATURELEN];
	int featurenum;
}One_Server;

class Frontend;

typedef struct
{
	Frontend* fe;
    int thread_count; 
    int filenum;
    int interval;
    char id;
    char beginchar;  
    char endchar;
    char *filename;
    int group;
}Arg_List;

typedef struct
{
    int thread_count; 
    int filenum;
    int interval;
    char id;
    char beginchar;  
    char endchar;
    char *filename;
    int existing_urls;
    int urls_put;
}Client_Arg_List;

class Frontend
{
	public:
   One_Server server_list[MAXSERVERNUM];
   Client_Arg_List client_args[MAXCLIENTTYPES];
	int server_num;
	int client_num;
    int thread_num;
    int put_urls;
    pthread_mutex_t lock;
	public:
	Frontend(char *);
	~Frontend(){};

	int Get_Servername(char* url, char* sname);
};

static char *trim(char *line)
{
    char *cp;
    for (cp=line; isspace(*cp) && (*cp) != 0; cp++);
    for (int i=strlen(line)-1; isspace(line[i]) && i>=0; i--)
        line[i]='\0';
    return cp;
}

static int split(char *line, char **key, char **value)
{
    char *cp = NULL;

    if ((line == NULL) || (strlen(line)==0) || (line[0]=='#'))
        return 0;
    for (size_t i=0; i<strlen(line); i++) {
        if (line[i] == '=') 
		{
            cp = &line[i];
            *cp++='\0';
            break;
        }
    }
    if (cp != NULL) 
	{
        (*key) = trim(line);
        (*value) = trim(cp);
        return 1;
    }
    else
        return 0;
}


Frontend::Frontend(char *client_file)
{
	char conf_file[MAXFILENAMLEN];
    //char client_file[MAXFILENAMLEN];
	sprintf(conf_file, "%scache_config", CONFIGPATH);
	//sprintf(client_file, "%sclient_config", CONFIGPATH);
    server_num = 0;
    client_num = 0;
    thread_num = 0;
    put_urls = 0;
    
    pthread_mutex_init(&lock,NULL);
    
	FILE* fp = fopen(conf_file, "r");
	if (fp == NULL)
	{
		fprintf(stderr, "Unable to open config file %s\n", conf_file);
		return;
	}

	char buffer[MAX_LINE];
	char *key, *value;
	char filename[MAXFILENAMLEN];

	while(fgets(buffer, MAX_LINE, fp))
	{
		if (!split(buffer, &key, &value))
			continue;
		if (!strcmp(key, "Domain"))
		{
			server_num++;
			server_list[server_num-1].featurenum = 0;
			strncpy(server_list[server_num-1].hostname, value, MAXHOSTNAMELEN);
		}
		if (!strcmp(key, "File_Name"))
		{
			strncpy(filename, value, MAXFILENAMLEN);
			char* end;
			char* begin;
			end = filename + strlen(filename);
			begin = end - 1;
			while ((begin != filename) && (begin[0] != '/'))
			{
				if (begin[0] == '.')
					end = begin;
				begin--;
			}
			if (begin[0] == '/')
			{
				strncpy(server_list[server_num-1].featurelist[server_list[server_num-1].featurenum], begin+1, MAXFEATURELEN);
				server_list[server_num-1].featurelist[server_list[server_num-1].featurenum][end-begin-1] = '\0';
			}
			else
			{
				strncpy(server_list[server_num-1].featurelist[server_list[server_num-1].featurenum], begin, MAXFEATURELEN);
				server_list[server_num-1].featurelist[server_list[server_num-1].featurenum][end-begin] = '\0';
			}
			server_list[server_num-1].featurenum++;
		}
	}
	fclose(fp);

    fprintf(stderr,"%d Servers Indexed\n",server_num);
    
	fp = fopen(client_file, "r");
	if (fp == NULL)
	{
		fprintf(stderr, "Unable to open client file %s\n", client_file);
		return;
	}

	while(fgets(buffer, MAX_LINE, fp))
	{
		if (!split(buffer, &key, &value))
			continue;
		if (!strcmp(key, "Type"))
		{
			client_num++;
            client_args[client_num-1].existing_urls = 0;
			client_args[client_num-1].id = value[0];
            client_args[client_num-1].filename = (char *)malloc(sizeof(char)*MAXFILENAMLEN);
		}
		if (!strcmp(key, "Interval"))
		{
			client_args[client_num-1].interval = atoi(value);
		}		
		if (!strcmp(key, "Threads"))
		{
			client_args[client_num-1].thread_count = atoi(value);
            thread_num += client_args[client_num-1].thread_count; 
		}		
		if (!strcmp(key, "Requests"))
		{
			client_args[client_num-1].filenum = atoi(value);
		}		
		if (!strcmp(key, "Begin"))
		{
			client_args[client_num-1].beginchar = value[0];
		}		
		if (!strcmp(key, "End"))
		{
			client_args[client_num-1].endchar = value[0];
		}		
		if (!strcmp(key, "URLS"))
		{
			strcpy(client_args[client_num-1].filename,value);
        }
	}

    fprintf(stderr,"%d Client Types OK to Start\n",client_num);
    fprintf(stderr,"%d Client Threads OK to Start\n",thread_num);
}


int Frontend::Get_Servername(char* url, char* sname)
{
	if (url == NULL)
		return 0;
	char ft[MAXFEATURELEN];
	strncpy(ft, url, 1);
	ft[1] = '\0';
	for(int i=0; i<server_num; i++)
	{
		for(int j=0; j<server_list[i].featurenum; j++)
		{
			if (!strcmp(server_list[i].featurelist[j], ft))
			{
				strncpy(sname, server_list[i].hostname, MAXHOSTNAMELEN);
				return 1;
			}
		}
	}
	return 0;
}

void* One_Connection(void* arg)
{
    Arg_List* arglist = (Arg_List *)arg;	
    Frontend* fe;
	fe = arglist->fe;
    int existing_urls;
    Web_Client *client = new Web_Client(arglist->filenum,
                                        arglist->interval,
                                        arglist->id,
                                        arglist->beginchar,
                                        arglist->endchar,
                                        arglist->filename); 
	int ret = (int)malloc(sizeof(int));
    int notopen;
    char *url = NULL;
    if(arglist->id == 'p'){
        notopen = client->Init_Dat_File("r+");
    }else if(arglist->id == 'g'){
        notopen = client->Init_Dat_File("r");
    }
    if(notopen){
        fprintf(stderr,"Could Not Initialize Client!\n"); 
        pthread_exit(NULL);
    }
    
    pthread_mutex_lock(&(fe->lock));
    if(fe->client_args[arglist->group].existing_urls == 0){
       fe->client_args[arglist->group].existing_urls = client->url_num;
       existing_urls = client->url_num;
    }else{
       existing_urls = fe->client_args[arglist->group].existing_urls;
    }
    pthread_mutex_unlock(&(fe->lock));
   
    fprintf(stderr,"%d URLs existed in: %s\n!",existing_urls,fe->client_args[arglist->group].filename);
    
    //Get First URL
    url = client->First_URL();
    if(url == NULL){
        fprintf(stderr, "Error retreiving URL!\n");
        pthread_exit(NULL);
    }
    fprintf(stderr,"First Client URL is: %s\n",url);
    
    
    //Get the Server Name    
    char *sname = (char *)calloc(MAXHOSTNAMELEN,sizeof(char));
    ret = fe->Get_Servername(url, sname);
    if (ret == 0){
		fprintf(stderr, "Cannot Find the Server for URL=%s\n", url);
        pthread_exit(NULL);
    }else
		fprintf(stderr, "URL=%s Will Be Forwarded to Server=%s\n", url, sname);
    
    //Create Socket
    int sid = (int)malloc(sizeof(int));
    struct sockaddr_in servaddr; 
    struct hostent* ssaddr = (hostent*)malloc(sizeof(struct hostent));
    int dot_addr = (int)malloc(sizeof(int));
    struct in_addr addr;

    sid = socket(AF_INET, SOCK_STREAM, 0);
    if (sid < 0)
    {
        fprintf(stderr, "Cannot Crate Socket!\n");
        fflush(stderr);
        pthread_exit(NULL);
    }
    bzero(&servaddr, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons(SERVER_PORT);
    dot_addr = inet_aton(sname, &addr);
    if (!dot_addr)
    {
        ssaddr = gethostbyname(sname);
        if (ssaddr == NULL)
        {
            fprintf(stderr, "Server [%s] addressing error!\n",sname);
            fflush(stderr);
            pthread_exit(NULL);
        }
        memcpy(&addr, ssaddr->h_addr, sizeof(addr));
    }
    memcpy(&servaddr.sin_addr, &addr, sizeof(addr));
    if (connect_with_timeout(sid, (struct sockaddr *)&servaddr, sizeof(servaddr), 
CONNECT_TIMEOUT) <0)
    {
        fprintf(stderr, "Connection to Server [%s] Failed with Error: %s!\n", sname, strerror(errno));
        fflush(stderr);
        pthread_exit(NULL);
    }

    //Send the URLS
    if(arglist->id == 'p'){
        client->Put_All(sid);
        fprintf(stderr,"Client Finished Putting %d URLS\n",client->url_num-existing_urls);
        pthread_mutex_lock(&(fe->lock));
        fe->client_args[arglist->group].urls_put += (client->url_num-existing_urls);
        fprintf(stderr,"All Clients in Group: %d Put %d URLS\n", arglist->group, fe->client_args[arglist->group].urls_put);
        pthread_mutex_unlock(&(fe->lock));
      
    }else if(arglist->id == 'g'){
        client->Get_All(sid);
        fprintf(stderr,"Client Finished Getting %d URLS\n",arglist->filenum);
    }else{
        fprintf(stderr,"Client Identification Error.  What is a '%c' Client?\n",arglist->id);
    }

    fflush(stderr);
    free(arglist);
    close(sid);
    pthread_exit(NULL);
}

int main(int argc, char **argv)
{
    if(argc != 2){
        fprintf(stderr,"usage: ./frontend client_config.txt\n");
        return 1;
    }
	Frontend *fe = new Frontend(argv[1]);
    assert(fe);
    
	struct sockaddr_in clientaddr;
	int clientaddr_len;
    Arg_List* arglist;
    
    pthread_t tids[fe->thread_num];
    for(int i=0;i<fe->client_num;i++)
	{
		clientaddr_len = sizeof(clientaddr);
        pthread_attr_t tattr; 
        pthread_attr_init(&tattr);
        /* set detach state for each child thread */
        if (pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_JOINABLE) != 0)
        {
            fprintf(stderr, "Set Joinable State Failed!\n");
	        fflush(stderr);
        }
   
        for(int j=0;j<fe->client_args[i].thread_count;j++){
          
		    if ((arglist=(Arg_List *) malloc(sizeof(Arg_List))) == NULL)
		    {
			    fprintf(stderr, "Not enough memory, forgetting everything now.. !\n");
			    fflush(stderr);
			    return 0;
		    }
            
            arglist->filenum = fe->client_args[i].filenum;
            arglist->interval = fe->client_args[i].interval;
            arglist->id = fe->client_args[i].id;
            arglist->beginchar = fe->client_args[i].beginchar;
            arglist->endchar = fe->client_args[i].endchar;
            arglist->filename = (char *)malloc(sizeof(char)*100);
            strcpy(arglist->filename,fe->client_args[i].filename);
		    arglist->fe = fe;
            arglist->group = i;
               
            
		    if (pthread_create(&tids[(i+1)*(j+1)], &tattr, One_Connection, arglist) != 0)
		    {
			    fprintf(stderr, "Front end create thread failed, connection for this client is closed!\n");
			    fflush(stderr);
			    arglist = NULL;
                continue;
		    }else{
               //fprintf(stderr,"%c Client Starting... Sending %d Requests From %c to %c from file: %s\n",fe->client_args[i].id,fe->client_args[i].filenum,fe->client_args[i].beginchar,fe->client_args[i].endchar,fe->client_args[i].filename);
            
            }
        }
        pthread_attr_destroy(&tattr); 
    }
    
    int status = 0;
    for(int i=0;i<fe->client_num;i++){  
        for(int k=0; k < fe->client_args[i].thread_count;k++){
            int rc = pthread_join(tids[(i+1)*(k+1)], (void **)&status);
            if (rc){
                switch(rc){
                    case ESRCH:  
                    fprintf(stderr,"No thread could be found corresponding to that specified by th.\n");
                    break;
                    case EINVAL:
                    fprintf(stderr,"Another thread is already waiting on termination of th.\n");
                    break;
                    case EDEADLK:
                    fprintf(stderr,"The th argument refers to the calling thread.\n");
                    default:
                    fprintf(stderr,"Unknown error...!\n");
                }
            }
            fprintf(stderr,"Completed join with thread %d,%d status= %d\n",i,k, status);
	    }
        if(fe->client_args[i].id == 'p'){
            FILE *fh = fopen(fe->client_args[i].filename,"r+");
            if(fh==NULL){
                fprintf(stderr,"Could not open %s!\n",fe->client_args[i].filename);
            }
            One_Url oneurl;
            oneurl.valid = fe->client_args[i].urls_put + fe->client_args[i].existing_urls;
            rewind(fh);
            fwrite(&oneurl, sizeof(One_Url), 1, fh);
            fclose(fh);
            fprintf(stderr,"Committing %d URLs to the Existing %d\n",fe->client_args[i].urls_put, fe->client_args[i].existing_urls);
        }
    }

    return 0; 
}
