#include "socket_opt.h"

void * my_malloc(int size, pthread_mutex_t* lock)
{
	void* result;
	pthread_mutex_lock(lock);
	result = (void *) malloc(size);
	pthread_mutex_unlock(lock);
	return result;
}

int write_all(int sockfd, char *buf, int size)
{
	fd_set wfds;
	struct timeval tv;
	int ow, aw;
	aw = 0;
	for(;aw < size;)
	{
		FD_ZERO(&wfds);
		FD_SET(sockfd, &wfds);
		tv.tv_sec = WRITE_TIMEOUT;
		tv.tv_usec = 0;
		if (select(sockfd+1, NULL, &wfds, NULL, &tv))
		{
			ow = write(sockfd, buf+aw, size - aw);
			if (ow <= 0)
			{
				if (errno == EINTR)
					continue;
				else
				{
					fprintf(stderr, "Socket error or socket closed!\n");
					fflush(stderr);
					return -1;
				}
			}
			aw = aw + ow;
		}
		else
		{
			fprintf(stderr, "Write time out, about to exit!\n");
			fflush(stderr);
			return -1;
		}
	}
	return aw;
}

int write_msg(int sockfd, char *buf, int size)
{
	int ok;
	char slength[10];
	if ((buf == NULL) || (size == 0))
		return 0;
	sprintf(slength, "Length%d\n", size);
	ok = write_all(sockfd, slength, strlen(slength));
	if (ok <= 0)
		return -1;
	ok = write_all(sockfd, buf, size);
	return ok;
}

int get_char(int fd, char *ch)
{
	fd_set rfds;
	struct timeval tv;
	int size = 0;
	for(; size < 1;)
	{
		FD_ZERO(&rfds);
		FD_SET(fd, &rfds);
		tv.tv_sec = READ_TIMEOUT;
		tv.tv_usec = 0;
		if (select(fd+1, &rfds, NULL, NULL, &tv))
		{
			size = read(fd, ch, 1);
			if (size <= 0)
			{
				if (errno == EINTR)
				{
					size = 0;
					continue;
				}
				else
				{	
					fprintf(stderr, "Socket error or socket closed!\n");
					fflush(stderr);
					return(-1);
				}
			}
		}
		else
		{
			fprintf(stderr, "Read time out, about to exit!\n");
			fflush(stderr);
			return -1;
		}
	}
	return(size);
}

int read_line(int fd, char *buf, int maxlen)
{
	int i, n;
	char ch;
	for (i=0; i<maxlen; )
	{
		n = get_char(fd, &ch);
		if (n == 1)
		{
			buf[i++] = ch;
			if (ch == '\n')
				break;
		}
		else
			if (n < 0)
				return(-1);
			else
				break;
	}
	buf[i] = '\0';
	return(i);
}

int read_all(int sockfd, char *buf, int size)
{
	fd_set rfds;
	struct timeval tv;
	int tlen, olen;
	tlen = 0;
	for(; tlen < size;)
	{
		FD_ZERO(&rfds);
		FD_SET(sockfd, &rfds);
		tv.tv_sec = READ_TIMEOUT;
		tv.tv_usec = 0;
		if (select(sockfd+1, &rfds, NULL, NULL, &tv))
		{
			olen = read(sockfd, buf+tlen, size-tlen);
			if (olen <= 0)
			{
				if (errno == EINTR)
				{
					olen = 0;
					continue;
				}
				else
				{
					fprintf(stderr, "Socket error or socket closed!\n");
					fflush(stderr);
					return(-1);
				}
			}
			tlen = tlen + olen;
		}
		else
		{
			fprintf(stderr, "Read time out, about to exit!\n");
			fflush(stderr);
			return -1;
		}
	}			
	buf[tlen] = '\0';
	return(tlen);
}

int read_msg(int sockfd, char **buf)
{
	char inbuf[128];
	int n;
	int i;
	int msglen;
	
	*buf = NULL;
	i = read_line(sockfd, inbuf, 128);
	if (i <= 0)
		return(i);
	if (strncmp(inbuf, "Length", 6) == 0)
		sscanf(inbuf+6, "%d", &n);
	else
		return(0);
	*buf = (char *) malloc((n+1)*sizeof(char));
	if (*buf == NULL)
	{
		fprintf(stderr, "Not enough memory, malloc failed!\n");
		fflush(stderr);
		return 0;
	}
	msglen = read_all(sockfd, *buf, n);
	if (msglen <= 0)
		free(*buf);
	return msglen;
}

static void connect_timeout_handler(int sig)
{
	fprintf(stderr, "connect timeout!\n");
	fflush(stderr);
	return;
}

int connect_with_timeout(int sockfd, const struct sockaddr *serv_addr, socklen_t addr_len, int time)
{
	void (*oldhandler)(int);
	int retval;

	if (siginterrupt(SIGALRM, 1) < 0)
	{
		fprintf(stderr, "Signal Interrupt unavailable!\n");
		retval = connect(sockfd, serv_addr, addr_len);
		return retval;
	}
	oldhandler = signal(SIGALRM, connect_timeout_handler);
	alarm(time);
	retval = connect(sockfd, serv_addr, addr_len);
	alarm(0);
	signal(SIGALRM, oldhandler);
	return retval;
}
