#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "argdef.h"
#include "webclt.h"
#include "socket_opt.h"

Web_Client::Web_Client(int filenum, 
                       int interval, 
                       char id,
                       char beginchar,
                       char endchar, 
                       char *filename){

	this->filenum = filenum;
	this->interval = interval;
	this->beginchar = beginchar;
	this->endchar = endchar;
	this->filename = (char *)malloc(sizeof(char)*100);
	strcpy(this->filename,filename);
	this->putting = false;
	this->getting = false;
	if(id == 'p')
	   this->putting = true;
	if(id == 'g')
	   this->getting = true;	

	pthread_mutex_init(&flock, NULL);

}

int Web_Client::Init_Dat_File(char *mode)
{
	pthread_mutex_lock(&flock);
	
	FILE* fh;
	fh = fopen(filename, mode);
	if (fh == NULL && (strcmp(mode,"r+")==0))
	{
		fprintf(stderr, "Client File does not exist, creating it...\n");
		fclose(fopen(filename, "w"));
		fh = fopen(filename, "r+");
		if (fh == NULL)
		{
			fprintf(stderr, "Client File cannot exist!\n");
            pthread_mutex_unlock(&flock);
			return 1;
		}
	}else if(fh == NULL){
        pthread_mutex_unlock(&flock);
        return 1;
    }

	One_Url oneurl;
	if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
	{
		fprintf(stderr, "Client File was empty\n");
        if (strcmp(mode,"r")==0){
            pthread_mutex_unlock(&flock);
            return 1;
        }
		oneurl.valid = 0;
		url_num = 0;
		rewind(fh);
		fwrite(&oneurl, sizeof(One_Url), 1, fh);
	}
	else
		url_num = oneurl.valid;

    if(strcmp(mode,"r+")==0)    
        fseek(fh, 0L, SEEK_END); 
    fclose(fh);
    pthread_mutex_unlock(&flock);
    fprintf(stderr,"URL Num: %d\n",url_num);
    return 0;
}

One_Url Web_Client::Generate_URL()
{ 
	One_Url oneurl;
    int randnum;
	randnum = 1 + (int) (1.0*(endchar-beginchar+1)*rand()/(RAND_MAX+1.0));
	randnum += beginchar - 1;
	oneurl.url[0] = randnum;
	for(int j=1; j<URLLEN-1; j++)
	{
		randnum = 1 + (int) (1.0*('z'-'a'+1)*rand()/(RAND_MAX+1.0));
		randnum += 'a' - 1;
		oneurl.url[j] = randnum;
	}
	oneurl.url[URLLEN-1] = '\0';
    oneurl.valid = 1;
	return oneurl;		
}

One_Url Web_Client::Retreive_URL(){

    FILE *fh;
    int randnum;
    One_Url oneurl;
    oneurl.valid = 0;
    pthread_mutex_lock(&flock);        
    fh = fopen(filename,"r");
    
    while(!oneurl.valid){
        randnum = 1 + (rand() % (url_num-1));
        fseek(fh, (long) randnum*sizeof(One_Url), SEEK_SET);
        if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1){
            fprintf(stderr,"Cannot fread to position: %d, given: %d\n",randnum*sizeof(One_Url),randnum);
            oneurl.valid = 0;
            url_num = randnum;
        }else{
            //should look like: hjsxbadkkcayxnjozrq
            if((oneurl.url[0] == endchar || oneurl.url[0] == beginchar) &&
                strlen(oneurl.url) == 19){
                oneurl.valid = 1;
            }else{
                oneurl.valid = 0;
            }
        }
    }
    fclose(fh);
    pthread_mutex_unlock(&flock);
    return oneurl; 
}

char* Web_Client::First_URL(){

   One_Url oneurl;
   if(putting){
      oneurl = Generate_URL();
   }else if(getting){
      oneurl = Retreive_URL();
   }else{
      return NULL;
   }
   if(oneurl.valid){
      char *returnedurl = strdup(oneurl.url);
      return returnedurl;
   }else{
      return NULL;
   }   
}

void Web_Client::Put_All(int sid)
{ 
	FILE* fh;
    char* data;
    int datalen;
    char* response;
    int reslen;
    int ret;
    char *urlnum;
    urlnum = (char *)malloc(sizeof(char)*10);
	ret = write_msg(sid, "PUT", 3);
	if (ret <= 0)
	{
		fprintf(stderr, "Socket error! PUT File aborted...\n");
		return;
	}
	
	sprintf(urlnum, "%d\n", filenum);
	ret = write_msg(sid, urlnum, strlen(urlnum));	
	if (ret <= 0)
	{
		fprintf(stderr, "Client Socket Error, exit...\n");
		return;
	}
    free(urlnum);

	struct timeval rseed;
	gettimeofday(&rseed, NULL);
	srand((rseed.tv_sec+rseed.tv_usec) % 100000);
	totalrequests = 0;
	totalresponsetime = 0;
	totalbytesent = 0;
	One_Url oneurl;	
	
    fprintf(stderr,"Client about to Put %d Documents...\n",filenum);
    for(int i=0; i<filenum; i++)
	{
		oneurl = Generate_URL();
		struct timeval begintime;
		struct timeval endtime;
		//the timer starts here
		int timevalid;
		timevalid = 1;
		if (gettimeofday(&begintime, NULL) != 0)
			timevalid = 0;
		
		ret = write_msg(sid, oneurl.url, strlen(oneurl.url));	
		if (ret <= 0)
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		datalen = 1 + (int) (MAXFILESIZE*1.0*rand()/(RAND_MAX+1.0));
		datalen = 1024*datalen;
		data = (char *) malloc(datalen);
		ret = write_msg(sid, data, datalen);
		free(data);
		if (ret <= 0)
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		
		reslen = read_msg(sid, &response); 
		if (reslen <= 0) 
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		if (strcmp(response, "ERROR") == 0)
		{
			fprintf(stderr, "Put url=%s with size %d ERROR.\n", oneurl.url, datalen);
		}
		if (strcmp(response, "EXIST") == 0)
		{
			fprintf(stderr, "Put url=%s with size %d EXIST.\n", oneurl.url, datalen);
		}
		if (strcmp(response, "OK") == 0)
		{
			fprintf(stderr, "Put url=%s with size %d OK.\n", oneurl.url, datalen);
			oneurl.valid = 1;    
 			pthread_mutex_lock(&flock);
			fh = fopen(filename, "a");
            fwrite(&oneurl, sizeof(One_Url), 1, fh);
			fclose(fh);
			pthread_mutex_unlock(&flock);
			url_num ++;
			
			//timer ends here
			if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
			{
				totalrequests ++;
				totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
						             (endtime.tv_usec - begintime.tv_usec);
				totalbytesent += datalen;
			}
		}
		free(response);
		struct timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 1000*interval;
		select(0, NULL, NULL, NULL, &tv);
	}
   	Dump_Stats();
	return;
}

void Web_Client::Get_All(int sid)
{	
	char* data;
	int datalen;
	char* response;
	int reslen;
	int ret;
	char *urlnum;

    urlnum = (char *)malloc(sizeof(char)*10);
	ret = write_msg(sid, "GET", 3);
	if (ret <= 0)
	{
		fprintf(stderr, "Socket error! GET File aborted...\n");
		close(sid);
		return;
	}
	
	sprintf(urlnum, "%d\n", filenum);
	ret = write_msg(sid, urlnum, strlen(urlnum));	
	if (ret <= 0)
	{
		fprintf(stderr, "Client Socket Error, exit...\n");
		close(sid);
		return;
	}
    free(urlnum);
	struct timeval rseed;
	gettimeofday(&rseed, NULL);
	srand((rseed.tv_sec+rseed.tv_usec) % 100000);
	totalrequests = 0;
	totalresponsetime = 0;
	totalbytesent = 0;

    fprintf(stderr,"Client about to Get %d Documents...\n",filenum);    
	for(int i=0; i<filenum; i++)
	{
        One_Url oneurl = Retreive_URL();
		if (!oneurl.valid){
			i --;
			continue;
		}

		struct timeval begintime;
		struct timeval endtime;
		//the timer starts here
		int timevalid;
		timevalid = 1;
		if (gettimeofday(&begintime, NULL) != 0)
			timevalid = 0;
		
		ret = write_msg(sid, oneurl.url, strlen(oneurl.url));	
		if (ret <= 0)
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		reslen = read_msg(sid, &response); 
		if (reslen <= 0) 
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		if (strcmp(response, "ERROR") == 0)
		{
			fprintf(stderr, "Get url=%s on socket %d ERROR.\n", oneurl.url, sid);
		}
		if (strcmp(response, "NOTFOUND") == 0)
		{
			fprintf(stderr, "Get url=%s on socket %d NOT FOUND.\n", oneurl.url, sid);
			//timer ends here
			if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
			{
				totalrequests ++;
				totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
						             (endtime.tv_usec - begintime.tv_usec);
			}
		}
				
		if (strcmp(response, "OK") == 0)
		{	
			datalen = read_msg(sid, &data); 
			if (datalen <= 0) 
			{
				fprintf(stderr, "Socket with server %d Error, GET File aborted...\n", sid);
			}
			else
			{
				fprintf(stderr, "Get url=%s with size %d on socket %d OK.\n", oneurl.url, datalen, sid);
				free(data);
				//timer ends here
				if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
				{
					totalrequests ++;
					totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
							             (endtime.tv_usec - begintime.tv_usec);
					totalbytesent += datalen;
				}
			}
		}
		free(response);
		struct timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 1000*interval;
		select(0, NULL, NULL, NULL, &tv);
	}
   	Dump_Stats();
	return;
}

void Web_Client::Dump_Stats()
{	
	int averesponsetime;
	int throughput1;
	int throughput2;
	double temp;
	if(totalrequests != 0)
	    averesponsetime = (int) (totalresponsetime/totalrequests);
    else
        averesponsetime = -1;
    temp = (double) 1.0*totalresponsetime/1000000;
    if(temp != 0){
    	throughput1 = (int) (1.0*totalrequests/temp);
	    throughput2 = (int) (1.0*totalbytesent/temp);
    }else{
        throughput1 = -1;
        throughput2 = -1;
    }    
    fprintf(stdout, "------------------------------------------\n");
	fprintf(stdout, "total requests: %d\n", totalrequests);
	fprintf(stdout, "average response time is %d us\n", averesponsetime);
	fprintf(stdout, "average throughput(URL) is %d urls/second\n", throughput1);
	fprintf(stdout, "average throughput(bytes) is %d bytes/second\n", throughput2);
	fprintf(stdout, "------------------------------------------\n");
}
