# include <stdio.h>
# include "gsldistrib.h"

static struct entry {
	double key;
	double acc_size;
	long long size;
} table[100000];

static int tab_size=0;

int tab_cmp(const void *a, const void *b)
{
	double da=*(double *)a;
	double db=((struct entry *)b)->key;

	if (da>=db)
		return 1;

	double dc=(((struct entry *)b)-1)->key;

	if (da<dc)
		return -1;

	return 0;
}

int main(int argc, char *argv[])
{
	if (argc!=3)  {
		fprintf(stderr, "Usage: %s <distrib> <n_obj>\n", argv[0]);
		exit(-1);
	}

	FILE *fp=fopen(argv[1], "r");
	if (!fp) {
		fprintf(stderr, "Cannot open file: %s.\n", argv[1]);
		exit(-1);
	}

	int n_obj=atoi(argv[2]);
	long long tot_size=0;
	long long obj_size=0;

	for (; tab_size<100000; tab_size++) {
		char line[1024];
		if (fgets(line, 1024, fp)==NULL) {
			break;
		}
		sscanf(line+2, "%lf\t%lf\t%lld", &(table[tab_size].key), &(table[tab_size].acc_size), &(table[tab_size].size));
	}
	tab_size--;

	Distrib_flat distrib_gen(0.0, 1.0);
	for (int i=0; i<n_obj; i++) {
		double x=distrib_gen.get_one()*100;
		if (x<table[0].key) {
			obj_size=table[0].size;
		}
		else if (x>=table[tab_size].key) {
			obj_size=(long long)gsldd_flat(1, table[tab_size].size);
		}
		else {
			struct entry *p;
			if ((p=(struct entry *)bsearch(&x, table, tab_size+1, sizeof(struct entry), &tab_cmp))!=NULL) {
				obj_size=p->size;
			}
			else {
				fprintf(stderr, "panic, internal error.\n");
				exit(-1);
			}
		}
		obj_size/=20;
		printf("%.3f\t%lld\n", x, obj_size);
		tot_size+=obj_size;
	}

	printf("total size=%lld\n", tot_size);

	return 0;
}
