// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: loc_events.h,v $ - Hearder file defining all loc-related events.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _LOC_EVENTS_H_
# define _LOC_EVENTS_H_

# include "calevent.h"

// application specific event types.
enum {EVENT_ERROR, EVENT_REQUEST_ADD, 
	  EVENT_REQUEST_DEL, EVENT_REQUEST_LOOKUP, 
	  EVENT_BCAST, EVENT_INSTANT_CHECK};

class RequestEvent: public Event {
 private:
	// nothing

 public:
	enum {ADD=EVENT_REQUEST_ADD, 
		  DEL=EVENT_REQUEST_DEL, 
		  LOOKUP=EVENT_REQUEST_LOOKUP};
	RequestEvent(int type, double t) : 
		Event(type, t) {}

	virtual bool good();
	virtual void dump();
	static bool trans_type(char *type, int &mtype);
	static char * type_to_str(int mtype);
};

class BcastEvent: public Event {
 private:
	int sid; // server ID
 public:
	BcastEvent(int server_id, double t) :
		Event(EVENT_BCAST, t),
		sid(server_id) {}

	int get_sid() { return sid; }

	virtual bool good() { return true; }
	virtual void dump();
};

class InstantCheckEvent: public Event {
 public:
	InstantCheckEvent(double t) :
		Event(EVENT_INSTANT_CHECK, t) {}

	virtual bool good() { return true; }
	virtual void dump();
};

# endif
