// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: loc_requestgen.cc,v $ - Request Event Generator
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <ctype.h>
# include <stdio.h>
# include <string.h>
# include <math.h>
// # include <zlib.h>

# include "gsldistrib.h"
# include "loc_events.h"
# include "loc_requestgen.h"

RequestGen::RequestGen() :
	trace_gen(true),
	t_prev_event(0)
{
	// trace_zfp()=Z_NULL;
	trace_fp()=NULL;
}

RequestGen::RequestGen(char *path, double scale) :
	trace_gen(true),
	t_prev_event(0)
{
	// trace_zfp()=Z_NULL;
	trace_fp()=NULL;
	t_scale()=scale;
	lastet()=0;
	
	if (path) {
		// trace_zfp()=gzopen(path, "r");
		trace_fp()=fopen(path, "r");
	}
}

RequestGen::RequestGen(int stable_objs, int max_objects, double r_add_del, double r_lookup) :
	trace_gen(false),
	t_prev_event(0)
{
	stb_n_obj()=stable_objs;
	max_n_obj()=max_objects;
	// stb_r_update()=r_add_del;
	// stb_r_lookup()=r_lookup;
	lookup_pcent()=0;
	double sum=sum_rate()=r_add_del+r_lookup;
	if (sum>0) {
		lookup_pcent()=r_lookup/sum;
	}

}

bool RequestGen::good()
{
	if (trace_gen) {
		// return ((trace_zfp()!=Z_NULL)&&(t_scale()>0));
		return ((trace_fp()!=NULL)&&(t_scale()>0));
	}

	// return ((stb_n_obj()>0)&&(stb_r_update()>0)&&(stb_r_lookup()>0));
	return ( 
			(stb_n_obj()>0) &&
			(max_n_obj()>stb_n_obj()) &&
			(lookup_pcent()>0) &&
			(sum_rate()>0)
			);
}

RequestGen::~RequestGen()
{
	if (trace_gen) {
		fclose(trace_fp());
		// gzclose(trace_zfp());
	}
}

RequestEvent *RequestGen::generate_next(int n_cur_objs)
{
	if (n_cur_objs<0)
		return NULL;

	RequestEvent *e=NULL;
	if (trace_gen) {
		e=gen_next_trace();
	}
	else {
		e=gen_next_synth(n_cur_objs);
	}

	if (e)
		t_prev_event=e->time();

	return e;
}

// removing trailing comments
// then removing leading and trailing spaces
static char *trim(char *line)
{
	char *p=strchr(line, '#');
	if (p) {
		*p='\0';
	}
	else {
		p=line+strlen(line);
	}

	p--;

	while ((p>=line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	p=line;

	while ((*p)&&(isspace(*p))) {
		p++;
	}

	return p;
}

// assuming each line contains the following info:
// time-stamp, relative-time, event-name
RequestEvent *RequestGen::gen_next_trace()
{
	if (!good())
		return NULL;

# define METHOD_LEN 15
	static char fmt[20]="";
	if (!fmt[0]) {
		snprintf(fmt, 20, "%s%ds", "%e\t%e\t%", METHOD_LEN);
	}

	do {
		char line[1024];
		char *p;

		/**
		   if (gzgets(trace_zfp(), line, 1024)==NULL) {
		   return NULL;
		   }
		**/
		if (fgets(line, 1024, trace_fp())==NULL) {
			return NULL;
		}

		p=trim(line);
		if (*p=='\0') // skip comments.
			continue;

		float t1, t2;
		char method[METHOD_LEN+1];
		method[METHOD_LEN]='\0';
		if (sscanf(p, fmt, &t1, &t2, method)!=3) {
			// bad format
			tmpi_error(DBG_VERBOSE, "Bad trace format: less parameters than expected.\n");
			continue;
		}

		double delta=t2-lastet();
		if (delta<0) {
			tmpi_error(DBG_VERBOSE, "Bad trace format: event time backtrack.\n");
			continue;
		}

		delta*=t_scale();
		
		int mtype;
		if (!RequestEvent::trans_type(method, mtype)) {
			// bad method
			tmpi_error(DBG_VERBOSE, "Bad trace format: bad method %s.\n", method);
			continue;
		}

		RequestEvent *e= new RequestEvent(mtype, t_prev_event+delta);
		if (e)
			lastet()=t2;
		else 
			tmpi_error(DBG_INTERNAL, "out of memory\n");

		return e;
	} while (1);
# undef METHOD_LEN

	// never will be reached!
	tmpi_error(DBG_FATAL, "PANIC: unreachable code.\n");
	abort();

	return NULL;
}

RequestEvent *RequestGen::gen_next_synth(int n_cur_obj)
{
	if (!good())
		return NULL;

	double delta=gsldd_exponential(1/sum_rate());
	double probLookup=gsldd_flat(0, 1);

	if (probLookup<=lookup_pcent()) {
		// create a lookup event;
		return new RequestEvent(RequestEvent::LOOKUP, t_prev_event+delta);
	}
	else {
		// mixing del and add requests.
		int mtype=-1;
		if (n_cur_obj>=max_n_obj()) {
			mtype=RequestEvent::DEL;
		}
		else {
			int m=stb_n_obj()-n_cur_obj;
			int n=-m;
			if (m<0)
				m=1;
			else if (m>0) 
				n=1;
			else {
				m=n=1;
			}

			if (m>10)
				m=(int)log(m);
			if (n>10)
				n=(int)log(n);

			double probADDThreshold=(double)m/(n+m);
			double probADD=gsldd_flat(0, 1);
			if (probADD<=probADDThreshold) {
				mtype=RequestEvent::ADD;
			}
			else {
				mtype=RequestEvent::DEL;
			}
		}

		return new RequestEvent(mtype, t_prev_event+delta);
	}
}

void RequestGen::release_event(RequestEvent *e)
{
	delete e;
}

// If we cannot get trace duration, we will return 0;
double RequestGen::get_trace_duration()
{
	if (!trace_gen)
		return 0.0;
	
	// we assume the last line of the trace file contains
	// the statistical info about the trace.
	char last_line[1000];
	memset(last_line, 0, 1000);
	fseek(trace_fp(), -999, SEEK_END);
	fread(last_line, 1, 999, trace_fp());
	char *p=last_line+strlen(last_line);
	p--;
	while ((p>=last_line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	while ((p>=last_line)&&(*p!='\n')) {
		p--;
	}

	double ret=0.0;
	if (*p!='\n') {
		goto _exit;
	}

	p++;

	if (*p!='#')
		goto _exit;

	p=strstr(p, "TOTAL_TIME=");
	if (!p)
		goto _exit;
	
	float trace_duration;
	if (sscanf(p+strlen("TOTAL_TIME="), "%f", &trace_duration)<1)
		goto _exit;
	
	ret=trace_duration*t_scale();

 _exit:
	fseek(trace_fp(), 0, SEEK_SET);

	return ret;
}
