// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: loc_requestgen.h,v $ - Header File for Request Event Generator
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _LOC_REQUEST_GEN_H_
# define _LOC_REQUEST_GEN_H_

// # include <zlib.h>
# include <stdio.h>

class RequestEvent;

class RequestGen {
 private:
	union {
		//////////////////////////////////////////
		// parameters for trace-driven simulation
		struct {
			// gzFile trace_zfp; // trace file
			FILE *trace_fp; // trace file
			double t_scale;
			double lastet;
		} trace_par;
		struct {
			int stb_n_obj;
			int max_n_obj;
			// double stb_r_update;
			// double stb_r_lookup;
			double lookup_pcent;
			double sum_rate;
		} synth_par;
	} param;
	bool trace_gen; // true if use trace to generate events
	double t_prev_event; // the time of the previous event
	// gzFile &trace_zfp() { return param.trace_par.trace_zfp; }
	FILE *&trace_fp() { return param.trace_par.trace_fp; }
	double &t_scale() { return param.trace_par.t_scale; }
	double &lastet() { return param.trace_par.lastet; }
	int &stb_n_obj() { return param.synth_par.stb_n_obj; }
	int &max_n_obj() { return param.synth_par.max_n_obj; }
	// int &stb_r_update() { return param.synth_par.stb_r_update; }
	// int &stb_r_lookup() { return param.synth_par.stb_r_lookup; }
	double &lookup_pcent() { return param.synth_par.lookup_pcent; }
	double &sum_rate() { return param.synth_par.sum_rate; }
	RequestEvent *gen_next_trace();
	RequestEvent *gen_next_synth(int n_cur_objs);

 public:
	RequestGen();
	RequestGen(char *fp, double scale);
	RequestGen(int stable_objects, int max_objects, double r_add_del, double r_lookup);
	~RequestGen();
	bool good();
	RequestEvent *generate_next(int n_cur_objs);
	void release_event(RequestEvent *e);
	double get_trace_duration();
};

# endif
