// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: loc_sim.h,v $ - Header File for CBS Bloom Filter Object 
 *            Locationing Simulator
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _LOC_SIM_H_
# define _LOC_SIM_H_

# include "usc_time.h"

class Server;
class Client;
class God;
class EventChain;
class Event;
class RequestGen;

class Simulator {
 private:
	void *prog; // progress bar object
	// t: simulation duration, <=0 okay: meaning duration unknown.
	// stb_obj: # of objects when stablized
	// max_obj: absolute maximum of objects
	// init_obj: # of existing objects when the simulation starts.
	void init(double update_int, int stb_obj, int max_obj, int init_obj);
	void release();

 private:
	double sim_time;
	int dup_degree;
	int svr_num;
	usc_time_t t_start;
	usc_time_t t_previous;
	double sim_t_previous;
	void update_prog(double sim_t_now);
	void finalize_prog();
	bool handle_request(Event *);
	bool handle_bcast(Event *);
	bool handle_icheck(Event *);
	EventChain *sim_core;
	RequestGen *req_gen; // event generator

 public:
	God *global; 
	Client *client;
	Server **servers;
	double pkt_drop;
	int snapshot;

 public:
	Simulator(double t, double update_int, double r_add_del, double r_lookup, 
			  int ns, int degree, int stb_obj, int max_obj, int init_obj, 
			  double pkt_loss, int instant_dump);
	Simulator(char *tc_path, double scale, double update_int, 
			  int ns, int degree, int stb_obj, int max_obj, int init_obj, 
			  double pkt_loss, int instant_dump);
	~Simulator();
	bool good();
	bool start();
	bool run();
	int get_dup_degree() { return dup_degree; }
	int get_svr_num() { return svr_num; }
	bool add_object();
	void report();
	double get_sim_time() { return sim_time; }
};

# endif
