// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: test_stub.cc,v $ - micro benchmark stub.
 *            create a bounch of files, truncate them to 512MB.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/03 05:14:41 $ by $Author: jzhou $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <sys/types.h>
# include <sys/stat.h>
# include <stdio.h>
# include <signal.h>
# include <string>
# include <unistd.h>

extern int run(int argc, char *argv[]);

int format_root(char *root_dir)
{
	char name[1024];
	for (int i=0; i<10; i++) {
		snprintf(name, 1024, "%s/%d", root_dir, i);
		if (access(name, R_OK|W_OK|X_OK)) {
			if (mkdir(name, S_IRWXU)<0) {
				return -1;
			}
		}
		for (int j=0; j<10; j++) {
			snprintf(name, 1024, "%s/%d/%d", root_dir, i, j);
			if (access(name, R_OK|W_OK|X_OK)) {
				if (mkdir(name, S_IRWXU)<0) {
					return -1;
				}
			}
		}
	}

	return 0;
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
        printf("Usage: %s <test_root_dir> ...\n", argv[0]);
        exit(-1);
    }

	char pwd[1024];
	if (getcwd(pwd, 1024) == NULL) {
		printf("Cannot determine current directory.\n");
		exit(-1);
	}

	if (format_root(argv[1]) < 0) {
		printf("Cannot create directory structure under root directory: %s.\n", argv[1]);
		exit(-1);
	}

	if (chdir(argv[1]) < 0) {
		printf("Cannot change to the root directory: %s.\n", argv[1]);
		exit(-1);
	}


	int ret = run(argc-2, argv+2);
	chdir(pwd);

	return ret;
}
