// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: usc_time.c,v $ - High-resolution timer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/03 05:14:41 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "usc_time.h"

static double MHZ_RATE=(double)-1; 
static const double MHZ_RATE_DEFAULT=(double)400;
static long long USC_START=0;

static int mhz_rate_init() {
	double res=MHZ_RATE_DEFAULT, mhz;
	char line[256], *s, search_str[] = "cpu MHz";
	FILE *f;

	/* open proc/cpuinfo */
	if((f = fopen("/proc/cpuinfo", "r")) == NULL)
		return -1;

	/* ignore all lines until we reach MHz information */
	while(fgets(line, 256, f) != NULL){
		if(strstr(line, search_str) != NULL){
			/* ignore all characters in line up to : */
			for(s = line; *s && (*s != ':'); ++s);
			/* get MHz number */
			if(*s && (sscanf(s+1, "%lf", &mhz) == 1)){
				res = mhz;
				break;
			}
		}
	}

	fclose(f);

	if (MHZ_RATE==(double)-1)
		MHZ_RATE=res;

	return 0;
}

long long usc_gettime(void)
{
  unsigned int tmp[2];

  __asm__ ("rdtsc"
	   : "=a" (tmp[1]), "=d" (tmp[0])
	   : "c" (0x10) );
  
  return ( ((long long)tmp[0] << 32 | tmp[1]) ) - USC_START ;
}

double usc_difftime(long long time2, long long time1)
{
	if (MHZ_RATE == (double)-1) 
		mhz_rate_init();
	return (time2-time1)>0 ? (time2-time1)/MHZ_RATE : (time1-time2)/MHZ_RATE ;
}

double usc_wtime(long long t)
{
	return t/MHZ_RATE;
}

void usc_init(){
	mhz_rate_init();

	if (USC_START==(long long)0)
		USC_START=usc_gettime();
}

double usc_mhzrate()
{
	return MHZ_RATE;
}

void usc_tmr_reset(usc_tmr_t *timer)
{
	if (timer) {
		timer->elaps=0;
		timer->started=0;
	}
}

void usc_tmr_start(usc_tmr_t *timer)
{
	if (timer && (!timer->started)) {
		timer->started=1;
		timer->lstart=usc_gettime();
	}
}

void usc_tmr_stop(usc_tmr_t *timer)
{
	if (timer && (timer->started)) {
		timer->started=0;
		timer->elaps+=(usc_gettime()-timer->lstart);
	}
}

double usc_tmr_read(usc_tmr_t *timer)
{
	if (timer && (!timer->started)) {
		return usc_wtime(timer->elaps);
	}
	else if (timer && (timer->started)) {
		return usc_wtime(timer->elaps+(usc_gettime()-timer->lstart));
	}

	return 0.0;
}

