// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: usc_time.h,v $ - header file for high-resolution timer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/03 05:14:41 $
 * Version :  $Revision: 1.1 $
 *
 * Originally from TMPI project.
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _TSC_TIME_H_
# define _TSC_TIME_H_

# if defined(__cplusplus)
extern "C" {
# endif

typedef long long usc_time_t;

typedef struct {
	usc_time_t elaps;
	usc_time_t lstart;
	int started;
} usc_tmr_t;

/** this needs to be called first **/
void usc_init();

double usc_difftime(usc_time_t, usc_time_t);

/** get the time in us related to usc_init **/
double usc_wtime(usc_time_t);

/**
 * get a reference point, use difftime to get the
 * meaningful time
 **/
usc_time_t usc_gettime();

/**
 * return the cpu speed in MHZ
 */
double usc_mhzrate();

void usc_tmr_reset(usc_tmr_t *timer);
void usc_tmr_start(usc_tmr_t *timer);
void usc_tmr_stop(usc_tmr_t *timer);
double usc_tmr_read(usc_tmr_t *timer);

# if defined(__cplusplus)
}
# endif

# endif
