// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: mb_nfs.cc,v $ - micro benchmark.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/07/14 18:26:08 $ by $Author: lkchu $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <getopt.h>
# include <stdio.h>
# include <signal.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <stdlib.h>
# include <errno.h>
# include <string.h>


# include "usc_time.h"
# include "tmpi_debug.h"

# define _XOPEN_SOURCE 500
# include <unistd.h>
extern "C" ssize_t pwrite(int fd, const void *buf, size_t count, off_t offset);
extern "C" ssize_t pread(int fd, void *buf, size_t count, off_t offset);

using namespace std;

static char *program_name=NULL;

/* range of files that will be read */
int cid=1;
int rmode = 0;

static void usage(int stat){

	printf("\nMicrobenchmark 3:\nThis bencmark writes at random offsets. microbenchmark 2 should be run before to populate the file system. \nParameters:\n  f: Configuration file.\n  u: Upper bound on the file names.\n  n: Number of write operations to be performed.\nLast argument needs to the the deployment ID.\n");
	exit(stat);
}

static void parse_cmd_args_and_init(int argc, char *argv[])
{
	static struct option const long_options[] = {
		{"upper", required_argument, 0, 'u'},
		{"ReadMode", required_argument, 0, 'r'},
		{"ClientID", required_argument, 0, 'n'},
		{"silent", no_argument, 0, 's'},
		{"debug", no_argument, 0, 'd'},
		{"help", no_argument, 0, 'h'},
		{"version", no_argument, 0, 'V'},
		{NULL, 0, 0, 0}
	};

	int optc;

	program_name=argv[0];

	while ((optc=getopt_long(argc, argv, \
					"ru:n:sdhV", long_options, NULL)) !=-1) {
		switch (optc) {
		case 0:
			break;

		case 'r':
			rmode = 1;
			break;
		case 'n':
			cid = atoi(optarg);
			break;
		
		case 's':
			tmpi_set_dbg_level(DBG_FATAL-1);
			tmpi_hide_location();
			break;
		case 'd':
			tmpi_set_dbg_level(DBG_ALL);
			tmpi_show_location();
			break;
		case 'V':
			exit(0);
			break;
		case 'h':
			usage(0);
			break;
		default:
			usage(0);
			;
		}
	}

	return;

_exit:
	exit(-1);
}

static void output_error(const char *cmd, bool do_output)
{
	if (do_output)
		printf("%s failed: %s.\n", cmd, strerror(errno));
}



/**
 *
 */
int main(int argc, char *argv[])
{
	signal(SIGPIPE, SIG_IGN);
	const int blockSize = 4*1024*1024;
	char *buf;

	tmpi_set_dbg_level(DBG_ALL);
	tmpi_show_location();

	bool do_output=true;
	bool do_timing=true;

	char objName[16];

	int i, offset;

	// might be no return.
	parse_cmd_args_and_init(argc, argv);

	usc_time_t t_start=0, t_end=0;
	int cmd_cnt=0;
	int err_cnt=0;

	int psf;

	buf = new char[blockSize];

	/* Start measurement */
	if (do_timing) {
		t_start=usc_gettime();
	}

	sprintf(objName, "%ld", cid);

	if (rmode)
		psf = open(objName,  O_RDONLY);
	else
		psf = open(objName,  O_CREAT|O_WRONLY, S_IRWXU);

		if (psf < 0) {
			printf("ERROR: Open (%s) : ", objName);
			output_error("Create", do_output);
			err_cnt++;
			exit(-1);
		}

		for(int j=0; j<64; j++){
		    offset = (rand()%131072)*4*1024;
		    if (rmode) {
			if (pread(psf,buf,blockSize,offset) < 0)
			{
				printf("ERROR: Read (%s) : ", objName);
				output_error("Read", do_output);
				err_cnt++;
			}
		    } else {
			if (pwrite(psf,buf,blockSize,offset) < 0)
			{
				printf("ERROR: Write (%s) : ", objName);
				output_error("Write", do_output);
				err_cnt++;
			}
		    }
		    cmd_cnt++;
		}

		/* close */
		if (!rmode) fsync(psf);
		if (close(psf) < 0) {
			printf("ERROR: close\n");
			output_error("Close", do_output);
				err_cnt++;
		}
		

	/* end measurement */

	if (do_timing) {
		t_end=usc_gettime();
		double t = usc_difftime(t_end, t_start)*1e-6;
		double mbp = (double)(cmd_cnt * blockSize) / (1024 * 1024 * t);
//		printf("Total %d commands executed, execution time %.3fms.(with %d errors)\n", 
//			   cmd_cnt, usc_difftime(t_end, t_start)/1000, err_cnt);
		printf("MBP = %.3f\n", mbp);
//		fprintf(stderr, "%.3f\n", usc_difftime(t_end, t_start)/1000);
	}

	return 0;
}
