// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: minimal.cc,v $ - micro benchmark.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/09/14 22:53:11 $ by $Author: gulbeden $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include <signal.h>

# include "sor_client.h"
# include "sor_fs.h"
# include "sor_file.h"
# include "subscriber.h"

using namespace std;

static Subscriber *sub=NULL;
static SorClient *scli=NULL;

static void output_error(const char *cmd){
	printf("%s failed: %s.\n", cmd, scli->str_error().c_str());
}

int main(int argc, char *argv[]){

	const int blockSize = 4;
	char buf[blockSize+1] = "ABCD";
	char objName[16] = "minimal.tet";

	if (argc < 3){
		printf("Usage %s <Conf-File> <DeploymentID>\n", argv[0]);
		printf("e.g. %s Sorrento.conf Mayo\n", argv[0]);
		return -1;
	}
	
	char *config=argv[1];
	char *ident = argv[2];
	SorFile *psf;
	off_seg_t written_len;

	signal(SIGPIPE, SIG_IGN);

	SorrentoConf sconf(config);
	sub = new Subscriber(sconf);
	sub->start();
	printf("Starting the proxy, please wait ...\n");
	sleep(3); // wait for 3 seconds.
	SorFS sorfs(sub, string(ident));
	scli = new SorClient(&sorfs);

   	psf=scli->create(objName);
	
	if (psf == NULL){
		output_error("Create");
		return -1;
	}
	
	if(scli->write(psf, 0, buf, blockSize, written_len)==false)
		output_error("Write");
	
	if (scli->close(psf) == false)
		output_error("Close");
		
	sub->stop();
	sub->finalize();
	delete sub;
	delete scli;
	return 0;
}
