// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: rem_exec_cli.cc,v $ - spawns programs on different
 *            cluster nodes.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/07/09 20:27:36 $ by $Author: bill $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/wait.h>

#include "../../shared/sock/sock_dg.h"
#include "../../shared/sock/sock_conn.h"

void genMCastAddr(int seed, char* mCastAddr, int &mCastPort);

using namespace std;

int main(int argc, char **argv){
	int pid, status;
	int numHosts;
	int i, seed = 0;
	FILE *fin;
	char mCastAddr[32];
	int mCastPort;
	char *command[32];

	if (argc < 2){
		fin = fopen("./exprhosts", "r");
		printf("usage %s [expr_hosts_file]\nReading from ./exprhosts\n",
			   argv[0]);
		seed = 'e'+'x'+'p'+'r'+'h'+'o'+'s'+'t'+'s';
	}

	else{
		fin = fopen(argv[1], "r");
		i=0;
		while(argv[1][i]!='\0')
			seed += (char)(argv[1][i++]);
	}

	if (fin == NULL){
		printf("No exprhosts file. Quiting\n");
		exit(-1);
	}
	
	if (fscanf(fin, "%d", &numHosts)<=0){
		printf("Format error in exprhosts file. Quiting\n");
		exit(-1);
	}

	//printf("Number of hosts is %d\n", numHosts);

	genMCastAddr(seed, mCastAddr, mCastPort);
	
	// non-changing parts of the command
	command[0] = "ssh";			// command name
	command[2]= "~/cbs/src/expr/remexec/rsorcli"; // remote program
	command[3] = mCastAddr;		// multicast address
	command[4] = new char[16];	
	sprintf(command[4], "%d", mCastPort); // multicast port
	
	for(i=0; i<numHosts; i++){

		command[1] = new char[256];
		fscanf(fin, "%s", command[1]); // host name

		// generate the command that rsorcli will execute
		int j = 5;
		char delim;
		do{
			command[j] = new char[1024];
			fscanf(fin, "%s", command[j]);
			j++;
			delim = fgetc(fin);
		} while (delim!='\n' && delim!=EOF);
		command[j] = NULL;

		// execute
		pid = fork();
		if (pid==0) {
			execvp(*command, command);
			perror("Error:\n");
			exit(1);
		}
	}
	
	// wait
	if (pid!=0) {
		wait(&status);
	}
}


void genMCastAddr(int seed, char* mCastAddr, int &mCastPort){
 	sprintf(mCastAddr, "234.%d.%d.%d", abs((seed*1234)%256), 
 			abs((seed*6784)%256), abs((seed*13434)%256));
	
 	mCastPort = abs(2000 + (seed * 4542)%40000);
}
