// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: rem_sor_cli.cc,v $ - waits for the start message to be
 *            sent to the multicast port, then runs the program. Not intended
 *            to be executed by the user, rexeccli runs this program.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/07/09 20:27:36 $ by $Author: bill $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

# include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>

#include "../../shared/sock/sock_dg.h"
#include "../../shared/sock/sock_conn.h"

using namespace std;

int main(int argc, char ** argv){
	int mcastSock;
	int bufLen = 2;
	char readBuf[bufLen];
	int writeSock;// = sock_udp(NULL, SOCK_PORT_ANY);
	char writeBuf[2] = "r";
	int maxIter = 1000, i;
	char **args = new char*[32];

	char *mCastAddr;
	int mCastPort;

	if (argc < 3){
		printf("Invalid number of arguments.\nUsage: %s <mcast addr> <mcast port> <command>...\nNote: This program should not be executed by the user.\n", argv[0]);
		exit(1);
	}
	mCastAddr = argv[1];
	mCastPort = atoi(argv[2]);
	
	for(i=3; i<argc; i++){
		args[i-3] = new char[128];
		args[i-3] = argv[i];
	}
	args[argc] = NULL;

	mcastSock = sock_mcast(mCastAddr, mCastPort, mCastAddr, NULL, NULL, 1);

    if (mcastSock<0)
        exit(1);

	writeSock = sock_mcast(NULL, SOCK_PORT_ANY, NULL, NULL, NULL, 1);

	if (writeSock <=0) {
		fprintf(stderr, "Unable to create write sock\n");
		exit(-1);
	}

	sock_writeto(writeSock, mCastAddr, mCastPort, writeBuf, &bufLen);
	
	i=0;

	// wait for start message
	do{
		sock_readfrom(mcastSock, NULL, 0, NULL, readBuf, &bufLen, 60000);
		i++;
	} while (readBuf[0] != 's' && i < maxIter);
	
	if (i >= maxIter && readBuf[0] != 's') exit(1); //give up waiting
	
	/* now start the executable */
	int pid;
	int status;
	
	pid = fork();
	if (pid==0) {
		execvp(*args, args);
		perror("Error:\n");
		exit(1);
	}
	
	if (pid!=0) {
		wait(&status);
	}
}
