#!/usr/bin/perl -w

use strict;
use FileHandle;

my $dryrun = 0;

if ($#ARGV < 1) {
	print <<EOF;
Usage: syncrun.pl [-n] <hostlist> <cmd> <args>
	<hostlist> contains a list of hosts to run the job.
	-n where n is an integer, to set the upper bound of nodes.
	<cmd> the executable
	<args> arguments
EOF
	exit(-1);
}

my $max_nodes=1000;
if ($ARGV[0] =~ /^-(\d+)$/) {
	$max_nodes=$1;
	shift(@ARGV);
}

if ($#ARGV < 1) {
	print <<EOF;
Usage: syncrun.pl [-n] <hostlist> <cmd> <args>
	<hostlist> contains a list of hosts to run the job.
	-n where n is an integer, to set the upper bound of nodes.
	<cmd> the executable
	<args> arguments
EOF
	exit(-1);
}

my $exe_rstartcli="./rstartcli";
my $exe_rexeccli="./rexeccli";

if ((!-e $exe_rstartcli) || (!-e $exe_rexeccli)) {
	print "Dependent files missing, check the existence of $exe_rstartcli and $exe_rexeccli.\n";
	exit(-1);
}

my $fh = new FileHandle($ARGV[0], "<");
if (!defined($fh)) {
	print "Failed to open file: $ARGV[1]\n";
	exit(-1);
}

my @hosts= ();

while (my $host = <$fh>) {
	chomp($host);
    $host =~ s/#.*$//;
    if ($host =~ /^\s*$/) {
        next;
    }
	push(@hosts, $host);
	# print "Host $host added.\n";
}

my $pwd=`pwd`;
chomp($pwd);
my $abscmd=`which $ARGV[1]`;
chomp($abscmd);

if ($abscmd =~ /^\//) {
	;
}
elsif ($abscmd =~ /^\./) {
	$abscmd=$pwd."/".$abscmd;
}
elsif ($abscmd =~ /alias/) {
	print "I don't know how to resolve the absolute path of $ARGV[0] because it is aliased.\n";
	exit(-1);
}

my $cmd=$abscmd;

for (my $i=2; $i<=$#ARGV; $i++) {
    $cmd = $cmd." $ARGV[$i]";
}

my $tmpname=`mktemp job.XXXXXX`;
chomp($tmpname);

my $fh_job = new FileHandle($tmpname, ">>");
my $n_nodes = $max_nodes;
if ($n_nodes > ($#hosts +1)) {
	$n_nodes = $#hosts +1;
}
print $fh_job $n_nodes, "\n";

foreach my $host (@hosts) {
	print $fh_job "$host $cmd\n";
	print "$host $cmd\n";
}

$fh_job->close();

my $cmd_line = "$exe_rstartcli $tmpname &";
print $cmd_line, "\n";
if (!$dryrun) {
	system($cmd_line);
}

$cmd_line = "sleep(2)";
print $cmd_line, "\n";
if (!$dryrun) {
	sleep(2);
}

$cmd_line = "$exe_rexeccli $tmpname";
print $cmd_line, "\n";
if (!$dryrun) {
	system($cmd_line);
}

if ($dryrun) {
	print "======Content of file $tmpname :\n";
	system("cat $tmpname");
	print "======End of file $tmpname :\n";
}

unlink($tmpname);
