// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sprite.h,v $ - header file for converted Sprite File System
 *            Trace parser.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:51 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

#ifndef _SPRITE_H_
#define _SPRITE_H_

# include <sys/time.h>
# include <stdio.h>
# include <string.h>
# include <zlib.h>

struct IDType {
	int p[4];
};

class SpriteRecord {
 public:
	enum OP {
		OP_START,
		OPEN, 
		UNLINK, 
		CREATE, 
		SEEK,
		CLOSE,
		TRUNCATE,
		WRITE,
		READ,
		OP_END
	};

 protected:
	enum OP type;
	struct timeval tv;

 public:
	SpriteRecord(void *buf, enum OP t=OP_START);

	int get_op() { return type; }

	bool get_time(struct timeval &tv) 
	{ 
		if (!is_good_op(type))
			return false; 

		tv=this->tv;
		return true;
	}

	virtual bool get_fid(IDType &fid) { return false; }
	virtual bool get_sid(IDType &sid ) { return false; }
	virtual bool get_mode(int &m) { return false; }
	virtual bool get_start(int &s) { return false; }
	virtual bool get_size(int &s) { return false; }
	virtual bool get_offset(int &off) { return false; }

	virtual void dump(FILE *fp=NULL) { printf("Base Record.\n"); }

	static bool is_good_id(IDType &id);
	static char *op_str(int op) {
		struct {
			int op;
			char *name;
		} *p, op_tab[] = {
			{ OP_START, "ERROR" },
			{ OPEN, "OPEN" },
			{ UNLINK, "UNLINK" },
			{ CREATE, "CREATE" },
			{ SEEK, "SEEK" },
			{ CLOSE, "CLOSE" },
			{ TRUNCATE, "TRUNCATE" },
			{ WRITE, "WRITE" },
			{ READ, "READ" },
			{ OP_END, "ERROR" },
			{ OP_END+1, NULL },
		};

		if (!is_good_op(op)) {
			return "ERROR";
		}

		p=op_tab;
		for (; p->name; p++) {
			if (op==p->op)
				return p->name;
		}

		return "ERROR";
	}

	static bool is_good_op(int op) 
	{ if ((op>=OP_END)||(op<=OP_START)) return false; return true; }
};

class SpriteParser {
	private:
		gzFile fp;
		struct timeval start_tv;
		struct timeval last_tv;
		bool first_record;
		int counters[SpriteRecord::OP_END];
		int nrec;
		
	public:
		SpriteParser(char *trace) : 
			first_record(true),
			nrec(0) 
		{
			fp=gzopen(trace, "rb"); 
			memset(counters, 0, sizeof(int)*SpriteRecord::OP_END);
		}

		~SpriteParser() { if (fp) { gzclose(fp); } }

		SpriteRecord *read_record();
		void free_record(SpriteRecord *&r);

		bool start_time(struct timeval &tv) {
			if (first_record)
				return false;
			tv=start_tv;
			return true;
		}

		bool end_time(struct timeval &tv) {
			if (first_record)
				return false;
			tv=last_tv;
			return true;
		}

		bool total_time(double &seconds) {
			struct timeval st, et;
			if (start_time(st) && end_time(et)) {
				if (et.tv_usec<st.tv_usec) {
					et.tv_usec+=1000000;
					et.tv_sec--;
				}
				et.tv_sec-=st.tv_sec;
				et.tv_usec-=st.tv_usec;
				seconds=(double)et.tv_sec+(double)et.tv_usec/1000000.0;
				return true;
			}
			return false;
		}

		void report(FILE *fp=NULL); 
};

# endif
