/**********************************************************************************
Copyright (c) 2003 The Regents of the University of California. 
All rights reserved.

Netptune project contact: Lingkun Chu (lkchu@cs.ucsb.edu),
                          Tao Yang (tyang@cs.ucsb.edu),
                          Kai Shen (kshen@cs.rochester.edu)
Web site: http://www.cs.ucsb.edu/projects/neptune
Department of Computer Science, UC Santa Barbara.

This software is distributed to individuals for personal non-commercial 
use and to non-profit entities for non-commercial purposes only. Permission 
is also granted to commercial enterprises to use this software for evaluation 
purposes only. It is licensed on a non-exclusive basis, free of charge for 
these uses. The license is subject to the following conditions:

1. Redistribution is not permitted. UC Santa Barbara will maintain a copy 
   of this software as a directly downloadable file, available under the terms 
   of license specified in this agreement.
2. All advertising materials mentioning features or use of this software must 
   display the following acknowledgment: "This product includes software 
   developed by the University of California, Santa Barbara." 
3. Neither the name of the University nor the names of its contributors may 
   be used to endorse or promote products derived from this software without 
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
**********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "argdef.h"
#include "socket_opt.h"

typedef struct
{
	char url[URLLEN];
	int valid;
}One_Url;

class Del_Client
{
	int url_num;
	public:
	char filename[MAXFILENAMLEN];

	public:
	//interval in millisecond
	void Delete_File_Server(char* server, int filenum, int interval);
	void Delete_File_Frontend(char* frontend, int filenum, int interval);
};
void Del_Client::Delete_File_Frontend(char* frontend, int filenum, int interval)
{
	int sid;
	struct sockaddr_in servaddr;
	struct hostent* ssaddr;
	int dot_addr;
	struct in_addr addr;
	sid = socket(AF_INET, SOCK_STREAM, 0);
	if (sid < 0)
	{
		fprintf(stderr, "Client creats socket error!\n");
		fflush(stderr);
		return;
	}
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(FRONT_PORT);
	dot_addr = inet_aton(frontend, &addr);
	if (!dot_addr)
	{
		ssaddr = gethostbyname(frontend);
		if (ssaddr == NULL)
		{
			fprintf(stderr, "The frontend address error, exit!\n");
			fflush(stderr);
			return;
		}
		memcpy(&addr, ssaddr->h_addr, sizeof(addr));
	}
	memcpy(&servaddr.sin_addr, &addr, sizeof(addr));
	if (connect_with_timeout(sid, (struct sockaddr *)&servaddr, sizeof(servaddr), CONNECT_TIMEOUT) <0)
	{
		fprintf(stderr, "Connect to the frontend error: %s, exit!\n", strerror(errno));
		fflush(stderr);
		return;
	}

	FILE* fh;
	fh = fopen(filename, "r+");
	if (fh == NULL)
	{
		fprintf(stderr, "Client File not exist!\n");
		close(sid);
		return;
	}
	One_Url oneurl;
	if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
	{
		fprintf(stderr, "Client File is empty! Save some Files first...\n");
		fclose(fh);
		close(sid);
		return;
	}
	else
		url_num = oneurl.valid;
	
	int randnum;
	char* response;
	int reslen;
	int ret;
	char urlnum[10];

	char sname[MAXHOSTNAMELEN];

	struct timeval rseed;
	gettimeofday(&rseed, NULL);
	srand((rseed.tv_sec+rseed.tv_usec) % 100000);
	
	sprintf(urlnum, "%d\n", filenum);
	ret = write_msg(sid, urlnum, strlen(urlnum));	
	if (ret <= 0)
	{
		fprintf(stderr, "Client Socket Error, exit...\n");
		fclose(fh);
		close(sid);
		return;
	}

	for(int i=0; i<filenum; i++)
	{
		randnum = 1 + (int) (url_num*1.0*rand()/(RAND_MAX+1.0));
		fseek(fh, (long) randnum*sizeof(One_Url), SEEK_SET);
		if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
			continue;
		if (oneurl.valid != 1)
		{
			i --;
			continue;
		}
	
		ret = write_msg(sid, oneurl.url, strlen(oneurl.url));	
		if (ret <= 0)
		{
			fprintf(stderr, "Socket with frontend Error, exit...\n");
			break;
		}
		reslen = read_msg(sid, &response);
		if (reslen <= 0) 
		{
			fprintf(stderr, "Socket with frontend Error, exit...\n");
			break;
		}
		if (strcmp(response, "NOTFOUND") == 0)
		{
			fprintf(stderr, "Cannot find the server for url=%s\n", oneurl.url);
			free(response);
		}
		else
		{
			strncpy(sname, response, MAXHOSTNAMELEN);
			free(response);
		
			int sids;
			struct sockaddr_in servaddrs;
			struct hostent* ssaddrs;
			int dot_addrs;
			struct in_addr addrs;
			sids = socket(AF_INET, SOCK_STREAM, 0);
			if (sids < 0)
			{
				fprintf(stderr, "Client creats socket with the server %s error!\n", sname);
				fflush(stderr);
				continue;
			}
			bzero(&servaddrs, sizeof(servaddrs));
			servaddrs.sin_family = AF_INET;
			servaddrs.sin_port = htons(SERVER_PORT);
			dot_addrs = inet_aton(sname, &addrs);
			if (!dot_addrs)
			{
				ssaddrs = gethostbyname(sname);
				if (ssaddrs == NULL)
				{
					fprintf(stderr, "The server %s address error, exit!\n", sname);
					fflush(stderr);
					close(sids);
					continue;
				}
				memcpy(&addrs, ssaddrs->h_addr, sizeof(addrs));
			}
			memcpy(&servaddrs.sin_addr, &addrs, sizeof(addrs));
			if (connect_with_timeout(sids, (struct sockaddr *)&servaddrs, sizeof(servaddrs), CONNECT_TIMEOUT) <0)
			{
				fprintf(stderr, "Connect to the server %s error: %s, exit!\n", sname, strerror(errno));
				fflush(stderr);
				close(sids);
				continue;
			}
			
			ret = write_msg(sids, "DEL", 3);
			if (ret <= 0)
			{
				fprintf(stderr, "Socket with server %s error! DEL File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				continue;
			}
			
			sprintf(urlnum, "1\n");
			ret = write_msg(sids, urlnum, strlen(urlnum));	
			if (ret <= 0)
			{
				fprintf(stderr, "Socket with server %s Error, DEL File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				continue;
			}
			
			ret = write_msg(sids, oneurl.url, strlen(oneurl.url));	
			if (ret <= 0)
			{
				fprintf(stderr, "Socket with server %s Error, DEL File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				break;
			}
			reslen = read_msg(sids, &response); 
			if (reslen <= 0) 
			{
				fprintf(stderr, "Socket with server %s  Error, DEL File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				break;
			}
			if (strcmp(response, "ERROR") == 0)
			{
				fprintf(stderr, "Del url=%s ERROR.\n", oneurl.url);
			}
			if (strcmp(response, "NOTFOUND") == 0)
			{
				fprintf(stderr, "Del url=%s Not Found.\n", oneurl.url);
			}
			if (strcmp(response, "OK") == 0)
			{
				fprintf(stderr, "Del url=%s OK.\n", oneurl.url);
				oneurl.valid = 0;
				fseek(fh,(long) -1*sizeof(One_Url), SEEK_CUR);
				fwrite(&oneurl, sizeof(One_Url), 1, fh);
				fflush(fh);
			}
			free(response);
			close(sids);
		}
		
		struct timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 1000*interval;
		select(0, NULL, NULL, NULL, &tv);
	}
	
	fclose(fh);
	close(sid);
	return;
}

void Del_Client::Delete_File_Server(char* server, int filenum, int interval)
{
	int sid;
	struct sockaddr_in servaddr;
	struct hostent* ssaddr;
	int dot_addr;
	struct in_addr addr;
	sid = socket(AF_INET, SOCK_STREAM, 0);
	if (sid < 0)
	{
		fprintf(stderr, "Client creats socket error!\n");
		fflush(stderr);
		return;
	}
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(SERVER_PORT);
	dot_addr = inet_aton(server, &addr);
	if (!dot_addr)
	{
		ssaddr = gethostbyname(server);
		if (ssaddr == NULL)
		{
			fprintf(stderr, "The server address error, exit!\n");
			fflush(stderr);
			return;
		}
		memcpy(&addr, ssaddr->h_addr, sizeof(addr));
	}
	memcpy(&servaddr.sin_addr, &addr, sizeof(addr));
	if (connect_with_timeout(sid, (struct sockaddr *)&servaddr, sizeof(servaddr), CONNECT_TIMEOUT) <0)
	{
		fprintf(stderr, "Connect to the server error: %s, exit!\n", strerror(errno));
		fflush(stderr);
		return;
	}
	
	FILE* fh;
	fh = fopen(filename, "r+");
	if (fh == NULL)
	{
		fprintf(stderr, "Client File not exist!\n");
		return;
	}
	One_Url oneurl;
	if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
	{
		fprintf(stderr, "Client File is empty! Save some Files first...\n");
		fclose(fh);
		return;
	}
	else
		url_num = oneurl.valid;
	
	int randnum;
	char* response;
	int reslen;
	int ret;
	char urlnum[10];

	ret = write_msg(sid, "DEL", 3);
	if (ret <= 0)
	{
		fprintf(stderr, "Socket error! DEL File aborted...\n");
		fclose(fh);
		close(sid);
		return;
	}
	sprintf(urlnum, "%d\n", filenum);
	ret = write_msg(sid, urlnum, strlen(urlnum));	
	if (ret <= 0)
	{
		fprintf(stderr, "Client Socket Error, exit...\n");
		fclose(fh);
		close(sid);
		return;
	}
	struct timeval rseed;
	gettimeofday(&rseed, NULL);
	srand((rseed.tv_sec+rseed.tv_usec) % 100000);
	for(int i=0; i<filenum; i++)
	{
		randnum = 1 + (int) (url_num*1.0*rand()/(RAND_MAX+1.0));
		fseek(fh, (long) randnum*sizeof(One_Url), SEEK_SET);
		if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
			continue;
		if (oneurl.valid != 1)
		{
			i --;
			continue;
		}
		ret = write_msg(sid, oneurl.url, strlen(oneurl.url));	
		if (ret <= 0)
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		
		reslen = read_msg(sid, &response); 
		if (reslen <= 0) 
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		if (strcmp(response, "ERROR") == 0)
		{
			fprintf(stderr, "Del url=%s ERROR.\n", oneurl.url);
		}
		if (strcmp(response, "NOTFOUND") == 0)
		{
			fprintf(stderr, "Del url=%s Not Found.\n", oneurl.url);
		}
		if (strcmp(response, "OK") == 0)
		{
			fprintf(stderr, "Del url=%s OK.\n", oneurl.url);
			oneurl.valid = 0;
			fseek(fh,(long) -1*sizeof(One_Url), SEEK_CUR);
			fwrite(&oneurl, sizeof(One_Url), 1, fh);
			fflush(fh);
		}
		free(response);
		struct timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 1000*interval;
		select(0, NULL, NULL, NULL, &tv);
	}
	fclose(fh);
	close(sid);
	return;
}

int main(int argc, char *argv[])
{
	Del_Client dclient;
	
	if ((argc < 5) || (argc > 6))
	{
		fprintf(stderr, "Parameters Error!\nUsage:delclt mode(f/s) server number clientfilename interval\n");
		exit(1);
	}

	strncpy(dclient.filename, argv[4], MAXFILENAMLEN);
	if (!strcmp(argv[1], "s"))
	{
		if (argc == 5) 
			dclient.Delete_File_Server(argv[2], atoi(argv[3]), 0);
		else
			dclient.Delete_File_Server(argv[2], atoi(argv[3]), atoi(argv[5]));
	}
	else
	{
		if (argc == 5) 
			dclient.Delete_File_Frontend(argv[2], atoi(argv[3]), 0);
		else
			dclient.Delete_File_Frontend(argv[2], atoi(argv[3]), atoi(argv[5]));
	}
	
	return 0;
}


