/**********************************************************************************
Copyright (c) 2003 The Regents of the University of California. 
All rights reserved.

Sorrento project contact: Hong Tang (hongt@cs.ucsb.edu)
			  Lingkun Chu (lkchu@cs.ucsb.edu),
                          Tao Yang (tyang@cs.ucsb.edu),
Web site: http://www.cs.ucsb.edu/projects/sorrento
Department of Computer Science, UC Santa Barbara.

This software is distributed to individuals for personal non-commercial 
use and to non-profit entities for non-commercial purposes only. Permission 
is also granted to commercial enterprises to use this software for evaluation 
purposes only. It is licensed on a non-exclusive basis, free of charge for 
these uses. The license is subject to the following conditions:

1. Redistribution is not permitted. UC Santa Barbara will maintain a copy 
   of this software as a directly downloadable file, available under the terms 
   of license specified in this agreement.
2. All advertising materials mentioning features or use of this software must 
   display the following acknowledgment: "This product includes software 
   developed by the University of California, Santa Barbara." 
3. Neither the name of the University nor the names of its contributors may 
   be used to endorse or promote products derived from this software without 
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <getopt.h>
#include <pthread.h>
#include <ctype.h>
#include "argdef.h"
#include "socket_opt.h"

#define MAX_LINE MAXFILENAMLEN+20

typedef struct
{
	char hostname[MAXHOSTNAMELEN];
	char featurelist[MAXFILENUMBER][MAXFEATURELEN];
	int featurenum;
}One_Server;

class Frontend;
typedef struct
{
	int sktid;
	Frontend* fe;
}Arg_List;

class Frontend
{
	public:
	One_Server server_list[MAXSERVERNUM];
	int server_num;

	public:

	Frontend();
	~Frontend(){};

	int Get_Servername(char* url, char* sname);
};

static char *trim(char *line)
{
    char *cp;
    for (cp=line; isspace(*cp) && (*cp) != 0; cp++);
    for (int i=strlen(line)-1; isspace(line[i]) && i>=0; i--)
        line[i]='\0';
    return cp;
}


static int split(char *line, char **key, char **value)
{
    char *cp = NULL;

    if ((line == NULL) || (strlen(line)==0) || (line[0]=='#'))
        return 0;
    for (size_t i=0; i<strlen(line); i++) {
        if (line[i] == '=') 
		{
            cp = &line[i];
            *cp++='\0';
            break;
        }
    }
    if (cp != NULL) 
	{
        (*key) = trim(line);
        (*value) = trim(cp);
        return 1;
    }
    else
        return 0;
}


Frontend::Frontend()
{
	char conf_file[MAXFILENAMLEN];
	sprintf(conf_file, "%scache_config", CONFIGPATH);
	
	server_num = 0;
	FILE* fp = fopen(conf_file, "r");
	if (fp == NULL)
	{
		fprintf(stderr, "Unable to open config file %s\n", conf_file);
		return;
	}

	char buffer[MAX_LINE];
	char *key, *value;
	char filename[MAXFILENAMLEN];

	while(fgets(buffer, MAX_LINE, fp))
	{
		if (!split(buffer, &key, &value))
			continue;
		if (!strcmp(key, "Domain"))
		{
			server_num++;
			server_list[server_num-1].featurenum = 0;
			strncpy(server_list[server_num-1].hostname, value, MAXHOSTNAMELEN);
		}
		if (!strcmp(key, "File_Name"))
		{
			strncpy(filename, value, MAXFILENAMLEN);
			char* end;
			char* begin;
			end = filename + strlen(filename);
			begin = end - 1;
			while ((begin != filename) && (begin[0] != '/'))
			{
				if (begin[0] == '.')
					end = begin;
				begin--;
			}
			if (begin[0] == '/')
			{
				strncpy(server_list[server_num-1].featurelist[server_list[server_num-1].featurenum], begin+1, MAXFEATURELEN);
				server_list[server_num-1].featurelist[server_list[server_num-1].featurenum][end-begin-1] = '\0';
			}
			else
			{
				strncpy(server_list[server_num-1].featurelist[server_list[server_num-1].featurenum], begin, MAXFEATURELEN);
				server_list[server_num-1].featurelist[server_list[server_num-1].featurenum][end-begin] = '\0';
			}
			server_list[server_num-1].featurenum++;
		}
	}
	fclose(fp);
}


int Frontend::Get_Servername(char* url, char* sname)
{
	if (url == NULL)
		return 0;
	char ft[MAXFEATURELEN];
	strncpy(ft, url, 1);
	ft[1] = '\0';
	for(int i=0; i<server_num; i++)
	{
		for(int j=0; j<server_list[i].featurenum; j++)
		{
			if (!strcmp(server_list[i].featurelist[j], ft))
			{
				strncpy(sname, server_list[i].hostname, MAXHOSTNAMELEN);
				return 1;
			}
		}
	}
	return 0;
}

void* One_Connection(void* arg)
{
	Arg_List* arglist;
	Frontend* fe;
	int sktid;

	arglist = (Arg_List *) arg;
	sktid = arglist->sktid;
	fe = arglist->fe;
	
	int ret;
	char* url;
	char* urlnumstr;
	int urlnum;
	
	ret = read_msg(sktid, &urlnumstr);
	if (ret <= 0)
	{
		fprintf(stderr, "socket error!\n");
		fflush(stderr);
		close(sktid);
		pthread_exit(NULL);
	}
	urlnum = atoi(urlnumstr);
	free(urlnumstr);

	for(int i=0; i<urlnum; i++)
	{
		ret = read_msg(sktid, &url);
		if (ret <= 0)
		{
			fprintf(stderr, "socket error!\n");
			fflush(stderr);
			close(sktid);
			pthread_exit(NULL);
		}
		else
		{
			char sname[MAXHOSTNAMELEN];
			ret = fe->Get_Servername(url, sname);
			if (ret == 0)
			{
				ret = write_msg(sktid, "NOTFOUND", 8);
				fprintf(stderr, "Cannot find the server for url=%s\n", url);
			}
			else
			{
				ret = write_msg(sktid, sname, strlen(sname));
				fprintf(stderr, "url=%s is forwarded to server=%s\n", url, sname);
			}
			free(url);
			if (ret <= 0)
			{
				fprintf(stderr, "Client socket error! exit...\n");
				fflush(stderr);
				close(sktid);
				pthread_exit(NULL);
			}
		}
	}
		
	close(sktid);
	free(arglist);
	pthread_exit(NULL);
}

int main()
{
	Frontend fe;
	
	int listenfd, connfd;
	struct sockaddr_in servaddr;
	struct sockaddr_in clientaddr;
	int clientaddr_len;
	int on;

	Arg_List* arglist;
	pthread_t tid;
	pthread_attr_t tattr;

	signal(SIGPIPE, SIG_IGN);

	listenfd = socket(AF_INET, SOCK_STREAM, 0);
	if (listenfd < 0)
	{
		fprintf(stderr, "Front end creats socket error, exit!\n");
		exit(1);
	}
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port = htons(FRONT_PORT);
	if (bind(listenfd, (struct sockaddr *)&servaddr, sizeof(servaddr)) < 0)
	{
		fprintf(stderr, "Front end bind error, exit!\n");
		exit(1);
	}
	on = 1;
	setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (char*)on, sizeof(on));
	if (listen(listenfd, BACKLOG) < 0)
	{
		fprintf(stderr, "Front end listen error, exit!\n");
		exit(1);
	}
	fprintf(stderr, "Front end started...\n");
	
	for(;;)
	{
		clientaddr_len = sizeof(clientaddr);
		connfd = accept(listenfd, (struct sockaddr*)&clientaddr, (socklen_t *)&clientaddr_len);
		if (connfd < 0)
		{
			fprintf(stderr, "Front end accept error!\n");
			continue;
		}
	
		if ((arglist=(Arg_List *) malloc(sizeof(Arg_List))) == NULL)
		{
			fprintf(stderr, "Not enough memory, socket closed!\n");
			fflush(stderr);
			close(connfd);
			continue;
		}
		arglist->sktid = connfd;
		arglist->fe = &fe;
		pthread_attr_init(&tattr);
		/* set detach state for each child thread */
		if (pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) != 0)
		{
			fprintf(stderr, "Set detach state failed!\n");
			fflush(stderr);
			close(connfd);
			continue;
		}
		pthread_attr_setscope(&tattr, PTHREAD_SCOPE_SYSTEM);
		if (pthread_create(&tid, &tattr, One_Connection, arglist) != 0)
		{
			fprintf(stderr, "Front end create thread failed, connection for this client is closed!\n");
			fflush(stderr);
			close(connfd);
			free(arglist);
			arglist = NULL;
			continue;
		}
		arglist = NULL;
	}
}
