/**********************************************************************************
Copyright (c) 2003 The Regents of the University of California. 
All rights reserved.

Netptune project contact: Lingkun Chu (lkchu@cs.ucsb.edu),
                          Tao Yang (tyang@cs.ucsb.edu),
                          Kai Shen (kshen@cs.rochester.edu)
Web site: http://www.cs.ucsb.edu/projects/neptune
Department of Computer Science, UC Santa Barbara.

This software is distributed to individuals for personal non-commercial 
use and to non-profit entities for non-commercial purposes only. Permission 
is also granted to commercial enterprises to use this software for evaluation 
purposes only. It is licensed on a non-exclusive basis, free of charge for 
these uses. The license is subject to the following conditions:

1. Redistribution is not permitted. UC Santa Barbara will maintain a copy 
   of this software as a directly downloadable file, available under the terms 
   of license specified in this agreement.
2. All advertising materials mentioning features or use of this software must 
   display the following acknowledgment: "This product includes software 
   developed by the University of California, Santa Barbara." 
3. Neither the name of the University nor the names of its contributors may 
   be used to endorse or promote products derived from this software without 
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
**********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "argdef.h"
#include "socket_opt.h"

typedef struct
{
	char url[URLLEN];
	int valid;
}One_Url;

class Get_Client
{
	int url_num;
	public:
	char filename[MAXFILENAMLEN];
	int beginchar;
	int endchar;

	unsigned int totalrequests;
	unsigned long totalresponsetime;
	unsigned long totalbytesent;	

	public:
	//interval in millisecond
	void Retrieve_File_Server(char* server, int filenum, int interval);
	void Retrieve_File_Frontend(char* frontend, int filenum, int interval);
};

void Get_Client::Retrieve_File_Frontend(char* frontend, int filenum, int interval)
{
	int sid;
	struct sockaddr_in servaddr;
	struct hostent* ssaddr;
	int dot_addr;
	struct in_addr addr;
	sid = socket(AF_INET, SOCK_STREAM, 0);
	if (sid < 0)
	{
		fprintf(stderr, "Client creats socket error!\n");
		fflush(stderr);
		return;
	}
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(FRONT_PORT);
	dot_addr = inet_aton(frontend, &addr);
	if (!dot_addr)
	{
		ssaddr = gethostbyname(frontend);
		if (ssaddr == NULL)
		{
			fprintf(stderr, "The frontend address error, exit!\n");
			fflush(stderr);
			return;
		}
		memcpy(&addr, ssaddr->h_addr, sizeof(addr));
	}
	memcpy(&servaddr.sin_addr, &addr, sizeof(addr));
	if (connect_with_timeout(sid, (struct sockaddr *)&servaddr, sizeof(servaddr), CONNECT_TIMEOUT) <0)
	{
		fprintf(stderr, "Connect to the frontend error: %s, exit!\n", strerror(errno));
		fflush(stderr);
		return;
	}
	
	FILE* fh;
	fh = fopen(filename, "r");
	if (fh == NULL)
	{
		fprintf(stderr, "Client File not exist, exit!\n");
		close(sid);
		return;
	}
	One_Url oneurl;
	if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
	{
		fprintf(stderr, "Client File is empty, exit!\n");
		close(sid);
		fclose(fh);
		return;
	}
	else
		url_num = oneurl.valid;
	
	int randnum;
	char* response;
	int reslen;
	int ret;
	char urlnum[10];

	char sname[MAXHOSTNAMELEN];

	struct timeval rseed;
	gettimeofday(&rseed, NULL);
	srand((rseed.tv_sec+rseed.tv_usec) % 100000);
	totalrequests = 0;
	totalresponsetime = 0;
	totalbytesent = 0;
	
	sprintf(urlnum, "%d\n", filenum);
	ret = write_msg(sid, urlnum, strlen(urlnum));	
	if (ret <= 0)
	{
		fprintf(stderr, "Socket Error, exit...\n");
		fclose(fh);
		close(sid);
		return;
	}
	
	for(int i=0; i<filenum; i++)
	{
		randnum = 1 + (int) (100*1.0*rand()/(RAND_MAX+1.0));
		if (randnum > HITRATE)
		{
			randnum = 1 + (int) (1.0*(endchar-beginchar+1)*rand()/(RAND_MAX+1.0));
			randnum += beginchar - 1;
			for(int j=0; j<URLLEN-1; j++)
				oneurl.url[j] = randnum;
			oneurl.url[URLLEN-1] = '\0';
		}
		else
		{		
			randnum = 1 + (int) (url_num*1.0*rand()/(RAND_MAX+1.0));
			fseek(fh, (long) randnum*sizeof(One_Url), SEEK_SET);
			if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
				continue;
			if (oneurl.valid != 1)
			{
				i --;
				continue;
			}
		}
	
		ret = write_msg(sid, oneurl.url, strlen(oneurl.url));	
		if (ret <= 0)
		{
			fprintf(stderr, "Socket with frontend Error, exit...\n");
			break;
		}
		reslen = read_msg(sid, &response);
		if (reslen <= 0) 
		{
			fprintf(stderr, "Socket with frontend Error, exit...\n");
			break;
		}
		if (strcmp(response, "NOTFOUND") == 0)
		{
			fprintf(stderr, "Cannot find the server for url=%s\n", oneurl.url);
			free(response);
		}
		else
		{
			strncpy(sname, response, MAXHOSTNAMELEN);
			free(response);
			
			struct timeval begintime;
			struct timeval endtime;

			int sids;
			struct sockaddr_in servaddrs;
			struct hostent* ssaddrs;
			int dot_addrs;
			struct in_addr addrs;
			sids = socket(AF_INET, SOCK_STREAM, 0);
			if (sids < 0)
			{
				fprintf(stderr, "Client creats socket with the server %s error!\n", sname);
				fflush(stderr);
				continue;
			}
			bzero(&servaddrs, sizeof(servaddrs));
			servaddrs.sin_family = AF_INET;
			servaddrs.sin_port = htons(SERVER_PORT);
			dot_addrs = inet_aton(sname, &addrs);
			if (!dot_addrs)
			{
				ssaddrs = gethostbyname(sname);
				if (ssaddrs == NULL)
				{
					fprintf(stderr, "The server %s address error, exit!\n", sname);
					fflush(stderr);
					close(sids);
					continue;
				}
				memcpy(&addrs, ssaddrs->h_addr, sizeof(addrs));
			}
			memcpy(&servaddrs.sin_addr, &addrs, sizeof(addrs));
			if (connect_with_timeout(sids, (struct sockaddr *)&servaddrs, sizeof(servaddrs), CONNECT_TIMEOUT) <0)
			{
				fprintf(stderr, "Connect to the server %s error: %s, exit!\n", sname, strerror(errno));
				fflush(stderr);
				close(sids);
				continue;
			}
			
			ret = write_msg(sids, "GET", 3);
			if (ret <= 0)
			{
				fprintf(stderr, "Socket with server %s error! GET File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				continue;
			}
			
			sprintf(urlnum, "1\n");
			ret = write_msg(sids, urlnum, strlen(urlnum));	
			if (ret <= 0)
			{
				fprintf(stderr, "Socket with server %s Error, GET File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				continue;
			}
			
			//the timer starts here
			int timevalid;
			timevalid = 1;
			if (gettimeofday(&begintime, NULL) != 0)
				timevalid = 0;
			
			ret = write_msg(sids, oneurl.url, strlen(oneurl.url));	
			if (ret <= 0)
			{
				fprintf(stderr, "Socket with server %s Error, GET File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				continue;
			}
			reslen = read_msg(sids, &response); 
			if (reslen <= 0) 
			{
				fprintf(stderr, "Socket with server %s Error, GET File aborted...\n", sname);
				fflush(stderr);
				close(sids);
				continue;
			}
			if (strcmp(response, "ERROR") == 0)
			{
				fprintf(stderr, "Get url=%s at server %s ERROR.\n", oneurl.url, sname);
			}
			if (strcmp(response, "NOTFOUND") == 0)
			{
				fprintf(stderr, "Get url=%s at server %s NOT FOUND.\n", oneurl.url, sname);
				//timer ends here
				if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
				{
					totalrequests ++;
					totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
							             (endtime.tv_usec - begintime.tv_usec);
				}
			}
			if (strcmp(response, "OK") == 0)
			{	
				char* data;
				int datalen;
				datalen = read_msg(sids, &data); 
				if (datalen <= 0) 
				{
					fprintf(stderr, "Socket with server %s Error, GET File aborted...\n", sname);
				}
				else
				{
					fprintf(stderr, "Get url=%s with size %d at server %s OK.\n", oneurl.url, datalen, sname);
					free(data);
					//timer ends here
					if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
					{
						totalrequests ++;
						totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
								             (endtime.tv_usec - begintime.tv_usec);
						totalbytesent += datalen;
					}
				}
			}
			free(response);
			close(sids);
		}
		
		struct timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 1000*interval;
		select(0, NULL, NULL, NULL, &tv);
	}
	
	fclose(fh);
	close(sid);
	return;
}

void Get_Client::Retrieve_File_Server(char* server, int filenum, int interval)
{	
	int sid;
	struct sockaddr_in servaddr;
	struct hostent* ssaddr;
	int dot_addr;
	struct in_addr addr;
	sid = socket(AF_INET, SOCK_STREAM, 0);
	if (sid < 0)
	{
		fprintf(stderr, "Client creats socket error!\n");
		fflush(stderr);
		return;
	}
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(SERVER_PORT);
	dot_addr = inet_aton(server, &addr);
	if (!dot_addr)
	{
		ssaddr = gethostbyname(server);
		if (ssaddr == NULL)
		{
			fprintf(stderr, "The server address error, exit!\n");
			fflush(stderr);
			return;
		}
		memcpy(&addr, ssaddr->h_addr, sizeof(addr));
	}
	memcpy(&servaddr.sin_addr, &addr, sizeof(addr));
	if (connect_with_timeout(sid, (struct sockaddr *)&servaddr, sizeof(servaddr), CONNECT_TIMEOUT) <0)
	{
		fprintf(stderr, "Connect to the server error: %s, exit!\n", strerror(errno));
		fflush(stderr);
		return;
	}

	FILE* fh;
	fh = fopen(filename, "r");
	if (fh == NULL)
	{
		fprintf(stderr, "Client File not exist, exit!\n");
		close(sid);
		return;
	}
	One_Url oneurl;
	if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
	{
		fprintf(stderr, "Client File is empty, exit!\n");
		fclose(fh);
		close(sid);
		return;
	}
	else
		url_num = oneurl.valid;

	int randnum;
	char* data;
	int datalen;
	char* response;
	int reslen;
	int ret;
	char urlnum[10];

	ret = write_msg(sid, "GET", 3);
	if (ret <= 0)
	{
		fprintf(stderr, "Socket error! GET File aborted...\n");
		fclose(fh);
		close(sid);
		return;
	}
	
	sprintf(urlnum, "%d\n", filenum);
	ret = write_msg(sid, urlnum, strlen(urlnum));	
	if (ret <= 0)
	{
		fprintf(stderr, "Client Socket Error, exit...\n");
		fclose(fh);
		close(sid);
		return;
	}
	struct timeval rseed;
	gettimeofday(&rseed, NULL);
	srand((rseed.tv_sec+rseed.tv_usec) % 100000);
	totalrequests = 0;
	totalresponsetime = 0;
	totalbytesent = 0;
	
	for(int i=0; i<filenum; i++)
	{
		randnum = 1 + (int) (100*1.0*rand()/(RAND_MAX+1.0));
		if (randnum > HITRATE)
		{
			randnum = 1 + (int) (1.0*(endchar-beginchar+1)*rand()/(RAND_MAX+1.0));
			randnum += beginchar - 1;
			for(int j=0; j<URLLEN-1; j++)
				oneurl.url[j] = randnum;
			oneurl.url[URLLEN-1] = '\0';
		}
		else
		{		
			randnum = 1 + (int) (url_num*1.0*rand()/(RAND_MAX+1.0));
			fseek(fh, (long) randnum*sizeof(One_Url), SEEK_SET);
			if (fread(&oneurl, sizeof(One_Url), 1, fh) != 1)
				continue;
			if (oneurl.valid != 1)
			{
				i --;
				continue;
			}
		}
		
		struct timeval begintime;
		struct timeval endtime;
		//the timer starts here
		int timevalid;
		timevalid = 1;
		if (gettimeofday(&begintime, NULL) != 0)
			timevalid = 0;
		
		ret = write_msg(sid, oneurl.url, strlen(oneurl.url));	
		if (ret <= 0)
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		reslen = read_msg(sid, &response); 
		if (reslen <= 0) 
		{
			fprintf(stderr, "Client Socket Error, exit...\n");
			break;
		}
		if (strcmp(response, "ERROR") == 0)
		{
			fprintf(stderr, "Get url=%s at server %s ERROR.\n", oneurl.url, server);
		}
		if (strcmp(response, "NOTFOUND") == 0)
		{
			fprintf(stderr, "Get url=%s at server %s NOT FOUND.\n", oneurl.url, server);
			//timer ends here
			if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
			{
				totalrequests ++;
				totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
						             (endtime.tv_usec - begintime.tv_usec);
			}
		}
				
		if (strcmp(response, "OK") == 0)
		{	
			datalen = read_msg(sid, &data); 
			if (datalen <= 0) 
			{
				fprintf(stderr, "Socket with server %s Error, GET File aborted...\n", server);
			}
			else
			{
				fprintf(stderr, "Get url=%s with size %d at server %s OK.\n", oneurl.url, datalen, server);
				free(data);
				//timer ends here
				if ((timevalid) && (gettimeofday(&endtime, NULL) == 0))
				{
					totalrequests ++;
					totalresponsetime += 1000000*(endtime.tv_sec - begintime.tv_sec) +
							             (endtime.tv_usec - begintime.tv_usec);
					totalbytesent += datalen;
				}
			}
		}
		free(response);
		struct timeval tv;
		tv.tv_sec = 0;
		tv.tv_usec = 1000*interval;
		select(0, NULL, NULL, NULL, &tv);
	}
	fclose(fh);
	close(sid);
	return;
}

int main(int argc, char *argv[])
{
	Get_Client gclient;

	gclient.beginchar = MINCHAR;
	gclient.endchar = MAXCHAR;
	if ((argc < 5) || (argc > 8))
	{
		fprintf(stderr, "Parameters Error!\nUsage:getclt mode(f/s) server number clientfilename beginchar endchar interval(ms)\n");
		exit(1);
	}
	
	strncpy(gclient.filename, argv[4], MAXFILENAMLEN);
	if (!strcmp(argv[1], "s"))
	{
		if (argc == 5) 
			gclient.Retrieve_File_Server(argv[2], atoi(argv[3]), 0);
		if (argc == 6)
			gclient.Retrieve_File_Server(argv[2], atoi(argv[3]), atoi(argv[5]));
		if (argc == 7)
		{
			gclient.beginchar = argv[5][0];
			gclient.endchar = argv[6][0];
			gclient.Retrieve_File_Server(argv[2], atoi(argv[3]), 0);
		}
		if (argc == 8)
		{
			gclient.beginchar = argv[5][0];
			gclient.endchar = argv[6][0];
			gclient.Retrieve_File_Server(argv[2], atoi(argv[3]), atoi(argv[7]));
		}
	}
	else
	{
		if (argc == 5) 
			gclient.Retrieve_File_Frontend(argv[2], atoi(argv[3]), 0);
		if (argc == 6)
			gclient.Retrieve_File_Frontend(argv[2], atoi(argv[3]), atoi(argv[5]));
		if (argc == 7)
		{
			gclient.beginchar = argv[5][0];
			gclient.endchar = argv[6][0];
			gclient.Retrieve_File_Frontend(argv[2], atoi(argv[3]), 0);
		}
		if (argc == 8)
		{
			gclient.beginchar = argv[5][0];
			gclient.endchar = argv[6][0];
			gclient.Retrieve_File_Frontend(argv[2], atoi(argv[3]), atoi(argv[7]));
		}
	}
	
	int averesponsetime;
	int throughput1;
	int throughput2;
	double temp;
	
	averesponsetime = (int) (gclient.totalresponsetime/gclient.totalrequests);
	temp = (double) 1.0*gclient.totalresponsetime/1000000;
	throughput1 = (int) (1.0*gclient.totalrequests/temp);
	throughput2 = (int) (1.0*gclient.totalbytesent/temp);
	fprintf(stdout, "------------------------------------------\n");
	fprintf(stdout, "the total read requests sent is %d\n", gclient.totalrequests);
	fprintf(stdout, "the total response time is %d us\n", gclient.totalresponsetime);
	fprintf(stdout, "the total bytes received is %d\n", gclient.totalbytesent);
	fprintf(stdout, "the average response time is %d us\n", averesponsetime);
	fprintf(stdout, "the average throughput(URL) is %d urls/second\n", throughput1);
	fprintf(stdout, "the average throughput(bytes) is %d bytes/second\n", throughput2);
	fprintf(stdout, "------------------------------------------\n");
	return 0;
}


