#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "indextable.h"
#include "md5.h"

Index_Table::Index_Table()
{
}

Index_Table::~Index_Table()
{
}

void Index_Table::Initialize(char* filename)
{
	for(int i=0; i<TABLESIZE; i++)
	{
		index_table[i].valid = 0;
		index_table[i].fileno = 0;
		index_table[i].beginblock = 0;
		index_table[i].endblock = 0;
		index_table[i].next = NULL;
	}
	item_num = 0;
	ifmodify = 0;
	Read_fromFile(filename);
}

void Index_Table::Finalize(char* filename)
{
	Write_toFile(filename);
	for(int i=0; i<TABLESIZE; i++)
	{
		One_Item* ptr;
		One_Item* cur;
		ptr = index_table[i].next;
		while (ptr != NULL)
		{
			cur = ptr;
			ptr = ptr->next;
			free(cur);
		}
	}
}

int Index_Table::Read_fromFile(char* filename)
{
	FILE* systemfile;
	item_num = 0;
	systemfile = fopen(filename, "r");
	if (systemfile == NULL)
	{
		fprintf(stderr, "Index Table File NOT Found! Create a NEW Table...\n");
		ifmodify = 1;
		return 0;
	}

	Save_Item oneitem;
	int curindex = -1;
	One_Item* ptr = NULL;
	while (fread(&oneitem, sizeof(Save_Item), 1, systemfile) == 1)
	{
		if (oneitem.index > TABLESIZE)
			break;
		if (curindex != oneitem.index)
		{
			curindex = oneitem.index;
			strncpy(index_table[curindex].url, oneitem.url, URLLEN);
			index_table[curindex].valid = oneitem.valid;
			index_table[curindex].fileno = oneitem.fileno;
			index_table[curindex].beginblock = oneitem.beginblock;
			index_table[curindex].endblock = oneitem.endblock;
			ptr = &index_table[curindex];
		}
		else
		{
			One_Item* newitem = (One_Item*) malloc(sizeof(One_Item));
			strncpy(newitem->url, oneitem.url, URLLEN);
			newitem->valid = oneitem.valid;
			newitem->fileno = oneitem.fileno;
			newitem->beginblock = oneitem.beginblock;
			newitem->endblock = oneitem.endblock;
			newitem->next = NULL;
			ptr->next = newitem;
			ptr = newitem;
		}
		if (oneitem.valid == 1)
			item_num ++;
	}
	fclose(systemfile);
	return 1;
}

int Index_Table::Write_toFile(char* filename)
{
	FILE* systemfile;
	systemfile = fopen(filename, "w");
	if (systemfile == NULL)
	{
		fprintf(stderr, "Index Table CANNOT be Saved! Table Lost...\n");
		return 0;
	}

	Save_Item oneitem;
	for(int i=0; i<TABLESIZE; i++)
	{
		One_Item* ptr;
		ptr = &index_table[i];
		while (ptr != NULL)
		{
			oneitem.index = i;
			strncpy(oneitem.url, ptr->url, URLLEN);
			oneitem.valid = ptr->valid;
			oneitem.fileno = ptr->fileno;
			oneitem.beginblock = ptr->beginblock;
			oneitem.endblock = ptr->endblock;
			if (fwrite(&oneitem, sizeof(Save_Item), 1, systemfile) != 1)
			{
				fprintf(stderr, "Save Index Table Error! Table Lost...\n");
				fclose(systemfile);
				return 0;
			}
			ptr = ptr->next;
		}
	}
	fclose(systemfile);
	ifmodify = 0;
	return 1;	
}

void Index_Table::Print_Table()
{
	fprintf(stderr, "The number of items in the table is %d\n\n", item_num);
	for(int i=0; i<TABLESIZE; i++)
	{
		One_Item* ptr;
		ptr = &index_table[i];
		while (ptr != NULL)
		{
			fprintf(stderr, "index = %d\n", i);
			fprintf(stderr, "valid = %d\n", ptr->valid);
			if (ptr->valid == 1)
			{
				fprintf(stderr, "url = %s\n", ptr->url);
				fprintf(stderr, "fileno = %d\n", ptr->fileno);
				fprintf(stderr, "beginblock = %d\n", ptr->beginblock);
				fprintf(stderr, "endblock = %d\n", ptr->endblock);
			}
			fprintf(stderr, "\n");
			ptr = ptr->next;
		}
	}
}

int Index_Table::Get_Index(char* url)
{
	unsigned char md5[16];
	unsigned long mdlong[4];
	unsigned long key;
	int index;

	md5String(url, md5);
	memcpy(mdlong, md5, 16);
	key = mdlong[0]^mdlong[1]^mdlong[2]^mdlong[3];
	index = (int) (key % TABLESIZE);
	return index;	
}

One_Item* Index_Table::Locate_File(char* url)
{
	int index;

	index = Get_Index(url);	
	One_Item* ptr;
	ptr = &index_table[index];
	while (ptr != NULL)
	{
		if (ptr->valid == 1)
		{
			if (strncmp(url, ptr->url, URLLEN) == 0)
				break;
		}
		ptr = ptr->next;
	}
	
	return ptr;	
}

int Index_Table::Retrieve_File(char* url, File_Position* fpos)
{
	One_Item* ptr = Locate_File(url);
	if (ptr == NULL)
		return 0;
	fpos->fileno = ptr->fileno;
	fpos->beginblock = ptr->beginblock;
	fpos->endblock = ptr->endblock;
	return 1;
}

int Index_Table::Add_File(char* url, int fileno, size_t bblock, size_t eblock)
{
	if (Locate_File(url) != NULL)
		return 0;
	
	int index = Get_Index(url);
	One_Item* ptr;
	ptr = &index_table[index];
	if (ptr->valid == 0)
	{
		strncpy(ptr->url, url, URLLEN);
		ptr->valid = 1;
		ptr->fileno = fileno;
		ptr->beginblock = bblock;
		ptr->endblock = eblock;
	}
	else
	{
		while (ptr->next != NULL)
			ptr = ptr->next;
		One_Item* newitem = (One_Item*)malloc(sizeof(One_Item));
		strncpy(newitem->url, url, URLLEN);
		newitem->valid = 1;
		newitem->fileno = fileno;
		newitem->beginblock = bblock;
		newitem->endblock = eblock;
		newitem->next = NULL;
		ptr->next = newitem;
	}
	item_num ++;
	ifmodify = 1;
	return 1;
}

int Index_Table::Delete_File(char* url)
{
	
	if (Locate_File(url) == NULL)
		return 0;

	int index = Get_Index(url);
	One_Item* ptr;
	ptr = &index_table[index];
	
	if ((ptr->valid == 1) && (strncmp(url, ptr->url, URLLEN) == 0))
	{
		One_Item* nptr;
		nptr = ptr->next;
		if (nptr == NULL)
		{
			ptr->valid = 0;
			ptr->fileno = 0;
			ptr->beginblock = 0;
			ptr->endblock = 0;
		}
		else
		{
			ptr->next = nptr->next;
			strncpy(ptr->url, nptr->url, URLLEN);
			ptr->valid = nptr->valid;
			ptr->fileno = nptr->fileno;
			ptr->beginblock = nptr->beginblock;
			ptr->endblock = nptr->endblock;
			free(nptr);
		}
	}
	else
	{
		One_Item* cur;
		cur = ptr->next;
		while(cur != NULL)
		{
			if ((cur->valid == 1) && (strncmp(url, cur->url, URLLEN) == 0))
				break;
			ptr = cur;
			cur = cur->next;
		}
		if (cur == NULL)
			return 0;
		ptr->next = cur->next;
		free(cur);		
	}
	item_num --;
	ifmodify = 1;
	return 1;		
}

int Index_Table::Ifmodified()
{
	return ifmodify;
}

/*
int main()
{
	Index_Table indextable;
	indextable.Add_File("http://www.microsoft.com/services", 0, 0, 10);
	indextable.Add_File("http://www.microsoft.com/networks", 0, 11, 20);
	indextable.Add_File("http://www.amozon.com/books", 0, 30, 40);
	indextable.Add_File("http://www.amozon.com/electronic/%s=344", 0, 41, 55);
	indextable.Add_File("http://www.dell.com/", 1, 11, 20);
	indextable.Add_File("http://www.dell.com/network", 1, 31, 31);

	indextable.Delete_File("http://www.dell.com/");
	indextable.Delete_File("http://www.dell.com/network");
	indextable.Print_Table();

	File_Position fp;
	if (indextable.Retrieve_File("http://www.amozon.com/books", &fp))
		fprintf(stderr, "the info for amozbooks is: fileno=%d, bb=%d, eb=%d\n", fp.fileno, fp.beginblock, fp.endblock);
	else
		fprintf(stderr, "Not Found for amozbooks\n");
	if (indextable.Retrieve_File("http://www.dell.com/", &fp))
		fprintf(stderr, "the info for dell is: fileno=%d, bb=%d, eb=%d\n", fp.fileno, fp.beginblock, fp.endblock);
	else
		fprintf(stderr, "Not Found for dell\n");
	
}
*/
