// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: simple_bloom.h,v $ - header file for the SIMPLE bloom filter object,
 *            which is used as a read-only object once loaded or assumes
 *            sequential execution when objects are added to the filter.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:52 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef __SIMPLE_BLOOM_H__
# define __SIMPLE_BLOOM_H__

# include <stdio.h>
///////////////////////////////////////
// The simpliest form of Bloom Filters
// Instantiate, add a number of objects
// and then query for their existence.
// Cannot remove objects from the filter.
// Adding objects must be done sequentially,
// no locking overhead at all.
class SimpleBF {
private:
	unsigned char *mask;	// bit mask
	unsigned int msize;	// size of the mask (in the unit of BF_PAGEs)
	unsigned int fsize;	// size of the filter (in bits)
	unsigned int n_bit_set; // number of bits set
	unsigned int n_object; // number of objects
	SimpleBF(); // illegal constructor
	SimpleBF(SimpleBF &); // illegal constructor
	SimpleBF & operator=(const SimpleBF& rhs); // illegal operator

public:
	// constructors/destructors
	SimpleBF(int max_objs);
	SimpleBF(FILE *fp);
	~SimpleBF();

	bool reset(); 		// clear the mask
	bool add(const void *key, int len);
	bool check(const void *key, int len);
	double set_bit_pcent();
	unsigned int get_num_obj() { return n_object; }
	bool serialize(FILE *fp);
	bool load(FILE *fp);
	int get_size();

private: 
	// Find the closest but smaller prime number to upper
	static unsigned int prime_within(unsigned int upper);
	static bool is_prime(unsigned int p);
};  

# ifndef _Solaris_

struct __dummy1 { unsigned long a[100]; };

# define asm_is_bit_set(ptr32, bit) \
    (__extension__ \
     ( { \
        register int oldbit;  \
        __asm__ __volatile__ ( \
        			"btl %2, %1\n\t" \
                    "sbbl %0, %0" \
                    : "=r" (oldbit), "=m" (*(volatile struct __dummy1 *)(ptr32)) \
                    : "Ir" (bit) \
        ); \
        oldbit; \
       } \
     ) \
    )

# endif // _Solaris_

# endif // __SIMPLE_BLOOM_H__

