// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t2.cc,v $ - Bloom filter tester II.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include "bloom.h"
# include "../guid/guid.h"
# include "../usc/usc_time.h"

# define TOTAL_ITEMS 100000

using namespace std;

int main()
{
	int ret=0;
	int fp=0;
	int i;
	usc_time_t t1, t2;
	double zbp=0;
	double addt=0, delt=0, checkt=0, check2t=0;

	cout << "Testing add/del/check." << endl;

	usc_init();
	BF bf(TOTAL_ITEMS);
	Guid gi[TOTAL_ITEMS];
	Guid uigi[TOTAL_ITEMS];
	t1=usc_gettime();
	for (int i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].gen();
		gi[i].pack(tmp, GUID_LEN);
		if (!bf.add(tmp, GUID_LEN)) {
			cout << "BF.add() failed!" << endl;
			goto _exit;
		}
		uigi[i].gen();
	}
	t2=usc_gettime();
	addt=usc_difftime(t2, t1)/TOTAL_ITEMS;

	zbp=bf.zero_pcent();
	t1=usc_gettime();
	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (!bf.check(tmp, GUID_LEN)) {
			cout << "BF.check() failed!" << endl;
			ret=-1;
			goto _exit;
		}
		uigi[i].pack(tmp, GUID_LEN);
		if (bf.check(tmp, GUID_LEN)) {
			fp++;
		}
	}
	t2=usc_gettime();
	checkt=usc_difftime(t2, t1)/TOTAL_ITEMS/2;

	t1=usc_gettime();
	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (!bf.del(tmp, GUID_LEN)) {
			cout << "BF.del() failed!" << endl;
			ret=-1;
			goto _exit;
		}
		if (bf.check(tmp, GUID_LEN)) {
			fp++;
		}
	}
	t2=usc_gettime();
	delt=usc_difftime(t2, t1)/TOTAL_ITEMS;

	t1=usc_gettime();
	for (i=0; i<TOTAL_ITEMS; i++)
	{
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (bf.check(tmp, GUID_LEN)) {
			cout << "BF.check() has inconsistent result!" << endl;
			ret=-1;
			goto _exit;
		}
	}
	t2=usc_gettime();
	check2t=usc_difftime(t2, t1)/TOTAL_ITEMS;

	cout << "False Positive:" << fp <<  endl;
	cout << "False Positive Percentage:" << (double)fp/TOTAL_ITEMS <<  endl;
	cout << "Percentage of zero bits:" << zbp <<  endl;
	cout << "2 Guid.gen(), 1 Guid.pack(), 1 BF.add() :" << addt << "us" << endl;
	cout << "1 Guid.pack(), 1 BF.check() :" << checkt << "us" << endl;
	cout << "1 Guid.pack(), 1 BF.check(), 1 BF.del() :" << delt << "us" << endl;
	cout << "1 Guid.pack(), 1 BF.check() :" << check2t << "us" << endl;

	cout << "Test succeed!" << endl;

	return 0;
_exit:
	cout << "Test failed" << endl;
	return ret;
}
