// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t4.cc,v $ - Bloom filter tester IV.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include "bloom.h"
# include "../guid/guid.h"

using namespace std;

# define TOTAL_ITEMS 10000
# define UPDATE_FREQ 10

int main()
{
	int ret=0;
	int i;
	int byte_update=0;
	int time_update=0;
	int lag_update=0;
	int byte_update2=0;
	int time_update2=0;
	int lag_update2=0;
	char page_buf[1024];
	int len=1024;
	int offset=0;
	int miss_item=0;
	int miss_item2=0;

	cout << "Testing page modification/updating." << endl;

	BF snd(TOTAL_ITEMS);
	BaseBF rcv(TOTAL_ITEMS);
	Guid gi[TOTAL_ITEMS];
	Guid uigi[TOTAL_ITEMS];
	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].gen();
		gi[i].pack(tmp, GUID_LEN);
		if (!snd.add(tmp, GUID_LEN)) {
			cout << "BF.add() failed!" << endl;
			goto _exit;
		}
		if (!((i+1)%UPDATE_FREQ)) {
			len=1024;
			if (snd.next_mod_page(page_buf, &offset, &len)) {
				rcv.update_block(page_buf, offset, len);
				byte_update+=len;
				time_update++;
			}
		}
		uigi[i].gen();
	}

	cout << "Random number generation done, filter partially replicated." << endl;

	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (!rcv.check(tmp, GUID_LEN)) {
			miss_item++;
		}
	}

	len=1024;
	while (snd.next_mod_page(page_buf, &offset, &len)) {
		rcv.update_block(page_buf, offset, len);
		byte_update+=len;
		time_update++;
		lag_update++;
		len=1024;
	}

	cout << "Filter fully replicated, checking consistency." << endl;

	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (rcv.check(tmp, GUID_LEN)!=snd.check(tmp, GUID_LEN)) {
			cout << "Inconsistent replication!" << endl;
			ret=-1;
			goto _exit;
		}
		uigi[i].pack(tmp, GUID_LEN);
		if (rcv.check(tmp, GUID_LEN)!=snd.check(tmp, GUID_LEN)) {
			cout << "Inconsistent replication!" << endl;
			ret=-1;
			goto _exit;
		}
	}

	cout << "Remove items from source filter." << endl;
	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (!snd.del(tmp, GUID_LEN)) {
			cout << "BF.del() failed!" << endl;
			ret=-1;
			goto _exit;
		}
		if (!((i+1)%UPDATE_FREQ)) {
			len=1024;
			if (snd.next_mod_page(page_buf, &offset, &len)) {
				rcv.update_block(page_buf, offset, len);
				byte_update2+=len;
				time_update2++;
			}
		}
	}

	cout << "Items removed, filter partially replicated." << endl;

	for (i=0; i<TOTAL_ITEMS; i++) {
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (rcv.check(tmp, GUID_LEN)) {
			miss_item2++;
		}
	}

	len=1024;
	while (snd.next_mod_page(page_buf, &offset, &len)) {
		rcv.update_block(page_buf, offset, len);
		byte_update2+=len;
		time_update2++;
		lag_update2++;
		len=1024;
	}

	cout << "Filter fully replicated, checking consistency." << endl;
	
	for (i=0; i<TOTAL_ITEMS; i++)
	{
		char tmp[GUID_LEN];
		gi[i].pack(tmp, GUID_LEN);
		if (rcv.check(tmp, GUID_LEN)!=snd.check(tmp, GUID_LEN)) {
			cout << "Inconsistent replication!" << endl;
			ret=-1;
			goto _exit;
		}
		uigi[i].pack(tmp, GUID_LEN);
		if (rcv.check(tmp, GUID_LEN)!=snd.check(tmp, GUID_LEN)) {
			cout << "Inconsistent replication!" << endl;
			ret=-1;
			goto _exit;
		}
	}

	cout << "Phase 1: Adding items:" << endl;
	cout << "Total bytes: " << byte_update << endl;
	cout << "Total times: " << time_update << endl;
	cout << "Lagged update: " << lag_update << endl;
	cout << "False miss due to lag: " << (double)miss_item/TOTAL_ITEMS << endl;
	cout << "Phase 2: Adding items:" << endl;
	cout << "Total bytes: " << byte_update2 << endl;
	cout << "Total times: " << time_update2 << endl;
	cout << "Lagged update: " << lag_update2 << endl;
	cout << "False Hit due to lag: " << (double)miss_item2/TOTAL_ITEMS << endl;
	cout << "Test Successful" << endl;
	return 0;
_exit:
	cout << "Test failed" << endl;
	return ret;
}
