// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t6.cc,v $ - Bloom filter tester VI.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:52 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include "simple_bloom.h"
# include "../guid/guid.h"
# include "../usc/usc_time.h"

# define TOTAL_ITEMS 100000
# define TOTAL_CHECKS 100000

int main()
{
	SimpleBF sbf(TOTAL_ITEMS);

	printf("Phase 1: Adding entries to bloom filter.\n");
	for (unsigned int i=0; i<TOTAL_ITEMS; i++) {
		Guid gi;
		gi.gen();
		char tmp[GUID_LEN];
		gi.pack(tmp, GUID_LEN);
		sbf.add(tmp, GUID_LEN);
		if (i%1000==0) {
			printf("\r%.3f%% done.", i*100.0/TOTAL_ITEMS);
			fflush(stdout);
		}
	}

	printf("\rPhase 2: Checking false positive rate.\n");

	usc_time_t t_start, t_end;
	t_start=usc_gettime();
	unsigned int fp=0;
	for (unsigned int i=0; i<TOTAL_CHECKS; i++) {
		Guid gi;
		gi.gen();
		char tmp[GUID_LEN];
		gi.pack(tmp, GUID_LEN);
		if (sbf.check(tmp, GUID_LEN)) {
			fp++;
		}
		if (i%1000==0) {
			printf("\r%.3f%% done.", i*100.0/TOTAL_CHECKS);
			fflush(stdout);
		}
	}
	t_end=usc_gettime();

	printf("\rPhase 3: Checking GUID generation overhead.\n");
	double t_gross=usc_difftime(t_end, t_start);
	t_start=usc_gettime();
	for (unsigned int i=0; i<TOTAL_CHECKS; i++) {
		Guid gi;
		gi.gen();
		char tmp[GUID_LEN];
		gi.pack(tmp, GUID_LEN);
		if (i%1000==0) {
			printf("\r%.3f%% done.", i*100.0/TOTAL_CHECKS);
			fflush(stdout);
		}
	}
	t_end=usc_gettime();
	double t_overhead=usc_difftime(t_end, t_start);

	double t_check=t_gross-t_overhead;

	printf("\rTotal %u keys, %.3f%% false positive rate, %.3f%% percent of zeros in the filter, %.3fus per check.\n", TOTAL_ITEMS, fp*100.0/TOTAL_CHECKS, (1-sbf.set_bit_pcent())*100, t_check/TOTAL_CHECKS);

	return 0;
}
