// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2000, 2001, 2002 The Regents of the University of 
  California, All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: tmpi_debug.c,v $ - an application debugging/logging module.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:52 $
 * Version :  $Revision: 1.10 $
 *
 * Originally from TMPI project.
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include <stdarg.h>
# include <string.h>
# include <time.h>
# include "tmpi_debug.h"

# ifndef TMPI_DBG_OUTPUT
# define TMPI_DBG_OUTPUT (stderr)
# endif

static int debug_level=0;
static int debug_line=-1;
static char *debug_file="<unknown>";
static int show_location=1;

static FILE *debug_output=NULL;

int tmpi_setparam(int line, char *file)
{
	debug_line=line;
	debug_file=file;
	
	return 0;
}

void tmpi_show_location()
{
	show_location=1;
}

void tmpi_hide_location()
{
	show_location=0;
}

int tmpi_get_dbg_level()
{
    return(debug_level);
}

void tmpi_set_dbg_level(int level)
{
	if (level>DBG_NONE) level=DBG_NONE;
	if (level<=DBG_ALL) level=DBG_ALL+1;
    debug_level = level;
}

int tmpi_error0(int level, char *fmt, ...)
{
    va_list ap;

	if (!debug_output) {
		debug_output=TMPI_DBG_OUTPUT;
	}
    va_start(ap, fmt);
    if (level <= debug_level)
		return 0;

	if (show_location) {
    	fprintf(debug_output, "[%d]:(%s:%d) ", level, debug_file, debug_line);
	}
    vfprintf(debug_output, fmt, ap);
    /* fprintf(debug_output, "\n"); */
    va_end(ap);
    fflush(debug_output);

	debug_line=-1;
	debug_file="<unknown>";

	return 0;
}

int tmpi_debug(int level)
{
	if (level <= debug_level)
		return 0;
	return 1;
}

int tmpi_error1(int level, int line, char *file, char *fmt, ...)
{
    va_list ap;

	if (!debug_output) {
		debug_output=TMPI_DBG_OUTPUT;
	}

    va_start(ap, fmt);
	debug_line=line;
	debug_file=file;
    if (level <= debug_level)
		return 0;
	if (show_location) {
    	fprintf(debug_output, "[%d]:(%s:%d) ", level, debug_file, debug_line);
	}

    vfprintf(debug_output, fmt, ap);
    va_end(ap);
    fflush(debug_output);

	debug_line=-1;
	debug_file="<unknown>";

	return 0;
}

int tmpi_set_output(FILE *output)
{
	char buf[256];

	if ((!output)||(fileno(output)<0))
		return -1;

	debug_output=output;

	fprintf(debug_output, "Logging started @ %s --->>>\n", tmpi_timestr(buf, 256));
	fflush(debug_output);

	return 0;
}

char *tmpi_timestr(char *buf, int len)
{
	time_t tt;
	struct tm ttm;

	time(&tt);
	memcpy(&ttm, gmtime(&tt), sizeof(struct tm));
	strftime(buf, len, "%a %b %e %T %Y %Z", &ttm);

	return buf;
}
