// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2000, 2001, 2002 The Regents of the University of 
  California, All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: tmpi_debug.h,v $ - header file for tmpi_debug.c
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:52 $
 * Version :  $Revision: 1.9 $
 *
 * Originally from TMPI project.
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _TMPI_DEBUG_H_
# define _TMPI_DEBUG_H_

# if defined(__cplusplus)
extern "C" {
# endif

# include <stdio.h>
# include <stdarg.h>

/**
 * One can set a certain code debug level and 
 * only those that are lower than the compile-
 * time debug level setting will be executed.
 */

/* print all debug messages */
# define DBG_ALL 0
/* print none of the debug messages */
# define DBG_NONE 10000
/** detailed error messages **/
# define DBG_VERBOSE 10
/** system fatal error, e.g. memory, socket, etc **/
# define DBG_FATAL 1000
/** internal error, something might be a result of programming logic error **/
# define DBG_INTERNAL 500
/** application caller error, less fatal than internal error **/
# define DBG_CALLER 400

/** warnings **/
# define DBG_APP 350
# define DBG_WARN1 300
# define DBG_WARN2 290

/* very detailed information */
# define DBG_INFO 50

# if defined(__GNUC__)
# define tmpi_error(l, fmt...) tmpi_error1(l, __LINE__, __FILE__, ## fmt)
# else
# define tmpi_error tmpi_setparam(__LINE__, __FILE__) || tmpi_error0
# endif

int tmpi_set_output(FILE *output);
void tmpi_show_location(void);
void tmpi_hide_location(void);
int tmpi_get_dbg_level(void);
void tmpi_set_dbg_level(int level);
int tmpi_error1(int , int , char *, char *, ...);
int tmpi_error0(int, char *, ...);
int tmpi_setparam(int line, char *file);
int tmpi_debug(int level); /* is current debug level higher than level */
char *tmpi_timestr(char *buf, int len);

# define ASSERT(s) { if (!(s)) tmpi_error(DBG_INTERNAL, "assertion failed\n"); }

# if defined(__cplusplus)
}
# endif

# endif
