// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: fc_test3.cc,v $ - File Cache Tester III.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include <string.h>
# include <stdlib.h>

# include "file_cache.h"

# define OP_ERROR 0
# define OP_OPEN 1
# define OP_CLOSE 2
# define OP_UNLINK 3
# define OP_TRUNCATE 4
# define OP_READ 5
# define OP_WRITE 6
# define OP_SEEK 7
# define OP_CREATE 8
# define OP_PAUSE 9

extern "C" {
# include <readline/readline.h>
# include <readline/history.h>
}

/* extern "C" char *readline(char *prompt); */

static int trans_op(char *start, char *end)
{
	struct OP_TABLE {
		char *op;
		int code;
	} *p=NULL, op_table[]=
	{
		{"OPEN", OP_OPEN},
		{"CLOSE", OP_CLOSE},
		{"UNLINK", OP_UNLINK},
		{"TRUNCATE", OP_TRUNCATE},
		{"READ", OP_READ},
		{"WRITE", OP_WRITE},
		{"SEEK", OP_SEEK},
		{"CREATE", OP_CREATE},
		{"PAUSE", OP_PAUSE},
		{NULL, -1}
	};

	p=op_table;
	while (p->op) {
		int l=strlen(p->op);
		if ((strncmp(start, p->op, l)==0)&&((end-start)==l))
			return p->code;
		p++;
	}

	return OP_ERROR;
}

static void get_int(char *line, char *token, int &i)
{
	char *p=NULL;
	if ((p=strstr(line, token))) {
		p=strchr(p, '=');
		if (p) {
			p++;
			i=atoi(p);
		}
	}
}

static void get_str(char *line, char *token, char *val)
{
	char *ps=NULL, *pe=NULL;
	if ((ps=strstr(line, token))) {
		ps=strchr(ps, '[');
		if (ps) {
			ps++;
			pe=strchr(ps, ']');
			if (pe) {
				strncpy(val, ps, pe-ps);
				val[pe-ps]='\0';
			}
		}
	}
}

int main(int argc, char *argv[])
{
	if (argc<2) {
		fprintf(stderr, "Usage: %s <file>.\n", argv[0]);
		exit(-1);
	}

	FILE *fp=fopen(argv[1], "r");
	if (!fp) {
		fprintf(stderr, "Cannot open file.\n");
		exit(-1);
	}

	File_Cache fcache(512);
	char dummy[128]="test.dummy";

	char line[1024];
	while (fgets(line, 1024, fp)) {
		char *op_start=line+1;
		char *op_end=op_start;
		while (((*op_end)>='A')&&((*op_end)<='Z')) {
			op_end++;
		}

		int op_code=trans_op(op_start, op_end);
		int size=-1, offset=0;
		char fid[64]="", sid[64]="";
		get_int(line, "size", size);
		get_int(line, "offset", offset);
		get_str(line, "fid", fid);
		get_str(line, "sid", sid);
		switch (op_code) {
			case  OP_OPEN:
				fcache.truncate(dummy, size);
				break;
			case  OP_CLOSE:
				break;
			case  OP_UNLINK:
				fcache.unlink(dummy);
				break;
			case  OP_TRUNCATE:
				fcache.truncate(dummy, size);
				break;
			case  OP_READ:
				{
					char data[1024];
					int len=size;
					while (len>0) {
						int len1=(len>1024)?1024:len;
						len1=fcache.read(dummy, offset, data, len1);
						if (len1<=0)
							break;
						len-=len1;
						offset+=len1;
					}
				}
				break;
			case  OP_WRITE:
				{
					char data[1024];
					int len=size;
					while (len>0) {
						int len1=(len>1024)?1024:len;
						len1=fcache.write(dummy, offset, data, len1);
						if (len1<=0)
							break;
						len-=len1;
						offset+=len1;
					}
				}
				break;
			case  OP_SEEK:
				break;
			case  OP_CREATE:
				fcache.touch(dummy);
				break;
			case  OP_PAUSE:
				char *line=readline("> ");
				free(line);
				break;
		}
	}

	return 0;
}
