// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: ch.cc,v $ - Consistent Hashing proposed by D. Karger, et. al.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/16 05:22:49 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <string.h>
# include "../md5/md5.h"
# include "ch.h"

class KEY_GEN {
private:
	unsigned char seed[MD5_LEN];
	unsigned int *uis;
	int ui_idx;

public:
	KEY_GEN(void *input, int len) :
		uis((unsigned int *)&seed),
		ui_idx(0)
	{
		if (len<=0) {
			std::cerr << "Bad key length,\n";
			throw -1;
		}
		Md5::md5((unsigned char *)input, len, seed);
	}

	unsigned int next_key() {
		unsigned int ret=uis[ui_idx];
		ui_idx++;
		if (ui_idx==4) {
			unsigned char old_seed[MD5_LEN];
			memcpy(old_seed, seed, MD5_LEN);
			Md5::md5((unsigned char *)old_seed, MD5_LEN, seed);
			ui_idx=0;
		}

		return ret;
	}
};

static float int2float(unsigned int key)
{
	float x=0.5;
	float ret=0; 
	for (int i=0; i<32; i++, x/=2) {
		if (key&(1U<<(31-i))) {
			ret+=x;
		}
	}

	return ret;
}

int CH::add_bucket(unsigned int buck_id)
{
	if ((max_buck*key_per_buck)<(buck_cnt+key_per_buck)) {
		std::cerr << "Too many buckets,\n";
		throw -1;
	}

	buck_cnt++;
	sorted=false;
	KEY_GEN key_gen(&buck_id, sizeof(unsigned int));

	for (int i=0; i<key_per_buck; i++, key_cnt++) {
		unsigned int key=key_gen.next_key();
		keys[key_cnt].key=int2float(key);
		keys[key_cnt].buck_id=buck_id;
	}

	return 0;
}

static int key_cmp(const void *a, const void *b)
{
	float fa=((CH::KEY_MAP *)a)->key;
	float fb=((CH::KEY_MAP *)b)->key;
	if (fa>fb)
		return 1;
	if (fa<fb)
		return -1;
	return 0;
}

unsigned int CH::hash(unsigned int item)
{
	if (!sorted) {
		qsort(keys, key_cnt, sizeof(KEY_MAP), &key_cmp);
		sorted=true;
	}

	int lb=0, ub=key_cnt-1;
	float ikey=int2float(item);
	if (ikey<=keys[lb].key) {
		return keys[lb].buck_id;
	}

	if (ikey>=keys[ub].key) {
		return keys[ub].buck_id;
	}

	while (lb<(ub-1)) {
		int mid=(lb+ub)/2;
		if (ikey==keys[mid].key) {
			return keys[mid].buck_id;
		}
		if (ikey<keys[mid].key) {
			ub=mid;
		}
		else {
			lb=mid;
		}
	}

	if ((ikey-keys[lb].key)<(keys[ub].key-ikey)) {
		return keys[lb].buck_id;
	}

	if ((ikey-keys[lb].key)>(keys[ub].key-ikey)) {
		return keys[ub].buck_id;
	}

	return (item%2)?(keys[ub].buck_id):(keys[lb].buck_id);
}

int CH::get_bucket_cnt(void)
{
	return buck_cnt;
}
