// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: ch.h,v $ - header file for the Consistent hashing 
 *            implementation proposed by D. Karger, et. al.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/16 05:22:49 $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _CH_H_
# define _CH_H_

# include <iostream>
# include <stdlib.h>
# include <math.h>
/**
 * Consistent-Hashing as proposed by D. Krager, et al. 
 * It uses a heap to sort bucket keys and uses binary
 * search to find the closet bucket for a given item.
 * For each bucket, O(log(N)) keys are generated, where
 * N is the total number of buckets. This is meant to
 * be used for comparison purposes. 
 */ 

# define KAPPA 1

# define KAPPA_LOG 8

class CH {
public:
	struct KEY_MAP {
		float key;
		unsigned int buck_id;
	};

private:
	int max_buck;
	int key_per_buck;
	int buck_cnt;
	int key_cnt;
	KEY_MAP *keys;
	bool sorted;
	
public:
	CH(int max_buckets) :
		max_buck(max_buckets),
		// key_per_buck((int)log(max_buck)*KAPPA),
		key_per_buck(KAPPA_LOG),
		buck_cnt(0),
		key_cnt(0),
		keys(NULL),
		sorted(true)
	{
		if (max_buck>0) {
			keys=(KEY_MAP *)calloc(max_buck*key_per_buck, sizeof(KEY_MAP));
		}
		else {
			std::cerr << "Out of memory.\n";
			throw -1;
		}
	}

	~CH()
	{
		if (keys) {
			free(keys);
		}
	}

	int add_bucket(unsigned int buck_id);
	unsigned int hash(unsigned int item);
	int get_bucket_cnt(void);
};

# endif
