// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t10.cc,v $ - HDCH tester X.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include <math.h>
# include <libgen.h>
# include <getopt.h>
# include <errno.h>
# include <string.h>

# include "../usc/usc_time.h"
# include "hdch.h"

static int random_fd=-1;
static unsigned int get_random_ui()
{
	unsigned int x=0;
	unsigned char *p=(unsigned char *)&x;
	unsigned int n=0;
	while (n<sizeof(int)) {
		int m=read(random_fd, p, (sizeof(int)-n));
		if (m<0) {
			if ((errno==EINTR)||(errno==EAGAIN))
				continue;
			else {
				fprintf(stderr, "Read failed.\n");
				throw -1;
			}
		}
		n+=m;
		p+=m;
	}

	return x;
}

static char *prog_name=NULL;

static void do_help()
{
	printf("\
Usage: %s [options] <#buckets>\n\
   -h                 Help, this screen.\n\
   -n <buckets>       Number of buckets.\n\
   -N <items>         Total number of items.\n\
   -f <rfile>         Getting random numbers from a file.\n\
", prog_name);

	exit(0);
}

int main(int argc, char *argv[])
{
	int n_buckets=0;
	int n_items=10000;
	int c;
	char *random_file="/dev/urandom";
	unsigned int x=0;

	prog_name=argv[0];
	while ((c=getopt(argc, argv, "n:N:i:d:f:h"))!=EOF) {
		switch (c) {
			case 'h': do_help(); break;
			case 'n': n_buckets=atoi(optarg); break;
			case 'N': n_items=atoi(optarg); break;
			case 'f': random_file=optarg; break;
			default: do_help();
		}
	}

	int n_missing_arg=0;
	if (!n_buckets) {
		n_missing_arg++;
	}

	if (n_missing_arg != (argc-optind)) {
		fprintf(stderr, "Syntax error.\n");
		do_help();
	}

	if (!n_buckets) {
		n_buckets=atoi(argv[optind++]);
	}

	random_fd=open(random_file, O_RDONLY);
	if (random_fd<0) {
		fprintf(stderr, "Cannot open file: %s.\n", random_file);
		exit(-1);
	}

	unsigned int *items=(unsigned int *)calloc(n_items, sizeof(unsigned int));
	char buck_ids1[3*n_buckets];
	unsigned int buck_id_base=(128<<24)|(111<<16)|(40<<8)|2;

	for (int i=0; i<n_items; i++) {
		items[i]=get_random_ui();
	}

	memset(buck_ids1, 0, 3*n_buckets);

	for (int i=0; i<n_buckets; i++) {
		int bid=0;
		do {
			bid=rand()%(3*n_buckets);
		} while (buck_ids1[bid]);
		buck_ids1[bid]=1;
	}

	HDCH h1(n_buckets);
	for (int i=0; i<3*n_buckets; i++) {
		if (buck_ids1[i]) {
			unsigned int bid=buck_id_base+i;
			bid=(bid<<24)|((bid&0xff00)<<8)|(bid>>16);
			h1.add_bucket(bid);
		}
	}

	usc_time_t t_start=usc_gettime();
	for (int i=0; i<n_items; i++) {
		unsigned int idx1=h1.hash(items[i]);
		x+=idx1;
	}
	usc_time_t t_end=usc_gettime();
	double t1=usc_difftime(t_end, t_start);

	t_start=usc_gettime();
	for (int i=0; i<n_items; i++) {
		x++;
	}
	t_end=usc_gettime();
	double t2=usc_difftime(t_end, t_start);

	printf("n_bucket=%d, per hash=%.3fus\n", n_buckets, (t1-t2)/n_items);

	return (x)?0:1;
}
