// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t2.cc,v $ - HDCH tester II.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include <math.h>
# ifndef _USE_CH_
# include "hdch.h"
# else
# include "ch.h"
# define HDCH CH
# endif
# include "../guid/guid.h"

using namespace std;
static unsigned int get_random_ui()
{
	Guid guid;
	unsigned int aui[4];
	guid.gen();
	guid.pack((unsigned char *)aui, sizeof(unsigned int)*4);
	return aui[0]^aui[1]^aui[2]^aui[3];
}

# define ITEM_NUM 10000
# define BUCK_NUM 10
int main()
{
	unsigned int items[ITEM_NUM];
	char buck_ids1[256];
	char buck_ids2[256];
	unsigned int buck_id_prefix=(128)|(111<<8)|(40<<16);

	cout << "Testing item distribution." << endl;

	for (int i=0; i<ITEM_NUM; i++) {
		items[i]=get_random_ui();
	}

	memset(buck_ids1, 0, 256);
	memset(buck_ids2, 0, 256);

	for (int i=0; i<BUCK_NUM; i++) {
		int bid=0;
		do {
			bid=rand()%256;
		} while (buck_ids1[bid]);
		buck_ids1[bid]=1;
		buck_ids2[bid]=1;
	}

	int bid=0;
	do {
		bid=rand()%256;
	} while (buck_ids2[bid]);
	buck_ids2[bid]=1;

	HDCH h1(256), h2(256);
	for (int i=0; i<256; i++) {
		if (buck_ids1[i]) {
			h1.add_bucket(buck_id_prefix|((unsigned)i<<24));
		}
	}
	for (int i=0; i<256; i++) {
		if (buck_ids2[i]) {
			h2.add_bucket(buck_id_prefix|((unsigned)i<<24));
		}
	}

	int diff_cnt=0;
	int buck_no1[ITEM_NUM];
	int buck_no2[ITEM_NUM];
	int buck_cnt1[256]; // how many items in each bucket for BUCK_NUM buckets?
	int buck_cnt2[256]; // how many items in each bucket for BUCK_NUM+1 buckets?
	memset(buck_cnt1, 0, sizeof(int)*256);
	memset(buck_cnt2, 0, sizeof(int)*256);

	for (int i=0; i<ITEM_NUM; i++) {
		unsigned int idx1=h1.hash(items[i]);
		unsigned int idx2=h2.hash(items[i]);
		if (idx1!=idx2)
			diff_cnt++;
		idx1>>=24;
		idx2>>=24;
		buck_no1[i]=idx1;
		buck_no2[i]=idx2;
		buck_cnt1[idx1]++;
		buck_cnt2[idx2]++;
	}

	cout << "After increasing # of buckets from "<< BUCK_NUM 
		<< " to " << BUCK_NUM+1 << ":" << endl;
	cout << ITEM_NUM-diff_cnt << " items remain in the same bucket, and " << 
		diff_cnt << " items changed buckets. Percentage is " 
		<< (double)diff_cnt/ITEM_NUM << "." << endl;
	cout << "Item distributions:" << endl;
	double avg=(double)ITEM_NUM/BUCK_NUM;
	double sdv=0;
	for (int i=0; i<256; i++) {
		if (buck_ids1[i]) {
			cout << buck_cnt1[i] << " ";
			sdv+=(double)(buck_cnt1[i]-avg)*(buck_cnt1[i]-avg);
		}
	}
	sdv/=(BUCK_NUM-1);
	sdv=sqrt(sdv);
	cout << endl;
	cout << "Standard deviation = "<< sdv << endl;

	avg=(double)ITEM_NUM/(BUCK_NUM+1);
	sdv=0;
	for (int i=0; i<256; i++) {
		if (buck_ids2[i]) {
			cout << buck_cnt2[i] << " ";
			sdv+=(double)(buck_cnt2[i]-avg)*(buck_cnt2[i]-avg);
		}
	}
	sdv/=(BUCK_NUM);
	sdv=sqrt(sdv);
	cout << endl;
	cout << "Standard deviation = "<< sdv << endl;

	cout << "Test succeed!" << endl;

	return 0;
}
