// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t4.cc,v $ - HDCH tester IV.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include <math.h>
# include "hdch.h"
# include "../guid/guid.h"
# include "../usc/usc_time.h"

using namespace std;

static unsigned int get_random_ui()
{
	Guid guid;
	unsigned int aui[4];
	guid.gen();
	guid.pack((unsigned char *)aui, sizeof(unsigned int)*4);
	return aui[0]^aui[1]^aui[2]^aui[3];
}

# define ITEM_NUM 10000
# define BUCK_NUM 100
int main()
{
	unsigned int items[ITEM_NUM];
	unsigned int buck_id_prefix=(128<<24)|(111<<16)|(40<<8);

	cout << "Performace test: hashing " << ITEM_NUM 
		<< " items into " << BUCK_NUM << " buckets." << endl;

	for (int i=0; i<ITEM_NUM; i++) {
		items[i]=get_random_ui();
	}

	HDCH h1(256);
	char buck_ids[256];
	memset(buck_ids, 0, 256);
	for (int i=0; i<BUCK_NUM; i++) {
		int bid=0;
		do {
			bid=rand()%256;
		} while (buck_ids[bid]);
		buck_ids[bid]=1;
		h1.add_bucket(buck_id_prefix|(unsigned)bid);
	}

	usc_time_t t1, t2;
	usc_init();

	t1=usc_gettime();
	unsigned int idx=0;
	for (int i=0; i<ITEM_NUM; i++) {
		idx+=h1.hash(items[i]);
	}
	t2=usc_gettime();
	cout << "Each hashing takes " << usc_difftime(t2, t1)/ITEM_NUM << "us." << endl;
	cout << "Final signature is "; HDCH::print_key(idx); cout << endl;

	cout << "Test succeed!" << endl;

	return 0;
}
