// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t7.cc,v $ - HDCH tester VII.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include <math.h>
# include "hdch.h"
# include "../guid/guid.h"

using namespace std;

static unsigned int get_random_ui()
{
	Guid guid;
	unsigned int aui[4];
	guid.gen();
	guid.pack((unsigned char *)aui, sizeof(unsigned int)*4);
	return aui[0]^aui[1]^aui[2]^aui[3];
}

# define ITEM_NUM 100000
# define BUCK_NUM 1000
# define BUCK_NUM2 (BUCK_NUM/100)
# define RANGE 4096

int main()
{
	unsigned int items[ITEM_NUM];

	cout << "Testing item distribution for buckets of small integers." << endl;

	for (int i=0; i<ITEM_NUM; i++) {
		items[i]=get_random_ui();
	}

	HDCH h1(RANGE), h2(RANGE);
	for (int i=0; i<BUCK_NUM; i++) {
		h1.add_bucket((unsigned int)(i+1));
	}

	for (int i=0; i<BUCK_NUM+BUCK_NUM2; i++) {
		h2.add_bucket((unsigned int)(i+1));
	}

	cout << "Buckets: " << h1.get_bucket_cnt() << "->" << h2.get_bucket_cnt() << endl;

	int diff_cnt=0;
	int buck_no1[ITEM_NUM];
	int buck_no2[ITEM_NUM];
	int buck_cnt1[RANGE]; // how many items in each bucket for BUCK_NUM buckets?
	int buck_cnt2[RANGE]; // how many items in each bucket for BUCK_NUM+BUCK_NUM2 buckets?
	memset(buck_cnt1, 0, sizeof(int)*RANGE);
	memset(buck_cnt2, 0, sizeof(int)*RANGE);

	for (int i=0; i<ITEM_NUM; i++) {
		unsigned int idx1=h1.hash(items[i]);
		unsigned int idx2=h2.hash(items[i]);
		if (idx1!=idx2)
			diff_cnt++;
		buck_no1[i]=idx1;
		buck_no2[i]=idx2;
		buck_cnt1[idx1]++;
		buck_cnt2[idx2]++;
	}

	cout << "After increasing # of buckets from "<< BUCK_NUM 
		<< " to " << BUCK_NUM+BUCK_NUM2 << ":" << endl;
	cout << ITEM_NUM-diff_cnt << " items remain in the same bucket, and " << 
		diff_cnt << " items changed buckets. Percentage is " 
		<< (double)diff_cnt/ITEM_NUM << "." << endl;
	cout << "Item distributions (first 10 buckets):" << endl;
	double avg=(double)ITEM_NUM/BUCK_NUM;
	double sdv=0;

	int sum=0;
	for (int i=1; i<=BUCK_NUM; i++) {
		sum+=buck_cnt1[i];
		if (i<=10) {
			cout << buck_cnt1[i] << " ";
		}
		sdv+=(double)(buck_cnt1[i]-avg)*(buck_cnt1[i]-avg);
	}

	sdv/=(BUCK_NUM-1);
	sdv=sqrt(sdv);
	cout << endl;

	if (sum!=ITEM_NUM) {
		cout << "sum1 wrong! Test failed!!!!!" << endl;
		return -1;
	}
	cout << "Standard deviation = "<< sdv << endl;

	avg=(double)ITEM_NUM/(BUCK_NUM+BUCK_NUM2);
	sdv=0;
	sum=0;
	int mov_buck=0;
	for (int i=1; i<=BUCK_NUM+BUCK_NUM2; i++) {
		sum+=buck_cnt2[i];
		if (i<=10) {
			cout << buck_cnt2[i] << " ";
		}
		sdv+=(double)(buck_cnt2[i]-avg)*(buck_cnt2[i]-avg);
		if (i<=BUCK_NUM) {
			if (buck_cnt1[i]!=buck_cnt2[i]) {
				mov_buck++;
			}
		}
	}
	sdv/=(BUCK_NUM+BUCK_NUM2-1);
	sdv=sqrt(sdv);
	cout << endl;
	if (sum!=ITEM_NUM) {
		cout << "sum2 wrong! Test failed!!!!!" << endl;
		return -1;
	}
	cout << "Standard deviation = "<< sdv << endl;
	cout << mov_buck << " buckets contributed itmes to the addtional " << BUCK_NUM2 
		<< " buckets." << endl;
	cout << "Test succeed!" << endl;

	return 0;
}
