// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: atomic.debug.h,v $ - debugging version of atomic.h. It is only
 *            correct under sequential execution. Comparing the result
 *            of the same program using both atomic.h and atomic_debug.h 
 *            under sequential execution.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _ATOMIC_H_
# define _ATOMIC_H_

# if defined(__cplusplus)
extern "C" {
# endif

inline char cas16(volatile unsigned short *addr, unsigned register short old, unsigned register short nw)
{
	if (*addr==old) {
		*addr=nw;
		return 1;
	}

	return 0;
}

inline char cas16s(volatile short *addr, register short old, register short nw)
{
	if (*addr==old) {
		*addr=nw;
		return 1;
	}

	return 0;
}

inline char cas32(volatile unsigned int *addr, unsigned register int old, unsigned register int nw)
{
	if (*addr==old) {
		*addr=nw;
		return 1;
	}

	return 0;
}

inline char cas32s(volatile int *addr, register int old, register int nw)
{
	if (*addr==old) {
		*addr=nw;
		return 1;
	}

	return 0;
}

/* test and set */
# define tas32(addr, bit) \
	(__extension__ \
	 ( { \
	   	register char oldbit;  \
		oldbit = ((*addr) & 0x1U<<(bit))?1:0; \
		*addr = (*addr) | (0x1U<<(bit)) \
		oldbit; \
	   } \
	 ) \
	)

/* test and clear */
# define tac32(addr, bit) \
	(__extension__ \
	 ( { \
	   	register char oldbit;  \
		oldbit = ((*addr) & 0x1U<<(bit))?1:0; \
		*addr = (*addr) & (~(0x1U<<(bit))) \
		oldbit; \
	   } \
	 ) \
	)

/* test and flip bit */
# define taf32(addr, bit) \
	(__extension__ \
	 ( { \
	   	register char oldbit;  \
		oldbit = ((*addr) & 0x1U<<(bit))?1:0; \
		*addr = (*addr) ^ (0x1U<<(bit)) \
		oldbit; \
	   } \
	 ) \
	)

/* add and fetch */
inline unsigned int anf32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old+val));

	return old+val;
}

inline int anf32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old+val));

	return old+val;
}

inline unsigned short anf16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old+val));

	return old+val;
}

inline short anf16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old+val));

	return old+val;
}

/* subtract and fetch */
inline unsigned int snf32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old-val));

	return old-val;
}

inline int snf32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old-val));

	return old-val;
}

inline unsigned short snf16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old-val));

	return old-val;
}

inline short snf16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old-val));

	return old-val;
}

/* fetch and add */
inline unsigned int fna32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old+val));

	return old;
}

inline int fna32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old+val));

	return old;
}

inline unsigned short fna16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old+val));

	return old;
}

inline short fna16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old+val));

	return old;
}

/* fetch and subtract */
inline unsigned int fns32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old-val));

	return old;
}

inline int fns32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old-val));

	return old;
}


inline unsigned short fns16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old-val));

	return old;
}

inline short fns16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old-val));

	return old;
}

/** commit a memory cell **/
# define commit32(ptr32) do { } while (0)

# define inc32(ptr32) \
	do { (*((unsigned int *)ptr32))++; } while (0)

# define inc16(ptr16) \
	do { (*((unsigned short *)ptr16))++; } while (0)

# define dec32(ptr32) \
	do { (*((unsigned int *)ptr32))--; } while (0)

# define dec16(ptr16) \
	do { (*((unsigned short *)ptr16))--; } while (0)

# define aadd32(pval, val) \
	do { *((unsigned int *)pval)+=(unsigned int)(val); } while (0)

# define aadd16(pval, val) \
	do { *((unsigned short *)pval)+=(unsigned short)(val); } while (0)

# define asub32(pval, val) \
	do { *((unsigned int *)pval)-=(unsigned int)(val); } while (0)

# define asub16(pval, val) \
	do { *((unsigned short *)pval)+=(unsigned short)(val); } while (0)

# if defined(__cplusplus)
}
# endif

# endif
