// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: atomic.h,v $ - Atomic synchronization primitives.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/11/08 00:37:00 $
 * Version :  $Revision: 1.10 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _ATOMIC_H_
# define _ATOMIC_H_

# if defined(__cplusplus)
extern "C" {
# endif

/* we can rely on smp_def.inc to detect whether
 * the underlying architecture is SMP or not.
 * But it is troublesome to add it in every Makefile
 * and this makes the binary less portable.
 * So I simply define it here.
 */
# ifndef __SMP__
# define __SMP__
# endif

# ifdef __SMP__
# define LOCK_PREFIX "lock ; "
# else
# define LOCK_PREFIX ""
# endif

/* some hacks to defeat gcc over-optimizations */
struct __dummy0 { unsigned long a[100]; };
#define SCS_ADDR(addr) (*(volatile struct __dummy0 *)(addr))

inline char cas16(volatile unsigned short *addr, unsigned register short old, unsigned register short nw)
{
	register char ret;
	__asm__ __volatile__ (
	LOCK_PREFIX "cmpxchg %2, %1\n\t"
				"sete %0"
				: "=q" (ret), "=m" (SCS_ADDR(addr))
				: "r" (nw), "a" (old)
	);

	return ret;
}

inline char cas16s(volatile short *addr, register short old, register short nw)
{
	register char ret;
	__asm__ __volatile__ (
	LOCK_PREFIX "cmpxchg %2, %1\n\t"
				"sete %0"
				: "=q" (ret), "=m" (SCS_ADDR(addr))
				: "r" (nw), "a" (old)
	);

	return ret;
}

inline char cas32(volatile unsigned int *addr, unsigned register int old, unsigned register int nw)
{
	char ret;
	__asm__ __volatile__ (
	LOCK_PREFIX "cmpxchg %2, %1\n\t"
				"sete %0"
				: "=q" (ret), "=m" (SCS_ADDR(addr))
				: "r" (nw), "a" (old)
	);

	return ret;
}

inline char cas32s(volatile int *addr, register int old, register int nw)
{
	char ret;
	__asm__ __volatile__ (
	LOCK_PREFIX "cmpxchg %2, %1\n\t"
				"sete %0"
				: "=q" (ret), "=m" (SCS_ADDR(addr))
				: "r" (nw), "a" (old)
	);

	return ret;
}

/* cas64 and casd in assembly cas64.s */
int cas64(volatile unsigned long long *pval, unsigned long long old, unsigned long long newv);

# define cas64s(pval, old, newv) \
	(cas64((volatile unsigned long long *)(pval), (unsigned long long)(old), (unsigned long long)(newv)))

int casd(volatile double *pval, double old, double newv);
unsigned long long anf64(volatile unsigned long long *pval, unsigned long long val);
unsigned long long snf64(volatile unsigned long long *pval, unsigned long long val);
unsigned long long fna64(volatile unsigned long long *pval, unsigned long long val);
unsigned long long fns64(volatile unsigned long long *pval, unsigned long long val);

# define anf64s(pval, val) \
	((long long)(anf64((volatile unsigned long long *)(pval), (unsigned long long)(val))))

# define snf64s(pval, val) \
	((long long)(snf64((volatile unsigned long long *)(pval), (unsigned long long)(val))))

# define fna64s(pval, val) \
	((long long)(fna64((volatile unsigned long long *)(pval), (unsigned long long)(val))))

# define fns64s(pval, val) \
	((long long)(fns64((volatile unsigned long long *)(pval), (unsigned long long)(val))))

/* test and set */
# define tas32(addr, bit) \
	(__extension__ \
	 ( { \
	   	register int oldbit;  \
		__asm__ __volatile__ ( \
		LOCK_PREFIX "btsl %2, %1\n\t" \
					"sbbl %0, %0" \
					: "=r" (oldbit), "=m" (SCS_ADDR(addr)) \
					: "Ir" (bit) \
		); \
		-oldbit; \
	   } \
	 ) \
	)

/* test and clear */
# define tac32(addr, bit) \
	(__extension__ \
	 ( { \
	   	register int oldbit;  \
		__asm__ __volatile__ ( \
		LOCK_PREFIX "btrl %2, %1\n\t" \
					"sbbl %0, %0" \
					: "=r" (oldbit), "=m" (SCS_ADDR(addr)) \
					: "Ir" (bit) \
		); \
		-oldbit; \
	   } \
	 ) \
	)

/* test and flip bit */
# define taf32(addr, bit) \
	(__extension__ \
	 ( { \
	   	register int oldbit;  \
		__asm__ __volatile__ ( \
		LOCK_PREFIX "btcl %2, %1\n\t" \
					"sbbl %0, %0" \
					: "=r" (oldbit), "=m" (SCS_ADDR(addr)) \
					: "Ir" (bit) \
		); \
		-oldbit; \
	   } \
	 ) \
	)

/* add and fetch */
inline unsigned int anf32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old+val));

	return old+val;
}

inline int anf32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old+val));

	return old+val;
}

inline unsigned short anf16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old+val));

	return old+val;
}

inline short anf16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old+val));

	return old+val;
}

/* subtract and fetch */
inline unsigned int snf32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old-val));

	return old-val;
}

inline int snf32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old-val));

	return old-val;
}

inline unsigned short snf16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old-val));

	return old-val;
}

inline short snf16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old-val));

	return old-val;
}

/* fetch and add */
inline unsigned int fna32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old+val));

	return old;
}

inline int fna32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old+val));

	return old;
}

inline unsigned short fna16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old+val));

	return old;
}

inline short fna16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old+val));

	return old;
}

/* fetch and subtract */
inline unsigned int fns32(volatile unsigned int *ptr, unsigned int val)
{
	unsigned int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32(ptr, old, old-val));

	return old;
}

inline int fns32s(volatile int *ptr, int val)
{
	int old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas32s(ptr, old, old-val));

	return old;
}


inline unsigned short fns16(volatile unsigned short *ptr, unsigned short val)
{
	unsigned short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16(ptr, old, old-val));

	return old;
}

inline short fns16s(volatile short *ptr, short val)
{
	short old;
	
	if (!val) return *ptr;

	do {
		old=*ptr;
	} while (!cas16s(ptr, old, old-val));

	return old;
}

/** commit a memory cell **/
#define MEMORY_BARRIER() asm __volatile__ ( " " : : : "memory" )

# ifndef __SMP__ 
# define commit32(ptr32) do { } while (0)
# else
# define commit32(ptr32) \
	asm __volatile__ ( \
		LOCK_PREFIX "addl $0, %0\n\t" \
					: "=m" (SCS_ADDR(ptr32)) \
					: \
	)
# endif

# ifndef __SMP__ 
# define commit64(ptr64) do { } while (0)
# else
# define commit64(ptr64) \
	asm __volatile__ ( \
		LOCK_PREFIX "addl $0, %0\n\t" \
		LOCK_PREFIX "addl $0, %1\n\t" \
					: "=m" (SCS_ADDR(ptr64)), "=m"(SCS_ADDR((char *)(ptr64)+4)) \
					: \
	)
# endif

# define inc32(ptr32) \
	asm __volatile__ ( \
		LOCK_PREFIX "addl $1, %0" \
					: "=m"(SCS_ADDR(ptr32)) \
					: \
	)

# define inc16(ptr16) \
	asm __volatile__ ( \
		LOCK_PREFIX "addw $1, %0" \
					: "=m"(SCS_ADDR(ptr16)) \
					: \
	)

# define dec32(ptr32) \
	asm __volatile__ ( \
		LOCK_PREFIX "subl $1, %0" \
					: "=m"(SCS_ADDR(ptr32)) \
					: \
	)

# define dec16(ptr16) \
	asm __volatile__ ( \
		LOCK_PREFIX "subw $1, %0" \
					: "=m"(SCS_ADDR(ptr16)) \
					: \
	)

# define aadd32(pval, val) \
	asm __volatile__ ( \
		LOCK_PREFIX "addl %1, %0" \
					: "=m"(SCS_ADDR(pval)) \
					: "r"(val) \
	)

# define aadd16(pval, val) \
	asm __volatile__ ( \
		LOCK_PREFIX "addw %1, %0" \
					: "=m"(SCS_ADDR(pval)) \
					: "r"((short)val) \
	)

# define asub32(pval, val) \
	asm __volatile__ ( \
		LOCK_PREFIX "subl %1, %0" \
					: "=m"(SCS_ADDR(pval)) \
					: "r"(val) \
	)

# define asub16(pval, val) \
	asm __volatile__ ( \
		LOCK_PREFIX "subw %1, %0" \
					: "=m"(SCS_ADDR(pval)) \
					: "r"((short)val) \
	)

# define bswap32(x) \
	(__extension__ \
	 ( { register unsigned int __v;  \
		if (__builtin_constant_p (x)) \
	   		__v = ((((x) & 0xff000000) >> 24) | \
				(((x) & 0x00ff0000) >> 8) | \
				(((x) & 0x0000ff00) << 8) | \
				(((x) & 0x000000ff) << 24)); \
			else \
				__asm__ __volatile__ ( \
					"bswap %0" \
					: "=r" (__v) \
					: "0" ((unsigned int) (x))\
				); \
			__v; \
		} \
	 ) \
	)

# if defined(__cplusplus)
}
# endif

# endif
