// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2000, Teoma Technologies, Inc.
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: md5.cc,v $ - MD5 digestion.
 * Authors :  Wei Wang, ww@teoma.com, Teoma Technologies,
 *            Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
#include <stdio.h>
#include <string.h>
#include "md5.h"

typedef unsigned char ub1;
typedef unsigned int ub4;

// static ub1 zeromd5[MD5_LEN]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

/*[]-----------------------------------------[]
  |  Local preprocessor defines               |
  []-----------------------------------------[]*/

/* Constants for MD5Transform routine. */
#define S11 7
#define S12 12
#define S13 17
#define S14 22
#define S21 5
#define S22 9
#define S23 14
#define S24 20
#define S31 4
#define S32 11
#define S33 16
#define S34 23
#define S41 6
#define S42 10
#define S43 15
#define S44 21

static ub1 padding[64] = {
	0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

/* F, G, H and I are basic MD5 functions. */
static inline ub4 F(ub4 x, ub4 y, ub4 z)
{
	return ((x & y) | ((~x) & z));
}

static inline ub4 G(ub4 x, ub4 y, ub4 z)
{
	return ((x & z) | (y & (~z)));
}

static inline ub4 H(ub4 x, ub4 y, ub4 z)
{
	return (x ^ y ^ z);
}

static inline ub4 I(ub4 x, ub4 y, ub4 z)
{
	return (y ^ (x | (~z)));
}

/* RotL rotates x left n bits. */
static inline ub4 RotL(ub4 x, int n)
{
	return ((x << n) | (x >> (32-n)));
}

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
   Rotation is separate from addition to prevent recomputation.
*/
static inline void FF(ub4& a, ub4 b, ub4 c, ub4 d, ub4 x, ub4 s, ub4 ac)
{
	a += F(b, c, d) + x + ac;
	a = RotL(a, s);
	a += b;
}

static inline void GG(ub4& a, ub4 b, ub4 c, ub4 d, ub4 x, ub4 s, ub4 ac)
{
	a += G(b, c, d) + x + ac;
	a = RotL(a, s);
	a += b;
}

static inline void HH(ub4& a, ub4 b, ub4 c, ub4 d, ub4 x, ub4 s, ub4 ac)
{
	a += H(b, c, d) + x + ac;
	a = RotL(a, s);
	a += b;
}

static inline void II(ub4& a, ub4 b, ub4 c, ub4 d, ub4 x, ub4 s, ub4 ac)
{
	a += I(b, c, d) + x + ac;
	a = RotL(a, s);
	a += b;
}

/* MD5 initialization. Begins an MD5 operation, writing a new context. */
static inline void md5Init(ub4 state[4])
{
	state[0] = 0x67452301;
	state[1] = 0xefcdab89;
	state[2] = 0x98badcfe;
	state[3] = 0x10325476;
}

/* Decodes input (ub1) into output (ub4). Assumes len is a multiple of 4. */
static inline void decode(ub4* output, ub1* input, int len)
{
	for (int i=0,j=0; j < len; j+=4,i++)
		output[i] = input[j] | (input[j+1] << 8) | (input[j+2] << 16) |
			(input[j+3] << 24);
}

/* Encodes input (ub4) into output (ub1). Assumes len is a multiple of 4. */
static inline void byte_encode(ub1* output, ub4* input, int len)
{
	ub4 temp;
	for (int i=0,j=0; j < len; j+=4,i++) {
		temp = input[i];
		output[j]   = temp & 0xff;
		output[j+1] = (temp >> 8) & 0xff;
		output[j+2] = (temp >> 16) & 0xff;
		output[j+3] = (temp >> 24) & 0xff;
	}
}

/* MD5 basic transformation. Transforms state based on block. */
static void md5Transform(ub4 state[4], ub1 block[64])
{
	ub4 a = state[0];
	ub4 b = state[1];
	ub4 c = state[2];
	ub4 d = state[3];

	ub4 x[16];
	decode(x, block, 64);

	/* Round 1 */
	FF(a, b, c, d, x[ 0], S11, 0xd76aa478); /* 1 */
	FF(d, a, b, c, x[ 1], S12, 0xe8c7b756); /* 2 */
	FF(c, d, a, b, x[ 2], S13, 0x242070db); /* 3 */
	FF(b, c, d, a, x[ 3], S14, 0xc1bdceee); /* 4 */
	FF(a, b, c, d, x[ 4], S11, 0xf57c0faf); /* 5 */
	FF(d, a, b, c, x[ 5], S12, 0x4787c62a); /* 6 */
	FF(c, d, a, b, x[ 6], S13, 0xa8304613); /* 7 */
	FF(b, c, d, a, x[ 7], S14, 0xfd469501); /* 8 */
	FF(a, b, c, d, x[ 8], S11, 0x698098d8); /* 9 */
	FF(d, a, b, c, x[ 9], S12, 0x8b44f7af); /* 10 */
	FF(c, d, a, b, x[10], S13, 0xffff5bb1); /* 11 */
	FF(b, c, d, a, x[11], S14, 0x895cd7be); /* 12 */
	FF(a, b, c, d, x[12], S11, 0x6b901122); /* 13 */
	FF(d, a, b, c, x[13], S12, 0xfd987193); /* 14 */
	FF(c, d, a, b, x[14], S13, 0xa679438e); /* 15 */
	FF(b, c, d, a, x[15], S14, 0x49b40821); /* 16 */

	/* Round 2 */
	GG(a, b, c, d, x[ 1], S21, 0xf61e2562); /* 17 */
	GG(d, a, b, c, x[ 6], S22, 0xc040b340); /* 18 */
	GG(c, d, a, b, x[11], S23, 0x265e5a51); /* 19 */
	GG(b, c, d, a, x[ 0], S24, 0xe9b6c7aa); /* 20 */
	GG(a, b, c, d, x[ 5], S21, 0xd62f105d); /* 21 */
	GG(d, a, b, c, x[10], S22,  0x2441453); /* 22 */
	GG(c, d, a, b, x[15], S23, 0xd8a1e681); /* 23 */
	GG(b, c, d, a, x[ 4], S24, 0xe7d3fbc8); /* 24 */
	GG(a, b, c, d, x[ 9], S21, 0x21e1cde6); /* 25 */
	GG(d, a, b, c, x[14], S22, 0xc33707d6); /* 26 */
	GG(c, d, a, b, x[ 3], S23, 0xf4d50d87); /* 27 */
	GG(b, c, d, a, x[ 8], S24, 0x455a14ed); /* 28 */
	GG(a, b, c, d, x[13], S21, 0xa9e3e905); /* 29 */
	GG(d, a, b, c, x[ 2], S22, 0xfcefa3f8); /* 30 */
	GG(c, d, a, b, x[ 7], S23, 0x676f02d9); /* 31 */
	GG(b, c, d, a, x[12], S24, 0x8d2a4c8a); /* 32 */

	/* Round 3 */
	HH(a, b, c, d, x[ 5], S31, 0xfffa3942); /* 33 */
	HH(d, a, b, c, x[ 8], S32, 0x8771f681); /* 34 */
	HH(c, d, a, b, x[11], S33, 0x6d9d6122); /* 35 */
	HH(b, c, d, a, x[14], S34, 0xfde5380c); /* 36 */
	HH(a, b, c, d, x[ 1], S31, 0xa4beea44); /* 37 */
	HH(d, a, b, c, x[ 4], S32, 0x4bdecfa9); /* 38 */
	HH(c, d, a, b, x[ 7], S33, 0xf6bb4b60); /* 39 */
	HH(b, c, d, a, x[10], S34, 0xbebfbc70); /* 40 */
	HH(a, b, c, d, x[13], S31, 0x289b7ec6); /* 41 */
	HH(d, a, b, c, x[ 0], S32, 0xeaa127fa); /* 42 */
	HH(c, d, a, b, x[ 3], S33, 0xd4ef3085); /* 43 */
	HH(b, c, d, a, x[ 6], S34,  0x4881d05); /* 44 */
	HH(a, b, c, d, x[ 9], S31, 0xd9d4d039); /* 45 */
	HH(d, a, b, c, x[12], S32, 0xe6db99e5); /* 46 */
	HH(c, d, a, b, x[15], S33, 0x1fa27cf8); /* 47 */
	HH(b, c, d, a, x[ 2], S34, 0xc4ac5665); /* 48 */

	/* Round 4 */
	II(a, b, c, d, x[ 0], S41, 0xf4292244); /* 49 */
	II(d, a, b, c, x[ 7], S42, 0x432aff97); /* 50 */
	II(c, d, a, b, x[14], S43, 0xab9423a7); /* 51 */
	II(b, c, d, a, x[ 5], S44, 0xfc93a039); /* 52 */
	II(a, b, c, d, x[12], S41, 0x655b59c3); /* 53 */
	II(d, a, b, c, x[ 3], S42, 0x8f0ccc92); /* 54 */
	II(c, d, a, b, x[10], S43, 0xffeff47d); /* 55 */
	II(b, c, d, a, x[ 1], S44, 0x85845dd1); /* 56 */
	II(a, b, c, d, x[ 8], S41, 0x6fa87e4f); /* 57 */
	II(d, a, b, c, x[15], S42, 0xfe2ce6e0); /* 58 */
	II(c, d, a, b, x[ 6], S43, 0xa3014314); /* 59 */
	II(b, c, d, a, x[13], S44, 0x4e0811a1); /* 60 */
	II(a, b, c, d, x[ 4], S41, 0xf7537e82); /* 61 */
	II(d, a, b, c, x[11], S42, 0xbd3af235); /* 62 */
	II(c, d, a, b, x[ 2], S43, 0x2ad7d2bb); /* 63 */
	II(b, c, d, a, x[ 9], S44, 0xeb86d391); /* 64 */

	state[0] += a;
	state[1] += b;
	state[2] += c;
	state[3] += d;
}

/* This function is ok for string with length <= 536870912
   Otherwise, count[0..1] should be changed to reflect the actual
   bits in string.
*/
static void md5common(ub1* string, int length, ub4 state[4])
{
	ub1 buffer[64];	/* input buffer */

	md5Init(state);

	ub1* input = string;
	int  len = length;
	while (len >= 64) {
		md5Transform(state, input);
		len -= 64;
		input += 64;
	}

	ub4 count[2];
	if (len < 56) {
		for (int i=0; i < len; i++)
			buffer[i] = input[i];
		for (int j=0,i=len; i < 56; i++,j++)
			buffer[i] = padding[j];
		count[0] = length << 3;
		count[1] = 0;
		byte_encode(buffer+56, count, 8);
		md5Transform(state, buffer);
	} else {
		for (int i=0; i < len; i++)
			buffer[i] = input[i];
		for (int j=0,i=len; i < 64; i++,j++)
			buffer[i] = padding[j];
		md5Transform(state, buffer);
		for (int j=0,i=64-len; j < 56; i++,j++)
			buffer[j] = padding[i];
		count[0] = length << 3;
		count[1] = 0;
		byte_encode(buffer+56, count, 8);
		md5Transform(state, buffer);
	}
}

/*[]-----------------------------------------[]
  |  Md5 member functions               |
  []-----------------------------------------[]*/

void Md5::md5(ub1* string, int length, ub1 digest[MD5_LEN])
{
	ub4 state[4];
	md5common(string, length, state);
	byte_encode(digest, state, MD5_LEN);
}


ub4 Md5::md5hash(ub1 *string, int length)
{
	ub4 state[4];
	md5common(string, length, state);
	return (state[0] + state[1] + state[2] + state[3]);
}

Md5::Md5()
{
	md5Init(state_);
	buf_len_ = 0;
	tot_len_ = 0;
}

void Md5::reset(void)
{
	md5Init(state_);
	buf_len_ = 0;
	tot_len_ = 0;
}

void Md5::stuff(char* string, int len)
{
	ub1* input = (ub1 *) string;
	tot_len_ += len;

	if (buf_len_ > 0) {
		int need = 64 - buf_len_;
		if (need <= len) {
			memcpy(buf_+buf_len_, input, need);
			md5Transform(state_, buf_);
			buf_len_ = 0;
			input += need;
			len -= need;
		} else {
			memcpy(buf_+buf_len_, input, len);
			buf_len_ += len;
			len = 0;
		}
	}
	// at this point, either buf_len_ or len is 0

	while (len >= 64) {
		md5Transform(state_, input);
		len -= 64;
		input += 64;
	}

	if (len > 0) {
		memcpy(buf_, input, len);
		buf_len_ = len;
	}
}

void Md5::close(void)
{
	ub4 count[2];
	if (buf_len_ < 56) {
		for (int j=0,i=buf_len_; i < 56; i++,j++)
			buf_[i] = padding[j];
		count[0] = tot_len_ << 3;
		count[1] = 0;
		byte_encode(buf_+56, count, 8);
		md5Transform(state_, buf_);
	} else {
		for (int j=0,i=buf_len_; i < 64; i++,j++)
			buf_[i] = padding[j];
		md5Transform(state_, buf_);
		for (int j=0,i=64-buf_len_; j < 56; i++,j++)
			buf_[j] = padding[i];
		count[0] = tot_len_ << 3;
		count[1] = 0;
		byte_encode(buf_+56, count, 8);
		md5Transform(state_, buf_);
	}
}

ub4 Md5::hash(void)
{
	return (state_[0] + state_[1] + state_[2] + state_[3]);
}

void Md5::pack(ub1 digest[MD5_LEN])
{
	byte_encode(digest, state_, 16);
}
