// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: pshash.h,v $ - header file for pshash.cc, a persistent hash
 *            table.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.7 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _PSHASH_H_
# define _PSHASH_H_

/* # include <db_cxx.h> */
# ifndef MAX_FILE_NAME
# define MAX_FILE_NAME 256
# endif

/**
 * The following is better to be replaced with a more dynamic solution.
 * Since it requires root previlige to obtain the file system parameters,
 * we are not able to get the I/O page size in the runtime. 
 * One way to get the I/O blcok size is using file system dump utilities. 
 * For example, the I/O block size for an ext2 file system can be obtained
 * through the following command:
 * % dumpe2fs /dev/hda6 | head -40 | grep Block
 * Then look for a line started with "Block size".
 *
 * Update: a function get_io_page_size has been added into pshash.cc
 * which uses the statfs() call to get the I/O block size for the underlying
 * file system. And the macro here IO_PAGE_SIZE now is merely a hint when
 * we are not able to determine the real I/O block size.
 */
# ifndef IO_PAGE_SIZE
# define IO_PAGE_SIZE 1024
# endif

class Db;
class Dbc;

/**
 * Persistent hashing
 */
class PsHash {
public:
	PsHash() : db(NULL), dbc(NULL) { return; }
	PsHash(char *backstore, unsigned int keylen, unsigned int datalen, unsigned int nelem, unsigned int cache_sz); // create a new hash table
	PsHash(char *backstore, unsigned int cache_sz); // load an existing hash table
	~PsHash();
	int load(char *backstore, unsigned int cache_sz);
	int create(char *backstore, unsigned int keylen, unsigned int datalen, unsigned int nelem, unsigned int cache_sz);
	int release();
	int get_backstore(char *backstore, unsigned int len);
	int set(void *key, unsigned int klen, void *data, unsigned int dlen);
	int get(void *key, unsigned int klen, void *data, unsigned int *dlen);
	int del(void *key, unsigned int len);
	
	// traversing the table
	int get_first(void *key, unsigned int *klen, void *data, unsigned int *dlen);
	int get_last(void *key, unsigned int *klen, void *data, unsigned int *dlen);
	int get_next(void *key, unsigned int *klen, void *data, unsigned int *dlen);
	int get_current(void *key, unsigned int *klen, void *data, unsigned int *dlen);
	int get_prev(void *key, unsigned int *klen, void *data, unsigned int *dlen);
	bool sync();
	
	// do we have a hash table loaded yet?
	int is_loaded();
private :
	Db *db;
	Dbc *dbc; // created on demand, and will never be destroyed till the hash table is to be released.
	char bsfile[MAX_FILE_NAME];
	int set_db_cache(unsigned int cache_sz, unsigned int io_page_size);
	int set_db_hash(unsigned int keylen, unsigned int datalen, unsigned int nelem, unsigned int io_page_size);
	int cursor_get(void *key, unsigned int *klen, void *data, unsigned int *dlen, int flags);
};

# endif
