// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t3.cc,v $ - persistent hashing tester III.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <iostream>
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>
# include <string.h>
# include "pshash.h"

using namespace std;

int main()
{
	int ret=0;
	int vals[100];

	cout << "Testing hash table set/get." << endl;

	FILE *fp=NULL;
	PsHash ph1, ph2;
	int i;
	int set_hash_err=0;
	int get_hash_err=0;

	unlink("t3.db");
	fp=fopen("t3.db", "rb");
	if (fp) {
		cout << "Failed to perform per-test cleanup: cannot remove t3.db!" << endl;
		ret=-1;
		goto _exit;
	}

	if (ph1.create("t3.db", 4, 4, 100, 100)) {
		cout << "PsHash create() failed!" << endl;
		ret=-1;
		goto _exit;
	}

	memset(vals, 0, sizeof(int)*100);

	for (i=0; i<100; i++) {
		int idx=rand()%100;
		int val=rand()%10000+1;

		if (!vals[idx]) {
			vals[idx]=val;
			if (ph1.set(&idx, sizeof(int), &val, sizeof(int))) {
				set_hash_err++;
			}
		}
	}

	if (set_hash_err) {
		cout << "PsHash set() failed!" << endl;
		ret=-1;
		goto _exit;
	}

	ph1.release();

	ph2.load("t3.db", 100);
	if (!ph2.is_loaded()) {
		cout << "PsHash load() failed!" << endl;
		ret=-1;
		goto _exit;
	}

	for (i=0; i<100; i++) {
		int val=0;
		int ret=0;
		unsigned int size=sizeof(int);
		ret=ph2.get(&i, sizeof(int), &val, &size);
		if ( ( (!vals[i]) && (ret==0) ) ||
				( (vals[i]) && (ret!=0) ) ||
				( (vals[i]) && (vals[i]!=val) ) ) {
				get_hash_err++;
		}
	}

	if (get_hash_err) {
		cout << "PsHash get() failed!" << endl;
		ret=-1;
		goto _exit;
	}

	ph2.release();

	cout << "Test passed!" << endl;

_exit:
	ph1.release();
	ph2.release();
	return ret;
}
