// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t1.cc,v $ - testing program for unit ring.
 *            this program tests the addition of elements into the ring,
 *            and whether the keys are sorted on the ring.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/11 08:28:26 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include "unit_ring.h"

//# define HOST_VNODE_MIN 10
# define HOST_VNODE_MIN 1
//# define HOST_VNODE_MAX 50
# define HOST_VNODE_MAX 5

//# define NUM_HOSTS 100
# define NUM_HOSTS 10
//# define NUM_OBJECTS 100000
# define NUM_OBJECTS 100

struct THost {
	guid_t host_id;
	int vnodes;
};

THost hosts[NUM_HOSTS];
guid_t objs[NUM_OBJECTS];

int main()
{
	UnitRing host_ring(true);
	UnitRing obj_ring(false);

	Guid guid;

	for (int i=0; i<NUM_HOSTS; i++) {
		guid.gen();
		guid_t x;
		guid.pack(x);
		int vn=random()%(HOST_VNODE_MAX-HOST_VNODE_MIN)+HOST_VNODE_MIN;
		hosts[i].vnodes=vn;
		memcpy((char *)&(hosts[i].host_id), (char *)&x, sizeof(guid_t));
		host_ring.add(x, vn);
	}

	for (int i=0; i<NUM_OBJECTS; i++) {
		guid.gen();
		guid_t x;
		guid.pack(x);
		memcpy((char *)&(objs[i]), (char *)&x, sizeof(guid_t));
		obj_ring.add(x);
	}

	printf("Hosts ring:\n");
	host_ring.dump();

	printf("Objects ring:\n");
	obj_ring.dump();

	return 0;
}
