// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t4.cc,v $ - testing program for unit ring.
 *            this program tests the get_neighbors() function.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/11 08:28:26 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include "unit_ring.h"

# define HOST_VNODE_MIN 1
# define HOST_VNODE_MAX 5

# define NUM_HOSTS 20

struct THost {
	guid_t host_id;
	int vnodes;
};

THost hosts[NUM_HOSTS];

int main()
{
	UnitRing host_ring(true);
	guid_t neighbors[NUM_HOSTS*2];

	Guid guid;
	guid.gen();
	guid_t x;
	guid.pack(x);

	int n_nb=host_ring.get_neighbors(neighbors, NUM_HOSTS, x);
	if (n_nb > 0) {
		printf("get_neighbors from an empty ring failed.\n");
		exit(-1);
	}

	int vn=random()%(HOST_VNODE_MAX-HOST_VNODE_MIN)+HOST_VNODE_MIN;
	hosts[0].vnodes=vn;
	memcpy((char *)&(hosts[0].host_id), (char *)&x, sizeof(guid_t));
	host_ring.add(x, vn);

	n_nb=host_ring.get_neighbors(neighbors, NUM_HOSTS, x);
	if (n_nb != 1) {
		printf("get_neighbors from a one-host ring with a host key failed.\n");
		exit(-1);
	}

	for (int i=0; i<100; i++) {
		guid.gen();
		guid.pack(x);
		n_nb=host_ring.get_neighbors(neighbors, 10, x);
		if (n_nb != 1) {
			printf("get_neighbors from a one-host ring with a random key failed.\n");
			exit(-1);
		}
	}

	for (int i=1; i<NUM_HOSTS; i++) {
		guid.gen();
		guid.pack(x);
		vn=random()%(HOST_VNODE_MAX-HOST_VNODE_MIN)+HOST_VNODE_MIN;
		hosts[i].vnodes=vn;
		memcpy((char *)&(hosts[i].host_id), (char *)&x, sizeof(guid_t));
		host_ring.add(x, vn);
	}

	printf("Hosts ring:\n");
	host_ring.dump();

	for (int i=1; i<NUM_HOSTS*2; i+=NUM_HOSTS/5) {
		printf("get_neighbor() with n_neighbor = %d.\n", i);
		guid.gen();
		guid.pack(x);
		n_nb=host_ring.get_neighbors(neighbors, i, x);
		for (int j=0; j<n_nb; j++) {
			Guid tmp(neighbors[j]);
			char guid_str[128];
			tmp.to_string(guid_str, 128);
			printf("%3d %s\n", j, guid_str);
		}
	}

	return 0;
}
