// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*- 

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t5.cc,v $ - testing program for unit ring.
 *            this program tests the addition of elements into the ring,
 *            and whether the keys are sorted on the ring.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/07/11 08:28:26 $
 * Version :  $Revision: 1.1 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <stdio.h>
# include "gsl/gsl_rng.h"

# include "unit_ring.h"

# define NUM_OBJECTS 100

UnitRing obj_ring(false);

void test_range(URRange & range)
{
	guid_t keys[NUM_OBJECTS];
	int n_keys=obj_ring.get_keys_in_range(range, keys, NUM_OBJECTS);
	if (n_keys>0) {
		printf("Totally %d keys in [%.5f, %.5f)\n", n_keys, range.from, range.to);
		for (int i=0; i<n_keys; i++) {
			Guid x(keys[i]);
			char guid_str[128];
			x.to_string(guid_str, 128);
			printf("%3d %s\n", i, guid_str);
		}
	}
	else {
		printf("There is no key in [%.5f, %.5f)\n", range.from, range.to);
	}

	return;
}

int main()
{
	Guid guid;

	for (int i=0; i<NUM_OBJECTS; i++) {
		guid.gen();
		guid_t x;
		guid.pack(x);
		obj_ring.add(x);
	}

	printf("Objects ring:\n");
	obj_ring.dump();

	{
		URRange ur(0.999, 0.001);
		test_range(ur);
	}

	{
		URRange ur(0.500, 0.500);
		test_range(ur);
	}

	gsl_rng *rng=gsl_rng_alloc(gsl_rng_default);
	unsigned long seed=(unsigned long)time(NULL);
	gsl_rng_set(rng, seed);

	for (int i=0; i<10; i++) {
		double from=gsl_rng_uniform(rng);
		double to=from+0.001;
		if (to>=1)
			to-=1;
		URRange ur(from, to);
		test_range(ur);
	}

	for (int i=0; i<10; i++) {
		URRange ur(gsl_rng_uniform(rng), gsl_rng_uniform(rng));
		test_range(ur);
	}

	return 0;
}
