// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: galois.h,v $ - header file for Galois Field operations.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _GALOIS_H_
# define _GALOIS_H_

# if defined(__cplusplus)
extern "C" {
# endif

# include "rs_types.h"

/**
 * An implementation of Galois GF(2^16) with the generation polynomial
 * x^16+x^12+x^3+x+1.
 */

int gal_init();
int gal_finalize();
int gal_field_size();
int gal_poly_gen();

/**
 * Galois addition for two vectors of n unsigned shorts
 */
int gal_plus_one(USHORT a, USHORT b, USHORT *c);
int gal_plus(USHORT *a, USHORT *b, USHORT *c, int n);

/**
 * Galois minus is the same as Galois plus
 */
# define gal_minus_one gal_plus_one
# define gal_minus gal_plus

/**
 * Galois multiplication for two vectors of n unsigned shorts
 */
int gal_mult_one(USHORT a, USHORT b, USHORT *c);
int gal_mult(USHORT *a, USHORT *b, USHORT *c, int n);

/**
 * Galois division for two vectors of n unsigned shorts
 */
int gal_div_one(USHORT a, USHORT b, USHORT *c);
int gal_div(USHORT *a, USHORT *b, USHORT *c, int n);

/**
 * Galois logarithm and inverse logarithm
 */
int gal_log_one(USHORT a, USHORT *c);
int gal_log(USHORT *a, USHORT *c, int n);

int gal_ilog_one(USHORT a, USHORT *c);
int gal_ilog(USHORT *a, USHORT *c, int n);

# if defined(__cplusplus)
}
# endif

# endif
