// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: rs.h,v $ - header file for Reed Solomon coding.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _RS_CODER_H_
# define _RS_CODER_H_

# include "rs_types.h"

/**
 * Initialize internal data structures.
 */
int rs_init();

/**
 * Finalization
 */
int rs_finalize();

struct _decode_handle;
typedef struct _decode_handle *Decode_Handle;

/* decoding */

/**
 * indices is an array of length n, storing the n encoded messages used to
 * decode original messages. m is the total number of redundant messages,
 * used to check the range of values in the array indice.
 */
int decode_precomp(Decode_Handle *dh, USHORT *indice, USHORT n, USHORT m);


/**
 * src_msgs: an array of n BYTE pointers, each of which points to a BYTE 
 * array of size msg_len.  
 * dest_msg: an array of BYTEs, whose size is msg_len.
 * msg_len: IMPORTANT: Internally we do the calculation based on 16 bits 
 * GF arithmatic. Thus, although an even msg_len is not a requirement, 
 * it is strongly advised to avoid odd msg_len as much as possible (e.g.
 * if we try to calculate a whole message by small blocks, we should break
 * the message upon even byte boundaries.
 * dest_msg_idx: The index of the fragment to be decoded [0 - (m+n-1)].
 */
int decode_comp(
	Decode_Handle *dh, 
	BYTE ** src_msgs, 
	BYTE *dest_msg, 
	UINT msg_len,
	USHORT dest_msg_idx 
);

/**
 * free the resource of the decoding handle
 */
int decode_release(Decode_Handle *dh);

/* encoding */
/**
 * Strictly speaking, it is not required to have a separate set of encoding 
 * functions. Encoding is just like the decoding using the original n fragments.
 * They are provided because it can reduce the cost of a matrix inversion
 * operation for an identity matrix.
 */
struct _encode_handle;
typedef struct _encode_handle *Encode_Handle;

/**
 * The fragmentation size n and the redundancy level m.
 */
int encode_precomp(Encode_Handle *eh, USHORT n, USHORT m);

/**
 * src_msgs: an array of n BYTE pointers, each of which points to a BYTE array
 * of size msg_len.
 * dest_msg: an array of BYTEs, whose size is msg_len. 
 * msg_len: IMPORTANT: Internally we do the calculation based on 16 bits 
 * GF arithmatic. Thus, although an even msg_len is not a requirement, 
 * it is strongly advised to avoid odd msg_len as much as possible (e.g.
 * if we try to calculate a whole message by small blocks, we should break
 * the message upon even byte boundaries.
 * dest_msg_idx: The index of the fragment to be decoded [0 - (m+n-1)].
 */
int encode_comp(
	Encode_Handle *eh, 
	BYTE **src_msgs, 
	BYTE *dest_msg, 
	UINT msg_len,
	USHORT dest_msg_idx
);

/**
 * free the resource of the encoding handle
 */
int encode_release(Encode_Handle *dh);

# endif
