// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: queue.h,v $ - header file for a dequeue. 
 *            The implementation is very simple and is *NOT* multi-thread
 *            safe! It supports the enqueue operation on 
 *            both ends of the queues; and dequeue operation on the front
 *            end of the queue.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/10 23:02:11 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

# ifndef _SIMPLE_QUEUE_H_
# define _SIMPLE_QUEUE_H_

# include <assert.h>

class Queue;

class Listable
{
private:
	Listable *next;
public:
	Listable() : next(NULL)
	{ }

	bool link(Listable **before)
	{
		if (next != NULL)
			return false;

		next=*before;
		*before=this;
		return true;
	}

	bool unlink(Listable **before)
	{
		if (*before != this)
			return false;

		*before = next;
		next = NULL;

		return true;
	}

	friend class Queue;
};

class Queue
{
private:
	Listable *head;
	Listable *tail;
	int n_items;

public:
	Queue() : head(NULL), tail(NULL), n_items(0)
	{}
	Listable *peek_head()
	{
		return head;
	}
	Listable *peek_tail()
	{
		return tail;
	}

	bool push_back(Listable *item)
	{
		bool ret;

		if (tail == NULL) { // first item.
			ret=item->link(&head);
		} else {
			ret=item->link(&(tail->next));
		}

		if (ret) {
			tail = item;
			n_items++;
		}

		return ret;
	}

	bool insert_after(Listable *before, Listable *item)
	{
		if (item->link(&(before->next))) {
			n_items++;
			return true;
		}
		else
			return false;
	}

	bool push_front(Listable *item)
	{
		if (item->link(&head) == true) {
			n_items++;

			if (tail == NULL)
				tail = head;

			return true;
		}

		return false;
	}

	Listable *pop_front()
	{
		Listable *ret=head;

		if (ret) {
			ret->unlink(&head);
			n_items--;

			if (head == NULL) {
				tail = NULL;
			}
		}

		return ret;
	}

	int size()
	{
		return n_items;
	}

	bool is_empty()
	{
        if (n_items == 0) {
            assert(head==NULL);

            return true;
        }
        else {
            assert(head!=NULL);

            return false;
        }

		// return (n_items == 0);
	}
};

# endif // _SIMPLE_QUEUE_H_
