// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: test_rwlock.cc,v $ - rwlock tester.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/10 23:02:11 $
 * Version :  $Revision: 1.8 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */


# include <getopt.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <unistd.h>
# include <ctype.h>

# include "../tpool/cbs_thread.h"
# include "../machine/atomic.h"
# include "rwlock.h"

# define MAX_READER_LEVEL 3
# define MAX_WRITER_LEVEL 5

static char *program_name;
static int n_readers=10;
static int n_writers=3;
static int test_level=MAX_WRITER_LEVEL;
static int reader_test_level=MAX_READER_LEVEL;

char *Id="$Id: test_rwlock.cc,v 1.8 2003/06/10 23:02:11 htang Exp $";

static char *chop(char *line)
{
	while (*line) {
		if (isspace(*line))
			line++;
		else
			break;
	}

	char *p=line+strlen(line)-1;
	while ((p>=line)&&(isspace(*p))) {
		*p='\0';
		p--;
	}

	return line;
}

static char *get_RCS_val(char *var)
{
	static char buf[1024];
	char *p;

	if ((p=strchr(var, ':'))==NULL) {
		return var;
	}
	buf[1023]='\0';
	strncpy(buf, p+1, 1024);
	p=buf+strlen(buf)-1;
	while ((p>=buf)&&(*p=='$')) {
		*p='\0';
		p--;
	}

	return chop(buf);
}
static void do_version()
{
	static char *version="$Revision: 1.8 $";
	static char *author="Hong Tang";
	static char *date="$Date: 2003/06/10 23:02:11 $";

	{
		printf("rwlock tester (%s) %s.\n", program_name, get_RCS_val(version));
		printf("Copyright (c) University of California, Santa Barbara.\n");
		printf("Author: %s.\n", get_RCS_val(author));
		printf("Last Modification Date: %s GMT.\n", get_RCS_val(date));
		printf("Specify the --help option for further information.\n");
	}

	exit(0);
}

static void usage(int status)
{
	if (status != 0) {
		fprintf(stderr, "Try '%s --help' for more information.\n", program_name);
	} else {
		printf ("\n\
Usage: %s [options]\n", program_name);
		printf ("\
The rwlock test program.\n\
  -r, --num-readers    Number of readers.\n\
  -w, --num-writers    Number of writers.\n\
  -l, --levels         Test levels [0-6], as follows:\n\
  ---------------------------------------------------\n\
   (0): no-op;            (3): (2) + try lock;       \n\
   (1): normal lock;      (4): (3) + lock promotion. \n\
   (2): (1) + timed lock; (5): (4) + try promotion.  \n\
\n\
  -h, --help           Display this help screen.\n\
  -V, --version        Display version number.\n\
");
	}

	exit(status);
}

static void parse_cmd_args(int argc, char *argv[])
{
	static struct option const long_options[] = {
		        {"num-readers", required_argument, 0, 'r'},
		        {"num-writers", required_argument, 0, 'w'},
		        {"levels", required_argument, 0, 'l'},
		        {"help", no_argument, 0, 'h'},
		        {"version", no_argument, 0, 'V'},
		        {NULL, 0, 0, 0}
	        };

	int optc;

	while ((optc=getopt_long(argc, argv, \
	                         "r:w:l:hV", long_options, NULL)) !=-1) {
		switch (optc) {
		case 'r':
			n_readers=atoi(optarg);
			break;
		case 'w':
			n_writers=atoi(optarg);
			break;
		case 'l':
			test_level=atoi(optarg);
			if (test_level>MAX_WRITER_LEVEL)
				test_level=MAX_WRITER_LEVEL;
			if (test_level<MAX_READER_LEVEL)
				reader_test_level=test_level;
			break;
		case 'h':
			usage(0);
			break;
		case 'V':
			do_version();
			break;
		default:
			usage(1);
		}
	}

	if ((test_level<0)||(n_readers<=0)||(n_writers<=0)) {
		fprintf(stderr, "Numbers of readers or writers must be positive.\n");
		usage(1);
	}
}

RWLock mylock;

struct SHARED {
	int x, y;
	SHARED() : x(0), y(0)
	{}
}
shared;

int action_counter[9];
int success_counter[9];

class Reader : public CBS_Thread {
private:
	int index;
	void check()
	{
		if (shared.x!=shared.y) {
			printf("Reader %i sees inconsistency (%d != %d).\n", index, shared.x, shared.y);
			stop();
		} else {
			usleep(rand()%50000);
		}
	}

public:
	Reader(int i) : index(i)
	{ }

	virtual void *run()
	{
		int rounds=400;

		while ((!should_quit()) && ((rounds--)>0)) {
			int which = (reader_test_level)?rand()%reader_test_level:-1;
			switch (which) {
			case 0:
				inc32(&(action_counter[0]));
				if (mylock.read_lock() == false ) {
					fprintf(stderr, "Reader %d: read_lock() failed.", index);
				} else {
					check();
					mylock.release();
					inc32(&(success_counter[0]));
				}
				break;

			case 1:
				inc32(&(action_counter[1]));
				if (mylock.read_timedlock(rand()%1000+10) == false) {
				} else {
					check();
					mylock.release();
					inc32(&(success_counter[1]));
				}
				break;

			case 2:
				inc32(&(action_counter[2]));
				if (mylock.read_trylock() == false) {
				} else {
					check();
					mylock.release();
					inc32(&(success_counter[2]));
				}
				break;

			default:
				break;

			}

			if (index == 0) {
				printf("\r\t\t[%d]     ", rounds);
				fflush(stdout);
			}
			usleep(rand()%50000);
		}

		return NULL;
	}
};

class Writer : public CBS_Thread {
private:
	int index;
	void update()
	{
		if (shared.x != shared.y) {
			printf("Writer %d found inconsistency. (%d != %d).\n", index, shared.x, shared.y);
			stop();
		} else {
			shared.x++;
			usleep(rand()%50000);
			shared.y++;
			printf("\r%d", shared.x);
			fflush(stdout);
		}
	}

public:
	Writer(int i) : index(i)
	{ }

	virtual void *run()
	{
		int rounds=200;

		while ((!should_quit()) && ((rounds--)>0)) {
			int which = (test_level)?rand()%test_level:-1;
			switch (which) {
			case 0:
				inc32(&(action_counter[3]));
				if (mylock.write_lock() == false ) {
					fprintf(stderr, "Writer %d: write_lock() failed.", index);
				} else {
					update();
					mylock.release();
					inc32(&(success_counter[3]));
				}
				break;

			case 1:
				inc32(&(action_counter[4]));
				if (mylock.write_timedlock(rand()%1000+10) == false) {
				} else {
					update();
					mylock.release();
					inc32(&(success_counter[4]));
				}
				break;

			case 2:
				inc32(&(action_counter[5]));
				if (mylock.write_trylock() == false) {
				} else {
					update();
					mylock.release();
					inc32(&(success_counter[5]));
				}
				break;
			case 3:
				inc32(&(action_counter[6]));
				if (mylock.read_lock() == false ) {
					fprintf(stderr, "Writer %d: read_lock() failed.", index);
				} else {
					if (mylock.promote_lock() == false ) {
						fprintf(stderr, "Writer %d: promote_lock() failed.", index);
					}
					else {
						inc32(&(success_counter[6]));
						update();
					}
					mylock.release();
				}
				break;
			case 4:
				inc32(&(action_counter[8]));
				if (mylock.read_lock() == false ) {
					fprintf(stderr, "Writer %d: read_lock() failed.", index);
				} else {
					if (mylock.promote_trylock() == false ) {
					}
					else {
						inc32(&(success_counter[8]));
						update();
					}
					mylock.release();
				}
				break;

			default:
				break;

			}

			usleep(rand()%50000);
		}

		return NULL;
	}
};

int main(int argc, char * argv[]) {
	program_name = argv[0];

	parse_cmd_args(argc, argv);

	printf("Readers=%d, Writers=%d, Level=%d\n", n_readers, n_writers, test_level);

	Reader ** readers = (Reader **)calloc(n_readers, sizeof(Reader *));
	Writer ** writers = (Writer **)calloc(n_writers, sizeof(Writer *));

	for(int i=0; i<n_readers; i++) {
		readers[i]=new Reader(i);
		readers[i]->start();
	}

	for(int i=0; i<n_writers; i++) {
		writers[i]=new Writer(i);
		writers[i]->start();
	}

	for(int i=0; i<n_readers; i++) {
		readers[i]->join();
	}
	for(int i=0; i<n_writers; i++) {
		writers[i]->join();
	}

	char * counter_name[] = {
		"read_lock",
		"read_timedlock",
		"read_trylock",
		"write_lock",
		"write_timedlock",
		"write_trylock",
		"promote_lock",
		"promote_timedlock",
		"promote_trylock"
	};

	printf("\n");
	for (int i=0; i<9; i++) {
		if (action_counter[i]>0) {
			printf("%s: %d/%d.\n", counter_name[i], success_counter[i], action_counter[i]);
		}
	}

	return 0;
}
