// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: cal_test.cc,v $ - calendar queue tester I.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <unistd.h>
# include <stdio.h>
# include <stdlib.h>
# include "gsldistrib.h"
# include "../usc/usc_time.h"
# include "calevent.h"

/**
 * Testing the simulation core
 * Total enqueues : NENQUEUE
 * Average queue length : QLEN (expressed as a percentage of the total queue length 
 * Event arrival rate : RRATE (requests/second, Poisson process)
 */

# define NENQUEUE 22000000
# define QLEN 0.01
# define ARRIVAL 100

int qsize=0;
int avg_qlen=(int)(NENQUEUE*QLEN);
int nenqueue=0;
int ndequeue=0;
double last_t=0.0;
EventChain sim_core;
Distrib_exponential et_gen(1.0/ARRIVAL);

inline void enqueue_one()
{
	// double delta_t=(double)((int)(et_gen.get_one()*10000))/10000;
	double delta_t=et_gen.get_one();
	Event *e=new Event(1, last_t+delta_t);
	if (sim_core.schedule(e)==true) {
		qsize++;
		nenqueue++;
	}
	else {
		delete e;
	}
}

inline void enqueue_one_void()
{
	// double delta_t=(double)((int)(et_gen.get_one()*10000))/10000;
	double delta_t=et_gen.get_one();
	Event *e=new Event(1, last_t+delta_t);
	qsize++;
	nenqueue++;
	delete e;
}

inline void dequeue_one()
{
	Event *e=sim_core.next_event();
	if (e) {
		last_t=e->time();
		delete e;
		qsize--;
		ndequeue++;
	}
}

inline void dequeue_one_void()
{
	last_t+=0.1;
	qsize--;
	ndequeue++;
}

void reset()
{
	qsize=0;
	nenqueue=0;
	ndequeue=0;
	last_t=0;
}

int main()
{
	Distrib_flat even_gen(0,1);

	usc_time_t start_t2=usc_gettime();
	int GI=NENQUEUE/100;
	int gi=0;
	while (nenqueue<NENQUEUE) {
		if (gi%GI==0) {
			printf("\r%d%%", (nenqueue+1)/GI);
			fflush(stdout);
		}
		gi++;
		int m=avg_qlen-qsize;
		int n=-m;
		if (m<0)
			m=1;
		else if (m>0) 
			n=1;
		else {
			m=n=1;
		}

		if (m>10)
			m=10;

		if (n>10)
			n=10;

		double p=(double)m/(n+m);
		double q=even_gen.get_one();
		if (q<p)
			enqueue_one_void();
		else
			dequeue_one_void();
	}
	printf("\r%d%%", (nenqueue+1)/GI);
	usc_time_t end_t2=usc_gettime();
	double void_time=usc_difftime(end_t2, start_t2);
	printf("\nVoid time: %.3fus/op.\n", void_time/(nenqueue+ndequeue));

	reset();
	enqueue_one();

	usc_time_t start_t=usc_gettime();
	gi=0;
	while (nenqueue<NENQUEUE) {
		if (gi%GI==0) {
			printf("\r%d%%", (nenqueue+1)/GI);
			fflush(stdout);
		}
		gi++;
		int m=avg_qlen-qsize;
		int n=-m;
		if (m<0)
			m=1;
		else if (m>0) 
			n=1;
		else {
			m=n=1;
		}

		if (m>10)
			m=10;

		if (n>10)
			n=10;

		double p=(double)m/(m+n);
		double q=even_gen.get_one();
		if (q<p)
			enqueue_one();
		else
			dequeue_one();
	}
	printf("\r%d%%", (nenqueue+1)/GI);
	usc_time_t end_t=usc_gettime();
	double dur=usc_difftime(end_t, start_t)-void_time;
	double per_op=dur/(nenqueue+ndequeue);

	printf("\n%.5f/%.5fms width, %d resizes, %.2f trav, %lld/%d enq, %lld deq, %d/%d/%d qlen, %.3fus/op.\n", 
			sim_core.get_width()*1000, sim_core.opt_width()*1000, sim_core.get_resize_cnt(), sim_core.get_num_traverse(), sim_core.num_enqueue(), nenqueue, sim_core.num_dequeue(),
			(int)sim_core.queue_length(), avg_qlen, qsize, per_op);

	return 0;
}
