// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: dist_test.cc,v $ - gsl distribution tester.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <stdio.h>
# include <math.h>
# include "gsldistrib.h"
/**
 * Testing the bended normal distribution.
 * Given an arbitrary number t in (0,1), we repeat
 * the following for 100 times for each k = 1 to 10
 * and report the mean and standard deviation:
 * we first segment a normal distribution (with sigma=1)
 * for the area (-s/k, s/k) (s>0), and then we squeeze 
 * the area (-s/k, s/k) to (0, 1) so that the point at 
 * -s/k is moved to 0, the point at 0 moved to t and 
 * the point at s/k to 1.
 * Meanings of the variables:
 * t: the observation of a random variable (load)
 * k: discrete measure of the stale-ness of t.
 * s: scaling constant, when the stale-ness factor k==s, 
 *    we only have the (-sigma, sigma) section of the
 *    normal distribution (which is fairly flat).
 */ 

# define REPEATI 10
# define REPEATJ 100
# define REPEATK 10
# define CONST_S 7
# define STEP 1
# define SIGMA 1

int main()
{
	Distrib_flat gen_a(0,1);
	for (int i=0; i<REPEATI; i++) {
		double t=gen_a.get_one();
		if ((t==0)||(t==1))
			continue;

		for (int k=1; k<=1+REPEATK*STEP; k+=STEP) {
		double s=0;
		double sq=0;
		double s_k=(double)CONST_S/k;
		for (int j=0; j<REPEATJ; j++) {
			double y=gsldd_gaussian(SIGMA);
			double t1=0;
			if ((y<-s_k)||(y>s_k)) {
				// y=gsldd_flat(-s_k, s_k);
				// t1=gen_a.get_one();
				t1=gsldd_flat(0, 1);
			}
			else if (y==s_k)
				t1=1;
			else if (y==-s_k)
				t1=0;
			else if (y==0)
				t1=t;
			else {
				if (t==0.5) {
					t1=0.5+0.5*y/s_k;
				}
				else {
					double a=(double)s_k*(2*t-1)/(t*(1-t));
					double b=(double)s_k*(2*t*(t-1)+1)/(t*(1-t));
					t1=(-b+sqrt(fabs(b*b+4*a*y)))/(2*a);
					t1+=t;
				}
			}

			s+=t1;
			sq+=t1*t1;
		}
		s/=REPEATJ;
		sq=sqrt(fabs((sq-s*s*REPEATJ)/(REPEATJ-1)));
		printf("[%4d-%4d] %.3f %.3f\t%.3f/%.3f.\n", i, k, s_k, t, s, sq);
		}
	}

	return 0;
}
