// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: prog_test.cc,v $ - testing progress bar.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:53 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <unistd.h>
# include <stdio.h>
# include <stdlib.h>
# include "../usc/usc_time.h"
# include "progress.h"

/**
 * Testing the simulation progress bar
 * Simulation time (sec): SIM_SEC
 *
 */

# define SIM_SEC 50
int main()
{
	if (!isatty(fileno(stdout))) {
		fprintf(stderr, "Progress bar will not show if the output device is not a terminal.\n");
		return 0;
	}

	set_progress_implementation("bar");
	void *prog=progress_create(0, SIM_SEC*1000);
	usc_time_t start_t=usc_gettime();
	srand((unsigned int)start_t);
	usc_time_t now;
	int total_time=0;
	while (1) {
		int sim_delta_ms=rand()%100+10; /* simulation time */
		if (total_time+sim_delta_ms>SIM_SEC*1000) {
			sim_delta_ms=SIM_SEC*1000-total_time;
		}
		int usleep_delta_us=rand()%100000+1; /* sleeping time */
		usleep(usleep_delta_us);
		now=usc_gettime();
		progress_update(prog, sim_delta_ms, (int)usc_difftime(now, start_t));
		total_time+=sim_delta_ms;
		if (total_time>=SIM_SEC*1000)
			break;
	}

	now=usc_gettime();
	progress_finish(prog, (long)usc_difftime(now, start_t));

	return 0;
}
