// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: echo_srv.c,v $ - a simple proxy.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/01 22:10:21 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/time.h>
# include <sys/types.h>
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include <string.h>
# include <ctype.h>
# include <signal.h>
# include "sock_conn.h"
# include "sock_sr.h"

int recvline(int fd, char *buf, int len, int tout)
{
	char *p;

	p=buf;
	for (; p<buf+len; p++) {
		int len=1;
		int errnum = sock_read_tout(fd, p, &len, tout);
		if (len == 0) {
			*p='\0';
			return -1;
		}

		if (*p=='\n')
			break;
	}

	*(p+1)='\0';
	
	return 0;
}

int sendline(int fd, char *buf)
{
	int ret=0;

	ret|=sock_write(fd, buf, strlen(buf));

	return ret;
}

int isempty_line(char *s)
{
	while (*s) {
		if (!isspace(*s))
			return 0;
		s++;
	}
	return 1;
}

void printline(char *host, char *s)
{
	static char last_host[1024]="";
	char *line=s;

	if (strcmp(host, last_host)) {
		printf("Write to %s:\n", host);
		strcpy(last_host, host);
	}

	while (line) {
		char *nextline=strchr(line, '\n');

		if (nextline) {
			*nextline='\0';
		}

		/**
		if (strncasecmp(line, "Host", 4)==0) {
			char *newhost="Host: login.prospero.com";
			int extra_blank=strlen(line)-strlen(newhost)-1;
			if (extra_blank>=0) {
				char *p=line+strlen(newhost);
				strcpy(line, newhost);
				while (extra_blank) {
					*p=' ';
					p++;
					extra_blank--;
				}
				*p++='\r';
			}
		}
		**/
		while (*line) {
			if (isprint(*line)) {
				printf("%c", *line);
			}
			else {
				printf("\\%u\\", *line);
				if (*line=='\n')
					printf("\n");
			}
			line++;
		}

		line=nextline;
		if (line) {
			printf("\n");
			*line='\n';
			line++;
		}
	}
}

int read_from_two(int h1, int h2, char *buf, int *len)
{
	int max_fd=(h1>h2)?(h1+1):(h2+1);

	fd_set rset;
	FD_ZERO(&rset);
	FD_SET(h1, &rset);
	FD_SET(h2, &rset);

	while (1) {
		int fd=-1;
		struct timeval tv={10, 0};
		switch (select(max_fd, &rset, NULL, NULL, &tv)) {
			case 0:
				break;
			case -1:
				return -1;
			default:
				if (FD_ISSET(h2, &rset)) {
					fd=h2;
				}

				if (FD_ISSET(h1, &rset)) {
					fd=h1;
				}

				if (fd<0)
					return -1;

				*len=read(fd, buf, *len);

				return fd;
		}
	}
}

void substitute(char *buf, int *len)
{
	char *ss[]={"login.prospero.com", "otto.cs.ucsb.edu"};
	char new_buf[2048];
	char *p=buf;
	char *q=new_buf;
	static int port=0;

	while (p<buf+*len) {
		char *n0=strstr(p, ss[0]);
		char *n1=strstr(p, ss[1]);
		char *n;
		char *endn;
		char subs[1024];

		if (n0||n1) {
			if (!n0) {
				n=n1;
			}
			else if (!n1) {
				n=n0;
			}
			else {
				n=(n0>n1)?n1:n0;
			}

			if (n==n0) {
				endn=n+strlen(ss[0]);
				if (port) {
					sprintf(subs, "%s:%d", ss[1], port);
				}
				else {
					strcpy(subs, ss[1]);
				}
			}
			else {
				strcpy(subs, ss[0]);

				endn=n+strlen(ss[1]);
				if (*endn==':') {
					endn++;
					port=atoi(endn);
					while (isdigit(*endn)) {
						endn++;
					}
				}
			}
			if (n-p>0) {
				strncpy(q, p, n-p);
				q+=n-p;
				p+=n-p;
			}

			strcpy(q, subs);
			q+=strlen(subs);
			p=endn;
		}
		else {
			int l=strlen(p);
			strncpy(q,p, l);
			q+=l+1;
			p+=l+1;
		}
	}
	memcpy(buf, new_buf, q-new_buf);
	*len=q-new_buf;
}

int main()
{
	int s=sock_listen(SOCK_PORT_ANY, 5);
	int port;
	int s2;
	char buf[2048];

	if (s<0) {
		perror("Cannot bind to a listen socket.\n");
		exit(-1);
	}
	port=sock_myport(s);
	if (port<0) {
		perror("Cannot get bind port.\n");
		exit(-1);
	}
	printf("My port=%d.\n", port);
	signal(SIGPIPE, SIG_IGN);

	while ((s2=sock_accept(s))>=0) {
		int s3=sock_conn("login.prospero.com", 80);

		if (s3<0) {
			printf("Cannot connect to login.prospero.com:80\n");
			break;
		}

		while (1) {
			int len=1023;
			int fd=read_from_two(s2, s3, buf, &len);
			int fd2;
			char *dest[]={"client", "login.prospero.com"};
			char *wdest;

			if (fd<0) {
				printf("Cannot read from either.\n");
				break;
			}

			if (fd==s2) {
				fd2=s3;
				wdest=dest[1];
			}
			else if (fd==s3) {
				fd2=s2;
				wdest=dest[0];
			}
			else {
				printf("Unexpected error.\n");
				break;
				break;
			}

			buf[len]='\0';
			substitute(buf, &len);
			buf[len]='\0';
			printline(wdest, buf);

			if (sock_write(fd2, buf, len)<0) {
				printf("Cannot write to %s.\n", wdest);
				break;
			}
		}

		printf("Client disconnected.\n");
		close(s2);
		close(s3);
	}

	close(s);
	return 0;
}
