// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: load_bcast.cc,v $ - load multicast program.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <unistd.h>
# include <stdio.h>
# include <mntent.h>
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
# include "mcast_annc.h"
# include "../res/resmon.h"
# include "load_struct.h"

# if defined(__DEBUG__)
void dump_anc(Mc_anc_data *data)
{
	Mc_rec **records=data->dump();
	for (Mc_rec **p=records; *p; p++) {
		Mc_rec *rec=*p;
		switch (rec->var) {
			case KEY_CPU_TOT: 
				printf("cputotal=%u\n", *(unsigned int *)rec->val());
				break;
			case KEY_CPU_USR:
				printf("user=%u\n", *(unsigned int *)rec->val());
				break;
			case KEY_CPU_KER:
				printf("kernel=%u\n", *(unsigned int *)rec->val());
				break;
			case KEY_CPU_IOW:
				printf("iowait=%u\n", *(unsigned int *)rec->val());
				break;
			case KEY_CPU_IDL:
				printf("idle=%u\n", *(unsigned int *)rec->val());
				break;
			case KEY_CPU_NUM:
				printf("cpu_num=%d\n", *(unsigned int *)rec->val());
				break;
			case KEY_CPU_CXW:
				printf("context=%u\n", *(unsigned int *)rec->val());
				break;
			case KEY_MEM_TOT:
				printf("memory=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_MEM_USE:
				printf("mused=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_MEM_FRE:
				printf("mfree=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_MEM_SHR:
				printf("mshared=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_MEM_BUF:
				printf("mbuffers=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_MEM_CCH:
				printf("mcached=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_SWP_TOT:
				printf("swap=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_SWP_FRE:
				printf("swap_free=%uKB\n", *(unsigned int *)rec->val());
				break;
			case KEY_DSK_FRE:
				break;
			default:
				;
		}
	}

	free(records);
}
# endif

int main(int argc, char *argv[])
{
	if (argc!=3) {
		printf("Usage: %s <mcast grp> <port>.\n", argv[0]);
		exit(-1);
	}

	Mc_ancer ancer(argv[1], atoi(argv[2]));
	while (1) {
		ResMon::CPU cpu;
		if (ResMon::get_cpu(&cpu)<0) {
			printf("Cannot obtain CPU load.\n");
			exit(-1);
		}

		ResMon::MEM mem;
		if (ResMon::get_mem(&mem)<0) {
			printf("Cannot obtain mem load.\n");
			exit(-1);
		}

		Mc_anc_data data;

		struct {
			unsigned char key;
			void *buf;
			unsigned int len;
		} *p, recs[] =
		{
			{KEY_CPU_TOT, &(cpu.cputotal), sizeof(cpu.cputotal)},
			{KEY_CPU_USR, &(cpu.user), sizeof(cpu.user)},
			{KEY_CPU_KER, &(cpu.kernel), sizeof(cpu.kernel)},
			{KEY_CPU_IOW, &(cpu.iowait), sizeof(cpu.iowait)},
			{KEY_CPU_IDL, &(cpu.idle), sizeof(cpu.idle)},
			{KEY_CPU_NUM, &(cpu.cpus), sizeof(cpu.cpus)},
			{KEY_CPU_CXW, &(cpu.cswitches), sizeof(cpu.cswitches)},
			{KEY_MEM_TOT, &(mem.mtotal), sizeof(mem.mtotal)},
			{KEY_MEM_USE, &(mem.mused), sizeof(mem.mused)},
			{KEY_MEM_FRE, &(mem.mfree), sizeof(mem.mfree)},
			{KEY_MEM_SHR, &(mem.mshared), sizeof(mem.mshared)},
			{KEY_MEM_BUF, &(mem.mbuffers), sizeof(mem.mbuffers)},
			{KEY_MEM_CCH, &(mem.mcached), sizeof(mem.mcached)},
			{KEY_SWP_TOT, &(mem.stotal), sizeof(mem.stotal)},
			{KEY_SWP_FRE, &(mem.sfree), sizeof(mem.sfree)},
			{0, NULL, 0}
		};

		p=recs;
		while (p->buf) {
			if (data.add(p->key, p->buf, p->len)<0) {
				printf("Mc_annc_data.add failed.\n");
				exit(-1);
			}
			p++;
		}

		static const char *MTAB_NAME = "/etc/mtab";
		FILE *fp;
		struct mntent *mnt;

		if ((fp=setmntent(MTAB_NAME, "r"))==NULL) {
			printf("Cannot open mtab.\n");
			exit(-1);
		}
	
		while ((mnt=getmntent(fp))) {
			if (strcmp(mnt->mnt_fsname, "none")==0) {
				continue;
			}
			ResMon::DISK *d=ResMon::get_disk(mnt->mnt_dir);
			if ((d->good)&&(d->btotal>0)&&(d->ftotal>0)) {
				struct disk dsk;
				dsk.btotal=d->btotal;
				dsk.bavail=d->bavail;
				dsk.ftotal=d->ftotal;
				dsk.ffree=d->ffree;

				if (data.add(KEY_DSK_FRE, &dsk, sizeof(dsk))<0) {
					printf("Mc_annc_data.add failed.\n");
					exit(-1);
				}

				delete d;
			}
		}
	
		endmntent(fp);

		ancer.announce(&data);
# if defined(__DEBUG__)
		{
			dump_anc(&data);
			char buf[10240];
			int len=10240;
			data.pack(buf, &len);
			Mc_anc_data data_tmp;
			data_tmp.unpack(buf, len);

			printf("---------------------------------\n");
			dump_anc(&data_tmp);
			printf("=================================\n");
		}
# endif

		usleep(100000);
	}
}
