// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: load_listener.cc,v $ - load listener sample program.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <unistd.h>
# include <stdio.h>
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
# include "mcast_annc.h"
# include "load_struct.h"

struct load *loads[1024];

void dump_loads()
{
	printf("Host Time CPU_USAGE MEM_USAGE DISK_USAGE\n");
	for (int i=0; i<1024; i++) {
		if (!loads[i])
			continue;
		printf("%s %s %.2f%%User %.2f%%Kernel %.2f%%IO %.2f%%Idle "
				"M=%uKB %.02f%%Used %.02ff%%Free S=%uKB %.02f%%Free", 
			loads[i]->host,
			ctime(&(loads[i]->when)),
			100.0*loads[i]->user/loads[i]->cputotal,
			100.0*loads[i]->kernel/loads[i]->cputotal,
			100.0*loads[i]->iowait/loads[i]->cputotal,
			100.0*loads[i]->idle/loads[i]->cputotal,
			loads[i]->mtotal,
			100.0*loads[i]->mused/loads[i]->mtotal,
			100.0*loads[i]->mfree/loads[i]->mtotal,
			loads[i]->stotal,
			100.0*loads[i]->sfree/loads[i]->stotal
		);
		for (int j=0; j<loads[i]->ndisks; j++) {
			printf(" (%luKB, %.2f%%Free)", 
				loads[i]->disks[j].btotal,
				100.0*loads[i]->disks[j].bavail/loads[i]->disks[j].btotal
			);
		}
		printf("\n");
	}
}

void add_loads(Mc_anc *anc)
{
	if (!anc)
		return;

	int idx=-1;
	int i;
	for (i=0; i<1024; i++) {
		if (loads[i]==NULL) {
			if (idx==-1)
				idx=i;
			continue;
		}
		if (strcmp(anc->addr(), loads[i]->host)==0)
			break;
	}

	if ((i>=1024)&&(idx==-1)) {
		// table full
		return;
	}

	if (i>=1024) {
		i=idx;
		loads[i]=(struct load *)malloc(sizeof(struct load));
		if (!loads[i])
			return;
	}

	memset(loads[i], 0, sizeof(struct load));
	strncpy(loads[i]->host, anc->addr(), 255);
	loads[255]='\0';
	loads[i]->when=anc->tstamp();
	Mc_rec **records=anc->dump();
	if (!records)
		return;

	for (Mc_rec **p=records; *p; p++) {
		Mc_rec *rec=*p;
		int ndisks;
		switch (rec->var) {
			case KEY_CPU_TOT: 
				memcpy(&(loads[i]->cputotal), rec->val(), sizeof(loads[i]->cputotal));
				break;
			case KEY_CPU_USR:
				memcpy(&(loads[i]->user), rec->val(), sizeof(loads[i]->user));
				break;
			case KEY_CPU_KER:
				memcpy(&(loads[i]->kernel), rec->val(), sizeof(loads[i]->kernel));
				break;
			case KEY_CPU_IOW:
				memcpy(&(loads[i]->iowait), rec->val(), sizeof(loads[i]->iowait));
				break;
			case KEY_CPU_IDL:
				memcpy(&(loads[i]->idle), rec->val(), sizeof(loads[i]->idle));
				break;
			case KEY_CPU_NUM:
				memcpy(&(loads[i]->cpus), rec->val(), sizeof(loads[i]->cpus));
				break;
			case KEY_CPU_CXW:
				memcpy(&(loads[i]->cswitches), rec->val(), sizeof(loads[i]->cswitches));
				break;
			case KEY_MEM_TOT:
				memcpy(&(loads[i]->mtotal), rec->val(), sizeof(loads[i]->mtotal));
				break;
			case KEY_MEM_USE:
				memcpy(&(loads[i]->mused), rec->val(), sizeof(loads[i]->mused));
				break;
			case KEY_MEM_FRE:
				memcpy(&(loads[i]->mfree), rec->val(), sizeof(loads[i]->mfree));
				break;
			case KEY_MEM_SHR:
				memcpy(&(loads[i]->mshared), rec->val(), sizeof(loads[i]->mshared));
				break;
			case KEY_MEM_BUF:
				memcpy(&(loads[i]->mbuffers), rec->val(), sizeof(loads[i]->mbuffers));
				break;
			case KEY_MEM_CCH:
				memcpy(&(loads[i]->mcached), rec->val(), sizeof(loads[i]->mcached));
				break;
			case KEY_SWP_TOT:
				memcpy(&(loads[i]->stotal), rec->val(), sizeof(loads[i]->stotal));
				break;
			case KEY_SWP_FRE:
				memcpy(&(loads[i]->sfree), rec->val(), sizeof(loads[i]->sfree));
				break;
			case KEY_DSK_FRE:
				ndisks=loads[i]->ndisks;
				if (ndisks>=10)
					break;
				memcpy(&(loads[i]->disks[ndisks]), rec->val(), sizeof(loads[i]->disks[ndisks]));
				loads[i]->ndisks++;
			default:
				;
		}
	}
	free(records);
}

int main(int argc, char *argv[])
{
	if (argc!=3) {
		printf("Usage: %s <mcast grp> <port>.\n", argv[0]);
		exit(-1);
	}

	Mc_ancee ancee(argv[1], atoi(argv[2]));
	memset(loads, 0, sizeof(struct load *)*1024);
	int i=10000;
	while (i--) {
		Mc_anc *anc=ancee.listen(-1);

		if (!anc) {
			continue;
		}

		if (i%50==0) {
			dump_loads();
		}

		add_loads(anc);
		delete anc;
	}

	return 0;
}
