// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: load_struct.h,v $ - structs for load monitors.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _LOAD_STRUCT_H_
# define _LOAD_STRUCT_H_

# include <netinet/in.h>
# include <time.h>

struct disk {
	unsigned long btotal;
	unsigned long bavail;
	unsigned long ftotal; 
	unsigned long ffree;
};

struct load {
	char host[256];
	time_t when;
	unsigned int cputotal, user, kernel, idle, iowait; // delta in 10ms
	unsigned int cpus; // # of CPUs
	unsigned int cswitches; // delta
	unsigned int mused, mtotal, mfree, mshared; // memory snapshot KB
	unsigned int mbuffers, mcached, stotal, sfree; // memory snapshot KB
	struct disk disks[10];
	int ndisks;
};

# define KEY_CPU_TOT ((unsigned char)0x10)
# define KEY_CPU_USR ((unsigned char)0x11)
# define KEY_CPU_KER ((unsigned char)0x12)
# define KEY_CPU_IOW ((unsigned char)0x13)
# define KEY_CPU_IDL ((unsigned char)0x14)
# define KEY_CPU_NUM ((unsigned char)0x15)
# define KEY_CPU_CXW ((unsigned char)0x16)
# define KEY_MEM_TOT ((unsigned char)0x17)
# define KEY_MEM_USE ((unsigned char)0x18)
# define KEY_MEM_FRE ((unsigned char)0x19)
# define KEY_MEM_SHR ((unsigned char)0x1A)
# define KEY_MEM_BUF ((unsigned char)0x1B)
# define KEY_MEM_CCH ((unsigned char)0x1C)
# define KEY_SWP_TOT ((unsigned char)0x1D)
# define KEY_SWP_FRE ((unsigned char)0x1E)
# define KEY_DSK_FRE ((unsigned char)0x1F)

# endif
