// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: mcast_annc.cc,v $ - multicast announcer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/09/29 00:15:14 $
 * Version :  $Revision: 1.11 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <netinet/in.h>
# include <time.h>
# include <unistd.h>
# include "sock_conn.h"
# include "sock_dg.h"
# include "udp_common.h"
# include "../guid/guid.h"
# include "mcast_annc.h"

using namespace std;

///////////////////////////////////////////
// buf points to the buf field of a Udp_msg
// len is the length of the buf field
// if a message is truncated, we process
// the available parts.
int Mc_anc_data::unpack(void *buf, int len)
{
	unsigned char *cbuf=(unsigned char *)buf;
	int l=sizeof(time_t);
	if (len<l) {
		return -1;
	}

	// already unpacked.
	// Even though it is not a problem to aggregate
	// multiple announcements, but it makes no
	// sense unless they are from the same announcer.
	if (records.size()>0) {
		return -1;
	}
	memcpy(&ts, cbuf, sizeof(time_t));
	while ((int)(l+sizeof(unsigned short)+sizeof(unsigned char))<=len) {
		unsigned short sll;
		int ll;
		memcpy(&sll, cbuf+l+sizeof(unsigned char), sizeof(unsigned short));
		ll=sll+sizeof(unsigned char)+sizeof(unsigned short);
		if (l+ll>len) {
			break;
		}
		if (add((int)cbuf[l], &(cbuf[l+sizeof(unsigned char)+sizeof(unsigned short)]), (int)sll)<0) {
			return -1;
		}
		l+=ll;
	}

	return 0;
}

Mc_anc_data::~Mc_anc_data() 
{ 
	vector<Mc_rec *>::iterator p=records.begin();
	for (; p!=records.end(); p++) {
		if (*p) {
			free(*p);
			*p=NULL;
		}
	}
	records.clear();
}

int Mc_anc_data::add(int var, void *val, int len)
{
	if (!val) {
		return -1;
	}

	if ((var>0xff)||(var<0)) {
		return -1;
	}

	if ((len>0xffff)||(len<0)) {
		return -1;
	}

	Mc_rec *rec=(Mc_rec *)malloc(MC_REC_HEADER_SZ+len);
	if (!rec)
		return -1;

	rec->var=(unsigned char)var;
	rec->len=(unsigned short)len;
	memcpy(rec->val(), val, len);
	records.push_back(rec);

	return 0;
}

int Mc_anc_data::pack(void *buf, int *len)
{
	int l=sizeof(time_t);
	unsigned char *cbuf=(unsigned char *)buf;

	if ((*len)<(int)sizeof(time_t)) {
		return -1;
	}
	time(&ts);
	memcpy(cbuf, &ts, sizeof(time_t)); 
	vector<Mc_rec *>::iterator p=records.begin();
	for (; p!=records.end(); p++) {
		if (*p) {
			int ll=sizeof(unsigned char)+sizeof(unsigned short)+(*p)->len;
			if ((l+ll)>(*len)) {
				return -1;
			}
			cbuf[l]=(*p)->var;
			memcpy(cbuf+l+sizeof(unsigned char), &((*p)->len), sizeof(short));
			memcpy(&cbuf[l+sizeof(unsigned char)+sizeof(unsigned short)], 
					(*p)->val(), (*p)->len);
			l+=ll;
		}
	}

	*len=l;

	return 0;
}

void Mc_anc_data::free_result(Mc_rec **result) 
{
	free(result);
}

Mc_rec **Mc_anc_data::dump() 
{
	Mc_rec **ret=(Mc_rec **)calloc(records.size()+1, sizeof(Mc_rec *));

	if (!ret)
		return NULL;

	memset(ret, 0, sizeof(Mc_rec *)*(records.size()+1));
	vector<Mc_rec *>::const_iterator p=records.begin();
	int i=0;
	for (; p!=records.end(); p++) {
		if (*p) {
			ret[i++]=*p;
		}
	}

	if (i==0) {
		free(ret);
		return NULL;
	}
	
	ret=(Mc_rec **)realloc(ret, (i+1)*sizeof(Mc_rec *));
	ret[i]=NULL;

	return ret;
}

Mc_rec **Mc_anc_data::lookup(int var) 
{
	Mc_rec **ret=(Mc_rec **)calloc(records.size()+1, sizeof(Mc_rec *));

	if (!ret)
		return NULL;

	memset(ret, 0, sizeof(Mc_rec *)*(records.size()+1));
	vector<Mc_rec *>::const_iterator p=records.begin();
	int i=0;
	for (; p!=records.end(); p++) {
		if (*p) {
			if ((int)((*p)->var)==var) {
				ret[i++]=*p;
			}
		}
	}

	if (i==0) {
		free(ret);
		return NULL;
	}
	
	ret=(Mc_rec **)realloc(ret, (i+1)*sizeof(Mc_rec *));
	ret[i]=NULL;

	return ret;
}

////////////////////////////////////////////////
// buf is the raw datagram received
Mc_anc::Mc_anc(void *buf, int len, const char *h, int p) 
{
	ad[0]='\0';
	pt=SOCK_PORT_ANY;
	sq=UDP_SEQ_ERR;

	if ((!buf)||(len<(int)UDP_MSG_HEADER_SZ)||(!h)||(p<=0)) {
		return;
	}
	Udp_msg *msg=(Udp_msg *)buf;

	if (msg->magic!=UDP_MSG_ANC_MAG)
		return;
	if (len!=msg->len)
		return;
	if ((msg->seq==UDP_SEQ_ANY)||(msg->seq==UDP_SEQ_ERR)) {
		return;
	}
	if (strlen(h)>15) {
		return;
	}
	sq=msg->seq;
	strcpy(ad, h);
	pt=p;
	unpack(msg->buf, len-UDP_MSG_HEADER_SZ);
}

Mc_ancer::Mc_ancer(const char *group, int p)
{
	addr[0]='\0';
	port=-1;
	next_seq=0;
	// char prime_ip[16];

	// strcpy(prime_ip, sock_prime_ip());
	s=sock_mcast(NULL, SOCK_PORT_ANY, NULL, NULL, NULL, 0);

	if (s<0)
		return;

	struct sockaddr_in sin;

	if (sock_filladdr(group, p, &sin)<0) {
		close(s);
		s=-1;
		return;
	}
	if (!sock_is_mcast_addr(&sin, sizeof(sin))) {
		close(s);
		s=-1;
		return;
	}
	// store IP string, avoid DNS lookup cost.
	if (sock_dotaddr(addr, 16, &(sin.sin_addr))<0) {
		close(s);
		s=-1;
		return;
	}
	port=p;
	next_seq=Guid::random_ui();
	next_seq=justify_seq(next_seq);
}

int Mc_ancer::announce(Mc_anc_data *acment)
{
	char buf[UDP_MAX_DG_SZ];
	int len=UDP_MAX_DG_SZ-UDP_MSG_HEADER_SZ;
	Udp_msg *msg=(Udp_msg *)buf;

	if (s<0) {
		return -1;
	}

	if (acment->pack(((Udp_msg *)buf)->buf, &len)<0) {
		return -1;
	}

	msg->magic=UDP_MSG_ANC_MAG;
	len+=UDP_MSG_HEADER_SZ;
	msg->len=len;
	msg->seq=next_seq++;

	if (sock_writeto(s, addr, port, buf, &len)<0) {
		return -1;
	}

	return 0;
}

Mc_anc *Mc_ancee::listen(int tout)
{
	char host[16]="";
	int port=SOCK_PORT_ANY;
	char buf[UDP_MAX_DG_SZ];
	int len=UDP_MAX_DG_SZ;

	if (sock_readfrom(s, host, 16, &port, buf, &len, tout)<0) {
		return NULL;
	}

	if (len<0) { // timeout
		return NULL;
	}

	Mc_anc *ac=new Mc_anc(buf, len, host, port);
	if (ac->seq()==UDP_SEQ_ERR) {
		delete ac;
		return NULL;
	}

	return ac;
}
