// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: mcast_annc.h,v $ - header file for multicast announcer.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/11/07 08:12:17 $
 * Version :  $Revision: 1.14 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _MCAST_ANNC_H_
# define _MCAST_ANNC_H_
# include <time.h>
# include <vector>
# include <unistd.h>
# include "../guid/guid.h"
# include "sock_dg.h"
# include "udp_common.h"

// an announcement, the value is always 4-byte aligned.
// is it enough for double value?
struct Mc_rec {
	unsigned char var;
	unsigned short len;
	union {
		char val[1]; // integer aligned 
		int pad;
	} u;

public :
	void *val() { return &(u.val); }
};

# define MC_REC_HEADER_SZ ((unsigned int)&(((Mc_rec *)0L)->u.val))

// used by ancer
class Mc_anc_data {
protected:	
	time_t ts;
	std::vector<Mc_rec *>records;

public:
	Mc_anc_data() { ts=(time_t)-1; return; }
	~Mc_anc_data(); 

	int add(int var, void *val, int len);
	int pack(void *buf, int *len);
	int unpack(void *buf, int len);
	time_t tstamp() { return ts;}
	Mc_rec **lookup(int var); 
	Mc_rec **dump(); 
	void free_result(Mc_rec **result);
};

// used by ancee
class Mc_anc : public Mc_anc_data {
private:
	char ad[16];
	int pt;
	seq_t sq;
public:
	Mc_anc(void *buf, int len, const char *h, int p); 
	~Mc_anc() { return;}
	seq_t seq() { return sq; }
	char *addr() { return ad; }
	int port() { return pt; }
	// invalidate methods in the base class.
	int add(int var, void *val, int len) { return -1;}
};

class Mc_ancer {
private :
	int s;
	char addr[16];
	int port;
	seq_t next_seq;

	static seq_t justify_seq(seq_t sq)
	{
		while ((sq==UDP_SEQ_ANY)||(sq==UDP_SEQ_ERR)) {
			sq++;
		}

		return sq;
	}

public :
	Mc_ancer(const char *group, int p);
	~Mc_ancer() {if (s>=0) close(s); }
	int announce(Mc_anc_data *acment);
	bool good() { return (s>=0); }
};

class Mc_ancee {
	int s;
public :
	Mc_ancee(const char *group, int port, bool loopback=true) 
	{
		s=sock_mcast(group, port, NULL, NULL, NULL, (loopback)?0:1);
	}

	~Mc_ancee() { if (s>=0) close(s); }

	Mc_anc *listen(int tout=-1);
	bool good() { return (s>=0); }
};

# endif
