// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: mtu_cli2.cc,v $ - UDP bandwidth tester II client.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/05 07:12:20 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <arpa/inet.h>
# include <sys/time.h>
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <unistd.h>
# include "sock_conn.h"
# include "sock_dg.h"

int main(int argc, char *argv[])
{
	if (argc!=3) {
		printf("Usage: %s <host> <port>\n", argv[0]);
		exit(-1);
	}

	int port=atoi(argv[2]);

	int fd=sock_udp(NULL, SOCK_PORT_ANY);

	if (fd<0) {
		printf("Cannot create UDP socket.\n");
		exit(-1);
	}
	
	struct sockaddr_in sin;
	sock_filladdr(argv[1], port, &(sin));
	char svrdotaddr[128];
	char buf[65536];
	sock_dotaddr(svrdotaddr, 128, &(sin.sin_addr));
	int i=8;
	while (1) {
		bool started=false;
		int begin=i;
		int len1=sizeof(int);
		while (sock_writeto(fd, svrdotaddr, port, &begin, &len1)==0)
		{
			if (len1 != sizeof(int))
				break;
			if (sock_readfrom(fd, svrdotaddr, 128, &port, &begin, &len1, 1000)<0)
				break;
			if (len1<0) {// timeout
				printf("Timeout\n");
				fflush(stdout);
				len1=sizeof(int);
				begin=i;
				continue;
			}
			if (len1 != sizeof(int))
				break;

			if (begin != i) {
				printf("size not match (%d, %d).\n", begin, i);
				fflush(stdout);
				len1=sizeof(int);
				begin=i;
				continue;
			}
			started=true;
			break;
		}

		i+=50;
		if (i>=10000)
			break;

		if (!started) {
			printf("SIZE=%d skipped.\n", i);
			continue;
		}

		printf("Begin flowding server with %d-byte packets.\n", i);
		fflush(stdout);
		for (int j=0; j<10000; j++) {
			int len=i;
			sock_writeto(fd, svrdotaddr, port, buf, &len);
		}
	}

	return 0;
}
