// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: mtu_srv2.cc,v $ - UDP bandwidth tester II server.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/time.h>
# include <unistd.h>
# include <iostream>
# include <stdlib.h>
# include <stdio.h>
# include "../usc/usc_time.h"
# include "sock_conn.h"
# include "sock_dg.h"

using namespace std;

static int run(char *host, int port)
{
	int fd=sock_udp(host, port);
	char *host_str=((host)&&(host[0]))?host:sock_hostname();

	if (fd<0) {
		cerr << "Cannot launch server: fail to bind to address: " 
			 << host_str << ":";
		if (port>0)
			cerr << port;
		else
			cerr << "<port_any>";
		cerr << endl;
		return -1;
	}

	cout << "Server started @ " 
		 << host_str << ":" << sock_myport(fd) << endl;

	char fromhost[128];
	int fromport;
	char buf[65536];
	usc_time_t start_t=0, end_t=0;
	int sum_bytes=0;
	int last_pkt_size=0;
	bool ooo_print=true;
	while (1) {
		int len=65536;
		fromhost[0]='\0';
		fromport=SOCK_PORT_ANY;
		if (sock_readfrom(fd, fromhost, 128, &fromport, buf, &len, -1)==0) {
			if (len==sizeof(int)) {
				if (sum_bytes==0) {
					start_t=usc_gettime();
				}
				else {
					end_t=usc_gettime();
					printf("Size=%d Bytes, Rate = %.3fMB/s\n", *(int *)buf, sum_bytes/usc_difftime(end_t, start_t));
					fflush(stdout);
					sum_bytes=0;
					start_t=usc_gettime();
				}
				sock_writeto(fd, fromhost, fromport, buf, &len);
			}
			else if (len>0) {
				sum_bytes+=len;
				if (last_pkt_size>len) {
					if (ooo_print) {
						printf("Out-of-order delivery detected.\n");
						fflush(stdout);
						ooo_print=false;
					}
				}
				last_pkt_size=len;
			}
		}
	}

	return 0;
}

int main(int argc, char *argv[])
{
	char *host;
	int port;

	if (argc==3) {
		host=argv[1];
		port=atoi(argv[2]);
	}
	else if (argc==2) {
		host=NULL;
		port=atoi(argv[1]);
	}
	else if (argc==1) {
		host=NULL;
		port=SOCK_PORT_ANY;
	}
	else {
		cout << "Usage: " << argv[0] << "[[host] port]" << endl;
		exit(-1);
	}

	run(host, port);

	return 0;
}
