// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: seltcp_cli.c,v $ - Testing TCP receive with timeout, the client.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.2 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/types.h>
# include <sys/ioctl.h>
# include <fcntl.h>
# include <termios.h>
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include <ctype.h>
# include "sock_conn.h"
# include "sock_sr.h"


/**
#ifndef SYSV
struct sgttyb savetty, settty;

int set_term()
{
	struct sgttyb savetty, settty;
	ioctl(0, TIOCGETP, &savetty);
	ioctl(0, TIOCGETP, &settty);
	settty.sg_flags |= CBREAK; 
	settty.sg_flags &= ~ECHO;
	ioctl(0, TIOCSETP, &settty);
}

int reset_term()
{
	ioctl(0, TIOCSETP, &savetty);
}
#else
**/

struct termio savetty, settty;

int set_term()
{
	ioctl(0, TCGETA, &savetty);
	ioctl(0, TCGETA, &settty);
	settty.c_lflag &= ~ICANON;
	/* settty.c_lflag &= ~ECHO; */
	ioctl(0, TCSETAF, &settty);

	return 0;
}

int reset_term()
{
	ioctl(0, TCSETAF, &savetty);

	return 0;
}
/** # endif **/

int readinput(char *buf, int len)
{
	int c;
	
	if (len<=0)
		return -1;

	c=getchar();
	if ((c>=0)&&(c<=255)) {
		*buf=c;
		return 1;
	}

	return -1;
}

int main(int argc, char *argv[])
{
	int s;
	int port;
	int len;
	char buf[1024];

	if (argc!=2) {
		printf("Usage: %s <server-port>\n", argv[0]);
		return -1;
	}
	
	set_term();

	port=atoi(argv[1]);
	if ((s=sock_conn(".", port))<0) {
		perror("Cannot connect to server.\n");
		reset_term();
		exit(-1);
	}
	while ((len=readinput(buf, 1024))>0) {
		if ((!isprint(buf[0]))&&(buf[0]!='\n'))
			break;
		sock_write(s, buf, len);
	}
	close(s);

	reset_term();

	return 0;
}
