// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: seludp_srv.c,v $ - testing UDP recv with timeout, the server.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <stdio.h>
# include <unistd.h>
# include <stdlib.h>
# include "sock_conn.h"
# include "sock_dg.h"

int recvline(int fd, char *buf, int len, int tout)
{
	char *p;

	p=buf;
	for (; p<buf+len; p++) {
		int len=1;
		if (sock_readfrom(fd, NULL, 0, NULL, p, &len, tout)<0) {
			*p='\0';
			return -1;
		}

		if (len<0) {
			*p='\0';
			return -1;
		}

		if (*p=='\n')
			break;
	}

	*p='\0';
	
	return 0;
}

void readline(char *pmt)
{
	char buf[1024];

	printf("%s", pmt);
	fgets(buf, 1024, stdin);
	return;
}

int main()
{
	int s=sock_udp(NULL, SOCK_PORT_ANY);
	int port;
	char buf[1024];

	if (s<0) {
		perror("Cannot create UDP socket.\n");
		exit(-1);
	}
	port=sock_myport(s);
	if (port<0) {
		perror("Cannot get bind port.\n");
		exit(-1);
	}

	printf("Please launch command \"seludp_cli %d\".\n", port);

	printf("Wait a second, type something from seludp_cli and hit <enter>.\n");

	if (recvline(s, buf, 1024, -1)<0) {
		printf("Failed to receive a line in blocked read.\n");
		printf("Echo partial message: %s\n", buf);
	}
	else {
		printf("Echo message: %s\n", buf);
	}

	printf("Type something from seltcp_cli and hit <enter>.\n");
	readline("Hit <enter> now ...");
	if (recvline(s, buf, 1024, 0)<0) {
		printf("Failed to receive a line in non-blocked read.\n");
		printf("Echo partial message: %s\n", buf);
	}
	else {
		printf("Echo message: %s\n", buf);
	}

	printf("I'm going to keep reading one char for 1 seconds and please give me something from seltcp_cli.\n");
	if (recvline(s, buf, 1024, 1000)<0) {
		printf("Failed to receive a line in time-out read.\n");
		printf("Echo partial message: %s\n", buf);
	}
	else {
		printf("Echo message: %s\n", buf);
	}

	printf("I'm going to keep reading one char for 5 seconds and please give me something from seltcp_cli.\n");
	if (recvline(s, buf, 1024, 5000)<0) {
		printf("Failed to receive a line in time-out read.\n");
		printf("Echo partial message: %s\n", buf);
	}
	else {
		printf("Echo message: %s\n", buf);
	}

	close(s);
	return 0;
}
