// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sock_conn.h,v $ - header file for sock_conn.c. Socket library
 *            wrapper of TCP connection and name/addr management.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/09/30 01:14:12 $
 * Version :  $Revision: 1.12 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _SOCK_CONN_H_
# define _SOCK_CONN_H_

# define SOCK_PORT_ANY (0)

# include <netdb.h>

# if defined(__cplusplus)
extern "C" {
# endif

struct in_addr;
struct sockaddr_in;

/*---------------------- client API ----------------------*/
/* return fd on success or -1 on error */
int sock_conn(const char *host, int port);
int sock_async_conn(const char *host, int port, int tout);
int sock_conn2(const char *host, int port, const char *bind_host);

/*---------------------- server API ----------------------*/
/* return fd on success or -1 on error */
int sock_listen(int port, int backlog);
int sock_listen2(int port, int backlog, const char *bind_host);

/* fd is returned by sock_listen */
int sock_accept(int fd);

/*---------------------- common API ----------------------*/
char *sock_hostname(void); /* my host name */
char *sock_prime_ip(void); /* my primary IP address */
int sock_myport(int fd);
int sock_peerport(int fd);
char *sock_myip(int fd);
char *sock_peerip(int fd);
char *sock_peerhost(int fd);
struct hostent *sock_gethostbyname(const char *host);
int sock_dotaddr(char *szAddr, int len, const struct in_addr *addr);
int sock_filladdr(const char *host, int port, struct sockaddr_in *addr);

/**
 * Set sending and receiving buffer size.
 */
# define SOCK_SNDBUF_SZ 100000
# define SOCK_RCVBUF_SZ 100000

int sock_setbufsize(int fd, int snd, int rcv);

/**
 * An IP address can be represented as four octets separated
 * by dots. E.g. 128.111.41.10. The following macros allows
 * us to extract the individual bytes from a network-ordered
 * IPv4 address. In the above example: 
 * OCTET_1=128, 
 * OCTET_2=111, 
 * OCTET_3=41, 
 * OCTET_4=10, 
 */
# define OCTET_1(n) (((unsigned char *)&n)[0])
# define OCTET_2(n) (((unsigned char *)&n)[1])
# define OCTET_3(n) (((unsigned char *)&n)[2])
# define OCTET_4(n) (((unsigned char *)&n)[3])

# if defined(__cplusplus)
}
# endif

# endif
