// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2000, 2001, 2002 The Regents of the University of 
  California, All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sock_sr.h,v $ - header file for sock_sr.c. Socket library wrapper
 *            for TCP traffic send/recv (with timeouts).
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/01 23:50:06 $
 * Version :  $Revision: 1.6 $
 *
 * Originally from TMPI project.
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# ifndef _SOCK_SR_H_
# define _SOCK_SR_H_

# include <sys/uio.h>
/* # include "sock_sos.h"  */

# if defined(__cplusplus)
extern "C" {
# endif

/**
 * the following functions return 0 on success and -1 on error 
 * It treats the data as an integral whole. And it only succeeds
 * when you completely read/write exactly len bytes of data.
 */

int sock_write(int fd, char *data, int len);
int	sock_writev(int fd, struct iovec *iov, int count);
int	sock_read(int fd, char *data, int len);

/**
 * sock_read_tout attempts to read as much as possible until
 * timeout occurs. On error or prematured timer, it returns the errno;
 * otherwise, it returns 0. In all cases, *len keeps the 
 * actual number of bytes that are valid for read (including 0).
 * (Timeout can be indicated by a return value of 0 and *len==0)
 * Note this is slightly different from previous version of
 * the implementation.
 */
int	sock_read_tout(int fd, char *data, int *len, int tout);

/**
 * The following return the actual bytes read by the read operation,
 * up to len bytes.
 */

/* Read until either the buffer is full or the peer closes the connection */
int	sock_read_till_close(int fd, char *data, int len);

/* Read once, will not block if there is data available */
int sock_readmax(int fd, char *data, int len);

/** 
 * the following return 1 for "yes" or 0 for "no", and "-1" for error 
 * Note since "error" also implies the subsequent read/write will not 
 * block, so even pepole don't check error return, they will discover
 * so afterwards.
 */

int sock_canread(int fd);
int sock_canwrite(int fd);

/** 
 * Socket version of fgets.
 */
char *sock_gets(char *data, int len, int fd);

# if defined(__cplusplus)
}
# endif

# endif
