/**
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include <inttypes.h>
**/

# include <netinet/in.h>
# include <signal.h>
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>
# include "sp_client.h"
# include "sock_conn.h"
# include "sock_sr.h"
# include "../usc/usc_time.h"

static SocketPoolClient *spclt=NULL;
static int size = 128*(1<<10);;
static int count = 1000;
static int port = 0;
static char host[1024] = "";
static int n_thr=1;

void usage(char *prog)
{
    fprintf(stderr,"The Client is compiled on %s %s\n",__DATE__,__TIME__);
    fprintf(stderr,"Usage: %s [options]\n",prog);
    fprintf(stderr,"Options are (default):\n");
    fprintf(stderr,"\t-s message size (%dKB)\n", size/1024);
    fprintf(stderr,"\t-n number of threads (%d).\n", n_thr);
    fprintf(stderr,"\t-m send count (%d)\n", count);
    fprintf(stderr,"\t-h host\n");
    fprintf(stderr,"\t-p port\n");
}

extern "C" void *thr(void *arg)
{
    char *data;
    int sockfd;
    int k;
    int tmp;
	int myid=(int)arg;

    data = (char *)malloc(size);
    if (data == NULL) {
        fprintf(stderr, "Out of memory!\n");
        return NULL;
    }

    for (k=0; k<count; k++)
    {
        sockfd = spclt->checkout(host, port);
        if (sockfd < 0) {
            if ((sockfd = sock_conn(host, port)) < 0) {
                fprintf(stderr, "Cannot connect to server: (%s:%d).\n", host, port);
                exit(1);
            }
        }
	    tmp = htonl(size);
        if (sock_write(sockfd, (char *)&tmp, sizeof(int)) != 0) {
			close(sockfd);
			continue;
		}
        if (sock_write(sockfd, data, size) != 0) {
			close(sockfd);
			continue;
		}

        spclt->checkin(sockfd, host, port);

		if (myid==0) {
			printf("\r%d    ", k);
			fflush(stdout);
		}
		usleep(random()%100000);
    }
    free(data);

	return NULL;
}

int main(int argc, char *argv[])
{
	usc_time_t st, et;
    char c;

    while ((c = getopt(argc, argv, "n:m:s:h:p:")) != EOF) {
        switch (c) {
        case 'n':
            n_thr=atoi(optarg);
            break;
        case 'm':
            count=atoi(optarg);
            break;
        case 's':
            size=atoi(optarg);
            break;
        case 'h':
            strcpy(host,optarg);
            break;
        case 'p':
            port=atoi(optarg);
            break;
        default:
            usage(argv[0]);
            exit(1);
        }
    }

    if ( (host[0] == '\0') || (port <= 0) || (count <= 0) || (size <= 0) || (n_thr <= 0) ) {
        usage(argv[0]);
        exit(1);
    }

    signal(SIGBUS, SIG_IGN);
    signal(SIGSEGV, SIG_IGN);
    signal(SIGPIPE, SIG_IGN);

	spclt=new SocketPoolClient(1024);

    st=usc_gettime();

	pthread_t thrs[n_thr];

	for (int i=0; i<n_thr; i++) {
		pthread_create(&thrs[i], NULL, &thr, (void *)i);
	}

	for (int i=0; i<n_thr; i++) {
		pthread_join(thrs[i], NULL);
	}

    et=usc_gettime();

	spclt->summary();
	spclt->dump();
	double total_time = usc_difftime(et, st)/1000000;
    fprintf(stderr, "Total Time: %.3f s, Average: %.3f s\n", total_time, total_time/count);

    return 0;
}
