// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: sp_singleton.cc,v $ - socket pool singleton 
 * 	          implementation. This module provides a simplified interface
 * 	          and maintains singleton instances of spclt and spsrv.
 * 	          Note that sp_client_xxx are stateless while spsrv requires
 *            proper initialization. This is because a live instance of
 *            SocketPoolServer takes precious resources of descriptors
 *            while SocketPoolClient only consumes small amount of memory.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/10/04 18:41:34 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <unistd.h>
# include "sp_server.h"
# include "sp_singleton.h"

# ifndef DISABLE_SOCKET_POOL

# include "sp_client.h"

static SocketPoolClient spclt(1024);
static SocketPoolServer *spsrv=NULL;

int sp_client_get_one(const char *ip, int port)
{
	return spclt.checkout(ip, port);
}

int sp_client_put_one(int fd, const char *ip, int port)
{
	return spclt.checkin(fd, ip, port);
}

void sp_client_dump()
{
	return spclt.dump();
}

int sp_server_init(int max_sz, int port)
{
	spsrv=new SocketPoolServer(max_sz, port);

	return 0;
}

int sp_server_finalize()
{
	if (spsrv)
		delete spsrv;

	spsrv=NULL;

	return 0;
}

int sp_server_get_port()
{
	if (spsrv) {
		return spsrv->get_port();
	}

	return -1;
}

int sp_server_get_one(int tout_in_ms)
{
	if (spsrv) {
		return spsrv->get_one(tout_in_ms);
	}

	return -1;
}

int sp_server_put_one(int fd)
{
	if (spsrv) {
		return spsrv->put_one(fd);
	}

	return -1;
}

# else

static SocketPoolServer *spsrv=NULL;

int sp_client_get_one(const char *ip, int port)
{
	return -1;
}

int sp_client_put_one(int fd, const char *ip, int port)
{
	close(fd);
	return 0;
}

void sp_client_dump()
{
	return;
}

int sp_server_init(int max_sz, int port)
{
	spsrv=new SocketPoolServer(max_sz, port);

	return 0;
}

int sp_server_finalize()
{
	if (spsrv)
		delete spsrv;

	spsrv=NULL;

	return 0;
}

int sp_server_get_port()
{
	if (spsrv) {
		return spsrv->get_port();
	}

	return -1;
}

int sp_server_get_one(int tout_in_ms)
{
	if (spsrv) {
		return spsrv->get_one(tout_in_ms);
	}

	return -1;
}

int sp_server_put_one(int fd)
{
	close(fd);

	return 0;
}

# endif
