// -*- mode:C; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t3.c,v $ - testing multicast send/recv.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/time.h>
# include <netinet/in.h>
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>
# include <string.h>
# include "sock_conn.h"
# include "sock_dg.h"
# include "../usc/usc_time.h"

struct load {
	char host[16];
	unsigned short load;
} ld_tab[100];

int ld_tab_size=0;

void add_load(char *h, int load)
{
	int i;
	for (i=0; i<ld_tab_size; i++) {
		if (strcmp(h, ld_tab[i].host)==0) {
			ld_tab[i].load=load;
			return;
		}
	}
	if (ld_tab_size==100)
		return;
	strcpy(ld_tab[ld_tab_size].host, h);
	ld_tab[ld_tab_size].load=load;
	ld_tab_size++;

	return;
}

void dump_load()
{
	int i;
	for (i=0; i<ld_tab_size; i++) {
		printf("%s\t: %d\n", ld_tab[i].host, ld_tab[i].load);
	}

	printf("================================================================\n");
}

int main(int argc, char *argv[])
{
	int i;
	int len;
	int port;
	int stop=0;
	int fd;
	struct timeval tv;

	if (argc!=3) {
		printf("Usage: %s <mcast grp> <port>.\n", argv[0]);
		return -1;
	}

	port=atoi(argv[2]);
	fd=sock_mcast(argv[1], port, NULL, NULL, NULL, 0);
	if (fd<0) {
		printf("Cannot create Multicast port @ %s:%d.\n", argv[1], port);
		exit(-1);
	}

	usc_init();
	gettimeofday(&tv, NULL);
	srand(usc_gettime()+tv.tv_sec+tv.tv_usec);
	i=rand()%100+2;

	while (!stop) {
		int fi;
		char host[16];
		int fp=port;

		len=sizeof(i);
		if (sock_writeto(fd, argv[1], port, &i, &len)<0) {
			printf("Cannot write to the multicast group.\n");
			dump_load();
			exit(-1);
		}
		if (len!=sizeof(int)) {
			printf("Failed to write the complete packet .\n");
			dump_load();
			exit(-1);
		}

		/* wait for a while and do a number of readfrom till nothing is received */
		strcpy(host, argv[1]);
		usleep(rand()%100000+10000);
		while (sock_readfrom(fd, host, 16, &fp, &fi, &len, 0)==0) {
			if (len!=sizeof(int))
				break;
			if (fi==0) {
				stop=1;
				break;
			}
			if (fi==1) {
				dump_load();
			}
			else {
				add_load(host, fi);
			}
		}
	}

	dump_load();
	close(fd);
	return 0;
}
