// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t4_cli.cc,v $ - UDP asynchrous request/response tester client.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 

# include <sys/time.h>
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <unistd.h>
# include "udp_cli.h"
# include "../usc/usc_time.h"
# include "sock_conn.h"

# define NUM_ITER 100

struct {
	seq_t seq;
	unsigned int val;
} data[NUM_ITER];

typedef struct {
	int idx;
	unsigned int ui;
} RES ;

void dump_data()
{
	for (int i=0; i<NUM_ITER; i++) {
		if (data[i].val!=0) {
			printf("0x%08X : 0x%08X\n", data[i].seq, data[i].val);
		}
	}
}

void handle_res(Udp_res *res)
{
	if (!res) {
		printf("Got no responses (all timeout?).\n");
		return;
	}

	if (res->status!=UDP_RES_OKAY) {
		printf("Bad response status.\n");
		res->dump();
		delete res;
		return;
	}

	if (res->length()!=sizeof(RES)) {
		printf("Bad response length.\n");
		res->dump();
		delete res;
		return;
	}

	int idx=((RES *)(res->result()))->idx;
	if ((idx<0)||(idx>=NUM_ITER)) {
		printf("Bad index number in response.\n");
		res->dump();
		delete res;
		return;
	}

	if (res->seq()!=data[idx].seq) {
		printf("Bad sequence number in response.\n");
		res->dump();
		delete res;
		return;
	}

	data[idx].val=((RES *)(res->result()))->ui;
	delete res;
}

int main(int argc, char *argv[])
{
	if (argc!=3) {
		printf("Usage: %s <host> <port>\n", argv[0]);
		exit(-1);
	}

	int port=atoi(argv[2]);

	Udp_cli *cli=new Udp_cli(NULL, SOCK_PORT_ANY, 100);

	if (!cli) {
		printf("Cannot create a UDP client.\n");
		exit(-1);
	}

	struct timeval tv;
	gettimeofday(&tv, NULL);
	srand(usc_gettime()+tv.tv_sec+tv.tv_usec);

	for (int i=0; i<NUM_ITER/10+1; i++) {
		int k;
		for (int j=0; j<10; j++) {
			k=i*10+j;
			if (k>=NUM_ITER) {
				break;
			}
			data[k].seq=cli->async_req(argv[1], port, &k, sizeof(int), 100);
			if (data[k].seq==UDP_SEQ_ERR) {
				printf("Cannot send async request, quitting...\n");
				exit(-1);
			}
			printf("0x%08X\n", data[k].seq);
		}
		Udp_res **ress=cli->wait_one(UDP_SEQ_ANY);
		Udp_res **p=ress;
		for (; *p!=NULL; p++) {
			handle_res(*p);
			if (*p)
				(*p)->dump();
			*p=NULL;
		}

		delete []ress;
		usleep(rand()%30000+5000);
	}

	Udp_res **ress=cli->wait_all();
	if (!ress) {
		printf("No responses returned from wait_all, weird.\n");

	}
	else {
		Udp_res **p=ress;
		for (; *p!=NULL; p++) {
			handle_res(*p);
			*p=NULL;
		}

		delete []ress;
	}

	delete cli;

	dump_data();
}
