// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t5_srv.cc,v $ - UDP async request/response tester II server.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/04/09 09:59:58 $
 * Version :  $Revision: 1.4 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/time.h>
# include <unistd.h>
# include <iostream>
# include "../guid/guid.h"
# include "../usc/usc_time.h"
# include "sock_conn.h"
# include "udp_srv.h"

using namespace std;

static unsigned int get_random_ui()
{
	Guid guid;
	unsigned int aui[4];
	guid.gen();
	guid.pack((unsigned char *)aui, sizeof(unsigned int)*4);
	return aui[0]^aui[1]^aui[2]^aui[3];
}

typedef struct {
	int idx;
} REQ;

typedef struct {
	int idx;
	unsigned int val;
} RES;

static int no_drop(double drop_prob)
{
	double r=(double)get_random_ui()/(double)((unsigned int)1<<31)/2;

	if (r>drop_prob)
		return 1;
	else
		return 0;
}

static void handle_the_request(Udp_srv *srv, Udp_srv_req *q)
{
	RES res;
	int len=sizeof(RES);

	res.idx=((REQ *)(q->request()))->idx;
	res.val=get_random_ui();

	if (no_drop(0.15)) {
		int repeat=rand()%10+1;
		for (int i=0; i<repeat; i++) {
			srv->respond(q, (void *)&res, len);
		}
	}

	delete q;
}

static int run(int s)
{
	Udp_srv *srv=new Udp_srv(s);

	if (!srv) {
		cout << "Cannot launch server.\n" << endl;
		return -1;
	}

	cout << "Server started @ " << 
		srv->host() << ":" << srv->port() << endl;

	while (Udp_srv_req *q=srv->wait_for_request()) {
		if (!q) {
			cout << "System error, server shutdown." << endl;
			break;
		}

		if (q->is_stop_req()) {
			cout << "Got shutdown request, server shutdown." << endl;
			break;
		}

		handle_the_request(srv, q);
	}

	delete srv;

	return 0;
}

int main(int argc, char *argv[])
{
	char *host;
	int port;

	if (argc==3) {
		host=argv[1];
		port=atoi(argv[2]);
	}
	else if (argc==2) {
		host=NULL;
		port=atoi(argv[1]);
	}
	else if (argc==1) {
		host=NULL;
		port=SOCK_PORT_ANY;
	}
	else {
		cout << "Usage: " << argv[0] << "[[host] port]" << endl;
		exit(-1);
	}

	struct timeval tv;
	gettimeofday(&tv, NULL);
	srand(usc_gettime()+tv.tv_sec+tv.tv_usec);

	int s=sock_mcast(host, port, NULL, NULL, NULL, 0);
	if (s<0) {
		cout << "Cannot create multicast port @ " << host << ":" << port << endl;
		exit(-1);
	}

	run(s);

	return 0;
}
