// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t6_cli.cc,v $ - testing the maximum size of a single UDP 
 *            write, the client.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2002/10/08 07:32:54 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <stdlib.h>
# include <stdio.h>
# include <unistd.h>
# include "sock_conn.h"
# include "sock_dg.h"

int main(int argc, char *argv[])
{
	if (argc!=3) {
		printf("Usage: %s <host> <port>\n", argv[0]);
		exit(-1);
	}

	int port=atoi(argv[2]);
	int last_success=1;
	int last_fail=0;
	int ssize=1;

	int s=sock_udp(NULL, SOCK_PORT_ANY);

	if (s<0) {
		printf("Cannot create a UDP client.\n");
		exit(-1);
	}

	while (1) {
		char *buf=(char *)malloc(ssize);
		int buf_len=ssize;
		int int_len=sizeof(buf_len);
		if (!buf) {
			printf("Cannot allocate %d bytes.\n", ssize);
			break;
		}
		if (sock_writeto(s, argv[1], port, buf, &buf_len)<0) {
			free(buf);
			printf("Writing %d bytes failed.\n", ssize);
			last_fail=ssize;
			ssize=(ssize+last_success)/2;
			if (ssize<=last_success)
				break;
			else 
				continue;
		}
		free(buf);

		bool failed=false;
		if (buf_len<ssize) {
			printf("Partial write %d/%d written.\n", buf_len, ssize);
			failed=true;
			break;
		}
		else if (buf_len>ssize) {
			printf("Weird, bad length from sock_writeto() %d>%d\n", buf_len, ssize);
			failed=true;
			break;
		}

		buf_len=0;
		if (sock_readfrom(s, NULL, 0, NULL, &buf_len, &int_len, 1000)<0) {
			printf("Didn't get server response for %d bytes.\n", ssize);
			failed=true;
		}
		else if (int_len<0) {
			printf("Read timed out for %d bytes.\n", ssize);
			failed=true;
		}
		else if (int_len!=sizeof(buf_len)) {
			printf("Weird, bad length from sock_readfrom()  size=%d for %d bytes request.\n", int_len, ssize);
			failed=true;
		}
		else if (buf_len < ssize) {
			printf("Server partial read %d/%d read.\n", buf_len, ssize);
			failed=true;
		}
		else if (buf_len>ssize) {
			printf("Bad acknowledgement %d>%d\n", buf_len, ssize);
			failed=true;
		}

		if (failed) {
			last_fail=ssize;
			ssize=(ssize+last_success)/2;
			if (ssize<=last_success)
				break;
		}
		else 
		{
			printf("%d-byte packet okay\n", ssize);

			last_success=ssize;
			if (last_fail==0) 
				ssize<<=1;
			else
				ssize=(ssize+last_fail)/2;

			if (ssize==last_success)
				break;
		}
	}

	close(s);

	return 0;
}
