// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t6_srv.cc,v $ - UDP maximum packet size tester, the server.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/06/05 07:12:20 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <sys/socket.h>
# include <iostream>
# include <stdlib.h>
# include "sock_conn.h"
# include "sock_dg.h"

using namespace std;

static int run(int s)
{
	static const int max_buf_sz=0x100000;
	char buf[max_buf_sz];
	char host[1024];
	int port;
	int buf_len=max_buf_sz;

	while (sock_readfrom(s, host, 1024, &port, buf, &buf_len, -1)==0) {
		int int_len=sizeof(buf_len);
		cout << "Got "<< buf_len << " bytes from " << host << ":" << port << endl;
		sock_writeto(s, host, port, &buf_len, &int_len);
		buf_len=max_buf_sz;
	}

	cout << "Cannot receive more data, quitting ... " << endl;

	return 0;
}

char *phost(char *host)
{
	if ((!host)||(host[0]=='\0')||(host[0]=='.')) {
		return sock_hostname();
	}
	return host;
}

int main(int argc, char *argv[])
{
	char *host;
	int port;

	if (argc==3) {
		host=argv[1];
		port=atoi(argv[2]);
	}
	else if (argc==2) {
		host=NULL;
		port=atoi(argv[1]);
	}
	else if (argc==1) {
		host=NULL;
		port=SOCK_PORT_ANY;
	}
	else {
		cout << "Usage: " << argv[0] << "[[host] port]" << endl;
		exit(-1);
	}

	int s=sock_mcast(host, port, NULL, NULL, NULL, 0);
	if (s<0) {
		s=sock_udp(host, port);
		if (s<0) {
			cout << "Cannot create socket @ " << phost(host) << ":" << port << endl;
			exit(-1);
		}
	}

	cout << "Starting server at "<< phost(host) << ":" << sock_myport(s) << endl;

	/**
	int rcvbuf_size=0;
	socklen_t len=sizeof(int);
	if (getsockopt(s, SOL_SOCKET, SO_RCVBUF, &rcvbuf_size, &len)<0) {
		cout << "Failed to get socket option: SO_RCVBUF." << endl;
	}
	else {
		cout << "Original socket receiver buffer size: " << rcvbuf_size << "."<< endl;
	}

	rcvbuf_size=100000;
	len=sizeof(int);
	if (setsockopt(s, SOL_SOCKET, SO_RCVBUF, &rcvbuf_size, len)<0) {
		cout << "Failed to set socket option: SO_RCVBUF." << endl;
	}
	**/

	run(s);

	return 0;
}
