// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: t7_cli.cc,v $ - testing the optimal packet size for UDP 
 *            send/recv.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/01/29 08:32:16 $
 * Version :  $Revision: 1.5 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# include <netinet/in.h>
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <unistd.h>
# include "sock_conn.h"
# include "sock_dg.h"
# include "../usc/usc_time.h"

# define UNIT 64
# define MAX_BUF (64*1024)
# define ITER (MAX_BUF/UNIT)
# define REPEAT 100

int lookup_host(char *name, char *ip, int len)
{
	struct sockaddr_in sin;
	memset(&sin, 0, sizeof(sin));

	sock_filladdr(name, 0, &sin);

	return sock_dotaddr(ip, len, &(sin.sin_addr));
}

/** UDP speed tester **/
int main(int argc, char *argv[])
{
	if (argc!=3) {
		printf("Usage: %s <host> <port>\n", argv[0]);
		exit(-1);
	}

	int port=atoi(argv[2]);
	char host[1024];

	if (lookup_host(argv[1], host, 1024)<0) {
		printf("Bad host.\n");
		exit(-1);
	}

	int s=sock_udp(NULL, SOCK_PORT_ANY);
	if (s<0) {
		printf("Cannot create a UDP client.\n");
		exit(-1);
	}

	int size=UNIT;
	double time_tab[ITER];

	for (int i=0; i<ITER; i++) {
		time_tab[i]=0.0;
	}

	for(; size<MAX_BUF; size+=UNIT) {
		char *buf=(char *)malloc(size);
		usc_time_t t0, t1;
		int failed=0;
		if (!buf) {
			printf("Cannot allocate %d bytes.\n", size);
			break;
		}
		t0=usc_gettime();
		for (int i=0; i<REPEAT; i++) {
			int buf_len=size;
			// int int_len=sizeof(buf_len);
			if (sock_writeto(s, host, port, buf, &buf_len)<0) {
				printf("Write to failed.\n");
				failed=1;
				break;
			}
			buf_len=size;
			if (sock_readfrom(s, NULL, 0, NULL, buf, &buf_len, 1000)<0) {
				printf("readfrom to failed.\n");
				failed=1;
				break;
			}
		}
		t1=usc_gettime();
		free(buf);
		if (failed)
			break;
		time_tab[size/UNIT]=(double)usc_difftime(t1,t0)/(size/UNIT)/REPEAT/2;
		usleep(1);
	}

	close(s);

	double optimal=0;
	for (int i=0; i<ITER; i++) {
		if (time_tab[i]!=0.0) {
			if (optimal==0.0) {
				optimal=time_tab[i];
			}
			else {
				if (optimal>time_tab[i])
					optimal=time_tab[i];
			}
		}
	}

	optimal=optimal*1.03;
	for (int i=0; i<ITER; i++) {
		if (time_tab[i]!=0.0) {
			if (time_tab[i]<optimal) {
				printf("***");
			}
			printf("Size=%d, %.2fus %d bytes\n", i*UNIT, time_tab[i], UNIT);
		}
	}


	return 0;
}
