// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: udp_common.h,v $ - shared constant/struct definition for UDP 
 *            servers and clients.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/08/01 05:40:57 $
 * Version :  $Revision: 1.10 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _UDP_COMMON_H_
# define _UDP_COMMON_H_

/** magic numbers **/
/* bad magic */
# define UDP_MSG_ERR_MAG ((unsigned short)0x0)
/* normal user request */
# define UDP_MSG_REQ_MAG ((unsigned short)0x8d2a)
/* normal user response */
# define UDP_MSG_RES_MAG ((unsigned short)0x7c19)
/* stop the server request */
# define UDP_MSG_STP_MAG ((unsigned short)0x6b08)
/* periodical announcement */
# define UDP_MSG_ANC_MAG ((unsigned short)0x5af7)

/* server shutdown response */
/** # define UDP_MSG_STD_MAG 0x5af7 **/

typedef unsigned int seq_t;

# define UDP_SEQ_ANY ((seq_t)0)
# define UDP_SEQ_ERR ((seq_t)-1)

// cast a datagram to structured data.
typedef struct udp_msg_ {
	unsigned short magic; // for basic verification
	unsigned short len;
	seq_t seq;
	char buf[4];
} Udp_msg ;

# define UDP_MSG_HEADER_SZ ((unsigned int)&(((Udp_msg *)0L)->buf))

/**
 * maximum 64KB datagrams
 */
# define UDP_MAX_DG_SZ (64*1024)
# define UDP_MAX_SND_DG_SZ (63*1024)
# define UDP_SAFE_SZ (60*1024)

/**
 * Optimal size seems to be the larger the better for ping-pong type test
 */
# define UDP_DG_SZ (32*1024)

# endif
