/**
% make test0004
gcc -lrt -Wall    test0004.c   -o test0004
% ./test0004
--> signal handler to be passed pointer:
0x8049180
<-- signal handler passed pointer:
0x8049180
Z
Z
Z
Z
(localhost) test% cat test0004.c
**/

/* An example of how to use signal notification as callback.
 * Amos Waterland <waterland@us.ibm.com>
 * 3 June 2002
 */

#include <aio.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define BYTES 20485760
#define TEMP "/tmp/test0004.dat"

static char buff[BYTES];

void handler( int val, siginfo_t *ptr, void *ignore )
{
	int i;
	void *data = (ptr->si_value).sival_ptr;

	printf( "<-- signal handler passed pointer: %p\n", data );
	fflush(
	    stdout );

	/* print the first four bytes of the array pointed to by pointer passed
	*/
	for (i = 0; i < 4; i++) {
		printf( "%c\n", ((char *)data)[i] );
	}
}

int main( int argc, char *argv[] )
{
	int i, fd;
	struct aiocb cb;
	struct sigaction sa;

	for (i = 0; i < BYTES; i++) {
		buff[i] = 'Z';
	}

	if ((fd = open( TEMP, O_CREAT | O_WRONLY, 0600 )) < 0) {
		perror( "error opening file" );
		exit( 1 );
	}

	sa.sa_sigaction = handler;
	sa.sa_flags = SA_SIGINFO;
	if (sigaction( SIGRTMIN, &sa, NULL)) {
		fputs( "error setting up signal handler\n", stderr );
		exit( 2 );
	}

	printf( "--> signal handler to be passed pointer: %p\n", (void *)buff
	      );
	fflush( stdout );

	cb.aio_fildes = fd;
	cb.aio_offset = 0;
	cb.aio_buf = buff;
	cb.aio_nbytes = BYTES;
	cb.aio_reqprio = 0;
	cb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
	cb.aio_sigevent.sigev_signo = SIGRTMIN;
	cb.aio_sigevent.sigev_value.sival_ptr = (void *)buff;

	if (aio_write( &cb )) {
		perror( "error writing to file" );
		exit( 3 );
	}

	while (aio_error( &cb ) == EINPROGRESS) {
		usleep( 10 );
	}

	if (aio_return( &cb ) != BYTES) {
		perror( "error returned" );
		exit( 4
		    );
	}

	if (close( fd )) {
		fputs( "error closing file\n", stderr );
		exit( 5 );
	}

	if (unlink( TEMP )) {
		perror( "error unlinking file" );
		exit( 6 );
	}

	return 0;
}
