// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: task_queue.h,v $ - header file for task_queue.cc. MT-safe task queue
 *            with optional admission control.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/08/11 23:40:10 $
 * Version :  $Revision: 1.9 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _TASK_QUEUE_H_
# define _TASK_QUEUE_H_

# include <pthread.h>
# include <deque>
/* # include "../machine/atomic.h" */

/**
 * Task queue with admission control
 */

class Task_Queue {
	private:
		pthread_mutex_t lock;
		pthread_cond_t cond;
		std::deque<void *> q;
		int a, b;
		int stopped;
		bool adm_control;

	public:
		// idle_qlen: the max queue length that considered as idle
		// work_batch: the number of tasks in the queue that will be considered as
		// 	decreasing the idle-ness by 1/e.
		Task_Queue(bool admission=false);
		Task_Queue(int idle_qlen, int work_batch);
		~Task_Queue();
		// normal enqueue, the item will be put to the tail of the queue.
		bool enqueue(void *item);
		// prioritized enqueue, the item will be put to the head of the queue.
		bool prio_enqueue(void *item);
		bool dequeue(void *&item); // will be blocked
		bool is_stopped() { return (stopped!=0);}
		bool stop();
		bool empty() { return q.empty(); }
		int size() { return q.size(); }
};

# define TQ_IDLE_QLEN 3
# define TQ_WORK_BATCH 3

# endif
