// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.

   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: thread_pool.h,v $ - header file for thread_pool.cc, a thread-pool
 *            implementation with admission control.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2003/08/11 23:40:10 $
 * Version :  $Revision: 1.11 $
 *
 * ISC project contact: htang@cs.ucsb.edu,  tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */ 
# ifndef _THREAD_POOL_H_
# define _THREAD_POOL_H_

# include "../machine/atomic.h"
# include "cbs_thread.h"
# include "task_queue.h"

typedef char *(*funcTP_CallBack)(void *state, void *req, int myid);

struct TP_Param {
	Task_Queue *q;
	funcTP_CallBack f; // callback function
	void *s; // state
	bool coe;

 public:
	TP_Param(Task_Queue *tq, void *ss, funcTP_CallBack callback) :
		q(tq), f(callback), s(ss), coe(false) 
	{ 
		if (!q) {
			q=new Task_Queue();
			coe=true;
		}

		return; 
	}

	TP_Param(int a, int b, void *ss, funcTP_CallBack callback) :
		q(NULL), f(callback), s(ss), coe(false) 
	{ 
		q=new Task_Queue(a, b);
		coe=true;

		return; 
	}

	~TP_Param()
	{
		if ((coe)&&(q)) {
			delete q;
		}
	}
};

class TP_Thread;

class Thread_Pool {
 private:
	TP_Thread **thrs;
	int thr_cnt;
	struct TP_Param param;
	int state; // 0: pre-start; 1: running; 2: stop signal sent; 
	int n_active_thread;
	void inc_at_cnt() { inc32(&n_active_thread); }
	void dec_at_cnt() { dec32(&n_active_thread); }

 public:
	Thread_Pool(Task_Queue *tq, int n_thr, void *ss, funcTP_CallBack callback) :
		thrs(NULL), thr_cnt(n_thr), 
		param(tq, ss, callback), 
		state(0), n_active_thread(0) { return; }
	Thread_Pool(int a, int b, int n_thr, void *ss, funcTP_CallBack callback) :
		thrs(NULL), thr_cnt(n_thr), 
		param(a, b, ss, callback), 
		state(0), n_active_thread(0) { return; }
	~Thread_Pool();
	bool start(); // spawn threads
	bool is_running() { return (n_active_thread>0); }
	bool enqueue(void *item) { if (!param.q) return false; return param.q->enqueue(item); }
	bool prio_enqueue(void *item) { if (!param.q) return false; return param.q->prio_enqueue(item); }


	void stop(); // signal threads that we are going to shutdown.
	bool empty(){ if (!param.q) return true; return param.q->empty(); }
	int size(){ if (!param.q) return -1; return param.q->size(); }
	void finalize(); // wait for all threads to stop.
	void join(); // wait for all threads to terminate naturally.
	struct TP_Param *get_param() { return &param; }
	void enqueue_eoq_mark() { if (is_running()) enqueue(NULL); }
	friend class TP_Thread;
};

class TP_Thread : public CBS_Thread {
private:
	Thread_Pool *pool;
	int myid;

public:
	TP_Thread(Thread_Pool *parent, int id) : pool(parent), myid(id)  {}
	virtual ~TP_Thread() {}
	virtual void *run();
};

# endif
