// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: block_map.h,v $ - stores which block maps to which 
 *                                        version.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/10 22:47:09 $
 * Version :  $Revision: 1.9 $ by $Author: gulbeden $
 *
 * ISC project contact: {htang, tyang, gulbeden}@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

#ifndef __BLOCK_MAP__
#define __BLOCK_MAP__

#include <stdio.h>
#include <math.h>
#include "../fcache/file_cache.h"

#define BLOCK_SIZE (4*1024)
//#define BLOCK_SIZE (4*1024*1024)
//#define BLOCK_SIZE (4)

class BlockMap {
public:
	BlockMap(char* bmf, File_Cache* fc);
	~BlockMap();
	int getBlockSize() { return BLOCK_SIZE; } // in bytes
	int readRange(unsigned int lower, unsigned int upper, unsigned int * ret);
	int updateBlock(int blockNum, int version);
	int updateLength(int newLen);
	int updateVersion(int newVer);
	int getLength() { return segmentLen; }
	int getVersion() { return version; }
	int flush();
	
private:
	int blockLoc(int i) { return (i+2)*sizeof(int); }
	int initializeBlocks(unsigned int lower, unsigned int higher);

	int maxSizeinKb;
	char* bmFileName;
	File_Cache *fcache;
	FILE* mapFile;
	int lastBlock;				// rank of the last written block, to keep 
								// block_map file size small.
	int segmentLen;
	int version;

};

#endif // __BLOCK_MAP__
