// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: test.cc,v $ - does a series of operations using VerFS 
 *            to check its correctness.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/10 22:44:34 $
 * Version :  $Revision: 1.18 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu, gulbeden@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */
#include "verfs.h"
#include <stdio.h>
#include <malloc.h>

#include "../guid/guid.h"

int main(){
	char buffer[5000];
	
	char **ws = new char *[3];

	ws[0] = "/home/storage/aziz/format";  
	ws[1] = NULL;
    
	Verfs *vfs;
	
	vfs = new Verfs(ws, "/tmp/");
	
	guid_t guids[20];
	int guidIndex=0;
	
	int verbose=1;
	
	char comm[10];
	int arg, offset, length;
	int end;
	
	for(int i=0; i<10; i++){
		guids[i].data[0]=0;  guids[i].data[1]=0;  guids[i].data[2]=0;
		guids[i].data[3]=0;  guids[i].data[4]=0;  guids[i].data[5]=0;
		guids[i].data[6]=0;  guids[i].data[7]=0;  guids[i].data[8]=0;
		guids[i].data[9]=0;  guids[i].data[10]=0; guids[i].data[11]=0;
		guids[i].data[12]=0; guids[i].data[13]=0; guids[i].data[14]=0;
		guids[i].data[15]=i;
	}
	
	do{
		int version;
		end = scanf("%s", comm);
		if (end == EOF) exit(0);
		
		printf(": %s\n", comm);
		
		if (comm[0] == 'c' && comm[1]=='1'){
			// create
			int d;
			d = vfs->create(guids[guidIndex], version);
			if (verbose) printf("created %d (version %d)\n", d, version);
			guidIndex++;
		}
		
		else if(comm[0] == 'o' && comm[1]=='r'){
			int desc;
			scanf("%d", &arg);
			desc = vfs->open(guids[arg], "r", version);
			printf("Opened %d (version %d)\n", desc, version);
		}

		else if(comm[0] == 'o' && comm[1]=='w'){
			int desc;
			scanf("%d", &arg);
			desc = vfs->open(guids[arg], "w", version);
			printf("Opened %d (version %d)\n", desc, version);
		}
		
		else if(comm[0] == 'c' && comm[1]=='l' ){
			scanf("%d", &arg);
			int version;
			vfs->close(arg, version);
			printf("Closed %d (version is now %d)\n", arg, version);
		}
		
		else if(comm[0] == 'r'){
			// read
			scanf("%d", &arg);
			scanf("%d", &offset);
			scanf("%d", &length);
			//if (vfs->read(arg, offset, buffer, length) == 0){
			vfs->read(arg, offset, buffer, length);
				buffer[length]='\0';
				printf("read:\n%s\n", buffer);
			//}
		}
		
		else if(comm[0] == 'w'){
			scanf("%d", &arg);
			scanf("%d", &offset);
			scanf("%d", &length);
			scanf("%s", buffer);
			if (vfs->write(arg, offset, buffer, length) >= 0)
				if (verbose) printf("wrote\n");
		}
		
		else if(comm[0] == 'a'){
			scanf("%d", &arg);
			scanf("%d", &length);
			scanf("%s", buffer);
			if (vfs->append(arg, buffer, length)) printf("appended\n");
		}
		
		else if(comm[0] == 'm'){
			scanf("%d", &arg);
			vfs->remove(guids[arg]);
			if (verbose) printf("removed\n");
		}
		
		else if(comm[0] == 'd'){
			// dump
			scanf("%d", &arg);
			vfs->dumpObject(arg);
			if (verbose) printf("Dumped :%d\n", arg);
		}
		
		else if(comm[0] == 't'){
			// truncate
			scanf("%d", &arg);
			scanf("%d", &length);
			if (vfs->truncate(arg, length)) printf("truncated :%d\n", arg);
		}
		
	} while (1);
	
	return 0;
}
