// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: verfs_desc.h,v $ -  Verfs descriptor header file, each
 *            open session has an associated descriptor.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/02/19 13:12:04 $ by $Author: gulbeden $
 * Version :  $Revision: 1.12 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */


#ifndef __VERFS_DESC_H__
#define __VERFS_DESC_H__

#include <pthread.h>

#include "verfs.h"
#include "rubber_vector.h"

class BlockMap;
class File_Cache;

#define READ_MODE 0
#define WRITE_MODE 1
#define BM_EXTENSION "b"

void initFName(char* c, char* wDir, char* suffix, guid_t &guid);

/* metadata pointed by the descriptor */
class VerfsDescriptor {
public:
	VerfsDescriptor(char *wDir, guid_t &guid, File_Cache *fc);
	~VerfsDescriptor();
	int openForWrite(int sessionID);
	int read(int offset, char *buff, int len);
	int write(int offset, char *buff, int len);
	int append(char *buff, int len);
	int flush();
	int close();
	int truncate(int length);
	int getFileEnd(){ return fileEnd; }
	int getVersion(){ return version; }
	int remove();
	guid_t getGuid(){ return guid; }

private:
	char h2c(int h);
	void getFileName(char *fileName, guid_t &guid, char *extension);
	int readBlock(int offset, char *buff, int version, int len);

	BlockMap *bm;				// block map for current file
	File_Cache *fcache;			// to do file ops in bm
	pthread_mutex_t fiLock;
	guid_t guid;
	char* wd;
	int sid;					// session ID
	RubberVector<bool> *modified;
	char mode;
	char *copyName;
	int fileEnd;
	int version;
};

#endif // __VERFS_FI_H__
