// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: verfs.cc,v $ -  A file system implementation that 
 *            uses logs for versioning.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/08 19:14:28 $ by $Author: gulbeden $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */


#include <stdio.h>
#include <stdlib.h> 
#include <string.h>				/* strncmp */
#include <sys/stat.h>
#include <dirent.h>
#include <queue>

#include "verfs.h"
#include "verfs_fi.h"
#include "../pshash/pshash.h"
#include "../res/resmon.h"
#include "../fcache/file_cache.h"
#include "../debug/tmpi_debug.h"
//#include "../../daemon/server/publisher.h"

//#include "logcleaner.h"
//#include <db3/db_cxx.h>

using namespace std;

#define MAX_LENGTH 1024			/* maximum filename length */


#define USE_FCACHE

//#define WD "/home/storage/aziz/"
#define WD "./"


/* If not defined, all operations will be done by c functions(see end of the code) */
//#define NORMAL_MODE


#ifdef NORMAL_MODE
/* use log (this part is not up-to-date) */

#define DISK_BLOCK_SIZE 4

#ifdef USE_FCACHE
File_Cache *fcache;
#endif


typedef struct inmEntry {
  int map [2*2*2*2*2*2*2*2];
}inmEntry;


typedef struct inmKey {
  char key[19];					/* 16 for guid_t, +3 for reduced size */
}inmKey;


/**
 * store metadata about objects in hashtable
 */
PsHash *iNodeMap=NULL;
PsHash *desc2vi=NULL;
PsHash *guid2desc=NULL;


/**
 * mutex needed for accessing the hashtable
 */
pthread_mutex_t tableMutex = PTHREAD_MUTEX_INITIALIZER;

int descriptor=0;


/**
 * Initialize hashtable, and file cache (if used)
 * and start log cleaner
 */
int vfsInit(){

	//guid2vi = InitializeTable( 5000 );    
	printf("Size of VersionInfo: %d\n", sizeof(VersionInfo));  
	printf("Size of inmEntry: %d\n", sizeof(inmEntry));
	
	/* constant hardcoded (1000) */
	/*
	  desc2guid = new PsHash("desc2guid.db", sizeof(int), sizeof(guid_t), 20, 20*sizeof(guid_t));
	  if (!desc2guid->is_loaded()) {
	  printf("second called\n");
	  delete desc2guid;
	  desc2guid = new PsHash("desc2guid.db",  1000);
	  if (!desc2guid->is_loaded()) {
      printf("Cannot load database.\n");
      exit(-1);
	  }
	  }
	  
  guid2vi= new PsHash("guid2vi.db", sizeof(guid_t), sizeof(VersionInfo), 20, 20*sizeof(VersionInfo));
  if (!guid2vi->is_loaded()) {
    delete guid2vi;
    guid2vi = new PsHash("guid2vi.db",  1000);
    if (!guid2vi->is_loaded()) {
      printf("Cannot load database.\n");
      exit(-1);
    }
  }*/


  guid2desc = new PsHash("guid2desc.db", sizeof(int), sizeof(guid_t), 20, 20*sizeof(guid_t));
  if (!guid2desc->is_loaded()) {
    printf("second called\n");
    delete guid2desc;
    guid2desc = new PsHash("guid2desc.db",  1000);
    if (!guid2desc->is_loaded()) {
      printf("Cannot load database.\n");
      exit(-1);
    }
  }
  
  desc2vi = new PsHash("desc2vi.db", sizeof(int), sizeof(guid_t), 20, 20*sizeof(VersionInfo));
  if (!desc2vi->is_loaded()) {
    printf("second called\n");
    delete desc2vi;
    desc2vi = new PsHash("desc2vi.db",  1000);
    if (!desc2vi->is_loaded()) {
      printf("Cannot load database.\n");
      exit(-1);
    }
  }

  iNodeMap = new PsHash("iNodeMap.db", sizeof(guid_t)+8, sizeof(inmEntry), 20000000, 200*sizeof(inmEntry));
  if (!iNodeMap->is_loaded()) {
    delete iNodeMap;
    iNodeMap = new PsHash("iNodeMap.db",  1000);
    if (!iNodeMap->is_loaded()) {
      printf("Cannot load database.\n");
      exit(-1);
    }
  }

  //startLogCleaner(&guid2vi);

#ifdef USE_FCACHE
  printf("File cache enabled.\n");
  fcache = new File_Cache(800);
#endif
  return 0;
}


/* CREATE:*/
/**
 * Create object without parent
 */
int vfsCreate(guid_t guid){

  VersionInfo* vi;
  char lName[MAX_LENGTH]; /*logname */
  int desc;

  desc = descriptor ++;
  
  vi = new VersionInfo();
  memcpy(vi->GUID.data, guid.data, 16);
  memcpy(vi->parent.data, guid.data, 16); /* parent itself */
  vi->committed = 0;
  vi->removed = 0;
  vi->dumped = 0;
  vi->finalized = 0;
  gettimeofday(&vi->lastAccessed, NULL);
  vi->fileEnd = 0;
  vi->logLength = 0;
  vi->desc = desc;
  
  sprintf(vi->logName, "%s%d", WD, desc); 
  sprintf(lName, "%s.log", vi->logName);

#ifndef USE_FCACHE
  vi->file = fopen(lName, "ab+");
#endif

  /* insert into hashtable as parent itself */
  pthread_mutex_lock(&tableMutex);

  //desc2guid->set(&desc, sizeof(int), &guid, sizeof(guid_t));
  //guid2vi->set(&guid, sizeof(guid_t), vi, sizeof(VersionInfo));
  desc2vi->set(&desc, sizeof(int), vi, sizeof(VersionInfo));
  guid2desc->set(&guid, sizeof(guid_t), &desc, sizeof(int));

  pthread_mutex_unlock(&tableMutex);
  
  delete vi;

  /* return handle */
  return desc;        /* descriptor is index in the hashtable */ 
}


/**
 * GUID: new object's ID
 * base_GUID: parent's ID
 */
int vfsCreate(guid_t GUID, guid_t baseGUID){

  int desc;
  unsigned int len;
  VersionInfo *vi, *viParent;

  char lName[MAX_LENGTH];
  
  viParent = new VersionInfo();
  len = sizeof(int);
  //guid2vi->get(&baseGUID, sizeof(guid_t), viParent, &len);
  guid2desc->get(&baseGUID, sizeof(guid_t), &desc, &len);
  len = sizeof(VersionInfo);
  desc2vi->get(&desc, sizeof(int), viParent, &len);

  if (viParent == NULL){
    fprintf(stderr, "parent object does not exist\n");
    return -1;
  }

  desc = descriptor++;

  vi = new VersionInfo();

  memcpy(vi->GUID.data, GUID.data, 16);
  memcpy(vi->parent.data, baseGUID.data, 16);
  vi->committed = 0;
  vi->removed = 0;
  vi->dumped = 0;
  vi->finalized = 0;
  gettimeofday(&vi->lastAccessed, NULL);

  // set file end

  vi->fileEnd = viParent->fileEnd;
  vi->logLength = 0;
  vi->desc = desc;

  sprintf(vi->logName, "%s%d", WD, desc);
  sprintf(lName, "%s.log", vi->logName);

#ifndef USE_FCACHE
  vi->file = fopen(lName, "ab+");
#endif

  /* insert into hashtable */
  pthread_mutex_lock(&tableMutex);

  //guid2vi->set(&GUID, 16, vi, sizeof(VersionInfo)); 
  //desc2guid ->set(&desc, sizeof(int), &GUID, sizeof(guid_t));
  desc2vi->set(&desc, sizeof(int), vi, sizeof(VersionInfo));
  guid2desc->set(&GUID, sizeof(guid_t), &desc, sizeof(int));

  pthread_mutex_unlock(&tableMutex);

  delete(vi);
  delete(viParent);

  /* return handle */
  return desc;        /* descriptor is index in the hashtable */ 
}


/*OPEN:*/
/**
 * Opens for read.
 */
int vfsOpen(guid_t GUID, char* flags){
  VersionInfo vi;
  unsigned int len;
  int desc;

  /* check is the object exists */
  //guid2vi->get(&GUID, sizeof(guid_t), &vi, &len);
  len = sizeof(int);
  guid2desc->get(&GUID, sizeof(guid_t), &desc, &len);
  len = sizeof(VersionInfo);
  desc2vi->get(&desc, sizeof(int), &vi, &len);

  if (vi.removed) return -1;

  return vi.desc;
}


/* CLOSE */
/**
 * Commits object, the object can no longer be modified.
 */
int vfsClose(int desc){
  unsigned int len;
  //guid_t GUID;

  VersionInfo vi;

/*
  desc2guid ->get(&desc, sizeof(int), &GUID, &len);
  guid2vi->get(&GUID, sizeof(guid_t), &vi, &len);
*/

  len = sizeof(VersionInfo);
  if(desc2vi->get(&desc, sizeof(int), &vi, &len) !=0){
    printf("ERROR (verfsClose): No such descriptor\n");
    return -1;
  }

  pthread_mutex_lock(&vi.objLock);
  vi.committed = 1;
  pthread_mutex_unlock(&vi.objLock);

#ifdef USE_FCACHE
  char logName[MAX_LENGTH];
  sprintf(logName, "%s.log", vi.logName);
  fcache->flush(logName);
#else
  fflush(vi.file);
  //fclose(vi->file);        /* keeping open for child */
#endif
  return 0;
}

int vfsReadExclusive(int GUID, int offset, int length, char *buffer);

VersionInfo *gvi= new VersionInfo;


inmKey key;
inmEntry entry;


/* WRITE */
/**
 * Write operation is appended into the log
 */
int vfsWrite(int desc, int offset, int length, char* data){

  int blockNum;     /* whick block is being written */
  int i=0;
  int wIndex=0;
  //VersionInfo *vi;
  int lOffset;
  int k;
  char cache[DISK_BLOCK_SIZE];
  //guid_t GUID;
  unsigned int len=sizeof(guid_t);

  lOffset = offset;
/*
  if(desc2guid ->get(&desc, sizeof(int), &GUID, &len)){
    printf("get descriptor error\n");
  }
  len = sizeof(VersionInfo);
  if(guid2vi->get(&GUID, sizeof(guid_t), gvi, &len) !=0){
    printf("get versionInfo error\n");
  }
*/

  len = sizeof(VersionInfo);
  if(desc2vi->get(&desc, sizeof(int), gvi, &len) !=0){
    printf("get versionInfo error\n");
  }

  if(gvi->committed) return -1;   /* objects cannot be modified after they are closed */

  pthread_mutex_lock(&gvi->objLock);
  gettimeofday(&gvi->lastAccessed, NULL);

  blockNum = offset / DISK_BLOCK_SIZE;

#ifdef USE_FCACHE
  char logName[MAX_LENGTH];
  sprintf(logName, "%s.log", gvi->logName);
  FC_Handle sLogFC = fcache->fopen(logName);
#endif

  /* write blocks one by one */
  for(i=0; i<=(length+offset%DISK_BLOCK_SIZE -1)/DISK_BLOCK_SIZE; i++){

    if(/*vi->blockIndex[blockNum]!=-1 &&*/ (offset%DISK_BLOCK_SIZE !=0 || length%DISK_BLOCK_SIZE!=0)){   // block has to be read
      printf("write not alligned, need to read existing data\n");

      /* read the block from the log */

      vfsReadExclusive(desc, blockNum * DISK_BLOCK_SIZE, DISK_BLOCK_SIZE, cache);
      
    }

    for(k=0; wIndex<length && wIndex+(offset%DISK_BLOCK_SIZE) < DISK_BLOCK_SIZE*(i+1); k++){
      cache[k + (lOffset%DISK_BLOCK_SIZE)] = data[wIndex];
      wIndex++;
    }

    lOffset = 0; /* now offset is 0, since next block will be written from index 0 */

    /* write data */
#ifdef USE_FCACHE
    fcache->fappend(sLogFC, cache, DISK_BLOCK_SIZE);
#else
    fwrite(cache, 1, DISK_BLOCK_SIZE, gvi->file);
    //fflush(gvi->file);
#endif

    /* update length */
    if(gvi->fileEnd < offset + length)
      gvi->fileEnd = offset + length;

    //char tmpChar[4];
    //int entryIndex;

    int high;
    int low;

    memcpy(&key, gvi->GUID.data, 16);

    high = blockNum / (2*2*2*2*2*2*2*2);
    low  = blockNum % (2*2*2*2*2*2*2*2);

    //printf("low is: %d", low);
    if ( high % (2*2*2*2*2*2*2*2) > 255 || (high / (2*2*2*2*2*2*2*2)) % (2*2*2*2*2*2*2*2) >255 || high / (2*2*2*2*2*2*2*2 * 2*2*2*2*2*2*2*2) > 255) 
      printf("Error in logic\n");

    key.key[18]= high % (2*2*2*2*2*2*2*2);
    key.key[17]=(high / (2*2*2*2*2*2*2*2)) % (2*2*2*2*2*2*2*2);
    key.key[16]= high / (2*2*2*2*2*2*2*2   *  2*2*2*2*2*2*2*2);
    

    len = sizeof(inmEntry);

    int exist;

    exist = iNodeMap->get(&key, sizeof(inmKey), &entry, &len);  /* can return empty - don't care*/

    if (exist == -1){
      // First time
      for(int a=0; a<2*2*2*2*2*2*2*2; a++)
	entry.map[a]= -1;
    }

    entry.map[low] = gvi->logLength++;
    //gvi->blockIndex[blockNum]=gvi->logLength++;

    iNodeMap->set(&key, sizeof(inmKey), &entry, sizeof(entry));

    blockNum++;
  }

#ifdef USE_FCACHE
  fcache->fflush(sLogFC);
  fcache->fclose(sLogFC);
#endif

  pthread_mutex_unlock(&gvi->objLock);

  desc2vi->set(&desc, sizeof(int), gvi, sizeof(VersionInfo));

  //delete vi;

  return 0;
}


/**
 * Reads data from an object for which data from child is already read 
 * and stored in buffer parameter. 
 */
char* readHelper(VersionInfo* vi, int offset, int length, char *buffer, char *readSoFar){

  char block[DISK_BLOCK_SIZE];
  char oName[MAX_LENGTH];
  unsigned int len;
  FILE* oFile;

  int lOffset; /* local offset variable */

  int blockNum;     /* whick block is being read */
  int logBlockNum;
  int i=0;

  int rIndex=0; /* index in read in readsofar */
  int rBIndex=0; /* index in read buffer */

  lOffset = offset;

  blockNum = offset / DISK_BLOCK_SIZE;

  //vfsFlush()

#ifdef USE_FCACHE
  char logName[MAX_LENGTH];
  sprintf(logName, "%s.log", vi->logName);
  FC_Handle sLogFC = fcache->fopen(logName);
  fcache->fflush(sLogFC);
#else
  fflush(vi->file);
#endif

  if (vi->dumped){
    // first read the block from the file
    sprintf(oName, "%s.object", vi->logName);
    oFile = fopen(oName,"rb+");
    fseek(oFile,offset,SEEK_SET);
    fread(buffer,1, length, oFile);
    fclose(oFile);
  }

  /* read blocks one by one */
  for(i=0; i< ceil( (double)(length+(offset%DISK_BLOCK_SIZE)) / (double) DISK_BLOCK_SIZE ); i++){

    //logBlockNum = vi->blockIndex[blockNum]; /* which block should be read from the log */

    inmKey key;
    inmEntry entry;

    int high;
    int low;
    int found;

    memcpy(&key, vi->GUID.data, 16);

    high = blockNum / (2*2*2*2*2*2*2*2);
    low  = blockNum % (2*2*2*2*2*2*2*2);

    //printf("low is: %d", low);

    if ( high % (2*2*2*2*2*2*2*2) > 255 || (high / (2*2*2*2*2*2*2*2)) % (2*2*2*2*2*2*2*2) >255 || high / (2*2*2*2*2*2*2*2 * 2*2*2*2*2*2*2*2) > 255) 
      printf("Error in logic\n");

    key.key[18]= high % (2*2*2*2*2*2*2*2);
    key.key[17]=(high / (2*2*2*2*2*2*2*2)) % (2*2*2*2*2*2*2*2);
    key.key[16]= high / (2*2*2*2*2*2*2*2   *  2*2*2*2*2*2*2*2);
    
    len = sizeof(inmEntry);
    found = iNodeMap->get(&key, sizeof(inmKey), &entry, &len);  /* can return empty - don't care*/

    logBlockNum = entry.map[low];

    //printf("Not Found: %d\n", found);

    if(found != -1 && logBlockNum != -1){

      /* complicated but seems to be working */
      rIndex = i * DISK_BLOCK_SIZE + lOffset % DISK_BLOCK_SIZE; 
      if (i != 0) rBIndex= i * DISK_BLOCK_SIZE - (offset % DISK_BLOCK_SIZE);

      /* read the block from the log */
#ifdef USE_FCACHE
      fcache->fread(sLogFC, logBlockNum * DISK_BLOCK_SIZE, block, DISK_BLOCK_SIZE);
#else
      fseek(vi->file, logBlockNum * DISK_BLOCK_SIZE, SEEK_SET);                /* NOT THREAD SAFE */
      fread(block, 1, DISK_BLOCK_SIZE, vi->file);
#endif

      /* summary block stored in the memory??? */

      /* now corresponding block is read, get the requested region */
      for(int k=0; rIndex<length + offset%DISK_BLOCK_SIZE && rIndex /*+(lOffset%DISK_BLOCK_SIZE)*/ < DISK_BLOCK_SIZE*(i+1) && blockNum*DISK_BLOCK_SIZE + k < vi->fileEnd; k++){
	
	/* if not read so far */
	if(readSoFar[rBIndex]=='u'){
	  
	  buffer[rBIndex] = block[k+(lOffset%DISK_BLOCK_SIZE)];
	  
	  readSoFar[rBIndex]='r';   /* mark read */
	}

	rIndex++;
	rBIndex++;
      }
      
    }
    blockNum++;
    lOffset = 0;
  }
#ifdef USE_FCACHE
  fcache->fclose(sLogFC);
#endif

  return buffer;
}

VersionInfo *gvi2 = new VersionInfo();

/* READ */
/* parse object log 
 * returns 0 if nothing is read, else returns a positive number
 */
int vfsRead(int desc, int offset, int length, char *buffer){

  char readSoFar[length+1];
  int readChars = 0;
  int i=0;
  unsigned int len;
  guid_t guid;

  for(i=0; i<length; i++)
    readSoFar[i]='u';          /* unread */

  len = sizeof(guid_t);

/*
  if(desc2guid->get(&desc, sizeof(int), &guid, &len)){
    printf("get descriptor error\n");
  }

  len = sizeof(VersionInfo);
  if(guid2vi->get(&guid, sizeof(guid_t), gvi, &len) !=0){
    printf("get versionInfo error\n");
  }
*/

  len = sizeof(VersionInfo);
  if(desc2vi->get(&desc, sizeof(int), gvi2, &len)!=0){
    printf("get versionInfo error\n");
  }

  if (pthread_mutex_lock(&gvi2->objLock) != 0)
	  return -1;

  vfsFlush(desc, gvi2);

  /* now see the log */
  //if(!gvi->finalized){

  for(i=0; i<length;i++)
    buffer[i]='\0';

  /* traverse parents starting from youngest */  
  len = sizeof(VersionInfo);

  do{ /* guid2vi is necessary since guid knows parent guid but not desc */
    //guid2vi->get(&guid, sizeof(guid_t), gvi2, &len);
    //desc2vi->get(&desc, sizeof(int), gvi2, &len);

    len = sizeof(int);
    guid2desc->get(&(gvi2->parent), sizeof(guid_t), &desc, &len);
    len = sizeof(guid_t);
    printf("descriptor is: %d\n", desc);
    desc2vi->get(&desc, sizeof(int), gvi2, &len);


    printf("will look at %s\n", gvi2->logName);

    readHelper(gvi2, offset, length, buffer, readSoFar);

    for(i=0; i<length; i++){
      if(readSoFar[i] == 'r') break;           /* still unread part reamain */
    }
    if (i == length-1 && readSoFar[i]== 'r')   /* everything already read */
      break; /*do while */
    printf("READ LOOP DONE %s, vi2 is %d, read: %s\n", readSoFar, gvi2->GUID.data[0], buffer);

    guid = gvi2->parent;
    
  } while(memcmp(gvi2->GUID.data, gvi2->parent.data, 16));
  
  pthread_mutex_unlock(&gvi->objLock);

  for(i=0; i<length; i++)
    if(readSoFar[i]=='r')
      readChars++;
  
  //delete vi;
  //delete vi2;

  return readChars;
}


/* READ */
/* this is same as the read function above except that it assumes object is locked and doesn't lock it.
 * (bad design?)
 */
int vfsReadExclusive(int desc, int offset, int length, char *buffer){

  char readSoFar[length+1];
  int readChars = 0;
  int i=0;
  unsigned int len;
  guid_t guid;

  for(i=0; i<length; i++)
    readSoFar[i]='u';          /* unread */

  len = sizeof(guid_t);

/*
  if(desc2guid->get(&desc, sizeof(int), &guid, &len)){
    printf("get descriptor error\n");
  }

  len = sizeof(VersionInfo);
  if(guid2vi->get(&guid, sizeof(guid_t), gvi, &len) !=0){
    printf("get versionInfo error\n");
  }
*/

  len = sizeof(VersionInfo);
  if(desc2vi->get(&desc, sizeof(int), gvi, &len)!=0){
    printf("get versionInfo error\n");
  }

  pthread_mutex_lock(&gvi->objLock);

  vfsFlush(desc, gvi);

  /* now see the log */
  //if(!gvi->finalized){

  for(i=0; i<length;i++)
    buffer[i]='\0';

  /* traverse parents starting from youngest */  
  len = sizeof(VersionInfo);

  do{ /* guid2vi is necessary since guid knows parent guid but not desc */
    //guid2vi->get(&guid, sizeof(guid_t), gvi2, &len);
    //desc2vi->get(&desc, sizeof(int), gvi2, &len);

    printf("will look at %s\n", gvi2->logName);

    readHelper(gvi2, offset, length, buffer, readSoFar);

    for(i=0; i<length; i++){
      if(readSoFar[i] == 'r') break;           /* still unread part reamain */
    }
    if (i == length-1 && readSoFar[i]== 'r')   /* everything already read */
      break; /*do while */
    printf("READ LOOP DONE %s, vi2 is %d, read: %s\n", readSoFar, gvi2->GUID.data[0], buffer);

    guid = gvi->parent;
    
    len = sizeof(int);
    guid2desc->get(&guid, sizeof(guid_t), &desc, &len);
    len = sizeof(guid_t);
    desc2vi->get(&desc, sizeof(int), gvi, &len);

  } while(memcmp(gvi2->GUID.data, gvi2->parent.data, 16));
  
  pthread_mutex_unlock(&gvi->objLock);

  for(i=0; i<length; i++)
    if(readSoFar[i]=='r')
      readChars++;
  
  //delete vi;
  //delete vi2;

  return readChars;

}



/* APPEND */
/**
 * Append operation is same ras writing to the end
 */
int vfsAppend(int desc, int length, char* data){
  VersionInfo vi;

  //guid_t GUID;
  unsigned int len = sizeof(VersionInfo);

  /*
  desc2guid ->get(&desc, sizeof(int), &GUID, &len);  
  guid2vi->get(&GUID, sizeof(guid_t), &vi, &len);
  */
  
  desc2vi->get(&desc, sizeof(int), &vi, &len);

  return (vfsWrite(desc, vi.fileEnd, length, data));
}


/**
 * Read log and write object block by block.
 */
int vfsDumpObject(int desc){
  char oName[MAX_LENGTH], logName[MAX_LENGTH];
  //printf("here\n");
  VersionInfo vi;

  char line[DISK_BLOCK_SIZE*1];
  int i, length;

  FILE *oLog, *oFile;

  unsigned int len;
  //guid_t guid;

  /*
  len = sizeof(guid_t);
  desc2guid ->get(&desc, sizeof(int), &guid, &len);
  len = sizeof(VersionInfo);
  guid2vi->get(&guid, sizeof(guid_t), &vi, &len);
  */

  len = sizeof(VersionInfo);
  desc2vi->get(&desc, sizeof(int), &vi, &len);

  if(vi.finalized) return 0; /* object is already in its final form */

  pthread_mutex_lock(&vi.objLock);
  
  sprintf(oName, "%s%s.object", WD, vi.logName);

  oFile = fopen(oName, "w");

  //printf("fileEnd:%ld, div : %f\n",vi.fileEnd, ceil((float)vi.fileEnd  / (float) DISK_BLOCK_SIZE));

  for (i=0; i<ceil( ( (float)(vi.fileEnd))  / ((float) (DISK_BLOCK_SIZE))); i++){
  //for (i=0; i<3; i++){
    length = vfsReadExclusive(desc, i*DISK_BLOCK_SIZE, 1*DISK_BLOCK_SIZE, line);
    fwrite(line, 1, DISK_BLOCK_SIZE, oFile);
    //printf("herer\n");
  }
  //printf("i:%d\n",i);
  
  fclose(oFile);
  vi.dumped = true;
  if(vi.committed) vi.finalized=true;

  /* empty log */
  sprintf(logName, "%s.log", vi.logName);
  oLog = fopen(logName, "w");
  fclose(oLog);

#ifdef USE_FCACHE
#else
  fseek(vi.file, 0, SEEK_SET);  /* seek to the beginning */
#endif

  pthread_mutex_unlock(&vi.objLock);

  desc2vi->set(&desc, sizeof(int), &vi, sizeof(VersionInfo));

  return 0;
}


/**
 * Flushes log
 */
void vfsFlush(int desc, VersionInfo* vi){
  char logName[MAX_LENGTH];
#ifdef USE_FCACHE
  sprintf(logName, "%d.log", desc);
  fcache->flush(logName);
#else
  //if(!vi->committed)
    fflush(vi->file);  
#endif
}


/* REMOVE */
/** 
 * just mark as removed since some other objects might need it as parent.
 */
int vfsRemove(guid_t GUID){
 /* VersionInfo *vi;
  unsigned int len;
  
  len = sizeof(VersionInfo);

  //guid2vi->get(&GUID, sizeof(guid_t), vi, &len);

  pthread_mutex_lock(&tableMutex);
  vi->removed=1;
  guid2vi->set(&GUID, sizeof(guid_t), vi, sizeof(VersionInfo));
  pthread_mutex_unlock(&tableMutex);
  
  delete vi;*/
  return 0;
}



/** C mode (log disabled) **/

#else



/* < operator for guid_t - required by map */
bool operator<(const guid_t &guid1, const guid_t &guid2){
	return memcmp(&guid1, &guid2, sizeof(guid_t));
}


/**
 * Hexical to char conversion
 */
static char h2c(int h)
{
	if ((h<0)||(h>15)) {
		return (char)0;
	}
	if (h<10) {
		return '0'+h;
	}
	return 'A'+(h-10);
}


/**
 * Checks if the path is correctly formatted.
 */
static bool check_format(char* path, bool f_mod_debug){

	bool result = true;
	char mp[MAX_LENGTH];
	char pwd[MAX_LENGTH];
	bool check = false;

	if (strlen(path)>=MAX_LENGTH) {
		tmpi_error(DBG_CALLER, "Mount point path too long.\n");
		exit(-1);
	}

	if (check) {
		if (ResMon::find_mount_point(path, mp, MAX_LENGTH)) {
			tmpi_error(DBG_CALLER, "Cannot determine mount point.\n");
			return false;
		}
	}

	if (getcwd(pwd, MAX_LENGTH)==NULL) {
		tmpi_error(DBG_CALLER, "Current directory path too long.\n");
		return false;
	}

	if (chdir(path)<0) {
		tmpi_error(DBG_CALLER, "Cannot access directory %s.\n", path);
		return false;
	}
	
	if (chdir(DIR_DATA)<0) {
		return false;
	}

	/*DIR *mp_dir=opendir(".");
	  if (mp_dir==NULL) {
	  tmpi_error(DBG_CALLER, "Cannot open directory %s.\n", mp);
	  return false;
	  }*/

	char cmd[1024];

	if (f_mod_debug){
		for(int a=0; a<16; a++){
			for(int b=0; b<16 && result; b++){
				snprintf(cmd, 1024, "%c/%c", h2c(a), h2c(b));
				if (chdir(cmd)<0) {
					result = false;
					break;
				}

				snprintf(cmd, 1024, "../..");
				if (chdir(cmd)<0) {
					result = false;
					break;
				}
			}
		}
	}
	else{
		for(int a=0; a<16; a++){
			for(int b=0; b<16 && result; b++){
				for(int c=0; c<16 && result; c++){
					for(int d=0; d<16 && result; d++){
						snprintf(cmd, 1024, "%c/%c/%c/%c", h2c(a), h2c(b), h2c(c), h2c(d));
						if (chdir(cmd)<0) {
							result = false;
							break;
						}
						
						snprintf(cmd, 1024, "../../../..");
						if (chdir(cmd)<0) {
							result = false;
							break;
						}
					}
				}
			}
		}
	}
	
	chdir(pwd);

	return result;
}


static int get_mode(char *f, mode_t &m){
	struct stat st;
	if (lstat(f, &st)<0)
		return -1;

	m=st.st_mode;

	return 0;
}

int Verfs::append(int desc, char *data, unsigned int len)
{
	// int ret;
	VerfsFileInfo * vfi = check_desc(desc);
	if (vfi == NULL)
		return -1;

	//pthread_mutex_lock(&vfi->fiLock);
	return fcache->append(vfi->name, data, len);
	//pthread_mutex_unlock(&vfi->fiLock);
	//return ret;
}

/**
 * decides on which partition to use.
 */ 
int Verfs::calcPartition(guid_t &guid)
{
	int maxFreeSp;
	int maxFreeInd = 0;
	time_t timeNow;
	bool status = false;
		
	time (&timeNow);

	if (timeNow - lastDIUpdate >= DI_UPDATE_INTERVAL)
		status = refreshDiskInfos();

	if (status == false || diskInfos == NULL)
		return descriptor % wDirCnt; // roundrobin

	maxFreeSp = diskInfos[0].bfree;

	for (int i=0; i<wDirCnt; i++){
		if (maxFreeSp > diskInfos[i].bfree){
			maxFreeSp = diskInfos[i].bfree;
			maxFreeInd = i;
		}
		// tmpi_error(DBG_VERBOSE, "Free space on %d is %d\n", i, diskInfos[i].bfree);
	}
	return maxFreeInd;
}


VerfsFileInfo * Verfs::check_desc(int desc)
{
	std::map<int, VerfsFileInfo *>::iterator iter = desc2fi.find(desc);
	
	if (iter == desc2fi.end()){
		return NULL;
	}

	return iter->second;
}

/**
 * just frees memory since fcache is being used 
 */
int Verfs::close(int desc, int &version){

	VerfsFileInfo *vfi = check_desc(desc);
	if (vfi == NULL)
		return -1;

	if(pthread_mutex_lock(&d2fLock) != 0)
		return -1;

	delete(desc2fi[desc]);
	desc2fi.erase(desc);

	pthread_mutex_unlock(&d2fLock);

	return 0;
}

/**
 * @return descriptor of the created file or -1 in case of failure.
 */
int Verfs::create(guid_t guid, int &version)
{
	// check if guid already exists
	int pIndex;
	unsigned int valSize = sizeof(int);

	// get locks
	if(pthread_mutex_lock(&d2fLock) != 0)
		return -1;
	if(pthread_mutex_lock(&g2pLock) != 0){
		pthread_mutex_unlock(&d2fLock);
		return -1;
	}

	if(guid2partition->get(&guid, sizeof(guid_t), &pIndex, &valSize) == 0 ){
		pthread_mutex_unlock(&g2pLock);
		pthread_mutex_unlock(&d2fLock);
		return -1;
	}

	pIndex = calcPartition(guid);

	guid2partition->set(&guid, sizeof(guid_t), &pIndex, sizeof(int));

	VerfsFileInfo *fi = new VerfsFileInfo(wDirs[pIndex], guid);
	
	// tmpi_error(DBG_VERBOSE, "file name %s\n", fi->name);

	pair<int, VerfsFileInfo*> *d2vfi;
	d2vfi = new pair<int, VerfsFileInfo*>(descriptor, fi);
	if(desc2fi.insert(*d2vfi).second == false){
		tmpi_error(DBG_INTERNAL, "insert into map failed\n");
		pthread_mutex_unlock(&g2pLock);
		pthread_mutex_unlock(&d2fLock);
		return -1;
	}

	descriptor++;

	pthread_mutex_unlock(&g2pLock);
	pthread_mutex_unlock(&d2fLock);
	
	// tmpi_error(DBG_VERBOSE, "create returns descriptor: %d\n", descriptor-1);

	return descriptor-1;
}


int Verfs::create(guid_t GUID, guid_t base_GUID, int &version){
  tmpi_error(DBG_CALLER, "Create with parent not implemented in C MODE");
  return -1;
}

int Verfs::dumpObject(int desc)
{
  return 0;
}

int Verfs::flush(int desc)
{
	int ret;
	VerfsFileInfo * vfi = check_desc(desc);
	if (vfi == NULL)
		return -1;
	//pthread_mutex_lock(&vfi->fiLock);
	ret = (fcache->flush(vfi->name))?0:-1;
	//pthread_mutex_unlock(&vfi->fiLock);
	return ret; 
}

/**
 * Sets the fileName to be the name of the file that corresponds to the GUID.
 * includes the path
 */
// void Verfs::getFileName(char *fileName, guid_t &guid){
// 	char dirName[128] = "";

// 	if(SCSD_DEBUG){
// 		for(int i=0; i<2; i++){
// 			sprintf(&(dirName[i*2]), "%c/", h2c(guid.data[i] % 16));
// 		}
// 	}

// 	else{						// not tested
// 		for(int i=0; i<4; i++){
// 			sprintf(&(dirName[i*2]), "%c/", h2c(guid.data[i] % 16));
// 		}
// 	}

// 	sprintf(fileName, "%s/%s%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X.s",
// 			DIR_DATA, dirName,
// 			guid.data[0],  guid.data[1],  guid.data[2],  guid.data[3],
// 			guid.data[4],  guid.data[5],  guid.data[6],  guid.data[7], 
// 			guid.data[8],  guid.data[9],  guid.data[10], guid.data[11], 
// 			guid.data[12], guid.data[13], guid.data[14], guid.data[15]);
// 	//printf("FILE NAME IS %s\n", fileName);
// }

/**
 * set iterator to the beginning and get first
 * @return true if successful, false if there is no first element or if 
 * failed.
 */
bool Verfs::getFirst(guid_t &guid)
{
	int pIndex;
	unsigned int valSize = sizeof(int);
	unsigned int keySize = sizeof(guid_t);
	
	if (guid2partition->get_first(&guid, &keySize,
								  &pIndex, &valSize) == 0) return true;
	return false;
}

/** 
 * Traverses the GUID list. 
 * @return true if operation is successful, false if operation fails or end
 * of the list is reached.
 */
bool Verfs::getNext(guid_t &guid)
{
	int pIndex;
	unsigned int valSize = sizeof(int);
	unsigned int keySize = sizeof(guid_t);

	if (guid2partition->get_next(&guid, &keySize, 
								  &pIndex, &valSize) == 0) return true;
	return false;
}

/**
 * common initializations among constructors.
 */
bool Verfs::init(){

	if (pthread_mutex_init(&d2fLock, NULL) != 0) 
		return false;
	if(pthread_mutex_init(&g2pLock, NULL) != 0)
		return false;

	char db_name[MAX_LENGTH];

	if (homeDir == NULL)
		sprintf(db_name, "guid2partition.db");
	
	else
		sprintf(db_name, "%s/guid2partition.db", homeDir);

	unlink(db_name);			// persistent hast not reused

	fcache = new File_Cache(800);

	if(time(&lastDIUpdate) <0)
		return -1;

	guid2partition = 
		new PsHash(db_name, sizeof(guid_t), 
				   sizeof(int),100, 100*(sizeof(guid_t)+sizeof(int)));
	if (!guid2partition->is_loaded()) {
		tmpi_error(DBG_FATAL, "guid2partition already exists.\n");
		delete guid2partition;
		guid2partition = new PsHash(db_name,  
									100*(sizeof(guid_t)+ sizeof(int)));
		if (!guid2partition->is_loaded()) {
			tmpi_error(DBG_FATAL, "Cannot load database.\n");
			return false;
		}
	}

	return true;
}

/**
 * since fcache is being used, just assigns a new descriptor 
 * problem: runs out of descriptors
 * returns -1 if object not found.
 */
int Verfs::open(guid_t guid, int flags, int &version){
	int pIndex;
	unsigned int valSize = sizeof(int);

	// get the locks
	if(pthread_mutex_lock(&d2fLock) != 0)
		return -1;
	if(pthread_mutex_lock(&g2pLock) != 0){
		pthread_mutex_unlock(&d2fLock);
		return -1;
	}

	if( guid2partition->get( &guid, sizeof(guid_t), 
							 &pIndex, &valSize ) != 0 ){
		tmpi_error(DBG_VERBOSE, "GUID to open not found\n");
		pthread_mutex_unlock(&g2pLock);
		pthread_mutex_unlock(&d2fLock);
		return -1; // not fnd
	}

	VerfsFileInfo *fi = new VerfsFileInfo(wDirs[pIndex], guid);
	
	// tmpi_error(DBG_VERBOSE, "file name %s\n", fi->name);

	pair<int, VerfsFileInfo*> *d2vfi;
	d2vfi = new pair<int, VerfsFileInfo*>(descriptor, fi);
	if(desc2fi.insert(*d2vfi).second == false){
		tmpi_error(DBG_INTERNAL, "insert into map failed\n");
		pthread_mutex_unlock(&g2pLock);
		pthread_mutex_unlock(&d2fLock);
		return -1;
	}

	descriptor++;

	// release the locks
	pthread_mutex_unlock(&g2pLock);
	pthread_mutex_unlock(&d2fLock);

	return descriptor-1;
}

int Verfs::read(int desc, unsigned int offset, char* buff, unsigned int len)
{
	//int ret;
	VerfsFileInfo * vfi = check_desc(desc);
	if (vfi == NULL){
		tmpi_error(DBG_VERBOSE, "Descriptor %d not found\n");
		return -1;
	}
	// tmpi_error(DBG_VERBOSE, "file name %s\n", vfi->name);
	//pthread_mutex_lock(&vfi->fiLock);
	//ret = fcache->read(vfi->name, offset, buff, len)?0:-1;
	//pthread_mutex_unlock(&vfi->fiLock);
	//tmpi_error(DBG_VERBOSE, "Read len %d, status %d\n", len, ret);
	return  fcache->read(vfi->name, offset, buff, len);
}


bool Verfs::refreshDiskInfos(void){

	bool success = true;
	int ndisks=0;

	if (wDirCnt>0) {
		for (int i=0; i<wDirCnt; i++) {
			ResMon::DISK *d=ResMon::get_disk(wDirs[i]);
			if ((d->good)&&(d->btotal>0)) {
				ndisks++;

				DiskInfo dsk;
				dsk.btotal=d->btotal;
				dsk.bfree=d->bavail;

				diskInfos[i] = dsk;
				delete d;
			}
			else {
				
				diskInfos[i].btotal = -1;
				diskInfos[i].bfree = -1;

				tmpi_error(DBG_VERBOSE, "Mount point not found: %s.\n", 
						   wDirs[i]);
				success = false;
			}
		}
	}
	return success;
}

/**
 * @return 0 if unlink successful, -1 otherwise
 */
int Verfs::remove(guid_t guid)
{
	int pIndex;
	int ret;
	unsigned int valSize = sizeof(int);
	
	if(pthread_mutex_lock(&g2pLock)!=0)
		return -1;
	
	if( guid2partition->get(&guid, sizeof(guid_t), &pIndex, &valSize) != 0){
		tmpi_error(DBG_VERBOSE, "GUID to be deleted is not found\n");
		pthread_mutex_unlock(&g2pLock);
		return -1;				// not found
	}
	
	VerfsFileInfo *fi = new VerfsFileInfo(wDirs[pIndex], guid);
	
	ret = (fcache->unlink(fi->name))?0:-1;
	
	pthread_mutex_unlock(&g2pLock);
	
	if (ret != 0)
		tmpi_error(DBG_VERBOSE, "Remove failed, status: %d\n", ret);

	return ret;
}

// void Verfs::setPublisher(Publisher& p){
// 	publisher = &p;
// }

/**
 * Truncates the file
 * @return new length if successful, -1 otherwise.
 */
int Verfs::truncate(int desc, unsigned int length)
{
	int ret;
	VerfsFileInfo * vfi = check_desc(desc);
	if (vfi == NULL)
		return -1;

	//pthread_mutex_lock(&vfi->fiLock);
	if (fcache->truncate(vfi->name, length) == true) {
		// tmpi_error(DBG_VERBOSE, "Truncated to %d bytes.\n", length);
        ret = length;
	}
	else {
		tmpi_error(DBG_VERBOSE, "Truncated failed.\n");
		ret = -1;
	}
	//pthread_mutex_unlock(&vfi->fiLock);

	return ret;
}

void Verfs::traverse(char *f, int partNum)
{
	mode_t m;

	if (get_mode(f, m)<0)
		return;

	//if (pthread_mutex_lock(&g2pLock) != 0)
	//	return;

	if (S_ISREG(m)) {
		char *req=strdup(f);
		char hex[5];
		if (req) {
			/* regular file : add to guid2partition table */

			/* convert string to GUID */
			guid_t guid;

			int offset=0;		// last position of /
			for(int a=0; req[a]!='\0'; a++){
				if(req[a] == '/') offset = a;
			}

			long dec;

			//printf("%s\n", req);
			for(unsigned int i=0; i<2*sizeof(guid_t); i+=2){
				sprintf(hex, "0x%c%c", req[i+offset+1], req[i+2+offset]);
				//printf("%s:", hex);
				dec= strtol(hex, NULL, 16);	// convert hex to int
				
				guid.data[i/2] = (char)dec;
				//printf("%ld:", dec);
			}
			// tmpi_error(DBG_VERBOSE, "found %s\n", req);

			guid2partition->set(&guid, sizeof(guid_t), &partNum,sizeof(int));
		}
	}

	else if (S_ISDIR(m)) {
		DIR *dir=opendir(f);
		if (!dir){
			pthread_mutex_unlock(&g2pLock);
			return;
		}
		struct dirent *de=NULL;
		queue<char *> dirlist;
		while ((de=readdir(dir))) {
			if ((de->d_name[0]=='.')&&(de->d_name[1]=='\0'))
				continue;

			if ((de->d_name[0]=='.')&&(de->d_name[1]=='.')
				&&(de->d_name[2]=='\0'))
				continue;

			char *d_name=strdup(de->d_name);
			if (d_name) {
				dirlist.push(d_name);
			}
		}
		closedir(dir);

		char ff[1024];
		while (!dirlist.empty()) {
			char *d_name=dirlist.front();
			dirlist.pop();
			snprintf(ff, 1024, "%s/%s", f, d_name);
			free(d_name);
			traverse(ff, partNum);
		}
	}
	//pthread_mutex_unlock(&g2pLock);
	return;
}

/**
 * Constructor
 */
Verfs::Verfs() :
	descriptor(0),
	fcache(NULL),
	guid2partition(NULL),
	wDirCnt(1),
	wDirs(NULL),
	diskInfos(NULL),
	lastDIUpdate(0),
	homeDir(NULL)
{
	wDirs = new char *[1];
	wDirs[0] = ".";

	char dirName[1024];

	if (!init()) throw -1;

	if(check_format(wDirs[0], true) == false){
		tmpi_error(DBG_CALLER, "Directory %s not correctly formatted\n", 
				   wDirs[0]);
		throw (-1);
	}
	
	sprintf(dirName, "%s/%s", wDirs[0], DIR_DATA);

	if (pthread_mutex_lock(&g2pLock)) 
		throw -1;
	traverse(dirName, 0);
	pthread_mutex_unlock(&g2pLock);
	
	diskInfos = new DiskInfo[wDirCnt];
}

/**
 * Constructor 2
 */
Verfs::Verfs(char **wds, int cnt) : 
	descriptor(0),
	fcache(NULL),
	guid2partition(NULL),
	wDirCnt(cnt),
	wDirs(wds),
	diskInfos(NULL),
	lastDIUpdate(0),
	homeDir(NULL)
{
	int i;
	char dirName[MAX_LENGTH];

	if (!init()) throw -1;

	for(i=0; i<wDirCnt; i++){
		if(check_format(wDirs[i], true) == false){
			tmpi_error(DBG_CALLER, 
					   "Directory %s not correctly formatted\n", wDirs[i]);
			throw(-1);
		}
		sprintf(dirName, "%s/%s", wDirs[i], DIR_DATA);
		if (pthread_mutex_lock(&g2pLock)) 
			throw -1;
		traverse(dirName, i);
		pthread_mutex_unlock(&g2pLock);
	}

	diskInfos = new DiskInfo[wDirCnt];
}

/**
 * Constructor 3
 * takes a NULL terminated array of working directories as parameter.
 */
Verfs::Verfs(char **wds, char* hd) :
	descriptor(0),
	fcache(NULL),
	guid2partition(NULL),
	wDirCnt(0),
	wDirs(wds),
	diskInfos(NULL),
	lastDIUpdate(0),
	homeDir(hd)
{
	int i;
	char dirName[MAX_LENGTH];

	if (!init()) throw -1;

	for(i=0; wDirs[i]!=NULL; i++){
		if(check_format(wDirs[i], true) == false){
			tmpi_error(DBG_CALLER, 
					   "Directory %s not correctly formatted\n", wDirs[i]);
			throw(-1);
		}
		sprintf(dirName, "%s/%s", wDirs[i], DIR_DATA);
			if (pthread_mutex_lock(&g2pLock)) 
		throw -1;
		traverse(dirName, i);
		pthread_mutex_unlock(&g2pLock);
		wDirCnt++;
	}
	diskInfos = new DiskInfo[wDirCnt];
}

Verfs::~Verfs(){
	delete guid2partition;
	delete fcache;
	delete diskInfos;
	pthread_mutex_destroy(&d2fLock);
	pthread_mutex_destroy(&g2pLock);
}

int Verfs::write(int desc, unsigned int offset, char* data, unsigned int len)
{
	//int ret;
	VerfsFileInfo * vfi = check_desc(desc);
	if (vfi == NULL)
		return -1;

	//pthread_mutex_lock(&vfi->fiLock);
	return fcache->write(vfi->name, offset, data, len);
	//pthread_mutex_unlock(&vfi->fiLock);
}






#endif
