// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: verfs.h,v $ -  Version based file system header file.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/08 19:14:28 $ by $Author: gulbeden $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */


#ifndef __VERFS_H__
#define __VERFS_H__

#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <pthread.h>
#include <map>

#include "../guid/guid.h"		/* from cbs */
//#include "../../daemon/server/sorrento_types.h"	// ObjID

#define SCSD_DEBUG 1
#define DIR_DATA ".data"
#define DIR_DB ".db"
#define DI_UPDATE_INTERVAL 1	// update interval for disk infos array

class File_Cache;
class PsHash;
class VerfsFileInfo;

typedef guid_t HostID;
typedef guid_t ObjID;

struct HostID_cmp : public std::less<HostID> {
	bool operator()(const HostID &x, const HostID &y) const {
		return(memcmp(x.data, y.data, sizeof(HostID))<0);
	}
};

struct ObjID_cmp : public std::less<ObjID> {
	bool operator()(const ObjID &x, const ObjID &y) const {
		return(memcmp(x.data, y.data, sizeof(ObjID))<0);
	}
};

struct Guid_cmp : public std::less<guid_t> {
	bool operator()(const guid_t &x, const guid_t &y) const {
		return(memcmp(x.data, y.data, sizeof(guid_t))<0);
	}
};

struct DiskInfo {
	int btotal;
	int bfree;
};


class Verfs{
public:
	Verfs();
	Verfs(char **wds, char* hd); // this one is called by the server
	Verfs(char **wds, int cnt);
	~Verfs();

	int create(guid_t GUID, int &version);
	int create(guid_t GUID, guid_t base_GUID, int &version);
	int open(guid_t GUID, int flags, int &version);
	int close(int desc, int &version);
	int read(int desc, unsigned int offset, char* buffer, unsigned int len);
	int write(int desc, unsigned int offset, char* data, unsigned int len);
	int append(int desc, char* data, unsigned int len);
	int truncate(int desc, unsigned int len);
	int remove(guid_t GUID);
	int flush(int desc);
	int dumpObject(int desc);

	/* for traversing the hashtable */
	bool getFirst(guid_t &guid);
	bool getNext(guid_t &guid);			/* returns NULL at the end */

	bool refreshDiskInfos(void);
	DiskInfo *getDiskInfos(void){ return diskInfos; }

	//void setPublisher(Publisher& p);
	
private:
	bool init();
	//void getFileName(char *fileName, guid_t &guid);
	int getPartition(int desc);
	int calcPartition(guid_t &guid);
	void traverse(char *path, int partNum);
	VerfsFileInfo *check_desc(int desc);

	int descriptor;
	File_Cache *fcache;
	PsHash *guid2partition;

	int wDirCnt;
	char **wDirs;

	DiskInfo *diskInfos;

	time_t lastDIUpdate;
	char* homeDir;

	std::map<ObjID, int, ObjID_cmp>::iterator iter;
	std::map<int, VerfsFileInfo *> desc2fi;

	pthread_mutex_t d2fLock;	// lock for descriptor-to-file-info table
								// and the descriptor variable

	pthread_mutex_t g2pLock;	// lock for guid-to-partition table

};


	

#endif // __VERFS_H__
