// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:t -*-

/************************************************************************
  Copyright (c) 2001, 2002 The Regents of the University of California
  All rights reserved.
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
 
   1. Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
 
   2. Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
 
   3. All advertising materials mentioning features or use of this
   software must display the following acknowledgement: This product
   includes software developed by the University of California, Santa
   Barbara.
 
   4. Neither the name of the University nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND THE AUTHORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/**
 * Name    :  $RCSfile: verfs_fi.cc,v $ -  File info class implementation that
              stores metadata for files.
 * Authors :  Hong Tang, htang@cs.ucsb.edu
 *            Aziz Gulbeden, gulbeden@cs.ucsb.edu
 *            Dept. of Computer Science, UC Santa Barbara
 * Last-Mod:  $Date: 2004/04/08 19:14:28 $ by $Author: gulbeden $
 * Version :  $Revision: 1.3 $
 *
 * ISC project contact: htang@cs.ucsb.edu, tyang@cs.ucsb.edu
 * Web site: http://www.cs.ucsb.edu/projects/isc
 */

# include <string>
# include "verfs_fi.h"
# include "verfs.h"

using namespace std;

VerfsFileInfo::VerfsFileInfo(char *wDir, guid_t &guid) :
  name(NULL)
{
	name = NULL;
	name = new char[strlen(wDir) + sizeof(guid_t)*2 + 100];
	
	if (name == NULL){
		throw -1;
	}
	
	strcpy(name, wDir);
	
	name[strlen(wDir)] = '/';
	getFileName(&(name[strlen(wDir)+1]), guid);
	
	if (pthread_mutex_init(&fiLock, NULL) != 0) 
		throw -1;				// do we really need this lock
}

/**
 * Hexical to char conversion
 */
char VerfsFileInfo::h2c(int h){
  if ((h<0)||(h>15)) {
    return (char)0;
  }
  if (h<10) {
    return '0'+h;
  }
  return 'A'+(h-10);
}

void VerfsFileInfo::getFileName(char *fileName, guid_t &guid)
{
  char dirName[1024] = "";
  
  if(SCSD_DEBUG){
    for(int i=0; i<2; i++){
      sprintf(&(dirName[i*2]), "%c/", h2c(guid.data[i] % 16));
    }
  }
  
  else{						// not tested
    for(int i=0; i<4; i++){
      sprintf(&(dirName[i*2]), "%c/", h2c(guid.data[i] % 16));
    }
  }
  
  sprintf(fileName, "%s/%s%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X.s",
	  DIR_DATA, dirName,
	  guid.data[0],  guid.data[1],  guid.data[2],  guid.data[3],
	  guid.data[4],  guid.data[5],  guid.data[6],  guid.data[7], 
	  guid.data[8],  guid.data[9],  guid.data[10], guid.data[11], 
	  guid.data[12], guid.data[13], guid.data[14], guid.data[15]);
}
